/*				FILE = edi_transfer_defs.h		*/

#define	DEFAULTS		TRUE
#define	MAX_DIR_SIZE		4096
#define	MAX_ERROR_COUNT		25
#define	WAIT_COUNT		60
#define	WAIT_INTERVAL		2
#define	INTERVAL_SECONDS	10	/* reduced for DEMO, set to 60*10 */
#define	COMPLETION_TIME		20
#define	MAX_HOSTNAME		256
#define	LOCK_FILE		"lock_file"
#define	FTAM_PROTOCOL		1
#define	XMODEM_PROTOCOL		2
#define	X25_PROTOCOL		3
#define	DEFAULT_INPUT_PROTOCOL	XMODEM_PROTOCOL

/* These are default definitions that specify where files will be found	*/
/* or will be created.  Change these definitions to relocate the	*/
/* directory base.  The PLATFORM_CFG file will provide the user defined	*/
/* directory definitions that will override these defaults.		*/
#ifndef	BASE_PLATFORM_DIR
#define	BASE_PLATFORM_DIR	"/mnt2/users/markovit/platform"
#endif
#define	BASE_OUTPUT_DIR		BASE_PLATFORM_DIR
#define	BASE_INPUT_DIR		BASE_PLATFORM_DIR"/does_not_exist"
#define	EXECUTABLES_DIR		BASE_PLATFORM_DIR"/bin/"

#define	PLATFORM_CFG		EXECUTABLES_DIR"platform.cfg"
#define	PROGRAM_LOCK_FILE	BASE_PLATFORM_DIR"/files/lockfile"

#ifdef	FTAM_OUT
#define	PROTOCOL	FTAM_PROTOCOL
#define	STARTUP_FILENAME	""
#define	TRANSFER_MODE	"EDI-FTAM-Output"
#define	TRANSFER_TYPE	TYPE_FTAM_OUT
#define	USAGE		"Usage: edi_out_ftam [options]\n\twhere 'options' are passed to ftam_out.\n"
#define	CASE		NO_CASE
#define	OUTPUT_XACTION_DIR	BASE_OUTPUT_DIR"/ftam/xaction"
#define	OUTPUT_CONTROL_DIR	BASE_OUTPUT_DIR"/ftam/control"
#define	OUTPUT_LOCK_DIR		BASE_OUTPUT_DIR"/ftam/lock"
#define	SLAVE_ARGVS	25
#define	SLAVE_PROCESS	EXECUTABLES_DIR"ftam_copy"
static	char	**parms = 0;
#define	XACTION_DIR_INDEX	2
#define	CONTROL_DIR_INDEX	3
#define	LOCK_DIR_INDEX		-1
#endif	/* FTAM_OUT	*/

#ifdef	X25_OUT
#define	PROTOCOL	X25_PROTOCOL
#define	STARTUP_FILENAME	""
#define	TRANSFER_MODE	"EDI-X.25-Output"
#define	TRANSFER_TYPE	TYPE_X25_OUT
#define	USAGE		"Usage: edi_out_x25 [options]\n\twhere 'options' are passed to x25_out.\n"
#define	CASE		NO_CASE
#define	OUTPUT_XACTION_DIR	BASE_OUTPUT_DIR"/x25/xaction"
#define	OUTPUT_CONTROL_DIR	BASE_OUTPUT_DIR"/x25/control"
#define	OUTPUT_LOCK_DIR		BASE_OUTPUT_DIR"/x25/lock"
#define	SLAVE_ARGVS	25
#define	SLAVE_PROCESS		EXECUTABLES_DIR"x25_out"
static	char	**parms = 0;
#define	XACTION_DIR_INDEX	-1
#define	CONTROL_DIR_INDEX	-1
#define	LOCK_DIR_INDEX		-1
#endif	/* X25_OUT	*/

#ifdef	XMODEM_OUT
#define	PROTOCOL	XMODEM_PROTOCOL
#define	STARTUP_FILENAME	"xmodem_startup_definitions"
#define	TRANSFER_MODE	"EDI-XMODEM-Output"
#define	TRANSFER_TYPE	TYPE_XMODEM_OUT
#define	USAGE		"Usage: xmodem_out [options]\n\twhere 'options' are passed to xmodem_copy.\n"
#define	CASE		NO_CASE
#define	OUTPUT_XACTION_DIR	BASE_OUTPUT_DIR"/xmodem/xaction"
#define	OUTPUT_CONTROL_DIR	BASE_OUTPUT_DIR"/xmodem/control"
#define	OUTPUT_LOCK_DIR		BASE_OUTPUT_DIR"/van/lock"
#define	SLAVE_ARGVS	25
#define	SLAVE_PROCESS		EXECUTABLES_DIR"xmodem_copy"
static	char	*parms[] = { "XMIT_PUT", 0};	/* added to slave_argv	*/
#define	XACTION_DIR_INDEX	4
#define	CONTROL_DIR_INDEX	5
#define	LOCK_DIR_INDEX		-1
#endif	/* XMODEM_OUT	*/

#ifdef	FTAM_IN
#define	PROTOCOL	FTAM_PROTOCOL
#define	STARTUP_FILENAME	""
#define	TRANSFER_MODE	"EDI-FTAM-Input"
#define	TRANSFER_TYPE	TYPE_FTAM_IN
#define	USAGE		"Usage: ftam_in [options]\n\twhere 'options' are passed to ftam_copy.\n"
#define	CASE		NO_CASE
#define	INPUT_HOST_FILE		BASE_INPUT_DIR"/remote_host_list"
#define	INPUT_XACTION_DIR	BASE_INPUT_DIR"/ftam/xaction"
#define	INPUT_CONTROL_DIR	BASE_INPUT_DIR"/ftam/control"
#define	SLAVE_ARGVS	25
#define	SLAVE_PROCESS	EXECUTABLES_DIR"ftam_copy"
static	char	**parms = 0;	// add to slave_argv
#define	INPUT_DIR_INDEX		1
#define	INPUT_HOST_INDEX	6
#define	INPUT_BLOCK_INDEX	-1
#define	FTAM_INPUT_LOCK_FILE	BASE_INPUT_DIR"/input_lock_file"
#define	DEFAULT_INPUT_CTRL_FILE	BASE_INPUT_DIR"/input_ctrl_file"
#endif	/* FTAM_IN	*/

#ifdef	XMODEM_IN
#define	PROTOCOL	XMODEM_PROTOCOL
#define	STARTUP_FILENAME	"van_startup_definitions"
#define	TRANSFER_MODE	"EDI-XMODEM-Input"
#define	TRANSFER_TYPE	TYPE_XMODEM_IN
#define	USAGE		"Usage: xmodem_in [options]\n\twhere 'options' are passed to xmodem_copy.\n"
#define	CASE		NO_CASE
#define	INPUT_HOST_FILE		BASE_INPUT_DIR"/remote_host_list"
#define	INPUT_XACTION_DIR	BASE_INPUT_DIR"/xmodem/xaction"
#define	INPUT_CONTROL_DIR	BASE_INPUT_DIR"/xmodem/control"
#define	SLAVE_ARGVS	25
#define	SLAVE_PROCESS	EXECUTABLES_DIR"xmodem_copy"
static	char	*parms[] = {"XMIT_GET", ERROR_DEF, 0};	/* add to slave_argv	*/
#define	INPUT_DIR_INDEX		1
#define	INPUT_HOST_INDEX	6
#define	INPUT_BLOCK_INDEX	-1
#define	XMODEM_INPUT_LOCK_FILE	BASE_INPUT_DIR"/input_lock_file"
#define	DEFAULT_INPUT_CTRL_FILE	BASE_INPUT_DIR"/input_ctrl_file"
#endif	/* XMODEM_IN	*/

#ifdef	X25_IN
#define	PROTOCOL	X25_PROTOCOL
#define	STARTUP_FILENAME	""
#define	TRANSFER_MODE	"EDI-X25-Input"
#define	TRANSFER_TYPE	TYPE_X25_IN
#define	USAGE		"Usage: x25_in [options]\n\twhere 'options' are passed to x25_copy.\n"
#define	CASE		NO_CASE
#define	INPUT_HOST_FILE		BASE_INPUT_DIR"/remote_host_list"
#define	INPUT_XACTION_DIR	BASE_INPUT_DIR"/x25/xaction"
#define	INPUT_CONTROL_DIR	BASE_INPUT_DIR"/x25/control"
#define	SLAVE_ARGVS	25
#define	SLAVE_PROCESS	EXECUTABLES_DIR"x25_copy"
static	char	**parms = 0;	/* add to slave_argv	*/
#define	INPUT_DIR_INDEX		1
#define	INPUT_HOST_INDEX	6
#define	INPUT_BLOCK_INDEX	-1
#define	X25_INPUT_LOCK_FILE	BASE_INPUT_DIR"/input_lock_file"
#define	DEFAULT_INPUT_CTRL_FILE	BASE_INPUT_DIR"/input_ctrl_file"
#endif	/* X25_IN	*/
