// FILE = linklist.h


//
// cell:
//
// Contains pointers allowing any set of objects to be maintained by a
// double linked link.
//
 
class Cell {
friend class LinkList;
		Cell( void *e, Cell *p, Cell *n ) {info=e; prev=p; next=n;}
	void	*info;
	Cell	*prev;
	Cell	*next;
};


//
// linklist:
//
// Maintains the front and rear list locations.  This class is used as a
// superclass for more specialized types of linked lists.
//

class LinkList {
public:
	int	empty() 		{ return front == NULL; }
	int	find( void * );		// find specific cell info in list
	void	del( void * );		// delete cell with specific info
	void	set_search()		{ curr = front; 	}
	void	*current();		// return current cell's info
protected:
		LinkList() 		{ front = rear = curr = NULL; }
		~LinkList();		// del all cells
	void	push( void * );		// add to front
	void	enter( void * );	// add to rear
	void	*pop();			// del from front
	void	*remove();		// del from rear
private:
	Cell 	*front, *rear, *curr;
};


//
// queue:
//
// A subclass of linklist allowing objects to be added at the rear and
// removed from the front.
//

class Queue : private LinkList {
public:
		Queue()			{ }
	void 	add( void *e ) 		{ enter( e ); }
	void 	*remove()		{ return pop(); }
		LinkList::current;
		LinkList::empty;
		LinkList::find;
		LinkList::del;
		LinkList::set_search;
};


//
// stack:
//
// A subclass of linklist allowing objects to be added and removed from
// the front.
//

class Stack : private LinkList {
public:
		Stack()			{ }
	void	push( void *e )		{ LinkList::push( e ); }
	void	*pop()			{ return LinkList::pop(); }
		LinkList::empty;
		LinkList::find;
		LinkList::del;
};


//
// deque:
//
// A subclass of linklist allowing objects to be added and removed from
// the front and the rear.
//

class Deque : private LinkList {
public:
		Deque()			{ }
	void	add_first( void *e )	{ push( e ); }
	void	add_last( void *e )	{ enter( e ); }
	void	*remove_first()		{ return pop(); }
	void	*remove_last()		{ return remove(); }
		LinkList::empty;
		LinkList::find;
		LinkList::del;
};
