
// FILE: log.h

#define BUF_SIZE      256
#define MES_SIZE      160
#define DATE_LEN      25

#define DELETING_INTER  "Deleting Interchange"
#define FILE_NOT_CREATED	"Could not create file."

// used in x400cntout.CC
#define RETURN_ERROR "Returning ERROR."
#define NOT_CREATED "api_create not successful."
#define NOT_PUT "api_put not successful."
#define NO_USER "User with id code and id qualifier does not exist."
#define DELETE_OBJECT "Deleted private object."


#define	NO_SUCH_PROTOCOL_ID	"Protocol id in TP File is not recognized"
#define	NO_SUCH_TRADING_PARTNER	"No Directory Entry Matched the Interchange's Trading Partner ID-CODE and ID-CODE-QUALIFIER"
#define	NO_SUCH_LOCAL_USER	"No Local User Matched the Interchange's ID-CODE and ID-CODE-QUALIFIER"
#define	CONFIG_OPEN_ERROR	"The Platform's Configuration File Could Not Be Opened"
#define	NO_LOCAL_USERS		"At Least One Local EDI User Must Be Configured"


#define	MOVING_INTERCHANGE	"The Current Interchange Will Be Relocated to the ERROR-INTERCHANGE Directory"
#define	DELETING_INTERCHANGE	"The Current Interchange Will be Deleted"
#define	EXITING			"The Platform Will Exit NOW!"
#define	INFORMATION		"This entry is for information only; no action is performed by the platform"


typedef enum {
     NONE_LOG, 
     BASIC_LOG, 
     TRACE_LOG, 
     INTERCHANGE_LOG, 
     VERBOSE_LOG
 } LOGLEVEL;


//
// CLASS: Log
//
// This class provides access to the log.
//

class Log {

         ofstream  *logfile;
         char      *logfile_name; // Name of log output file.
         LOGLEVEL   loglevel;    
         char      *loc;
    
	 char	   *ascii( OM_return_code );

 public:
                   Log(LOGLEVEL, char *); // Log constructor.
                  ~Log();                 // Log destructor.
         
	 void      log_event( char *loc,
                              char *msg,
                              char *action );
         
	 void      log_event( char *, OM_return_code, char * );
         
	 STATUS    set_logfile( char *logfile_name );
         void      set_loglevel( LOGLEVEL );
	 LOGLEVEL  get_loglevel();
	 ofstream  *get_logfile();
};


//
// Log()
//
// Class Members:  loglevel, logfile_name
// Parameters:	   loglevel : the logging level set by user.
//                 log_name : the log file name set by user. 
// Called By:	   public/constructor
//
// This method initializes the class member identifying the loglevel and
// log file name.
//

inline Log::Log( LOGLEVEL log_level, char *log_name ) { 

        // Set the loglevel and file name to what the user desires.
        set_loglevel(log_level);
        // set and open the "logfile_name" to be written to.
        set_logfile(log_name);
}

inline Log::~Log () {

  // If the logfile still exists close it.
  if (logfile)
    logfile->close();
}

inline LOGLEVEL Log::get_loglevel() {
	return loglevel;
}

inline ofstream *Log::get_logfile() {
	return logfile;
}

