#include <dirent.h>
#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include <sys/file.h>
#include <string.h>
#include "log.h"


//**********************************************************************
//
//  PROJECT:       EDI Communicaitons Platform
//  PROGRAMMER:    John J. Garguilo (105-56-6043)
//  CREATION DATE: 7/8/92
//  FILE NAME:    ~platform/src/lib/log/log_mgr.cc
//  DESCRIPTION:  This is the source file for the library class Log.
//                This file contains two methods named log_event() and 
//                set_logfile().  These methods will enable programs,
//                perhaps running concurrently, to write time-tagged 
//                text messages to a default or specified log file.   
//                The log file name can be changed at any time.
//
//***********************************************************************

//
// Method: set_logfile
// Class: Log
// Calls: new, close()
//
// Description:  This method allows the user to change the file used for
//               the logging of messages for a particular process.
//               The new log file will be name by the argument "logfile_name".
// 

STATUS Log::set_logfile(char *logfile_name)
{
   ofstream* newlogFile; // The new file to be opened.

   // Try to open the file identified by "logfile_name".  If the open
   // is not successful return UNSUCCESSFUL.

   newlogFile = new ofstream(logfile_name, ios::out|ios::app);
   // Check to make sure the open is successful.
   if (!newlogFile) // open failed? {
#ifdef DEBUG
cout << "\"" << logfile_name << "\" not successfully opened." << endl;
#endif
      return (UNSUCCESSFUL); // Keep the existing log file open.
      }
   else {
     // logfile_name was successfully opened so the previous log file
     // needs to be closed.  The new logfile will now be used for
     // all subsequent log_event calls.
	 if (!logfile)       // Make sure the logFile was opened 
           logfile->close(); // before it can be closed.
	 // Give the new file stream to the "logfile" stream. 

#ifdef DEBUG
cout << "New logfile: \"" << logfile_name << "\" created." << endl;
#endif
         logfile = newlogFile; 
   }
  return(SUCCESSFUL);
 }

//
// Method: log_event
// Class: Log
// Calls: asctime(), strncpy(), strcpy(), strncat(), strcat(), time()
//
//  Description:  This function takes the argument information, formats it
//                into  a time - tagged line of text, and appends it to the
//                current log file.
// 

void Log::log_event (char *loc, char *msg, char *action)
{

  char date[DATE_SIZE];   // The date and time when the logging took place. //
  char buffer[BUF_SIZE];  // Buffer to store the string to be written to    //
                          // the logfile.                                   //
  time_t totsecs;         // Number of total seconds since end of 1969.     //
  int i;                  // Counter.                                       //

  // First clear out the buffers. //
     for (i = 0; i < BUF_SIZE; i++)
       buffer[i] = ' ';
     for (i = 0; i < MES_SIZE; i++)
         fmt_message[i] = ' ';
     for (i = 0; i < DATE_SIZE; i++)
         date[i] = ' ';

  // Now record the date and time. //
     totsecs = time(NULL);

     strncpy(date, asctime(localtime(&totsecs)), (DATE_SIZE - 1)); 
     date[DATE_SIZE - 1] = '\0';

 // Copy the value of date into the buffer (which will eventually
 // be written). 
     strncpy(buffer, date, DATE_SIZE); 

 // Concatenate the message line of text on the end of the buffer 
 // which already contains the date.
  strncat(buffer, ":MSG:", 5);
  strncat(buffer, msg, strlen(msg));

 // Concatenate the "action" to be taken on the end of the buffer 
 // which already contains the date and message.
  strncat(buffer, ":ACTION:", 8);
  strncat(buffer, action, strlen(action));

  // Now write the buffer to the appropriate log file. //
    (*logFile) << buffer << endl;
}


//
// Method: set_loglevel
// Class: Log
// Calls: none
//
//  Description:  This method sets the loglevel the user desires.
// 

inline Log::set_loglevel(LOGLEVEL log_level) {

    loglevel = log_level;

{



