//				FILE = ipc_msg.C

#include	"functions.h"
#include	"msg_defs.h"

//	The constructor has four parameters, defaults supplied if called
//	with only two or three parameters.
	FUNCTION ipc_msg::ipc_msg(key_t key,long type,int max_text,int flags)	{
	if (max_text > MSG_MAX_TEXT)	max_text = MSG_MAX_TEXT;
	if (max_text < 0)		max_text = MSG_DEFAULT_TEXT;
	if ((msgp = (struct msg_cmd *) new char (sizeof(struct msg_cmd) +
			max_text)) != 0)
		if ((msg_qid = msgget(key,(IPC_CREAT | 0764))) == -1)	{
			delete [] msgp;
			msgp = 0;
		}	else	{
			msg_max_text = max_text;
			msg_my_type = type;
			msg_key = key;
			msg_flags = flags;
			if (!msg_my_pid)	msg_my_pid = getpid();
			msgp->msg_text[msg_max_text] = '\0';
		}
}


	FUNCTION ipc_msg::~ipc_msg()	{
	if (msgp)	{
		if (msg_flags & MSG_RMID)
			if (msgctl(msg_qid,IPC_RMID,0) == -1)
				perror("~ipc_msg()");
		delete [] msgp;
		msgp = 0;
	}
}


// Public Functions

// Receive a waiting message, return T/F.  Will get next message with 
// specified 'type' if type is positive or any message with type field
// less than or equal to magnitude of 'type' if 'type' is negative.
	FUNCTION ipc_msg::msg_rcv(long type)	{
	int	ret = FALSE;
	if (!msgp)	goto	fini;
	msgp->msg_type = type;
	if (msgrcv(msg_qid,(struct msgbuf *)msgp,
		sizeof(struct msg_cmd)+msg_max_text-sizeof(type),
		type,IPC_NOWAIT)  != -1)	ret = TRUE;
	else	if (errno != ENOMSG)	perror("msg_rcv");
fini:	return(ret);
}

// Return the 'type' of received message
	FUNCTION ipc_msg::msg_get_type()
	{return (msgp ? msgp->msg_type : 0);	}

// Return the 'command' from the received message
	FUNCTION ipc_msg::msg_get_cmd()
	{return (msgp ? msgp->msg_cmd : 0);	}

// Return the 'sub_command' from the received message
	FUNCTION ipc_msg::msg_get_sub_cmd()
	{return (msgp ? msgp->msg_sub_cmd : 0);	}

// Return the text of the received message
	char	* FUNCTION ipc_msg::msg_get_text()
	{ return (msgp ? msgp->msg_text : 0);	}


// Send a message with a specified 'type' field
	FUNCTION ipc_msg::msg_snd(long type,int cmd,char *text,int sub_cmd)	{
	int	ret = FALSE;
	if (!msgp)	goto	fini;
	msgp->msg_type = type;
	msgp->msg_snd_pid = msg_my_pid;
	msgp->msg_snd_type = msg_my_type;
	msgp->msg_cmd = cmd;
	msgp->msg_sub_cmd = sub_cmd;
	if (text)	strncpy(msgp->msg_text,text,msg_max_text);
	else		msgp->msg_text[0] = '\0';
	if (msgsnd(msg_qid,(struct msgbuf *)msgp,
		sizeof(struct msg_cmd)+strlen(msgp->msg_text)-sizeof(type),
		0) != -1)	ret = TRUE;
	else perror("msg_snd");
fini:	return(ret);
}

// Print the private ipc_msg variables, the contents of a message and the
// system structures associated with the queue_id. For debug only.
	FUNCTION ipc_msg::msg_print()	{
	sprintf(log_str,"msg_print: Dump of msg variables\n");
	logit(log_str);
	sprintf(log_str,"	max_text = %d. pid = %x my_type = %d. qid = %d.\n",
		msg_max_text,msg_my_pid,msg_my_type,msg_qid);
	logit(log_str);
	msg_print_cmd();
	msg_print_ds();
	return(TRUE);
}

// Print the contents of a message
	FUNCTION ipc_msg::msg_print_cmd()	{
	if (msgp)	{
		sprintf(log_str,"msg_print_cmd: Dump of msg_cmd = 0x\n",msgp);
		logit(log_str);
		sprintf(log_str,"	type = %d. pid = %x snd_type = %d. cmd = %d. sub_cmd = %d.\n",
			msgp->msg_type,msgp->msg_snd_pid,
			msgp->msg_snd_type,msgp->msg_cmd,
			msgp->msg_sub_cmd);
		logit(log_str);
		sprintf(log_str,"	text = '%s'\n",msgp->msg_text);
		logit(log_str);
	}
	return(TRUE);
}

// Private Functions

// Print the system structure msqid_ds
	FUNCTION ipc_msg::msg_print_ds()	{
	struct	msqid_ds ds;
	int	ret = FALSE;
	if (msgctl(msg_qid,IPC_STAT,&ds) == -1)
		goto	fini;
	sprintf(log_str,"msg_print_ds: Dump of msqid_ds = %d.\n",msg_qid);
	logit(log_str);
	msg_print_ipc((struct ipc_perm *)&ds);
	sprintf(log_str,"	first = %x last = %x\n",
		ds.msg_first,ds.msg_last);
	logit(log_str);
	sprintf(log_str,"	cbytes = %d. qnum = %d. qbytes = %d.\n",
		ds.msg_cbytes,ds.msg_qnum,ds.msg_qbytes);
	logit(log_str);
	sprintf(log_str,"	lspid = %x lrpid = %x\n",
		ds.msg_lspid,ds.msg_lrpid);
	logit(log_str);
	sprintf(log_str,"	stime = %x rtime = %x ctime = %x\n",
		ds.msg_stime,ds.msg_rtime,ds.msg_ctime);
	logit(log_str);
	ret = TRUE;
fini:	return(ret);
}

// Print the system structure ipc_perm (part of msqid_ds)
	FUNCTION ipc_msg::msg_print_ipc(struct ipc_perm *perm)	{
	int	ret = FALSE;
	if (!perm)	goto	fini;
	sprintf(log_str,"msg_print_ipc: Dump of ipc_perm = %x\n",perm);
	logit(log_str);
	sprintf(log_str,"	uid = %d. gid = %d. cuid = %d. cgid = %d.\n",
		perm->uid,perm->gid,perm->cuid,perm->cgid);
	logit(log_str);
	sprintf(log_str,"	mode = %o seq = %d. key = %x\n",
		perm->mode,perm->seq,perm->key);
	logit(log_str);
	ret = TRUE;
fini:	return(ret);
}
