// FILE:  api_obj.C

#include	<xom.h>
#include	<xmh.h>
#include	<iostream.h>
#include	<fstream.h>
#include	<assert.h>
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/api_obj.h"

//
// PUBLIC: create a private object.
//

STATUS APIObject::api_create( char *class_string ) {

	OM_return_code		om_status;
	OM_object_identifier	class_id;

	MAKE_OM_STRING( class_id, class_string, 7 );

	om_status = om_create( class_id, 	// Identify the class
		   	       OM_TRUE,         // Use default values
		   	       workspace,       // Value from mt_open()
		   	       &api_private	// Output: private object
			     );

	if ( om_status != OM_SUCCESS ) {
#ifdef	DEBUG
		cout << "length=" << class_id.length << endl;
		cout << "elements=" << class_id.elements << endl;
#endif
		log->log_event( "api_create", om_status, EXITING );
		return ERROR;
	}
	else {
		object_to_delete |= PRIVATE;
		return SUCCESS;
	}
}


//
// PUBLIC: delete an API object.
//

void APIObject::api_delete() {
	OM_return_code	om_status;

	if ( object_to_delete & PRIVATE ) {
		if ( ( om_status = om_delete( api_private ) ) != OM_SUCCESS )
			log->log_event( "api_delete", om_status, INFORMATION );
		object_to_delete &= NO_PRIVATE;
	}

	if ( object_to_delete & PUBLIC ) {
		if ( ( om_status = om_delete( api_public ) ) != OM_SUCCESS )
			log->log_event( "api_delete", om_status, INFORMATION );
		object_to_delete &= NO_PUBLIC;
	}
}


//
// PUBLIC: copy the class' private object to its private object.
//

 STATUS APIObject::api_get( OM_exclusions excl, OM_type_list types ) {

	OM_return_code	om_status;

	om_status = om_get( api_private,// set by mt_start_transfer_in()
			    excl,       // list of exclusions
			    types, 	// get only certain types
			    OM_FALSE,	// copy strings into local char set
			    0,		// used w/ exclude-all-but-these-values
			    0,		// used w/ exclude-all-but-these-values
			    &api_public,// public object set by API
			    &api_total	// # of attribute values public object
		    );
	if ( om_status != OM_SUCCESS ) {
		log->log_event( "api_get", om_status, EXITING );
		return ERROR;
	}
	else {
		object_to_delete |= PUBLIC;
		return SUCCESS;
	}
}


//
// PUBLIC: copy the class' private object to its private object.
//

STATUS APIObject::api_get( OM_exclusions excl ) {

	OM_return_code	om_status;

	om_status = om_get( api_private,// set by mt_start_transfer_in()
			    excl,       // list of exclusions
			    0, 		// get only certain types
			    OM_FALSE,	// copy strings into local char set
			    0,		// used w/ exclude-all-but-these-values
			    0,		// used w/ exclude-all-but-these-values
			    &api_public,// public object set by API
			    &api_total	// # of attribute values public object
		    );
	if ( om_status != OM_SUCCESS ) {
		log->log_event( "api_get", om_status, EXITING );
		return ERROR;
	}
	else {
		object_to_delete |= PUBLIC;
		return SUCCESS;
	}
}


//
// PUBLIC: copy the class' public object to its private object, replacing 
// all values.
//

STATUS APIObject::api_put() {

	OM_return_code	om_status;

	om_status = om_put( api_private, OM_REPLACE_ALL, api_public, 0, 0, 0 );

	if ( om_status != OM_SUCCESS ) {
		log->log_event( "api_put", om_status, EXITING );
		return ERROR;
	}
	else
		return SUCCESS;
}

	
//
// PUBLIC: copy the class' public object to its private object, replacing
// specific types.
//

STATUS APIObject::api_put( OM_type_list types ) {

	OM_return_code	om_status;

	om_status = om_put( api_private,OM_REPLACE_ALL,api_public,types,0,0 );
	if ( om_status != OM_SUCCESS ) {
		log->log_event( "api_put", om_status, EXITING );
		return ERROR;
	}
	else
		return SUCCESS;
}

	
//
// PUBLIC:  add the class attribute first in the descriptor array.
//

void APIObject::make_class( void *class_oid ) { 
	
	OM_descriptor *desc = &api_public[ 0 ];

	desc->type = OM_CLASS;
	desc->syntax = OM_S_OBJECT_IDENTIFIER_STRING;
	desc->value.string.length = strlen( ( char * ) class_oid );
	desc->value.string.elements = class_oid;
}


//
// PUBLIC:  add the OM_NO_MORE_TYPES attribute to the descriptor array.
//

void APIObject::end_object() {

	OM_descriptor *desc = &api_public[ put_index ];

	desc->type = OM_NO_MORE_TYPES;
	desc->syntax = OM_S_NO_MORE_SYNTAXES;
	desc->value.string.length = 0;
	desc->value.string.elements = 0;
}

//
// PUBLIC:  test if an object belongs to a specific class.
//

BOOL APIObject::is_class( char *class_id ) {

	// use om_instance ??

	if ( ! memcmp( class_id, 
		       api_public[ 0 ].value.string.elements, 
		       strlen( class_id ) ) )
		return TRUE;
	
	return FALSE;
}


void APIObject::put_desc( OM_type	type,
			  OM_syntax	syntax,
			  OM_integer	value ) {

	OM_descriptor *desc = &api_public[ put_index++ ];

#ifdef	DEBUG
	cout << "Putting type/value:  " << type << " / " << value << endl;	
#endif

	desc->type = type;
	desc->syntax = syntax;
	if ( syntax == OM_S_ENUMERATION ) 
		desc->value.enumeration = value; 
	else if ( syntax == OM_S_BOOLEAN ) 
		desc->value.boolean = value;
	else
		desc->value.integer = value;
}


void APIObject::put_desc( OM_type	type,
			  OM_syntax	syntax,
			  char		*value ) {

	OM_descriptor *desc = &api_public[ put_index++ ];

#ifdef	DEBUG
	cout << "Putting type/value:  " << type << " / " << value << 
		" [" << strlen( value ) << "]" << endl;	
#endif

	desc->type = type;
	desc->syntax = syntax;
	desc->value.string.length = (OM_element_position) strlen( value );
	desc->value.string.elements = value;
}

void APIObject::put_desc( OM_type	type,
			  OM_syntax	syntax,
			  APIObject	*value ) {

	OM_descriptor *desc = &api_public[ put_index++ ];

#ifdef	DEBUG
	cout << "Putting type:  " << type << endl;	
#endif
	
	desc->type = type;
	desc->syntax = syntax;
	desc->value.object.object = ( OM_object ) ( value->pri_object() );
}

void APIObject::put_desc( OM_type	   type,
			  OM_public_object value ) {

	OM_descriptor *desc = &api_public[ put_index++ ];
	
#ifdef	DEBUG
	cout << "Putting type:  " << type << endl;	
#endif

	desc->type = type;
	desc->syntax = OM_S_OBJECT;
	desc->value.object.object = ( OM_object ) value;
}



APIObject::APIObject( OM_workspace ws, Log *l ) { 
		api_public = public_desc;
		assert( api_public != 0 );
		workspace = ws;
		log = l; 
		put_index = 1;
		get_index = 0;
		log_index = 0;
		object_to_delete = NO_OBJECT;
	}

APIObject::APIObject( APIObject *obj ) {
		api_public = public_desc;
		assert( api_public != 0 );
		workspace = obj->api_workspace();
		log = obj->api_log();
		put_index = 1;
		get_index = 0;
		log_index = 0;
		object_to_delete = NO_OBJECT;
	}
