// FILE:  config.C

#include	<iostream.h>
#include	<fstream.h>
#include	<string.h>
#include	<assert.h>
#include	<xom.h>
#include	<xedip.h>
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/interchange.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/edi_user.h"
#include	"../../../include/config.h"


//
// PUBLIC:  function to read the configuration file.
//

STATUS Config::read_configuration_file() {
	ifstream	config_file ( CONFIG_FILE, ios::in );
	BOOL		create_config_file;
	STATUS		status = SUCCESS;

	if ( ! config_file ) {
		cout << endl << "Configuration file does not exist.  Do you "
		     << "want to create it [y:n]?  ";
		GET_BOOL( create_config_file );
		if ( create_config_file )
			status |= configure();
		else {
			cout << CONFIG_OPEN_ERROR << endl << EXITING << endl;
			status |= ERROR;
		}
	}
	else {
		config_file.getline( edi_user_output_dir, MAX_GETLINE );
		config_file.getline( trans_input_dir, MAX_GETLINE );
		config_file.getline( ftam_ic_dir, MAX_GETLINE );
		config_file.getline( ftam_ctl_dir, MAX_GETLINE );
		config_file.getline( van_ic_dir, MAX_GETLINE );
		config_file.getline( van_ctl_dir, MAX_GETLINE );
		config_file.getline( van_scr_dir, MAX_GETLINE );

		if ( end_of_config_file( &config_file ) ) {
			cout << NO_LOCAL_USERS << endl << EXITING << endl;
			status |= ERROR;
		}
		else 
			edi_user_list = read_edi_user_list( &config_file );
	}

	return status;
}


//
// PRIVATE:  read the list of EDI users.
//

Queue *Config::read_edi_user_list( ifstream *config_file ) {

	Queue *users = new Queue;
	assert( users != 0 );
	while ( ! end_of_config_file( config_file ) ) {
		EDIUser	*edi_user = new EDIUser;
		assert( edi_user != 0 );
		edi_user->read_edi_user( config_file );
		users->add( edi_user );
	}

	return users;
}


//
// PUBLIC:  match the arguments against the EDI User List in the 
// configuration file.
//

EDIUser *Config::get_local_user( char *id_code, char *id_qual ) {
	Queue	*users;
	EDIUser	*user;

	users = edi_user_list;
	if ( users ) {
		users->set_search();
		user = ( EDIUser * ) users->current();
		while ( user ) {
#ifdef	DEBUG
			cout << "Comparing '" << user->id_code() << "' to '"
			     << id_code << "'" << endl;
			cout << "Comparing '" << user->id_code_qualifier() 
			     << "' to '" << id_qual << "'" << endl;
#endif
			if ( ! strcmp( user->id_code(), id_code ) &&
			     ! strcmp( user->id_code_qualifier(), id_qual ) )
			     	return user;
			user = ( EDIUser * ) users->current();
		}
	}

	return NULL;
}


#ifdef	DEBUG

//
// PUBLIC:  Display the parsed values from the configuration file to the screen.
//

void Config::display_configuration() {
	Queue	*users;
	EDIUser	*user;

	cout << "USER DIR:  " << edi_user_directory_name() << endl;
	cout << "TRAN DIR:  " << transmission_directory_name() << endl;
	cout << "FTAM IC :  " << ftam_interchange_directory_name() << endl;
	cout << "FTAM CTL:  " << ftam_control_directory_name() << endl;
	cout << "VAN  IC :  " << van_interchange_directory_name() << endl;
	cout << "VAN  CTL:  " << van_control_directory_name() << endl;
	cout << "VAN  SCR:  " << van_script_directory_name() << endl;
	users = edi_user_list;
	if ( users ) {
		users->set_search();
		user = ( EDIUser * ) users->current();
		while ( user ) {
			user->display_user();
			user = ( EDIUser * ) users->current();
		}
	}
	else
		cout << "NO USERS REGISTERED!";
}

#endif	// DEBUG


//
// PUBLIC:  Build the configuration file.
//

STATUS Config::configure() {
	ofstream	config_file( CONFIG_FILE, ios::out );
	char		buf[ MAX_LINE ];
	STATUS	status = SUCCESS;


	if ( ! config_file ) {
		cout << "Cannot open " << CONFIG_FILE << endl;
		status |= ERROR;
	}
	else {
		cout << endl << endl;
		cout << "The following questions prompt for the information ";
		cout << "required to configure the" << endl;
		cout << "EDI Communications Platform.  For any question ";
		cout << "requiring a  YES / NO  answer " << endl;;
		cout << "please type n for NO, or y for YES:" << endl;
		
		//
		// Get the directory information.
		//
		
		cout << endl << endl << "Enter User Output Directory [";
		cout << edi_user_output_dir << "]" << endl << ">> ";
		cin.getline( buf, MAX_GETLINE );
		if ( buf[ 0 ] != NULL )
			strcpy( edi_user_output_dir, buf );

		cout << "Enter Transmission Input Directory [";
		cout << trans_input_dir << "]" << endl << ">> ";
		cin.getline( buf, MAX_GETLINE );
		if ( buf[ 0 ] != NULL )
			strcpy( trans_input_dir, buf );
		
		cout << "Enter FTAM Interchange Directory [";
		cout << ftam_ic_dir << "]" << endl << ">> ";
		cin.getline( buf, MAX_GETLINE );
		if ( buf[ 0 ] != NULL )
			strcpy( ftam_ic_dir, buf );
		
		cout << "Enter FTAM Control Directory [";
		cout << ftam_ctl_dir << "]" << endl << ">> ";
		cin.getline( buf, MAX_GETLINE );
		if ( buf[ 0 ] != NULL )
			strcpy( ftam_ctl_dir, buf );
		
		cout << "Enter VAN Interchange Directory [";
		cout << van_ic_dir << "]" << endl << ">> ";
		cin.getline( buf, MAX_GETLINE );
		if ( buf[ 0 ] != NULL )
			strcpy( van_ic_dir, buf );
		
		cout << "Enter VAN Control Directory [";
		cout << van_ctl_dir << "]" << endl << ">> ";
		cin.getline( buf, MAX_GETLINE );
		if ( buf[ 0 ] != NULL )
			strcpy( van_ctl_dir, buf );
		
		cout << "Enter VAN script file [";
		cout << van_scr_dir << "]" << endl << ">> ";
		cin.getline( buf, MAX_GETLINE );
		if ( buf[ 0 ] != NULL )
			strcpy( van_scr_dir, buf );
		
		//
		// Get information about the local users.
		//
		
		get_local_user_data();
		write_configuration( &config_file );
	}
	
	return status;
}


//
// PRIVATE:  Add and Delete users.
//

void Config::get_local_user_data() {
	char	sel;

	if ( edi_user_list == NULL ) {
		edi_user_list = new Queue;
		assert( edi_user_list != 0 );
		edi_user_list->add( add_edi_user() );
	}
	
	do {
		cout << endl << "Menu Selections:" << endl << endl;
		cout << "     [A]dd a user" << endl;
		cout << "     [D]el a user" << endl;
		cout << "     [E]xit configuration" << endl << endl;
		cout << "Select A, D, or E >>  ";
		cin >> sel;
		cin.ignore();	// newline

		LOWER_CASE( sel );
		if ( sel == 'a' )
			edi_user_list->add( add_edi_user() );
		else if ( sel == 'd' )
			delete_edi_user();
		else if ( sel == 'e' )
			;	// do nothing 
		else
			cout << endl << "Invalid selection" << endl;
	}
	while ( sel != 'e' );
}


//
// PRIVATE:  Delete one user.
//

void Config::delete_edi_user() {
	EDIUser	*edi_user;
	char	del_user;

	edi_user_list->set_search();
	while ( ( edi_user = ( EDIUser * ) edi_user_list->current() ) != NULL ){
		cout << endl << "EDI User: " << endl 
		     << "    ID Code: " << edi_user->id_code() << endl
		     << "    ID Qual: " << edi_user->id_code_qualifier() << endl
		     << endl << "Delete User [y:yes, n:no, x:exit delete]?  ";
		cin >> del_user;
		LOWER_CASE( del_user );
		if ( del_user == 'x' )
			break;
		if ( del_user == 'y' )
			edi_user_list->del( edi_user );
	}
}

//
// PRIVATE:  Add one user.
//

EDIUser *Config::add_edi_user() {
	
	EDIUser	*edi_user = new EDIUser;
	assert( edi_user != 0 );
	cout << endl << "Please enter information for a local user:";
	cout << endl << endl;
	edi_user->build_edi_user();

	return edi_user;
}


//
// PRIVATE:  Write the configuration information to the configuration file.
//

void Config::write_configuration( ofstream *config_file ) {
	EDIUser	*edi_user;

	*config_file << edi_user_output_dir << endl;
	*config_file << trans_input_dir << endl;
	*config_file << ftam_ic_dir << endl;
	*config_file << ftam_ctl_dir << endl;
	*config_file << van_ic_dir << endl;
	*config_file << van_ctl_dir << endl;
	*config_file << van_scr_dir << endl;
	edi_user_list->set_search();
	while ( ( edi_user = ( EDIUser * ) edi_user_list->current() ) != NULL )
		edi_user->write_edi_user( config_file );
}


//
// PUBLIC:  match the arguments against the EDI User List in the 
// configuration file.
//

EDIUser *Config::get_local_user( ORName *name ) {
	Queue	*users;
	EDIUser	*user;
	ORName	*local_name;

	users = edi_user_list;
	if ( users ) {
		users->set_search();
		user = ( EDIUser * ) users->current();
		while ( user ) {
			local_name = user->x400_address();
			if ( local_name->compare_or_name( name ) == TRUE )
			     	return user;
			user = ( EDIUser * ) users->current();
		}
	}

	return NULL;
}

