// FILE:  edi_user.C

#include	<iostream.h>
#include	<fstream.h>
#include	<string.h>
#include	<stdlib.h>
#include	<assert.h>
#include	<xom.h>
#include	<xedip.h>
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/interchange.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/edi_user.h"

//
// PUBLIC:  convert the ASCII string of 0's and 1's into their binary
// equivalent values and store into the edi_stds array.
//

void EDIUser::set_edi_standards( char *stds_buf ) {
	int	i;

	for ( i=0; i<MAX_EDI_STANDARDS; i++ )
		edi_stds[ i ] = stds_buf[ i ] - 0x30;
}

#ifdef	DEBUG

//
// PUBLIC:  display all attributes of a user to the screen.
//

void EDIUser::display_user() {
	int	i;

	cout << "EDI USER:" << endl;
	cout << TAB << "ID CODE=" << code << endl;
	cout << TAB << "CODE QUAL=" << code_qual << endl;
	cout << TAB << "INPUT DIR=" << input_dir << endl;
	cout << TAB << "ADD DIR=" << add_dir << endl;
	cout << TAB << "TSDS=" << tsds << endl;
	cout << TAB << "USE X.400=" << use_ua << endl;
	cout << TAB << "USE X.435=" << use_pedi << endl;
	cout << TAB << "REJ COPY=" << rej_copy << endl;
	cout << TAB << "REJ INC=" << rej_incomplete << endl;
	cout << TAB << "TEST SENDER=" << test_send << endl;
	cout << TAB << "TEST STDS=" << test_stds << endl;
	cout << TAB << "EDI STDS=";

	for ( i=0; i<MAX_EDI_STANDARDS; i++ )
		cout << edi_stds[ i ];
	cout << endl;

	( x400_address() )->display_name();
	display_syntax_ids();

	cout << endl;
}


//
// PRIVATE:  display the syntax ids for one user to the screen.
//

void EDIUser::display_syntax_ids() {
	Queue		*user_syntaxes;
	SYNTAX_ID	*syntax_id;

	user_syntaxes = syntaxes;
	if ( user_syntaxes ) {
		user_syntaxes->set_search();
		syntax_id = ( SYNTAX_ID * ) user_syntaxes->current();
		while ( syntax_id ) {
			cout << TAB << "ID STRING=";
			cout << syntax_id->id_string << endl;
			cout << TAB << "ID VERSION=";
			cout << syntax_id->id_version << endl;
			syntax_id = ( SYNTAX_ID * ) user_syntaxes->current();
		}
	}
}

#endif	// DEBUG


//
// PRIVATE: build one EDI user.
//

void EDIUser::build_edi_user() {
	char	buf[ MAX_LINE ];

	do {
		cout << "Enter the Id Code for the User:            ";
		cin.getline( code, INTERCHANGE_ID_SIZE+1 );
	}
	while ( code[ 0 ] == NULL );
	
	cout << "Enter the Id Code Qualifier for the User:  ";
	cin.getline( code_qual, ID_QUALIFIER_SIZE+1 );
	
	cout << "Deliver Inbound Interchanges to the TSDS?  ";
	GET_BOOL( tsds );

	if ( tsds == FALSE ) {
		do {
			cout << "Enter the EDI User Input Directory:        ";
			cin.getline( input_dir, MAX_GETLINE );
		}
		while ( input_dir[ 0 ] == NULL );
	}
	
	cout << "Does the local EDI user use the UA?        ";
	GET_BOOL( use_ua );
	if ( ! use_ua )
		return;

	x400_addr = new ORName();
	assert( x400_addr != 0 );
	x400_addr->build_or_name();
	
	cout << endl << 
	     "Does the local EDI user use the X.435 service?             ";
	GET_BOOL( use_pedi );
	if ( ! use_x435() )
		return;
	
	cout << endl << "The following questions are specific to ";
	cout << "the X.435 service." << endl << endl;
	do {
		cout << "Enter the EDI User Additional Directory:                   ";
		cin.getline( add_dir, MAX_GETLINE );
	}
	while ( add_dir[ 0 ] == NULL );
	
	cout << "Reject an inbound X.435 message marked FOR COPY?           ";
	GET_BOOL( rej_copy );

	cout << "Reject an inbound X.435 message marked as INCOMPLETE?      ";
	GET_BOOL( rej_incomplete );

	cout << "Reject an inbound X.435 message if the sender is not " << endl;
	cout << "     a registered trading partner?                         ";
	GET_BOOL( test_send );

	cout << "Reject an inbound X.435 message if the standard is not ";
	cout << "supported?  ";
	GET_BOOL( test_stds );

	if ( test_stds ) {
		cout <<endl<< "Please enter y or n as to whether each of the ";
		cout << "following standards is supported:" << endl << endl;
		cout << "Support EDIFACT-ISO646?   ";
		GET_BOOL( edi_stds[ EDIFACT_ISO646 ] );
		cout << "Support EDIFACT-T61?      ";
		GET_BOOL( edi_stds[ EDIFACT_T61 ] );
		cout << "Support EDIFACT-Octet?    ";
		GET_BOOL( edi_stds[ EDIFACT_OCTET ] );
		cout << "Support ANSI X12-ISO646?  ";
		GET_BOOL( edi_stds[ ANSIX12_ISO646 ] );
		cout << "Support ANSI X12-T61?     ";
		GET_BOOL( edi_stds[ ANSIX12_T61 ] ); 
		cout << "Support ANSI X12-Octet?   ";
		GET_BOOL( edi_stds[ ANSIX12_OCTET ] );
		cout << "Support ANSI X12-EBCDIC?  ";
		GET_BOOL( edi_stds[ ANSIX12_EBCDIC ] );
		cout << "Support UN/TDI-ISO646?    ";
		GET_BOOL( edi_stds[ UNTDI_ISO646 ] );
		cout << "Support UN/TDI-T61 ?      ";
		GET_BOOL( edi_stds[ UNTDI_T61 ] );
		cout << "Support UN/TDI-Octet?     ";
		GET_BOOL( edi_stds[ UNTDI_OCTET ] );
		cout << "Support Private-Octet?    ";
		GET_BOOL( edi_stds[ PRIVATE_OCTET ] );
		cout << "Support Undefined-Octet?  ";
		GET_BOOL( edi_stds[ UNDEFINED_OCTET ] );
	}

	syntaxes = build_syntax_ids();
}


//
// PRIVATE:  build the syntax ids for one EDI user.
//

Queue *EDIUser::build_syntax_ids() {
	SYNTAX_ID	*syntax_id;
	Queue 		*syntaxes = new Queue;
	assert( syntaxes != 0 );

	cout << endl << 
	     "Enter Supported Standard Versions; Finish List With a ."
	     << endl << endl;
	do {
		cout << "ID-String  >>  ";
		syntax_id = new SYNTAX_ID;
		assert( syntax_id != 0 );
		cin.getline( syntax_id->id_string, ID_STRING_LEN+1 );
		if ( syntax_id->id_string[ 0 ] == '\0' )
			delete syntax_id;
		else if ( syntax_id->id_string[ 0 ] != '.' ) {
			cout << "ID-Version >>  ";
			cin. getline( syntax_id->id_version, ID_VERSION_LEN+1 );
			syntaxes->add( syntax_id );
			syntax_count++;
		}
	}
	while ( syntax_id->id_string[ 0 ] != '.' );
	
	delete syntax_id;
	return syntaxes;
}


//
// PUBLIC:  validate the syntax ids for a local user.
//

BOOL EDIUser::validate_syntax_id( char *str, char *ver ) {
	Queue		*user_syntaxes;
	SYNTAX_ID	*syntax_id;

	user_syntaxes = syntaxes;
	if ( user_syntaxes ) {
		user_syntaxes->set_search();
		syntax_id = ( SYNTAX_ID * ) user_syntaxes->current();
		while ( syntax_id ) {
			if ( ! strcmp( str, syntax_id->id_string ) &&
			       strcmp( ver, syntax_id->id_version ) )
				return TRUE;
		}
		return FALSE;
	}
	return TRUE;
}


//
// PUBLIC: read one EDI user.
//

void EDIUser::read_edi_user( ifstream *ifile ) {

	ORName *address = new ORName;
	assert( address != 0 );

	char	buf[ MAX_LINE ];
	
	ifile->getline( buf, INTERCHANGE_ID_SIZE+1, USER_DELIMITER );
	set_id_code( buf );
	ifile->getline( buf, ID_QUALIFIER_SIZE+1, USER_DELIMITER);
	set_id_code_qualifier( buf );
	set_tsds_queue( ifile->get() );
	set_use_x400( ifile->get() );
	set_use_x435( ifile->get() );
	( void ) ifile->get();		// newline
	
	ifile->getline( buf, MAX_GETLINE );
	set_input_directory( buf );
	ifile->getline( buf, MAX_GETLINE );
	set_additional_directory( buf );
	
	if ( ! use_x400() )
		return;
	
	set_reject_copy( ifile->get() );
	set_reject_incomplete_copy( ifile->get() );
	set_test_sender( ifile->get() );
	set_test_edi_standards( ifile->get() );
	ifile->getline( buf, MAX_EDI_STANDARDS+2 );
	set_edi_standards( buf );
	
	address->read_x400_address( ifile );
	set_x400_address( address );

	set_syntax_ids( read_syntax_ids( ifile ) );
}


//
// PRIVATE:  read the syntax ids for one EDI user.
//

Queue *EDIUser::read_syntax_ids( ifstream *ifile ) {
	int 		i;
	SYNTAX_ID	*syntax_id;
	char		count_buf[ MAX_COUNT ];
	Queue 		*syntaxes = new Queue;
	assert( syntaxes != 0 );

	ifile->getline( count_buf, MAX_COUNT+1 );
	set_number_syntax_ids( atoi( count_buf ) );

	for ( i=0; i<number_syntax_ids(); i++ ) {
		syntax_id = new SYNTAX_ID;
		assert( syntax_id != 0 );
		ifile->getline( syntax_id->id_string, 
				      ID_STRING_LEN+1, 
				      ADDR_DELIMITER );
		ifile->getline( syntax_id->id_version, 
				      ID_VERSION_LEN+1 );
		syntaxes->add( syntax_id );
	}

	return syntaxes;
}


//
// PUBLIC:  Write one EDI User to the configuration file.
//

void EDIUser::write_edi_user( ofstream *ifile ) { 
	
	*ifile << id_code() << USER_DELIMITER << id_code_qualifier() << 
		USER_DELIMITER << tsds_queue() << use_x400() << use_x435() <<
		endl << input_directory() << endl << additional_directory() << 
		endl;

	if ( ! use_x400() )
		return;
			
	*ifile << reject_copy() << reject_incomplete_copy() << test_sender()
	       << test_edi_standards();
	for ( int i=0; i<MAX_EDI_STANDARDS; i++ ) 
		*ifile << edi_standard( i );
	*ifile << endl;

	ORName *addr = x400_address();
	addr->write_or_name( ifile ); 
	write_syntax_ids( ifile );
}


//
// PRIVATE:  Write the syntax ids supported by one EDI User to 
// the configuration file.
//

void EDIUser::write_syntax_ids( ofstream *ifile ) {
	SYNTAX_ID	*syntax_id;

	*ifile << number_syntax_ids() << endl;
	Queue *syntaxes = syntax_ids();
	if ( number_syntax_ids() )
		syntaxes->set_search();
	for ( int i=0; i<number_syntax_ids(); i++ ) {
		syntax_id = ( SYNTAX_ID * ) syntaxes->current();
		*ifile << syntax_id->id_string << USER_DELIMITER <<
			syntax_id->id_version << endl;
	}
}
