static char rcsid[]="$Header: feature.c,v 2.6 93/03/25 14:50:54 khiem Exp $";
/*
 * $Header: feature.c,v 2.6 93/03/25 14:50:54 khiem Exp $
 * $Revision: 2.6 $
 * $Locker:  $
 * $Log:	feature.c,v $
 * Revision 2.6  93/03/25  14:50:54  14:50:54  khiem ()
 * Add Pedi hook to be able to negotiate Pedi package, and add to 
 * feature_selector[]. Use symbolic names for feature_selector[] indices.
 * Modify potential bug of assignment to memory outside of feature_list[]
 * when setting defaults in previous version. Selected packages are in
 * feature_selector[]. Other non-package features are no-op, implicitly
 * set if defaulted. Khiem.
 * 
 * 
 * Revision 2.5  92/09/18  00:48:07  00:48:07  khiem ()
 * Assign negotiated package flag: MH88, MH84, etc.
 * 
 *
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "xom.h"               /* object management */
#include "xomhp.h"             /* internal data structures */
#include "xmh.h"               /* message handling */
#include "xmhp.h"              /* message handling */
#include "xedip.h"             /* Pedi package */
#include "interr.h"            /* internal error handling */
#include "osdeps.h"

/* 
 * global variables for packages selected.
 */
extern boolean    mhp_is88MH;
extern boolean    mhp_is84MH;
extern boolean    mhp_is88IM;
extern boolean    mhp_is84IM;

void activate_feature_list(
    MH_feature         feature_list[],
    OM_boolean         feature_selector[],
    int                *status
)

{ /* start activate_feature_list */

int         exact_count, tmp_count, match, i;
OM_boolean  feature_available;
OM_boolean  basic_xfer, xfer_in, xfer_out;
OM_boolean  im_84, im_88, mh_84, mh_88;

/* initialize all the input activated bits of feature_list to 0 */ 

exact_count = 0;
for (i=0; i < MAX_FEATURES; i++)
{
    feature_selector[i] = OM_FALSE;
}
        
while ( (feature_list) && feature_list[exact_count].feature.length != 0) 
{ /* while loop : feature_list.. */
   if (exact_count < MAX_FEATURES)
   {
       feature_list[exact_count].activated = 0;
       exact_count++;
   }
   else /* more than maximum of features allowed, return error */
   {
       *status = TOO_MANY_FEATURES;
       return;    
   }
} /* while loop : feature_list.. */

/* 
 * feature_selector[] is an boolean array. Each element indicates whether
 * a package feature has been selected as follows:
 *     [0] -- IM84P_SET
 *     [1] -- IM88P_SET
 *     [2] -- MH84P_SET
 *     [3] -- MH88P_SET
 *     [4] -- XOMP_SET
 *     [5] -- XEDIP_SET
 *
 * There is not need to indicate other features, such as basic transfer
 * or basic xferin, as these are mandatory defaults and have no effect.
 * Feature selector might be expanded in the future for more packages,
 * and for more advanced function sets.
 */

/* 
 * Compare each feature given by the application with the set of supported 
 * features, and set the feature in feature_selector as appropriate.
 * There will ALWAYS be support for basic transfer, basic mt_xferin, 
 * and basic mt_xferout as defaults, and it's no-op.
 * Cannot negotiate out of these bacics. If required features not present, 
 * default will be set.
 * The current defaults are:
 *   -- basic xfer   -- need not be set
 *   -- basic xferin -- need not be set
 *   -- basic xferout-- need not be set
 *   -- MH_88
 *   -- IM_88
 *   -- XOM
 *
 * Other options are:
 *   -- MH_84 (MH_88 selected if both are present)
 *   -- IM_84 (IM_88 selected if both are present)
 *   -- EDI
 */   

tmp_count = exact_count = 0;
basic_xfer = xfer_in = xfer_out = OM_FALSE;
im_84 = im_88 = mh_84 = mh_88 = OM_FALSE;
feature_selector[IM88P_SET] = feature_selector[MH88P_SET] = OM_TRUE;
feature_selector[XOMP_SET] = OM_TRUE;

while ( (feature_list) && feature_list[tmp_count].feature.length !=0) 
{ /* while loop : feature_list[tmp_count].feature.length !=0 */
  feature_available = OM_FALSE;
  if (( match = strcmp (feature_list[tmp_count].feature.elements, 
          OMP_O_MH_FE_BASIC_TRANSFER)) == 0)
  {
    feature_available = OM_TRUE;
    if (!basic_xfer)
    { /* if (!basic_xfer) */
       feature_list[tmp_count].activated = 1;
       exact_count++;
       basic_xfer = OM_TRUE;
       *status = 0;
       tmp_count++ ;
       continue;
     } /* if (!basic_xfer) */
     else
     { /* if duplicate */
        tmp_count++ ;
     	continue;
     }
  } /* if OMP_O_MH_FE_BASIC_TRANSFER */ 

  if (( match = strcmp (feature_list[tmp_count].feature.elements, 
          OMP_O_MH_FE_TRANSFER_IN)) == 0)
  {
    feature_available = OM_TRUE;
    if (!xfer_in)
    { /* if (!xfer_in) */
        feature_list[tmp_count].activated = 1;
     	exact_count++;
     	xfer_in = OM_TRUE;
        *status = 0;
        tmp_count++ ;
        continue;
    } /* if (!xfer_in) */ 
    else
    { /* if duplicate */
        tmp_count++ ;
     	continue;
    }
   } /* if OMP_O_MH_FE_TRANSFER_IN */

   if (( match = strcmp (feature_list[tmp_count].feature.elements, 
          OMP_O_MH_FE_TRANSFER_OUT)) == 0)
   {
     feature_available = OM_TRUE;
     if (!xfer_out)
     { /* if (!xfer_out) */
        feature_list[tmp_count].activated = 1;
     	exact_count++;
     	xfer_out = OM_TRUE;
        *status = 0;
        tmp_count++ ;
        continue;
     } /* if (!xfer_out) */
     else
     { /* if duplicate */
        tmp_count++ ;
     	continue;
     }
   } 

   if (( match = strcmp (feature_list[tmp_count].feature.elements, 
        OMP_O_MH_FE_IM_84)) == 0)
   {
     feature_available = OM_TRUE;
     if (!im_84)
     { /* if (!im_84) */
        feature_list[tmp_count].activated = 1;
     	exact_count++;
     	im_84 = OM_TRUE;
        *status = 0;
        tmp_count++ ;
        continue;
     } /* if (!im_84) */ 
     else
     { /* if duplicate */
        tmp_count++ ;
     	continue;
     }
   } 
   if (( match = strcmp (feature_list[tmp_count].feature.elements, 
         OMP_O_MH_FE_IM_88)) == 0)
   {
     feature_available = OM_TRUE;
     if (!im_88)
     { /* if (!im_88) */
        feature_list[tmp_count].activated = 1;
     	exact_count++;
     	im_88 = OM_TRUE;
        *status = 0;
        tmp_count++ ;
        continue;
     } /* if (!im_88) */
     else
     { /* if duplicate */
        tmp_count++ ;
     	continue;
     }
   } 
   if (( match = strcmp (feature_list[tmp_count].feature.elements, 
        OMP_O_MH_FE_MH_84)) == 0)
   {
    feature_available = OM_TRUE;
    if (!mh_84)
    { /* if (!mh_84) */
        feature_list[tmp_count].activated = 1;
        exact_count++;
        mh_84 = OM_TRUE;
        *status = 0;
        tmp_count++ ;
        continue;
     } /* if (!mh_84) */ 
     else
     { /* if duplicate */
        tmp_count++ ;
     	continue;
     }
   } 

   if (( match = strcmp (feature_list[tmp_count].feature.elements, 
        OMP_O_MH_FE_MH_88)) == 0)
   {
     feature_available = OM_TRUE;
     if (!mh_88)
     { /* if (!mh_88) */
        feature_list[tmp_count].activated = 1;
     	exact_count++;
     	mh_88 = OM_TRUE;
        *status = 0;
        tmp_count++ ;
        continue;
       } /* if (!mh_88) */ 
     else
     { /* if duplicate */
         tmp_count++ ;
     	continue;
     }
   } 

   if (( match = strcmp (feature_list[tmp_count].feature.elements, 
        OMP_O_EDI_PACKAGE)) == 0)
   {
     feature_available = OM_TRUE;
     feature_list[tmp_count].activated = 1;
     exact_count++;
     tmp_count++ ;
     feature_selector[XEDIP_SET] = OM_TRUE;
     continue;
   } 

   if (!feature_available)
   {
       *status = MH_RC_FEATURE_UNAVAILABLE;
       return;
   }
} /* while loop : feature_list[tmp_count].feature.length !=0 */


if ((im_84) && (im_88))
{
    /* only return im_88 */
    feature_selector[IM88P_SET] = OM_TRUE;
    feature_selector[IM84P_SET] = OM_FALSE;
    mhp_is88IM = TRUE;
}
else if (im_88) {
    feature_selector[IM88P_SET] = OM_TRUE;
    mhp_is88IM = TRUE;
} 
else if (im_84) {
    feature_selector[IM84P_SET] = OM_TRUE;
    feature_selector[IM88P_SET] = OM_FALSE;
    mhp_is84IM = TRUE;
}

if ((mh_84) && (mh_88))
{
    /* only return mh_88 */
    feature_selector[MH88P_SET] = OM_TRUE;
    feature_selector[MH84P_SET] = OM_FALSE;
    mhp_is88MH = TRUE;
}
else if (mh_88) {
    feature_selector[MH88P_SET] = OM_TRUE;
    mhp_is88MH = TRUE;
}
else if (mh_84) {
    feature_selector[MH84P_SET] = OM_TRUE;
    feature_selector[MH88P_SET] = OM_FALSE;
    mhp_is84MH = TRUE;
}

*status = OM_SUCCESS;
return;
} /* end activate_feature_list */

