// FILE:  ifile.C

#include	<sys/types.h>
#include	<dirent.h>
#include	<fstream.h>
#include	<string.h>
#include	<assert.h>
#include	<xom.h>
#include	<xedip.h>
#include	<sys/unistd.h>
extern	"C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/subject.h"
#include	"../../../include/interchange.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/edi_user.h"
#include	"../../../include/platfile.h"
#include	"../../../include/config.h"
#include	"../../../include/tpfile.h"
#include	"../../../include/ifile.h"
#include	"../../../include/api_obj.h"
#include	"../../../include/x400api.h"
#include	"../../../include/ftam.h"
#include	"../../../include/van.h"
#include	"../../../include/edi_control_defs.h"

#define	FTAM_SHIT	1

//
// PUBLIC:  process interchanges placed in the EDI user output file directory.
//

STATUS InterchangeFile::send( Config		*config,
			      Log		* log ) { 

	Interchange	*interchange;
	STATUS		status = SUCCESS;


	while ( read_file_directory() == SUCCESS ) {
		
		// Read file the containing the interchange.
		if ( read_file() == ERROR ) {
			status |= ERROR;
			break;
		}
		
		// Create an Interchange object for the interchange.
		interchange = new Interchange( file_contents() );
		assert( interchange != 0 );
		
		// Parse specific fields from the interchange.
		interchange->parse_edi_names();

		// Transmit the interchange via X.400, FTAM, or DIAL-UP.
		status |= transmit_interchange( interchange, config, log );
		status |= ACTIVITY_OCCURRED;
	}

#ifdef	DEBUG
	cout << "send(): returning " << status << endl;
#endif
	
	return status;
}


//
// Once it has been determined that the transmission medium is FTAM.
// a corresponding control file and ftam file need to be created and
// and put to specific platform determined directories.
//
STATUS InterchangeFile::transmit_ftam( Log	   	   *log,
				       char        	   *control_dir, 
				       char        	   *ftam_dir, 
				       TradingPartnerFile  *tp_entry ) {

	STATUS		status = SUCCESS;
	ofstream        *ftam_control_file;
	char		control_filename[ MAX_FILENAME ];
 
#ifdef	DEBUG
	cout << "transmit_ftam() entry" << endl;
#endif

	strcpy( control_filename, control_dir );
        strcat( control_filename, "/" );
	strcat( control_filename, filename() );

        // Open the ftam control file.
	ftam_control_file = new ofstream( control_filename, ios::out );

        // Make sure the (control) file was opened.
	if ( !ftam_control_file ) {
		log->log_event( "transmit_ftam", FILE_NOT_CREATED, EXITING );
	    	return ( status |= ERROR );
	}

#ifdef DEBUG
	cout << "FTAM control file: " << control_filename << endl;
#endif	
	
	// Build the Control File.
		*ftam_control_file << EDI_FTAM_CONTROL_FILE << endl;
        *ftam_control_file << DESTINATION_INIT_ID 
                           << tp_entry->user_name() << endl;
        *ftam_control_file << DESTINATION_FILESTORE_PW 
                           << tp_entry->user_password() << endl;
        *ftam_control_file << DESTINATION_DIRNAME 
                           << tp_entry->x500_dirname() << endl;

	// Check to make sure there is a directory name given in X500.
	if ( tp_entry->file_dirname() != NULL ) 
             *ftam_control_file << DESTINATION_DIRECTORY 
                                << tp_entry->file_dirname() << endl;
        
        // Close the control file.
	ftam_control_file->close();

	// Rename the interchange file to the platform designated ftam 
	// directory.
        status |= move_file( ftam_dir );

	return status;
}

			     
//
// Once it has been determined that the transmission medium is a VAN,
// a VAN file and corresponding control file need to be created and
// and put to specific platform determined directories.
//
STATUS InterchangeFile::transmit_dial_up( char        	*control_dir, 
                                          char        	*van_interchange_dir, 
				          TradingPartnerFile  *tp_entry ) {
	
	STATUS		status = SUCCESS;
	char		control_filename[ MAX_FILENAME ];
	char		van_script_filename[ MAX_FILENAME ];
  
	// First name the control file with the complete directory
	// path.  The van script file will be copied from this directory.
	strcpy( control_filename, control_dir );
        strcat( control_filename, "/" );
	strcat( control_filename, filename() );
	
	// The van script file will be copied to this directory.
	strcpy( van_script_filename, VAN_DIR );
        strcat( van_script_filename, "/" );
	strcat( van_script_filename, tp_entry->van_name() );

	// Rename the VAN control file, corresponding to the VAN used by
	// the user, to the designated control directory . 
	// The control_filename is necessary because a single user may
	// use multiple van services.
	if ( link( van_script_filename, control_filename ) == -1 )
		status |= ERROR;

#ifdef	DEBUG
	cout << "Van Control = " << control_filename << endl;
	cout << "Script = " << van_script_filename << endl;
#endif

	// Rename the interchange file to the platform designated van 
	// directory.
        status |= move_file( van_interchange_dir );

	return status;
}


//
// PRIVATE:  determine the appropriate transmission medium for the interchange 
// argument, and send a message to the appropriate object to transmit the 
// interchange.
//

STATUS InterchangeFile::transmit_interchange( Interchange *interchange,
					      Config      *config,
					      Log 	  *log ) {
	TradingPartnerFile	tp_entry( log );
	X400Message		*x400_message;
	EDIUser			*local_user;
	STATUS			status = SUCCESS;
	ofstream        	*logfile = log->get_logfile();
        char			*control_dir;
	char			*ftam_dir;
	char			*dial_up_dir;

	// Read the communications attributes from the Directory.
	
	if ( tp_entry.read_communications_attributes(
			interchange->get_sender_ID_code(),
			interchange->get_sender_ID_qualifier(),
			interchange->get_recipient_ID_code(),
			interchange->get_recipient_ID_qualifier() ) == ERROR ) {
		delete interchange;
		return ( move_file( ERROR_DIR ) );
	}

	switch ( tp_entry.transmission_medium() ) {
		case MHS_IPM:
		case MHS_EDI:
			*logfile << endl << "***************************************************************" << endl << endl;
			interchange->parse_interchange();
			local_user = config->get_local_user( 
				interchange->get_sender_ID_code(),
				interchange->get_sender_ID_qualifier() );
			if ( local_user == NULL ) {
				log->log_event( "transmit_interchange",
						NO_SUCH_LOCAL_USER,
						MOVING_INTERCHANGE );
				delete interchange;
				return ( move_file( ERROR_DIR ) );
			}
			x400_message = new X400Message( &tp_entry, 
					      local_user->x400_address(),
					      log, interchange );
			if ( tp_entry.transmission_medium() == MHS_EDI )
				status |= x400_message->build_edi_message( 
								EDI_MESSAGE);
			else
				status |= x400_message->build_edi_message( 
								IPM_MESSAGE);
			
			if ( ! OK( status ) )  {
				log->log_event( "transmit_interchange",
						"EDI Build Error",  
						MOVING_INTERCHANGE );
				status |= move_file( ERROR_DIR );
			}
			else
				status |= delete_file();
			delete interchange;
			delete x400_message;
			break;

		case FTAM:
			*logfile << endl << "***************************************************************" << endl << endl;
			control_dir = config->ftam_control_directory_name(); 
			ftam_dir = config->ftam_interchange_directory_name(); 
			status |= transmit_ftam( log, control_dir, ftam_dir,
                                                 &tp_entry );
			*logfile << "OUTBOUND FTAM TRANSFER" << endl
				<< "Local User:  " 
				<< interchange->get_sender_ID_code() << endl
				<< "Partner ID:  "
				<< interchange->get_recipient_ID_code() << endl
				<< endl;
		        delete interchange;
			break;

		case DIAL_UP:
			// Find out where to put the control and dial-up files 
			// from the directory.
			*logfile << endl << "***************************************************************" << endl << endl;
			control_dir = config->ftam_control_directory_name(); 
			control_dir = config->van_control_directory_name();
			dial_up_dir = config->van_interchange_directory_name();
			status = transmit_dial_up( control_dir, 
						   dial_up_dir,
						   &tp_entry );
			*logfile << "OUTBOUND VAN TRANSFER"
				 << "Local User:  " 
				 << interchange->get_sender_ID_code() << endl
				 << "Partner ID:  "
				 << interchange->get_recipient_ID_code() << endl
				 << endl;
                        delete interchange;
			break;
	}

	return status;
}


//
// PUBLIC:  process interchanges placed in the transmission input directory.
//

STATUS InterchangeFile::receive( Config *config, Log *log ) { 
	EDIUser		*local_user;
	Interchange	*interchange;
	ofstream        *logfile = log->get_logfile();
	char		ftam_shit_file[ MAX_FILENAME ];
	STATUS		status = SUCCESS;

#ifdef	DEBUG
	cout << "receive():  Entry" << endl;
#endif

	while ( read_file_directory() == SUCCESS ) {

		// Read file the containing the interchange.
		if ( read_file() == ERROR ) {
			status |= ERROR;
			break;
		}

#ifdef	FTAM_SHIT
		if ( ! strncmp( filename(), "._", 2 ) ) {
			unlink( full_filename() );
			continue;
		}
#endif
		
		// Create an Interchange object for the interchange.
		interchange = new Interchange( file_contents() );
		assert( interchange != 0 );
		
		// Parse the recipient (local user) from the interchange.
		interchange->parse_edi_names();

		*logfile << endl << "***************************************************************" << endl << endl;
		// Match the recipient to a local user.
		local_user = config->get_local_user( 
				interchange->get_recipient_ID_code(),
				interchange->get_recipient_ID_qualifier() );

#ifdef	DEBUG
	if ( local_user == NULL )
		cout << "Found no matching local user!" << endl;
	else 
		cout << "local user: " << interchange->get_recipient_ID_code()
		     << " Qual: " << interchange->get_recipient_ID_qualifier()
		     << endl << "local dir: " << local_user->input_directory()
		     << endl;
#endif
		
		// Move the file to the local EDI user's directory.
		if ( local_user != NULL ) 
			status |= move_file( local_user->input_directory() );
		else {
			log->log_event( "receive()", 
					NO_SUCH_LOCAL_USER, 
					MOVING_INTERCHANGE );
			status |= move_file( ERROR_DIR );
		}
		*logfile << "INBOUND INTERCHANGE" << endl
			 << "Local User:  " 
			 << interchange->get_sender_ID_code() << endl
		         << "Partner ID:  " 
			 << interchange->get_recipient_ID_code() << endl
		     	 << "Destination: " << local_user->input_directory()  
			 << endl << endl;
		delete interchange;
		status |= ACTIVITY_OCCURRED;
	}

#ifdef	DEBUG
	cout << "receive status = " << status << endl;
#endif

	return status;
}
