// FILE:  interchange.C

#include	<iostream.h>
#include	<fstream.h>
#include	<xom.h>
#include	<assert.h>
#include	<xedip.h>
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/interchange.h"
 

//
// parse_interchange()
//
// Class Members:  
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method determines the format of the interchange 
// (X12, EDIFACT or OTHER) and
// calls the corresponding parse methods if the interchange is a valid type.
//

STATUS Interchange::parse_interchange() {

  char *value = '\0';  // Individual values parsed off of the content.
  char *temp_contents = '\0'; // Copy of the contents to be parsed. 
  char *inter_type = '\0';  // The interchange type. 

#ifdef DEBUG
 cout << "-----In parse_interchange().-----  " << endl; 
#endif
        // Reserve space for the interchange type.
  	inter_type = new char[INTER_TYPE_SIZE]; 
	assert ( inter_type != NULL );
        // Reserve space for a copy of contents.
  	temp_contents = new char[ strlen( contents ) ]; 
	assert ( temp_contents != NULL );
        // Reserve space for the values parsed off the contents.
  	value = new char[ strlen( contents ) ]; 
	assert ( value != NULL );
         
#ifdef DEBUG
 cout << "contents:  " << contents << endl; 
#endif
        // the element separator is the forth character in the interchange.
	data_element_separator[0] = contents[ELEMENT_SEPARATOR_POS];
        data_element_separator[1] = '\0';
#ifdef DEBUG
 cout << "data element separator:  " << data_element_separator << endl; 
#endif
        // The first call to strtok gives the whole interchange (which is) 
        // to be parsed.  After this initial call, a value of NULL needs to
        // be sent as the first argument so strtok can return the remaining
        // parts (tokens) separated by the data element separator.

        // Copy the contents to a temporary buffer so that the tokens
        // can be parsed using strtok (which returns a nulled out string
        // of that sent).
        strcpy(temp_contents, contents);
#ifdef DEBUG
 cout << "temp_contents:  " << temp_contents << endl; 
#endif
        strcpy(value, strtok( temp_contents, data_element_separator ));

	// Get the interchange type and record the bodypart and  
	// edi message types.
	strcpy(inter_type, get_interchange_type( value ));
#ifdef DEBUG
 cout << "contents (interchange) header type: "<< inter_type << endl;         
#endif

        // Based on the interchange type call the appropriate parse function.
	// Note only two types are currently supported: X12 and EDIFACT.
        if (!strncmp( inter_type, "ISA", 3 )) 
	    // X12 Interchange
	     parse_X12_interchange();
	else if (!strncmp( inter_type, "UNA", 3 ))  // EDIFACT Interchange
	         parse_EDIFACT_interchange();
	     else return(UNSUCCESSFUL); // Interchange type not supported.
#ifdef	DEBUG
	display_interchange();
#endif
	// Free up reserved space.
	if ( temp_contents != NULL )
           delete temp_contents;
        if ( inter_type != NULL )
	   delete inter_type;
	if ( value != NULL )
	   delete value;

#ifdef DEBUG
 cout << "----- End of parse_interchange().-----  " << endl; 
#endif
return(SUCCESS);
}

//
// parse_X12_interchange()
//
// Class Members:  
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method parses the contents member to set specific interchange 
// attributes.
//
// The following is an X12 interchange header, which is the only part of
// the interchange structure needed to fill the corresponding X.435 fields.
//
//      I01	Authorization Information Qualifier, type: ID, M, 2/2
//      I02	Authorization Information, type: AN, 10/10
//      I03	Security Information Qualifier, type: ID, 2/2
//      I04	Security Information, type: AN, 10/10
//      I05	Interchange ID Qualifier, type: ID, 2/2
//      I06	Interchange Sender ID, type: ID, 15/15
//      I05	Interchange ID Qualifier, type: ID, 2/2
//      I07  	Interchange Receiver ID, type: ID, 15/15
//      I08  	Interchange Date, type: DT, 6/6
//      I09     Interchange Time, type: TM, 4/4
//      I10	Interchange Control Standards Identifier, type: ID, 1/1
//      I11	Interchange Control Version Number, type: ID, 5/5
//      I12	Interchange Control Number, type: NO, 9/9
//      I13	Acknowledgment Requested, type: ID, 1/1
//      I14	Test Indicator, type: ID, 1/1
//      I15	Subelement Separator, type AN, 1/1
//      <tr>    terminator

void Interchange::parse_X12_interchange() {
  char *dummy = '\0';
  int  len;

#ifdef DEBUG
  cout << "-----In parse_X12_Interchange().-----" << endl;
#endif
  // strtok is used to parse each field of the contents.  It is important
  // that strtok is called in the order that makes up an X12 Interchange
  // header.  Also strtok must be called with the first argument "NULL" in 
  // order to parse the same contents and return sequential tokens.

  //      I01	Authorization Information Qualifier, type: ID, M, 2/2
          strcpy( authorization_info_qualifier, strtok( NULL, (char *) data_element_separator )); 
          authorization_info_qualifier[AUTH_INFO_QUAL_SIZE - 1] = '\0';
#ifdef DEBUG
  cout << "authorization _info_qualifier: " << authorization_info_qualifier << endl;
#endif	
  
  //      I02	Authorization Information, type: AN, 10/10
          strcpy( authorization_info, strtok(NULL, (char *) data_element_separator)); 
          authorization_info[AUTH_INFO_SIZE - 1] = '\0';
#ifdef DEBUG
  cout << "authorization _info: " << authorization_info << endl;
#endif	

  //      I03	Security Information Qualifier, type: ID, 2/2
  //      No mapping to X.435
          dummy =  strtok(NULL, (char *) data_element_separator); 

  //      I04	Security Information, type: AN, 10/10
  //      No mapping to X.435
          dummy =  strtok(NULL, (char *) data_element_separator); 

  //      I05	Interchange ID Qualifier, type: ID, 2/2
  //      No mapping to X.435
          sender_ID_qualifier = new char[ X12_ID_QUAL_SIZE ];
	  assert( sender_ID_qualifier != NULL );
          strcpy(  sender_ID_qualifier, strtok(NULL, (char *) data_element_separator)); 
          len = X12_ID_QUAL_SIZE - 2;
	  while ( len > -1 && sender_ID_qualifier[len] == ' ' )
		len --;
	  sender_ID_qualifier[len+1] = '\0';
#ifdef DEBUG
  cout << "sender id qual: " << sender_ID_qualifier << endl;
#endif

  //      I06	Interchange Sender ID, type: ID, 15/15
	  sender_ID_code = new char[ SENDER_ID_CODE_SIZE ];
	  assert( sender_ID_code != NULL );
          strcpy( sender_ID_code, strtok(NULL, (char *) data_element_separator)); 
          len = SENDER_ID_CODE_SIZE - 2;
	  while ( len > -1 && sender_ID_code[len] == ' ' )
		len --;
	  sender_ID_code[len+1] = '\0';
#ifdef DEBUG
  cout << "sender id code: '" << sender_ID_code << "'" << endl;
#endif

  //      I05	Interchange ID Qualifier, type: ID, 2/2
  //      No mapping to X.435
          recipient_ID_qualifier = new char[ X12_ID_QUAL_SIZE ];
	  assert( recipient_ID_qualifier != NULL );
          strcpy(  recipient_ID_qualifier, strtok(NULL, (char *) data_element_separator)); 
          len = X12_ID_QUAL_SIZE - 2;
	  while ( len > -1 && recipient_ID_qualifier[len] == ' ' )
		len --;
	  recipient_ID_qualifier[len+1] = '\0';
#ifdef DEBUG
  cout << "recipient id qual: " << recipient_ID_qualifier << endl;
#endif

  //      I07  	Interchange Receiver ID, type: ID, 15/15
	  recipient_ID_code = new char[ RECIP_ID_CODE_SIZE ];
	  assert( recipient_ID_code != NULL );
          strcpy( recipient_ID_code, strtok(NULL, (char *) data_element_separator )); 
          len = RECIP_ID_CODE_SIZE - 2;
	  while ( len > -1 && recipient_ID_code[len] == ' ' )
		len --;
	  recipient_ID_code[len+1] = '\0';

#ifdef DEBUG
  cout << "recipient id code: '" << recipient_ID_code << "'" <<  endl;
#endif
  //      I08  	Interchange Date, type: DT, 6/6
//change for EDIFACT?	  date_preparation = new char[ DATE_SIZE ];
	  date_preparation = new char[ DATE_PREP_SIZE+TIME_PREP_SIZE ];
	  assert( date_preparation != NULL );
          strcpy( date_preparation, strtok(NULL, (char *) data_element_separator )); 
          date_preparation[DATE_PREP_SIZE - 1] = '\0';
#ifdef DEBUG
  cout << "date: " << date_preparation << endl;
#endif

  //      I09     Interchange Time, type: TM, 4/4
	  time_preparation = new char[ TIME_PREP_SIZE ];
	  assert( time_preparation );
          strcpy( time_preparation, strtok(NULL, (char *) data_element_separator )); 
          time_preparation[TIME_PREP_SIZE - 1] = '\0';
#ifdef DEBUG
  cout << "time: " << time_preparation << endl;
#endif

  //      I10	Interchange Control Standards Identifier, type: ID, 1/1
  //?????
	  syntax_ID_string = new char[ SYNTAX_ID_STRING_SIZE ];
	  assert( syntax_ID_string );
          strcpy( syntax_ID_string, strtok(NULL, (char *) data_element_separator )); 
          syntax_ID_string[SYNTAX_ID_STRING_SIZE - 1] = '\0';

  //
  //  Concatenate the Interchange version number and control number
  //  to get the X.435 field "interchange control reference.
  //
  //      I11	Interchange Control Version Number, type: ID, 5/5
          strcpy( syntax_version, strtok(NULL, (char *) data_element_separator )); 
     
#ifdef DEBUG
  cout << "syntax_ID_string: " << syntax_ID_string << endl;
  cout << "syntax_version: " << syntax_version << endl;
#endif
  //      I12	Interchange Control Number, type: NO, 9/9
          strncat(interchange_control_reference, 
                  (strtok(NULL, (char *) data_element_separator)), 9);
          interchange_control_reference[INTERCHANGE_CNTL_SIZE - 1] = '\0';

#ifdef DEBUG
  cout << "interchangte cntl ref: " << interchange_control_reference << endl;
#endif

  //      I13	Acknowledgment Requested, type: ID, 1/1
           strcpy( acknowledgement_request, strtok(NULL, (char *) data_element_separator ));
           acknowledgement_request[ACK_REQUEST_SIZE - 1] = '\0';
#ifdef DEBUG
  cout << "acknowledgement request: " << acknowledgement_request << endl;
#endif

  //      I14	Test Indicator, type: ID, 1/1
         strcpy( test_indicator, strtok(NULL, (char *) data_element_separator));
         test_indicator[TEST_INDICATOR_SIZE - 1] = '\0';
#ifdef DEBUG
  cout << "test indicator: " << test_indicator << endl;
#endif

  //      I15	Subelement Separator, type AN, 1/1
          comp_data_element_separator[0] = contents[strlen(contents) - 2];
          comp_data_element_separator[ELEMENT_SEPARATOR_SIZE - 1] = '\0';
#ifdef DEBUG
  cout << "comp data element separator: " << comp_data_element_separator[0] << endl;
#endif

  //      <tr>    terminator
  //      Simply represents  the end of Interchange information (contents).
          segment_terminator[0] = contents[strlen(contents) - 1];
          segment_terminator[SEGMENT_TERMINATOR_SIZE - 1] = '\0';
#ifdef DEBUG
  cout << "segment_terminator: " << segment_terminator << endl;
#endif

  //  Fields not filled from X12 Interchange (defaulted to NULL here)
  //  which are needed:
     // Decimal Notation must be present.  For lack of a better idea,
     // set it equal to a space.
     strcpy( decimal_notation, " " );
     recipient_reference[0] = '\0';
     recipient_reference_qualifier[0] = '\0';
     process_priority_code[0] = '\0';
     comm_agreement_ID[0] = '\0';
     release_indicator[0] = '\0';
     reserved[0] = '\0';
//?????     syntax_version[0] = '\0';
     sender_routing_addr[0] = '\0';
     application_ref[0] = '\0';
     recipient_routing_addr[0] = '\0';

#ifdef DEBUG
  cout << "-----End of parse_X12_Interchange().-----" << endl;
#endif
 } // parse_X12_interchange end.


void parse_sender_recip(char* field1, char* field2, char *field3,
                        char* value,
                        char* comp_separator ) {

     char *first_token = '\0';
     char *second_token = '\0';
     int  code_len = 0;
     int  i = 0;

#ifdef DEBUG
 cout << "------In parse_sender_recip()------ " << endl;
#endif
   // Create temporary space
   first_token = new char[TOKEN_SIZE];
   assert ( first_token != NULL );
   second_token = new char[TOKEN_SIZE];
   assert ( second_token != NULL );

  // Get the first_token.
  strcpy( first_token, strchr( value, comp_separator[0] ));
#ifdef DEBUG
 cout << "first_token = " << first_token << endl;
#endif

  // Check to see if any of the conditional Sender/Recipient fields are present.
  // Partner ID Code and Address for reverse routing are conditional.
  // If none are present set the sender/Recipient ID to value, otherwise
  // parse the conditional fields partner ID qualifier and reverse routing
  // address (which both may or may not be present).
  if (first_token == NULL) {
     strcpy( field1, value );
#ifdef DEBUG
 cout << "ID_code (no conditionals): " << field1 << endl;
#endif

  }     
  else {  // parse the sub elements of sender information.
    code_len = strlen(value) - strlen(first_token);
#ifdef DEBUG
cout << "code_length: " << code_len << endl;
#endif

    for (i = 0; i < code_len; i++)
        field1[i] = value[i];
    field1[i] = '\0';
#ifdef DEBUG
cout << "sender-recip_ID_code: " << field1 << endl;
#endif

    for(i = 0; i < strlen(first_token); i++)
         first_token[i] = first_token[i + 1];
#ifdef DEBUG
 cout << "first_token = " << first_token << endl;
#endif

    // Check to see if there is a second field.  If not fill the third field.
    if ( first_token[0] == comp_separator[0] ) {
       for ( i = 0; i < strlen( first_token ); i++)
           field3[i] = first_token[i + 1];
	field2[0] = '\0';
    }
    else {
      strcpy ( second_token, strchr( first_token, comp_separator[0] ));
      if ( second_token != NULL ) {
         code_len = strlen( first_token ) - strlen( second_token );
         for (i = 0; i < code_len; i++)
             field2[i] = first_token[i];
         field2[i] = '\0';
#ifdef DEBUG
cout << "ID_code_qualifier: " << field2 << endl;
#endif

         for (i = 0; i < strlen( second_token ); i++)
              field3[i] = second_token[i + 1];
         field3[strlen( field3 )] = '\0';
#ifdef DEBUG
cout << "routing_address: " << field3 << endl;
#endif
       }
     else {
       strcpy( field2, first_token );
#ifdef DEBUG
cout << "ID_code_qualifier: " << field2 << endl;
#endif
     }
   } 
  } // end else there are conditional fields

  // Free up space.

    if ( first_token != NULL)
      delete first_token;
    if ( second_token != NULL )
       delete second_token;

#ifdef DEBUG
 cout << "------End parse_sender_recip()------ " << endl;
#endif

}

void Interchange::parse_EDIFACT_interchange() {

  char *dummy = '\0';
  char * value = '\0';
  int i = 0;
  int j = 0; // basic loop counters.
  
#ifdef DEBUG
cout << "-----In parse_EDIFACT_Interchange().-----" << endl;
#endif

    // strtok is used to parse each field of the contents.  It is important
    // that strtok is called in the order that makes up an X12 Interchange
    // header.  Also strtok must be called with the first argument "NULL" 
    // in order to parse the same contents and return sequential tokens.

    // Since EDIFACT gives the component data element separator first,
    // and we set the first token to the data_element separator above
    // (in parse_interchange) the actual character must be switched.
    strcpy( comp_data_element_separator, data_element_separator );
#ifdef DEBUG
    cout << "comp_data_element_separator: " << comp_data_element_separator << endl;
#endif

    // Data Element Separator.
    data_element_separator[0] = contents[DATA_ELE_SEP_POS + 1];
    data_element_separator[1] = '\0';
#ifdef DEBUG
cout << "data_element_separator: " << data_element_separator << endl;
#endif

    // Decimal Notation. Comma or full stop.
    decimal_notation[0] =  contents[DATA_ELE_SEP_POS + 2];
    decimal_notation[1] = '\0';
#ifdef DEBUG
cout << "decimal_notation: " << decimal_notation << endl;
#endif

    // Release Indicator. If not used, a space character.
    release_indicator[0] = contents[DATA_ELE_SEP_POS + 3];
    release_indicator[1] = '\0';
#ifdef DEBUG
cout << "release_indicator: " << release_indicator << endl;
#endif

    // Reserved for future.
    reserved[0] = contents[DATA_ELE_SEP_POS + 4];
    reserved[1] = '\0';
#ifdef DEBUG
cout << "reserved: " << reserved << endl;
#endif
    
    // Segment Terminator.
    segment_terminator[0] = contents[DATA_ELE_SEP_POS + 5];
    segment_terminator[1] = '\0';
#ifdef DEBUG
cout << "segment_terminator: " << segment_terminator << endl;
#endif

    // UNB Segment identifier (not needed to fill any X.435 values)
    dummy = strtok(NULL, (char *) data_element_separator);

   // Syntax Identifier. Manditory
    value = new char[MAX_FIELD_SIZE];
    assert( value != NULL );
    strcpy(value, strtok( NULL, (char *) data_element_separator ));
#ifdef DEBUG
cout << "value: " << value << endl;
#endif
    syntax_ID_string = new char[ SYNTAX_ID_SIZE ];
    assert( syntax_ID_string );
    strcpy( syntax_ID_string, value );
    syntax_version[0] =  value[5];
    syntax_ID_string[SYNTAX_ID_SIZE - 1] = '\0';
    syntax_version[1] = '\0';
#ifdef DEBUG
cout << "syntax_ID_string: " << syntax_ID_string << endl;
cout << "syntax_version: " << syntax_version << endl;
#endif

   // Interchange Sender. 
   strcpy ( value, strtok( NULL, (char *) data_element_separator ));
#ifdef DEBUG
 cout << "value = " << value << endl;
#endif

  // reserve space.
  sender_ID_code = new char[ INTERCHANGE_ID_SIZE ];
  assert( sender_ID_code != NULL );
  sender_ID_qualifier = new char[ ID_QUALIFIER_SIZE ];
  assert( sender_ID_qualifier );
  // Parse the Interchange Sender.
  parse_sender_recip( sender_ID_code, sender_ID_qualifier, 
                      sender_routing_addr, value, 
                      comp_data_element_separator );
#ifdef DEBUG
 cout << "sender_ID_code = " << sender_ID_code << endl;
 cout << "sender_ID_qualifier = " << sender_ID_qualifier << endl;
 cout << "sender_routing_addr = " << sender_routing_addr << endl;
#endif

  // Interchange Recipient. Manditory
  strcpy( value, strtok( NULL, (char *) data_element_separator ));
#ifdef DEBUG
 cout << "recipient value = " << value << endl;
#endif

  // Parse the Interchange Sender.
  recipient_ID_code = new char[ INTERCHANGE_ID_SIZE ];
  assert( recipient_ID_code );
  recipient_ID_qualifier = new char[ ID_QUALIFIER_SIZE ];
  assert( recipient_ID_qualifier );
  parse_sender_recip( recipient_ID_code, recipient_ID_qualifier, 
                      recipient_routing_addr, value, 
		      comp_data_element_separator );
#ifdef DEBUG
 cout << "recipient_ID_code = " << recipient_ID_code << endl;
 cout << "recipient_ID_qualifier = " << recipient_ID_qualifier << endl;
 cout << "recipient_routing_addr = " << recipient_routing_addr << endl;
#endif

  // DATE and TIME of Preparation. Manditory
  strcpy( value, strtok( NULL, (char *) data_element_separator ));
  date_preparation = new char[ DATE_SIZE + 1 ];
  assert( date_preparation );
  time_preparation = new char[ TIME_SIZE ];
  assert( time_preparation );
  strcpy( date_preparation, value );

  for (j = 0, i = TIME_SIZE; i <= strlen(value); i++, j++) {
      time_preparation[j] =  value[i];
  }
  date_preparation[DATE_SIZE] = '\0';
  time_preparation[strlen(time_preparation)] = '\0';
#ifdef DEBUG
cout << "date_preparation: " << date_preparation << endl;
cout << "time_preparation: " << time_preparation << endl;
#endif

    // INTERCHANGE CONTROL REFERENCE. Manditory
    strcpy( interchange_control_reference,  
       strtok(NULL, (char *) data_element_separator) );
//    interchange_control_reference[strlen(interchange_control_reference)] = '\0';
#ifdef DEBUG
cout << "interchange_control_reference: " << interchange_control_reference << endl;
#endif

   // RECIPIENT'S REFERENCE PASSWORD. Conditional
    strcpy( recipient_reference, 
       strtok(NULL, (char *) data_element_separator) );

    // Make sure this is not the last field (since all remaining
    // fields are optional. 
    if ( strchr( recipient_reference, segment_terminator[0] )
	       != NULL ) {
       for (i = 0; i < strlen( recipient_reference ); i++)
	   if (  recipient_reference[i] == segment_terminator[0] ) {
	      recipient_reference[i] = '\0';
	      i = strlen( recipient_reference );
           }
      }  

//    recipient_reference[strlen(recipient_reference)] = '\0';

//  if it is agreed upon in the Interchange Argeement
//  this field needs to be filled.  ???????????    
    // recipient's reference password qualifier.
    recipient_reference_qualifier[0] = '\0';   // redipient's reference password qualifier
#ifdef DEBUG
cout << "recipient_reference: " << recipient_reference << endl;
cout << "recipient_reference_qualifier: " << recipient_reference_qualifier << endl;
#endif

    // APPLICATION REFERENCE. Conditional.
    strcpy( application_ref, strtok(NULL, (char *) data_element_separator) );
    if (application_ref[0] == segment_terminator[0]) 
       application_ref[0] = '\0';
//    application_ref[strlen(application_ref)] = '\0';
#ifdef DEBUG
cout << "application_ref: " << application_ref << endl;
#endif

    // PROCESSING PRIORITY CODE. Conditional.
    strcpy( process_priority_code, strtok(NULL, (char *) data_element_separator));
    if ( process_priority_code[0] == segment_terminator[0]) 
       process_priority_code[0] = '\0';
//    process_priority_code[1] = '\0';
#ifdef DEBUG
cout << "process_priority_code: " << process_priority_code << endl;
#endif

    // ACKOWLEDGEMENT REQUEST. Conditional.
    strcpy( acknowledgement_request, strtok(NULL, (char *) data_element_separator) );
    if (acknowledgement_request[0] == segment_terminator[0]) 
       acknowledgement_request[0] = '\0';
//    acknowledgement_request[ACK_REQUEST_SIZE - 1] ='\0';
#ifdef DEBUG
cout << "acknowledgement_request: " << acknowledgement_request << endl;
#endif
						
    // COMMUNICATIONS AGREEMENT ID. Conditional.
    strcpy( comm_agreement_ID, strtok(NULL, (char *) data_element_separator) );
    if (comm_agreement_ID[0] == segment_terminator[0]) 
       comm_agreement_ID[0] = '\0';

//    comm_agreement_ID[strlen(comm_agreement_ID)] ='\0';
#ifdef DEBUG
cout << "communication agreement ID: " << comm_agreement_ID << endl;
#endif

    // TEST INDICATOR. Conditional.
    strcpy( test_indicator, strtok(NULL, (char *) segment_terminator) );
    if (test_indicator[0] == segment_terminator[0]) 
       test_indicator[0] = '\0';
//    test_indicator[TEST_INDICATOR_SIZE - 1] = '\0';
#ifdef DEBUG
cout << "test indicator: " << test_indicator << endl;
#endif
   
    // Free up reserved space that is no longer needed.
    if ( value != NULL )
       delete value;

#ifdef DEBUG
cout << "-----End of parse_EDIFACT_Interchange().-----" << endl;
#endif
}

//
// parse_edi_names()
//
// Class Members:  Interchange
// Class Methods:  
// Parameters:     
// Called By:	   public
//
// This method parses the incoming interchange for the recipient_name,
// and sender_name, and fills the id_code and id_code_qualifier for each.
//
void Interchange::parse_edi_names() {

    char *temp_sender = '\0';  
    char *temp_recipient = '\0';  
    char *value = '\0';
    char *temp_value = '\0';
    char *interchange_type = '\0';
    char *comp_separator = '\0';
    char *separator = '\0';
    int  i = 0, len;

#ifdef DEBUG
  cout << "-----In parse_edi_names().----- " << value << endl;
#endif
     // Allocate space for separators.
     comp_separator = new char[ELEMENT_SEPARATOR_SIZE];
     assert( comp_separator != NULL );
     separator = new char[ELEMENT_SEPARATOR_SIZE];
     assert( separator != NULL );
     temp_sender = new char[20];
     assert( temp_sender != NULL );
     temp_recipient = new char[20];
     assert( temp_recipient != NULL );
     interchange_type = new char[4];
     assert( interchange_type != NULL);
temp_value = new char[20];
assert( temp_value != NULL );
     
     // Create space for Sender and Recipient edi name info. 
     sender_ID_code = new char[ SENDER_ID_CODE_SIZE ];
     assert( sender_ID_code != NULL );
     sender_ID_qualifier = new char[ ID_QUALIFIER_SIZE ];
     assert( sender_ID_qualifier != NULL );
//   sender_routing_addr = new char[ ROUTING_ADDR_SIZE ];
//   assert( sender_routing_addr != NULL );
     recipient_ID_code = new char[ RECIP_ID_CODE_SIZE ];
     assert( recipient_ID_code != NULL );
     recipient_ID_qualifier = new char[ ID_QUALIFIER_SIZE ];
     assert( recipient_ID_qualifier != NULL );
//   recipient_routing_addr = new char[ ROUTING_ADDR_SIZE ];
//   assert( recipient_routing_addr != NULL );

     // Copy the contents (interchange) into value.
     value = new char[ strlen(contents) ];
     assert( value != NULL );
     strcpy(value, contents);

     // Determine the interchange type and record the bodypart and
     // edi message types.
     strncpy(interchange_type,  get_interchange_type(value), 3);
     interchange_type[3] = '\0';
#ifdef DEBUG
  cout << "value: " << value << endl;
  cout << "interchange_type:  " << interchange_type << endl;
#endif

     // Based off the interchange type parse the interchange to
     // get the recipient name.
     if (!strncmp(interchange_type, "ISA", 3)) { //X12 Interchange
#ifdef DEBUG
  cout << "in ISA..." << endl;
#endif
             separator[0] = contents[ELEMENT_SEPARATOR_POS];
             separator[1] = '\0';
             strcpy( temp_value, strtok( value, separator ));
#ifdef DEBUG
  cout << "strtoc value = " << temp_value << endl;
#endif
             // Skip to the sender name info in the content.
             for ( i = 0; i < 5; i++ ) {
               strcpy( temp_value, strtok( NULL, separator ) );
#ifdef DEBUG
  cout << "strtoc value = " << temp_value << endl;
#endif
             }

             // Create and set the sender name info 
             strcpy( sender_ID_code, strtok(NULL, separator));
             len = SENDER_ID_CODE_SIZE - 2;
	     while ( len > -1 && sender_ID_code[len] == ' ' )
		   len --;
	     sender_ID_code[len+1] = '\0';
	     sender_ID_qualifier[0] = '\0';
	     sender_routing_addr[0] = '\0';

             // Now create and set the recipient info.
strcpy( recipient_ID_code, strtok( NULL, separator ) );
             strcpy( recipient_ID_code, strtok( NULL, separator ) );
             len = RECIP_ID_CODE_SIZE - 2;
	     while ( len > -1 && recipient_ID_code[len] == ' ' )
		   len --;
	     recipient_ID_code[len+1] = '\0';
	     recipient_ID_qualifier[0] = '\0';
	     recipient_routing_addr[0] = '\0';
       }  
       else if (!strncmp( interchange_type, "UNA", 3 )) { // EDIFACT Interchange
#ifdef DEBUG
  cout << "in UNA..." << endl;
#endif
                comp_separator[0] = contents[DATA_ELE_SEP_POS];
                comp_separator[1] = '\0';
                separator[0] = contents[DATA_ELE_SEP_POS + 1];
                separator[1] = '\0';
#ifdef DEBUG
  cout << "comp_separator = " << comp_separator << endl;
  cout << "separator      = " << separator << endl;
#endif
                strcpy( temp_value, strtok( value, separator ));
#ifdef DEBUG
  cout << "strtoc value = " << temp_value << endl;
#endif

                // Skip to the sender name info in the content.
                for (i = 0; i < 2; i++) {
                  strcpy( temp_value, strtok( NULL, separator ));
#ifdef DEBUG
  cout << "strtoc value = " << temp_value << endl;
#endif
                }
                // Fill the sender edi name info.
		strcpy(temp_sender, strtok( NULL, separator ));

		// Fill the recipient edi name info.
                strcpy(temp_recipient, strtok(NULL, separator));
#ifdef DEBUG
  cout << endl << "temp_sender    = " << temp_sender << endl;
  cout << endl << "temp_recipient = " << temp_recipient << endl;
#endif

		// Parse and set the Sender information.
		parse_sender_recip( sender_ID_code, sender_ID_qualifier,
	        	sender_routing_addr, temp_sender, comp_separator );
#ifdef DEBUG
  cout << "sender_ID_code = " << "'" << sender_ID_code << "'" << endl;
  cout << "sender_ID_qual = " << sender_ID_qualifier << endl;
  cout << "sender_routing_addr = " << sender_routing_addr << endl;
#endif

		// Parse and set the recipient information.
		parse_sender_recip( recipient_ID_code, recipient_ID_qualifier,
	              recipient_routing_addr, temp_recipient, comp_separator );
#ifdef DEBUG
  cout << "recipient_ID_code = " << recipient_ID_code << "'" << endl;
  cout << "recipient_ID_qual = " << recipient_ID_qualifier << endl;
  cout << "recipient_routing_addr = " << recipient_routing_addr << endl;
#endif
             }
	         else { // Interchange type not supported.
#ifdef DEBUG
  cout << "else... no interchange supported... " << endl;
#endif
	              sender_ID_code[0] = '\0';
	              recipient_ID_code[0] = '\0';
#ifdef DEBUG
  cout << "sender_ID_code    = " << temp_sender << endl;
  cout << "recipient_ID_code = " << temp_recipient << endl;
#endif
	              sender_ID_qualifier[0] = '\0';
	              sender_routing_addr[0] = '\0';
	              recipient_ID_qualifier[0] = '\0';
	              recipient_routing_addr[0] = '\0';
                  }


#ifdef DEBUG
  cout << endl << "sender id code: " << "'" << get_sender_ID_code() << "'" << endl;
  cout << "sender id qualifier: " << get_sender_ID_qualifier() << endl;
  cout << "sender routing addr: " << get_sender_routing_addr() << endl;
  cout << "recipient id code: " << "'" << get_recipient_ID_code() << "'" << endl;
  cout << "recipient id qualifier: " << get_recipient_ID_qualifier() << endl;
  cout << "recipient routing addr: " << get_recipient_routing_addr() << endl;
#endif
     // Free up the space created by this function.
     if ( comp_separator[0] != NULL )
      delete comp_separator;
     if ( separator[0] != NULL )
	delete separator;
     if ( temp_sender[0] != NULL )
	delete temp_sender;
     if ( temp_recipient[0]!= NULL )
	delete temp_recipient;
     if ( interchange_type[0] != NULL )
        delete interchange_type;
     if ( temp_value[0] != NULL )
	delete temp_value;
     if ( value[0] != NULL )
        delete value;
#ifdef DEBUG
  cout << "----- End of parse_edi_names().-----" <<  endl << endl;
#endif
}


void Interchange::log_interchange(LOGLEVEL loglevel, ofstream *logfile) {

// Log the interchange only if the loglevel is appropriate.
if ( loglevel >= INTERCHANGE_LOG ) {

  *logfile << "------INTERCHANGE------" << endl;

  if ( recipient_ID_code != NULL )
   *logfile << TAB << "recipient_ID_code:       " << recipient_ID_code << endl;

  if ( recipient_ID_qualifier != NULL )
   *logfile << TAB << "recipient_ID_qualifier:  " << recipient_ID_qualifier << endl;

  if ( recipient_routing_addr != NULL )
   *logfile << TAB << "recipient_routing_addr:  " << recipient_routing_addr << endl;

  if ( recipient_reference != NULL )
   *logfile << TAB << "recipient_reference:     " << recipient_reference << endl;

  if ( recipient_reference_qualifier != NULL )
   *logfile << TAB << "recipient_ref_qual:      " << recipient_reference_qualifier << endl;

  if ( interchange_control_reference != NULL )
   *logfile << TAB << "interchange_control_ref: " << interchange_control_reference << endl;

  if ( process_priority_code != NULL )
   *logfile << TAB << "process_priority_code:   " << process_priority_code << endl;

  if ( acknowledgement_request != NULL )
   *logfile << TAB << "acknowledgement_request: " << acknowledgement_request << endl;

  if ( comm_agreement_ID != NULL )
   *logfile << TAB << "comm_agreement_ID:       " << comm_agreement_ID << endl;

  if ( test_indicator != NULL )
   *logfile << TAB << "test_indicator:          " << test_indicator << endl;

  if ( authorization_info != NULL )
   *logfile << TAB << "authorization_info:      " << authorization_info << endl;

  if ( authorization_info_qualifier != NULL )
   *logfile << TAB << "authorization_info_qual: " << authorization_info_qualifier << endl;

  if ( comp_data_element_separator != NULL )
   *logfile << TAB << "comp_data_element_sep:   " << comp_data_element_separator << endl;

  if (  data_element_separator!= NULL )
   *logfile << TAB << "data_element_separator:  " << data_element_separator << endl;

  if ( decimal_notation != NULL )
   *logfile << TAB << "decimal_notation:        " << decimal_notation << endl;

  if ( release_indicator != NULL )
   *logfile << TAB << "release_indicator:       " << release_indicator << endl;

  if ( reserved != NULL )
   *logfile << TAB << "reserved:                " << reserved << endl;

  if ( segment_terminator != NULL )
   *logfile << TAB << "segment_terminator:      " << segment_terminator << endl;

  if ( syntax_ID_string != NULL )
   *logfile << TAB << "syntax_ID_string:        " << syntax_ID_string << endl;

  if ( syntax_version != NULL )
   *logfile << TAB << "syntax_version:          " << syntax_version << endl;

  if ( sender_ID_code != NULL )
   *logfile << TAB << "sender_ID_code:          " << sender_ID_code << endl;

  if ( sender_ID_qualifier != NULL )
   *logfile << TAB << "sender_ID_qualifier:     " << sender_ID_qualifier << endl;

  if ( sender_routing_addr != NULL )
   *logfile << TAB << "sender_routing_addr:     " << sender_routing_addr << endl;

  if (  date_preparation != NULL )
   *logfile << TAB << "date_preparation:        " << date_preparation << endl;

  if ( time_preparation != NULL )
   *logfile << TAB << "time_preparation:        " << time_preparation << endl;

  if ( application_ref != NULL )
   *logfile << TAB << "application_ref:         " << application_ref << endl;

 }
}

#ifdef	DEBUG
//
// Display any attribute for which a value is present to the screen.
//

void Interchange::display_interchange() {

  cout << "-----------BEGIN DISPLAY INTERCHANGE-----------" << endl;

   cout << TAB << "recipient_ID_code:       " << recipient_ID_code << endl;
   cout << TAB << "recipient_ID_qualifier:  " << recipient_ID_qualifier << endl;
   cout << TAB << "recipient_routing_addr:  " << recipient_routing_addr << endl;
   cout << TAB << "recipient_reference:     " << recipient_reference << endl;
   cout << TAB << "recipient_ref_qual:      " << recipient_reference_qualifier << endl;
   cout << TAB << "interchange_control_ref: " << interchange_control_reference << endl;
   cout << TAB << "process_priority_code:   " << process_priority_code << endl;
   cout << TAB << "acknowledgement_request: " << acknowledgement_request << endl;
   cout << TAB << "comm_agreement_ID:       " << comm_agreement_ID << endl;
   cout << TAB << "test_indicator:          " << test_indicator << endl;
   cout << TAB << "authorization_info:      " << authorization_info << endl;
   cout << TAB << "authorization_info_qual: " << authorization_info_qualifier << endl;
   cout << TAB << "comp_data_element_sep:   " << comp_data_element_separator << endl;
   cout << TAB << "data_element_separator:  " << data_element_separator << endl;
   cout << TAB << "decimal_notation:        " << decimal_notation << endl;
   cout << TAB << "release_indicator:       " << release_indicator << endl;
   cout << TAB << "reserved:                " << reserved << endl;
   cout << TAB << "segment_terminator:      " << segment_terminator << endl;
   cout << TAB << "syntax_ID_string:        " << syntax_ID_string << endl;
   cout << TAB << "syntax_version:          " << syntax_version << endl;
   cout << TAB << "sender_ID_code:          " << sender_ID_code << endl;
   cout << TAB << "sender_ID_qualifier:     " << sender_ID_qualifier << endl;
   cout << TAB << "sender_routing_addr:     " << sender_routing_addr << endl;
   cout << TAB << "date_preparation:        " << date_preparation << endl;
   cout << TAB << "time_preparation:        " << time_preparation << endl;
   cout << TAB << "application_ref:         " << application_ref << endl;
  cout << "-----------END DISPAY INTERCHANGE-----------" << endl;
}

#endif
