// FILE = linklist.C

#include	<iostream.h>
#include	<assert.h>
#include	"../../../include/linklist.h"


LinkList::~LinkList() {		// deallocate memory for all cells in list
	if ( front == NULL )
		return;
	Cell *cell = front;
	while ( cell->next != NULL ) {
		cell = cell->next;
		delete cell->prev;
	}
	delete cell;
}


void LinkList::push( void *e ) {	// add cell to list front
	Cell *new_cell = new Cell( e, NULL, front );
	assert( new_cell != 0 );
	if ( rear == NULL ) 
		rear = curr = new_cell;
	else
		front->prev = new_cell;
	if ( curr == front )
		curr = new_cell;
	front = new_cell;
}


void *LinkList::pop() {			// delete cell at list front
	if ( front == NULL )		//   and return its info
		return NULL;
	void *e = front->info;
	if ( front == rear ) {
		delete front;
		front = rear = curr = NULL;
	}
	else {
		if ( curr == front )
			curr = front->next;
		front = front->next;
		delete front->prev;
		front->prev = NULL;
	}
	return e;
}


void LinkList::enter( void *e ) {	// add cell to list rear
	rear = new Cell( e, rear, NULL );
	assert( rear != 0 );
	if ( front == NULL )
		front = curr = rear;
	else
		rear->prev->next = rear;
}


void *LinkList::remove() {		// delete cell at list rear
	if ( front == NULL )      	//  and return its info
		return NULL;
	void *e = rear->info;
	if ( front == rear ) {
		delete front;
		front = rear = curr = NULL;
	}
	else {
		if ( curr == rear )
			curr = rear->prev;
		rear = rear->prev;
		delete rear->next;
		rear->next = NULL;
	}
	return e;
}


int LinkList::find( void *e ) {		// find specific cell info in list
	Cell *cell = front;		//  and return 0 or 1
	while ( cell != NULL ) {
		if ( cell->info == e )
			return 1;
		cell = cell->next;
	}
	return 0;
}


void LinkList::del( void *e ) {		// delete specific cell from list;
	Cell *cell = front;		//   do not return cell's info
	while ( cell != NULL ) {
		if ( cell->info == e ) {
			if ( curr == cell )
				if ( curr == rear )
					curr = cell->prev;
				else
					curr = cell->next;
			if ( cell == front )
				front = cell->next;
			else
				cell->prev->next = cell->next;
			if ( cell == rear )
				rear = cell->prev;
			else
				cell->next->prev = cell->prev;
			delete cell;
			return;
		}
		cell = cell->next;
	}
}

void *LinkList::current() {		// return current cell's info
	void	*e;

	if ( curr == NULL )
		return NULL;
	e = curr->info;
	curr = curr->next;
	return e;
}
