/*******************
/* mt_sxferin.c
********************
#ifndef lint
static char rcsid[]="$Header: mt_sxferin.c,v 2.8 93/07/12 12:54:10 trang Exp $";
#endif

/*
 * If this file looks strangely indented, it might be because you're using
 * Tabs that aren't equal to 8 spaces. I use tabs=8spaces; if you're using
 * vi, you can type ":se tabstop=8" to make this file more readable
 */

/*****************************************************************************
 *
 * FILENAME:	mt_sxferin.c
 *
 * DESCRIPTION:	This module contains the API function mt_start_transfer_in()
 *		and other assorted sub-functions called by it.
 *
 *****************************************************************************/

/* System includes */
#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>		/* for trace */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

/* Include files from other parts of X.400 product */
#include "conf.h"		/* x4admin interface */

/* APIA/XOPEN-supplied include files */
#include "xom.h"		/* object management */
#include "xmh.h"		/* message handling */

/* Internal include files */
#include "xomhp.h"		/* internal data structures */
#include "xmhp.h"		/* message handling */
#include "om_pkstruc.h"
#include "x4client.h"		/* trace */
#include "x4cat.h"		/* trace msg catalog */
#include "mac.h"		/* trace macro */

/* New include files created by me */
#include	<misc.h>

/* Some #defines */
#define	os_O_CREAT	O_CREAT	/* Should go into osdeps.h */
#define		DIR_LOCKFILE	".lock"	/* Maybe this should go into misc.h */

/*
 * Extern functions
 */
extern	OM_return_code		omp_is_object_valid(OM_object);
extern	boolean			os_untemp(char *, char *, char);
extern	boolean			os_prefix_file(char *, char *, char);
extern	int	mhp_parse(	OM_private_object, char *, char *,
				OM_int_private_object_header **, os_FILE **,
				char *,char *);

/*
 * Extern variables
 */
extern	int			errno;
extern	char			*strerror();
extern  OM_package_def		*XOM_package;
extern  int_OMP_workspace	x4_i_workspace;

/*
 * Static and other locally defined functions
 */
void           get_prefix_letter(char *, char *);
OM_return_code unreserve_pdu(char*, int, os_FILE *, OM_boolean);
OM_return_code lock_directory(char *, int *);
OM_return_code unlock_directory(char *, int);
static OM_return_code unlock_unreserve_move_to_uq(char	*, int);

/*
 * Static vars
 */
static	time_t	tloc;		/* Used in Procedure tracing */

/*
 * Global vars
 */
static	char			intbuf[20]; /* Used to sprintf integers */

/**************
* THIS HAS BEEN IMPLEMENTED DIFFERENTLY - the testing oq name is directly
* copied into globalconfig->oq_path, so it's transparent to this function

boolean	x4glob_fakedirs = FALSE;	 * Indicates if a fake set of dirs
					 * should be used (e.g, can use OQ =
					 * ./oq100 while testing, instead of
					 * /usr/spool/x400/oq/oq003)
					 *
char	x4glob_testoq[MAXFILE + 1];	 * Name of test oq dir to use, if
					 * x4glob_fakedirs is TRUE
**************/


/************************************************************************
**
** mt_start_transfer_in()
**
************************************************************************/

OM_return_code
mt_start_transfer_in(
	OM_private_object	session,
	OM_private_object	*object,
	OM_string		*identifier)
{
	extern struct map_global	*globalconfig;	/* Config struct */
	extern struct apiclient		*apiclient;	/* Config struct */
	OM_return_code	rc = OM_SUCCESS;
	char		oqpath[MAXFILE + 1]; /* Pathname of oq dir */
	char		fbuf[MAXFILE + 1]; /* Pathname of received file */
	char		reserved[MAXFILE + 1]; /* To store new filename once
						* it's been reserved
						*/
	char		prefix;		/* N/H/L on incoming msg */
	char		resv_prefix;	/* Lower-case equivalent of
					 * incoming prefix
					 */
	char		*p;	/* Temp pointer */
	boolean		try_again = TRUE;
	OM_int_private_object_header	*session_header;
	OM_descriptor   		*tmpdesc;
	OM_int_xferin_list		*headlist;
	OM_int_private_object_header	*tmpobject;
	OM_int_xferin_list		*entry;
	OM_int_xferin_list		*nextp, *curp;
	boolean				list_empty = TRUE;
	int		dird;		/* Directory descriptor */
	int		pdud;		/* File descriptor for PDU file */
	os_FILE         *parsefd=NULL;  /* fd for mhp_parse() asn1 msg */

	/* Trace the procedure entry */
	PROC_ENTRY("mt_start_transfer_in", "session <%d>", session, 0, 0, 0, 0);

	/******************************************
	 * Check if the session is a valid object
	 ******************************************/

	/* Make sure session ptr is legal... */
	if (!session)
	{
		log_error(OM_POINTER_INVALID, OMP_DF_POINTER_INVALID, 0);
		PROC_EXIT("mt_start_transfer_in", 0,session,OM_POINTER_INVALID);
		return(OM_POINTER_INVALID);
	}

	/* then check that it's a valid object that hasn't been deleted... */
	if ((rc = omp_is_object_valid(session)) != OM_SUCCESS)
	{
		/* Error already logged in lower-level function */
		PROC_EXIT("mt_start_transfer_in", 0, session, rc);
        	return(rc);
        }
		
	/* and check that it is a private object! */
	if (session->type != OM_PRIVATE_OBJECT )
	{
		log_error(OM_NOT_PRIVATE, OMP_DF_NOT_PRIVATE,
					(session->type==OM_CLASS) ?
					omp_hexme(session->value.string) : 0);
		PROC_EXIT("mt_start_transfer_in", 0, session, OM_NOT_PRIVATE);
		return(OM_NOT_PRIVATE);
	}

	/* Obtain the name of the OQ to monitor */
/*** This has been implemented in a different way **
	if (x4glob_fakedirs)	* Use a command-line specified directory *
		strcpy(oqpath, x4glob_testoq);
	else	* Use real directory location *
***/
		sprintf(oqpath, "%s/%s", globalconfig->oq_path, apiclient->oq);
	
	/* Loop until one is able to reserve one of the msgs in the queue */
	while (try_again)
	{

	/*
	 * Check if there are any messages in the input queue to this client
	 * (this queue is called an oq because it is an output queue from the
	 * point of view of the MTA) if so, try to read in the next available
	 * message in sequence
	 */

	/*
	 * Since there may be multiple processes that access the directory
	 * and they may be in different states, we need to ensure the
	 * atomicity of operations across several functions:
	 * The relevant functions are:
	 *	mt_start_transfer_in() (picking up a new message)
	 *	mt_finish_transfer_in() (retaining a message if needed)
	 *	mt_open() (recovering reserved messages from dead processes)
	 *
	 * We use the following algorithm here in mt_start_transfer_in:
	 *	open, lock parent directory
	 *	find first unreserved PDU (H/N/M/L)
	 *	reserve it (h/n/m/l)
	 *	open file, save descriptor in OM_int_xferin_list
	 *	lock the file
	 *	unlock, close the directory
	 */

	/*
	 * Open and lock the oq, into which incoming msgs would
	 * have been placed
	 */
	if ((rc = lock_directory(oqpath, &dird)) != OM_SUCCESS)
	{
		/* Error already logged in called function */
		unlock_directory(oqpath, dird);
		PROC_EXIT("mt_start_transfer_in", 0, session, rc);
		return(rc);
	}

	/*
	 * If a file is found with High-Prefix ('H'), process it; if not,
	 * check if a file with the Normal-Prefix ('N') is found, and
	 * process that, and so on down, through the priority levels.
	 * The alphabetically (and therefore chronologically, because of
	 * the way os_uniq_name() works) first file with the given prefix
	 * is picked up
	 *
	 * Note that as soon as one of the ORed conditions below evaluates
	 * to TRUE, the others will not be checked. This is crucial to
	 * the order of Priority being maintained
	 */
	if ( (!os_prefix_file(fbuf, oqpath, HIGH_PREFIX)) ||
		(!os_prefix_file(fbuf, oqpath, NORMAL_PREFIX)) ||
		(!os_prefix_file(fbuf, oqpath, MEDIUM_PREFIX)) ||
		(!os_prefix_file(fbuf, oqpath, LOW_PREFIX)) )

	{	/* File was found in queue */

		/*
		 * Reserve the file 
		 * 
		 * The prefix letter in the original filename was 'H', 'N',
		 * 'M, or 'L', depending on the priority of the message.
		 * The file is reserved by changing this prefix to the
		 * corresponding lower-case letter ('h' or 'n' or 'm' or 'l').
		 * So for example, the file N2828d09da.001 would be 
		 * reserved by changing its name to n2828d09da.001 .
		 */

		/* Obtain the prefix from the original filename */
		(void)get_prefix_letter(fbuf, &prefix);

		/* Obtain the prefix to use to reserve it */
		resv_prefix = RESERVED(prefix);

		if (!os_untemp(reserved, fbuf, resv_prefix))
		{	/* Check if os_untemp() failed because some other
			 * process/session snuck in and reserved this file.
			 * If so, pick up a different message and try again.
			 * (This is why the while-loop exists)
			 *
			 * Otherwise, some unexpected system error occurred
			 * inside os_untemp(). Log the system error, return.
			 */
			if ((errno == ENOENT) || (errno == EEXIST))
				continue;
			else
			{
				log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR,
						__FILE__, __LINE__,
						MH_INT_DF_FAILED_TO_RESV_UNRESV,
						fbuf, strerror(errno),0);
				unlock_directory(oqpath, dird);
				PROC_EXIT("mt_start_transfer_in", 0, session,
							OM_SYSTEM_ERROR);
				return(OM_SYSTEM_ERROR);
			}
		}
		else
			try_again = FALSE;

	}
	else
	{	/* Error, no message waiting to be processed */
		log_error(MH_RC_QUEUE_EMPTY, MH_RC_DF_QUEUE_EMPTY, oqpath);
		unlock_directory(oqpath, dird);
		PROC_EXIT("mt_start_transfer_in", 0, session,MH_RC_QUEUE_EMPTY);
		return(MH_RC_QUEUE_EMPTY);
	}
	
	}	/* End of while (try_again) loop */

	/* If we've reached here, we've found a file and reserved it.
	 * Now lock the file and unlock the directory. The file descriptor
	 * should be saved in the int_xferin_list entry, so it can later
	 * be unlocked (say in mt_finish_transfer_in)
	 */

	/* Need to open the PDU file first */
	if ((pdud = os_std_open(reserved, os_O_RDWR)) <= 0)
	{
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_OPEN,
				reserved, strerror(errno),0);
		unlock_directory(oqpath, dird);
		PROC_EXIT("mt_start_transfer_in", 0, session, OM_SYSTEM_ERROR);
		return(OM_SYSTEM_ERROR);
	}

	/* Now lock it. Given our algorithm, there is no way that another
	 * process could have locked this file, so it's an error if it
	 * doesn't work...
	 * (unless there's a flaw in our algorithm for atomicity...!)
	 */
	if (os_lockf(pdud, os_F_TLOCK, 0) == -1)
	{
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_LOCK,
				reserved, strerror(errno),0);
		unlock_directory(oqpath, dird);
		PROC_EXIT("mt_start_transfer_in", 0, session, OM_SYSTEM_ERROR);
		return(OM_SYSTEM_ERROR);
	}

	/* Finally, unlock and close the oq directory */
	if ((rc = unlock_directory(oqpath, dird)) != OM_SUCCESS)
	{
		PROC_EXIT("mt_start_transfer_in", 0, session, rc);
		return(rc);
	}

	
	/*
	 * With all that junk out of the way, now we can start processing the
	 * message (finally!)
	 */

	/* Try to parse the message and save it in a private object */
	rc = mhp_parse(session, reserved, globalconfig->tmp_path, &tmpobject,
				&parsefd,
				globalconfig->iq_path, globalconfig->uq_path);

	/* Now check this return-code for various specific error codes */
	switch (rc)
	{
		case	OM_SUCCESS:
			/* Break out of this switch stmt, continue processing */
			break;

		case    OM_MEMORY_INSUFFICIENT: 
			/* This is treated as temporary error, so we don't
			 * want to generate reports for them. Unreserve msg,
			 * leave it in oq and return. The client can try
			 * again later if he/she/(/it?!) wants to
			 */

			/* Unreserve msg */
			(void)unreserve_pdu(reserved, pdud, parsefd, OM_FALSE);

			/* Whether the os_untemp() in unreserve_pdu succeeded
			 * or not, the return code is not SYSTEM_ERROR, but
			 * the original error code
			 */
			PROC_EXIT("mt_start_transfer_in", 0, session, rc);
			return(rc);

		default: /* anything else, move the pdu to uq */
			/* The function mhp_parse() has logged the error. Let's
			 * just unreserve, unlock the file, and move it to
			 * the UQ. 
			 */
			(void)unlock_unreserve_move_to_uq(reserved, pdud);
      			enddp_specified_fd (parsefd); 

			/* Whatever happened in the above function call,
			 * the return code is not SYSTEM_ERROR, but
			 * the original error code
			 */
			PROC_EXIT("mt_start_transfer_in", 0, session, rc);
			return(rc);

	}	/* End of switch stmt */

	/*
	 * The message has been parsed successfully..
	 *
	 * Any errors encountered from this point onwards are also treated
	 * as temporary errors, so do the same thing as in the default case
	 * of the switch stmt above - i.e. unreserve the msg and leave it
	 * in oq. Remove any structures added to linked-list xferin
	 */

	/*
	 * Create an entry for this message (object) to be added to the
	 * session's transfer-in list
	 */
	if (!(entry =
		(OM_int_xferin_list *) os_calloc(sizeof(OM_int_xferin_list))))
	{
		log_error(OM_MEMORY_INSUFFICIENT, OMP_DF_MEMORY_INSUFFICIENT);
		os_free(entry);

		/* Unreserve msg */
		(void)unreserve_pdu(reserved, pdud, parsefd, OM_FALSE);

		/* Whether the os_untemp() succeeded or not, the
		 * return code is not SYSTEM_ERROR, but the original
		 * error code
		 */
		PROC_EXIT("mt_start_transfer_in", 0, session,
						OM_MEMORY_INSUFFICIENT);
		return(OM_MEMORY_INSUFFICIENT);
	}
	
	entry->prefix = prefix;
	strcpy(entry->xferin_file_name, reserved);
	entry->lockfd = pdud;	/* To be unlocked in mt_finish_transfer_in,
				 * or elsewhere
				 */
	entry->setpfd = parsefd; /* to be closed when unreserve */
	tmpdesc = (OM_descriptor *)tmpobject;
	entry->object = (OM_private_object)++tmpdesc;

	/*
	 * Now add the entry created, to the transfer_in_list
	 */

	tmpdesc = (OM_descriptor *)session;
	session_header = (OM_int_private_object_header *)--tmpdesc;
	
	if (headlist = session_header->context.xferin_list)
	{	/* List is not empty */
		list_empty = FALSE;
		curp = headlist;
		nextp = (OM_int_xferin_list *)curp->next;

		/* Move to end of list */
		while(nextp)
		{
			curp = nextp;
			nextp = (OM_int_xferin_list *)curp->next;
		}
		
		/* Add entry to end of list */
		curp->next = (struct int_xferin_list *)entry;
		entry->prev = (struct int_xferin_list *)curp;
		/* entry->next is already NULL because it was calloc'ed */
	}
	else
	{	/* List is empty */
		session_header->context.xferin_list = entry;
		/* entry->next and entry->prev are already NULL because 
		 * they were calloc'ed
		 */
	}

	/* Object to be returned */
	*object = entry->object;

	/* Identifier to be returned - user must provide enough space */
	identifier->length = strlen(fbuf);
	strcpy(identifier->elements, fbuf);

	session_header->context.modifier &= OM_INT_XFERIN;

	PROC_EXIT("mt_start_transfer_in", session, object, OM_SUCCESS);
	return(OM_SUCCESS);

}	/* mt_start_transfer_in() */


/************************************************************************
**
** get_prefix_letter() - Given a filename, obtains the prefix letter 
** Not static, called in mt_open()
**
*************************************************************************/
void
get_prefix_letter(
	char	*file,
	char	*prefix)
{
	char	*p;	/* Temp pointer */

	if ((p = strrchr(file, '/')) != NULL)
		p++;
	else
		p = file;
	*prefix = *p;
		
}	/* get_prefix_letter() */

/************************************************************************
**
** get_parent_dir_name() - Given a filename, obtains the pathname of its
** parent directory (or the name of the queue the file resides in)
**
*************************************************************************/
static void
get_parent_dir_name(
	char	*file,
	char	*parent)
{
	char	*p, *q;	/* Temp pointers */

	strcpy(parent, file);

	q = parent;
	if ((p = strrchr(parent, '/')) != NULL) /* The name contains a '/' */
		if (p == q)	/* The only / is at the beginning */
			parent[1] = '\0';	/* Parent dir is "/" */
		else	/* There is a pathname preceding the '/' */
			*p = '\0';	/* Terminate string before the '/' */
	else	/* No '/' in the name */
		strcpy(parent, ".");	/* File is in the current directory */

}	/* get_parent_dir_name() */

/************************************************************************
**
** lock_directory() - Opens and locks the given directory, returns descriptor
** 	INPUT:	Directory pathname
**	OUTPUT: Pointer to file descriptor of opened directory name
**
** Actually, it doesn't lock the directory, but instead a well-known filename
** (DIR_LOCKFILE) inside the directory. This is because one can't lock dirs -
** because lockf(2) needs the file to have been opened with O_WRONLY or O_RDWR,
** and open(2) only allows you to open directories with O_RDONLY.
**
** Not static, called in mt_open()
**
*************************************************************************/
OM_return_code
lock_directory(
	char	*queuename,
	int	*dird)
{

	char	dirlockfile[MAXFILE + 1];
	mode_t	prevmask;
    uid_t               x4_uid;
	gid_t               x4_gid;
	struct passwd       *pwdp;
    boolean             new_lock_file=FALSE;

	/* Obtain name of dirlockfile */
	sprintf(dirlockfile, "%s/%s", queuename, DIR_LOCKFILE);

	/* Open the directory lock file. If it doesn't exist, it is
	 * created with permissions (0666 - umask) typically, this might
	 * be 0644 (since umask is typically 022)
	 */
	
	/* make sure lock file readable and writable by user and group as umask */
	prevmask = umask( S_IWOTH | S_IROTH );

	if ((*dird = os_std_open(dirlockfile, os_O_RDWR ))
								== -1)
	{ 
	/* file does not exit */
	if ((*dird = os_std_open(dirlockfile, os_O_RDWR | os_O_CREAT, 0666))
								== -1)
	{
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_OPEN,
				dirlockfile, strerror(errno),0);
		return(OM_SYSTEM_ERROR);
	}

	pwdp = getpwnam("x400");

	x4_uid = pwdp->pw_uid;
	x4_gid = pwdp->pw_gid;

	/* Change ownership of file */

		if (chown(dirlockfile, x4_uid, x4_gid) != 0)
			{
		       log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__,"cannot chown to x400" ,
			   dirlockfile, strerror(errno),0);
		       return(OM_SYSTEM_ERROR);

			}
	}
	(void) umask(prevmask); /* reset umask */

	/* and lock it. This waits till a lock can be obtained, so it should
	 * always succeed
	 */
	if (os_lockf(*dird, os_F_LOCK, 0) == -1)
	{
		/* The os_F_LOCK option should make it wait if the dirlockfile
		 * is currently locked by someone else, so if it gives an error,
		 * it is a genuine error...
		 */
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_LOCK,
				dirlockfile, strerror(errno),0);
		return(OM_SYSTEM_ERROR);
	}
	
	return(OM_SUCCESS);

}	/* lock_directory() */

/************************************************************************
**
** unlock_directory() - Unlocks given directory, closes it, returns descriptor
** 	INPUT:	Directory pathname, directory file-descriptor
**	OUTPUT: Return code indicating success/failure
**
** Not static, called in mt_open()
**
*************************************************************************/
OM_return_code
unlock_directory(
	char	*queuename,
	int	dird)
{

	char	dirlockfile[MAXFILE + 1];

	/* Obtain name of dirlockfile */
	sprintf(dirlockfile, "%s/%s", queuename, DIR_LOCKFILE);

	/* Unlock the directory lock file */
	if (os_lockf(dird, os_F_ULOCK, 0) == -1)
	{
		sprintf(intbuf, "%d", dird);
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_UNLOCK, intbuf,
				dirlockfile, strerror(errno));
		return(OM_SYSTEM_ERROR);
	}

	/* and close it */
	if (os_std_close(dird) == -1)
	{
		sprintf(intbuf, "%d", dird);
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_CLOSE, intbuf,
				dirlockfile, strerror(errno));
		return(OM_SYSTEM_ERROR);
	}

	return(OM_SUCCESS);
	
}	/* unlock_directory() */

/************************************************************************
**
** unreserve_pdu() - This leaves the PDU in whatever directory it is in
** currently, but changes the prefix letter back to the equivalent
** Unreserved prefix. 
**
** This function has now been modified so it also unlocks the PDU file.
** To do this, it has to lock and unlock the directory, so the unlocking
** and 'unreserving' of the PDU file are atomic. The file descriptor to
** unlock with, has been added as a parameter. --Ram 11/13/91
**
** Also added as a parameter is the boolean 'remove' - if it is set to
** OM_TRUE, it deletes the file altogether.
**
** The algorithm to ensure atomicity is:
**	Lock parent directory
**	Unlock file and close it
**	If 'remove' is TRUE, then
**		delete file
**	else
**		unreserve it
**	Unlock parent directory
**	
**
** NOTE: This function is not static because it is called from mt_fxferin.c
**
*************************************************************************/
OM_return_code
unreserve_pdu(
	char	*reserved,
	int	lockfd,
	os_FILE *parsefd,
	OM_boolean	remove)
{

	char		unreserved[MAXFILE + 1]; /* To hold new filename*/
	char		resv_prefix, unresv_prefix;
	int		dird;		/* Directory descriptor */
	char		queuename[MAXFILE + 1]; /* To hold name of queue */
	OM_return_code	rc;

	(void)get_prefix_letter(reserved, &resv_prefix);
	unresv_prefix = UNRESERVED(resv_prefix);
	
	/* Get pathname of oq, from reserved filename */
	(void) get_parent_dir_name(reserved, queuename);

	/* Lock parent directory */
	if ((rc = lock_directory(queuename, &dird)) != OM_SUCCESS)
		return(rc);

	/* Now unlock the PDU file */
	if (os_lockf(lockfd, os_F_ULOCK, 0) == -1)
	{
		sprintf(intbuf, "%d", lockfd);
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_UNLOCK, intbuf,
				reserved, strerror(errno));
		unlock_directory(queuename, dird);
		return(OM_SYSTEM_ERROR);
	}

	/* close it */
	if (os_std_close(lockfd) == -1)
	{
		sprintf(intbuf, "%d", lockfd);
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_CLOSE, intbuf,
				reserved, strerror(errno));
		unlock_directory(queuename, dird);
		return(OM_SYSTEM_ERROR);
	}

	/* close the file descriptor  created in mhp_parse for the pdu file */
	if (parsefd)
		enddp_specified_fd (parsefd);

	/* And if 'remove' is TRUE, delete the file. Otherwise unreserve it */
	if (remove)
	{
		if(os_delete(reserved) == -1)
		{
			log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR,__FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_LINK_UNLINK,
				reserved, strerror(errno),0);
			unlock_directory(queuename, dird);
			return(OM_SYSTEM_ERROR);
		}
	}
	else
	{
		if (!os_untemp(unreserved, reserved, unresv_prefix))
		{
			log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR,__FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_RESV_UNRESV,
				reserved, strerror(errno),0);
			unlock_directory(queuename, dird);
			return(OM_SYSTEM_ERROR);
		}
	}

	/* Finally, unlock and close the parent directory */
	if ((rc = unlock_directory(queuename, dird)) != OM_SUCCESS)
		return(rc);

	/* Made it! */
	return(OM_SUCCESS);
}	/* unreserve_pdu() */

/************************************************************************
**
** unlock_unreserve_move_to_uq() - This moves the PDU to the UQ, 
** unlocks it and changes the prefix letter back to the equivalent
** Unreserved prefix.
**
*************************************************************************/
static OM_return_code
unlock_unreserve_move_to_uq(
	char	*reserved,
	int	lockfd)
{

	char		unreserved[MAXFILE + 1]; /* To hold new filename*/
	char		resv_prefix, unresv_prefix;
	int		dird;		/* Directory descriptor */
	char		queuename[MAXFILE + 1]; /* To hold name of queue */
	OM_return_code	rc;
	extern struct map_global	*globalconfig;	/* Config struct */

	(void)get_prefix_letter(reserved, &resv_prefix);
	unresv_prefix = UNRESERVED(resv_prefix);
	
	/* Get pathname of oq (parent directory), from reserved filename */
	(void) get_parent_dir_name(reserved, queuename);

	/* Lock parent directory */
	if ((rc = lock_directory(queuename, &dird)) != OM_SUCCESS)
		return(rc);

	/* Now unlock the PDU file */
	if (os_lockf(lockfd, os_F_ULOCK, 0) == -1)
	{
		sprintf(intbuf, "%d", lockfd);
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_UNLOCK, intbuf,
				reserved, strerror(errno));
		unlock_directory(queuename, dird);
		return(OM_SYSTEM_ERROR);
	}

	/* close it */
	if (os_std_close(lockfd) == -1)
	{
		sprintf(intbuf, "%d", lockfd);
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_CLOSE, intbuf,
				reserved, strerror(errno));
		unlock_directory(queuename, dird);
		return(OM_SYSTEM_ERROR);
	}

	/* And unreserve it */
	if (!os_untemp(unreserved, reserved, unresv_prefix))
	{
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_RESV_UNRESV,
				reserved, strerror(errno),0);
		unlock_directory(queuename, dird);
		return(OM_SYSTEM_ERROR);
	}

	/* Now move the file to the UQ */
	if (os_move_file(unreserved, globalconfig->uq_path) == FALSE)
	{
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FAILED_TO_MOVE, 
				globalconfig->uq_path,
				unreserved, strerror(errno));
	}
	else	/* move to UQ succeeded */
	{
		log_error(OM_SYSTEM_ERROR, OMP_DF_SYSTEM_ERROR, __FILE__,
				__LINE__, MH_INT_DF_FILE_MOVED, 
				globalconfig->uq_path, reserved,0);
	}

	
	/* Finally, unlock and close the parent directory */
	if ((rc = unlock_directory(queuename, dird)) != OM_SUCCESS)
		return(rc);

	/* Made it! */
	return(OM_SUCCESS);
}	/* unlock_unreserve_move_to_uq() */
