
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include "p1p2.h"
#include "xom.h"
#include "xmhp.h"
#include "ximp.h"
#include "xomhp.h"
#include "x4apifp.h"
#include "climport.h"


extern boolean list_append();
extern boolean list_alloc();
extern int free_dlist();
/* extern char *String_dir; */
extern int mhp_p_eits();

static OM_return_code mhp_p_im_ipm();
static OM_return_code mhp_p_im_ipn();
static OM_return_code mhp_p_im_id();
static OM_return_code mhp_p_im_or_desc();
static OM_return_code mhp_p_im_recip();
static OM_return_code mhp_p_im_ia5();
static OM_return_code mhp_p_im_binary();
static OM_return_code mhp_p_im_iso6937();
static OM_return_code mhp_p_im_msgbp();
static OM_return_code mhp_p_im_delenvelop();
static OM_return_code mhp_p_im_ttx();
static OM_return_code mhp_p_im_external();


/* declare commonly used functions */
extern int putvalue(OM_int_descriptor *, OM_int_value **, OM_type, 
                    OM_syntax, void*);
extern int p_strcpy(OM_int_descriptor *, OM_int_value **, OM_type,
             OM_syntax, void*, Uint32);
extern OM_return_code write_str(OM_int_descriptor *, OM_int_value **,
            Uint16, Uint16, char *);
extern OM_return_code mhp_p_external(struct ext_type *, OM_boolean,
        OM_int_private_object_header **);
extern char *get_delenvl_extension(P22MessageBodyPart *msgbp, sint16 type);



/*
 * mhp_parse_ipm(ipm, object)
 *
 * Input:
 *
 * Output:
 *        object - handle to the output object
 *
 */

int mhp_parse_ipm(ipm, object)
P22InformationObject *ipm;
OM_int_private_object_header **object;
{
    OM_return_code mhp_p_im_ipm();
    OM_return_code mhp_p_im_ipn();
    int status;
    
    switch (ipm->t) {
    case InformationObject_ipm:
        if ((status = mhp_p_im_ipm(ipm->v.ipm, object)) != OM_SUCCESS)
            return(status);
        break;
    case InformationObject_ipn:
        if ((status = mhp_p_im_ipn(ipm->v.ipn, object)) != OM_SUCCESS)
            return(status);
        break;
    default:
        return(OM_NO_SUCH_OBJECT);
        break;
    } /* end switch */
    return(OM_SUCCESS);
} /* end mhp_parse_ipm() */


/*************************************************************************
* mhp_p_im_ipm(ipm, object)
**************************************************************************/

static OM_return_code mhp_p_im_ipm(ipm, object)
P22IPM *ipm;
OM_int_private_object_header **object;
{
/*
OM_return_code mhp_p_im_ia5();
OM_return_code mhp_p_im_binary();
OM_return_code mhp_p_im_iso6937();
OM_return_code mhp_p_im_ttx();
OM_return_code mhp_p_im_id();
OM_return_code mhp_p_im_or_desc();
OM_return_code mhp_p_im_recip();
*/
    OM_int_private_object_header *subobject;
    P22Heading *heading;
    char *buf;
    int status;
    OM_boolean flag;
    OM_enumeration enumer;
    OM_int_descriptor *desc;
    OM_int_value *baseval, *curval, *nextval;
    ListPointer *plist;
    Uint32 len;
    Uint16 i=0, count;
    P22HeadingExtension *p22hdext;
    int incomplete_copy = OM_FALSE;
    
    /*
    TRACE2("Start mhp_p_im_ipm in file %s\n", __FILE__);
    */
    
    if ((status = omp_template(IM_C_INTERPERSONAL_MSG,object)) !=OM_SUCCESS)
        return(status);
    
    heading = (P22Heading *)&ipm->heading;
    
    /* Index through IPM descriptor list sequentially and fill in attribute
     * if it exists. */
    for (i = TOP;       
        (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
        i++) 
    {
        desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
        switch (desc->attr_type) {
        case IM_AUTO_FORWARDED:
            flag = OM_FALSE;        /* default is FALSE */
            if (heading->m & Heading_auto_forwarded) {
                if (heading->auto_forwarded)
                    flag = OM_TRUE;
            } /* end if */
            if ((status = putvalue(desc,&desc->attr_value,IM_AUTO_FORWARDED,OM_S_BOOLEAN,(void*)flag)) != OM_SUCCESS)
                return(status);
            break;
    
        case IM_AUTHORIZING_USERS:
            if (heading->m & Heading_authorizing_users) {
            for (plist = (ListPointer *)&(heading->authorizing_users),
                plist->current = plist->first, count = 0,
                baseval = curval = nextval = desc->attr_value;
                plist->current != NULL; plist->current = plist->current->next) {
                /* Create and fill in the Authorizing Users sub-object */
                if ((status = mhp_p_im_or_desc(plist->current->data,&subobject)) != OM_SUCCESS)
                return(status);
        
                /* Create a value structure for the subobject and link it */
                if ((status = putvalue(desc,&nextval,IM_AUTHORIZING_USERS,OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        
                append_struct(&baseval,&curval,nextval,&count);
            } /* end for */
        
            desc->attr_value = baseval;
            desc->count = count;        
            } /* end if */
            break;
        case IM_BLIND_COPY_RECIPIENTS:
            if (heading->m & Heading_blind_copy_recipients) {
            for (plist = (ListPointer *)&(heading->blind_copy_recipients),
                plist->current = plist->first, count = 0,
                baseval = curval = nextval = desc->attr_value;
                plist->current != NULL; plist->current = plist->current->next) {
                /* Create and fill in the Blind Copy Recipient sub-object */
                if ((status = mhp_p_im_recip(plist->current->data,&subobject)) != OM_SUCCESS)
                return(status);
        
                /* Create a value structure for the subobject and link it */
                if ((status = putvalue(desc,&nextval,IM_BLIND_COPY_RECIPIENTS,OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        
                append_struct(&baseval,&curval,nextval,&count);
            } /* end for */
            desc->attr_value = baseval;
            desc->count = count;        
            } /* end if */
            break;
        case IM_BODY:
            if (ipm->m & IPM_Body) {
            for (plist = (ListPointer *)&(ipm->body),
                plist->current = plist->first, count = 0,
                baseval = curval = nextval = desc->attr_value;
                plist->current != NULL; plist->current = plist->current->next) 
            {
                subobject = NULL;
                switch(((struct bodypart *)plist->current->data)->type) {
                case MH_IA5TEXTPART:
                    if ((status = mhp_p_im_ia5(plist->current->data,&subobject)) != OM_SUCCESS)
                        return(status);
                    break;
                case MH_FWDEDIPMSG:
                    if ((status = mhp_p_im_msgbp(plist->current->data,&subobject))
                                != OM_SUCCESS)
                        return(status);
                    break;
    
                case MH_TELETEXPART:
                    if ((status = mhp_p_im_ttx(plist->current->data,&subobject)) 
                                != OM_SUCCESS)
                        return(status);
                    break;
    
                case MH_UNIDENTIFIED:
                    if ((status = mhp_p_im_binary(plist->current->data,&subobject)) != OM_SUCCESS)
                        return(status);
                    break;
                case MH_EXTERNALPART:
                    if ((status = mhp_p_im_external(plist->current->data,
                                    &subobject)) != OM_SUCCESS)
                            return(status);
                    break;
                case MH_ISO6937PART:
                    if ((status = mhp_p_im_iso6937(plist->current->data,&subobject)) != OM_SUCCESS)
                        return(status);
                    break;
                default:
                    break;
                } /* end switch */
        
                if (subobject == NULL)
                /* No body part */
                    break;
        
                /* Create a value structure for the subobject and link it */
                if ((status = putvalue(desc,&nextval,IM_BODY,OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
                append_struct(&baseval,&curval,nextval,&count);
            } /* end for */
        
            desc->attr_value = baseval;
            desc->count = count;        
            } /* end if */
            break;
        case IM_COPY_RECIPIENTS:
            if (heading->m & Heading_copy_recipients) {
            for (plist = (ListPointer *)&(heading->copy_recipients),
                plist->current = plist->first, count = 0,
                baseval = curval = nextval = desc->attr_value;
                plist->current != NULL; plist->current = plist->current->next) 
            {
    
                /* Create and fill in the Copy Recipient sub-object */
                if ((status = mhp_p_im_recip(plist->current->data,&subobject)) 
                                        != OM_SUCCESS)
                    return(status);
        
                /* Create a value structure for the subobject and link it */
                if ((status = putvalue(desc,&nextval,IM_COPY_RECIPIENTS,
                            OM_S_OBJECT,subobject)) != OM_SUCCESS)
                    return(status);
                append_struct(&baseval,&curval,nextval,&count);
            } /* end for */
    
            desc->attr_value = baseval;
            desc->count = count;        
            } /* end if */
            break;
    
        case IM_EXPIRY_TIME:
            if (heading->m & Heading_expiry_time) {
            if ((buf = (char *)mhp_to_utc(&heading->expiry_time)) == NULL)
                return(OM_MEMORY_INSUFFICIENT);
        
            if ((status = putvalue(desc,&desc->attr_value,IM_EXPIRY_TIME,OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
                return(status);
            } /* end if */
            break;
    
        case IM_IMPORTANCE:
            if (heading->m & Heading_importance) {
            switch (heading->importance) {
            case IMPORTANCE_low:
                enumer = IM_LOW;
                break;
            case IMPORTANCE_normal:
            default:
                enumer = IM_ROUTINE;
                break;
            case IMPORTANCE_high:
                enumer = IM_HIGH;
                break;
            } /* end switch */
            } /* end if */
            else { /* default is normal */
                enumer = IM_ROUTINE;
            }
            if ((status = putvalue(desc,&desc->attr_value,IM_IMPORTANCE,OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
                return(status);
            break;
    

    case IM_INCOMPLETE_COPY:
    if ( ipm->heading.m &  Heading_extensions )
    {
    int i;   /* Not to be confused with outer loop i */
    P22ExtensionsField *p22extfld;
      p22extfld = (P22ExtensionsField *) ipm->heading.extensions;
      for (i=0; i<p22extfld->n; i++) {
            p22hdext = (P22HeadingExtension *) p22extfld->v[i];
        if ( p22hdext->m & HeadingExtension_headingextensiontype )
        {
            if ( (p22hdext->headingextensiontype.length == 5 ) &&
                 (p22hdext->headingextensiontype.id[0] == 2 ) &&
                 (p22hdext->headingextensiontype.id[1] == 6 ) &&
                 (p22hdext->headingextensiontype.id[2] == 1 ) &&
                 (p22hdext->headingextensiontype.id[3] == 5 ) &&
                 (p22hdext->headingextensiontype.id[4] == 0 ) )
            {
                incomplete_copy = OM_TRUE;
                break;
            }
        }
      }
    }
    if ((status = putvalue(desc,&desc->attr_value,IM_INCOMPLETE_COPY,
        OM_S_BOOLEAN,(void*)incomplete_copy)) != OM_SUCCESS)
                return(status);
        break;
    case IM_LANGUAGES:
    if ( ipm->heading.m &  Heading_extensions )
    {
    int i;   /* Not to be confused with outer loop i */
    P22ExtensionsField *p22extfld;
      p22extfld = (P22ExtensionsField *) ipm->heading.extensions;
      for (i=0; i<p22extfld->n; i++) {
            p22hdext = (P22HeadingExtension *) p22extfld->v[i];
        if ( p22hdext->m & HeadingExtension_headingextensiontype )
        {
            if ( (p22hdext->headingextensiontype.length == 5 ) &&
                 (p22hdext->headingextensiontype.id[0] == 2 ) &&
                 (p22hdext->headingextensiontype.id[1] == 6 ) &&
                 (p22hdext->headingextensiontype.id[2] == 1 ) &&
                 (p22hdext->headingextensiontype.id[3] == 5 ) &&
                 (p22hdext->headingextensiontype.id[4] == 1 ) )
            {
            P22Languages *lang;
            int j;
                parse_p2hding_extensions( p22hdext->headingextensionvalue, 1, &(p22hdext->extstruct) );
                lang = (P22Languages *) p22hdext->extstruct;
                for (j=0; j <lang->n; j++)
                {
                     if ((status = putvalue(desc,&nextval,
                      IM_LANGUAGES,OM_S_PRINTABLE_STRING,
                      (void *)lang->v[i])) != OM_SUCCESS)
                            return(status);
                             append_struct(&baseval,&curval,nextval,&count);
                            } /* end for */
                    desc->attr_value = baseval;
                    desc->count = count;        
            }
        }
      }
    }
        break;

    
        case IM_OBSOLETED_IPMS:
            if (heading->m & Heading_obsoleted_IPMs) {
            for (plist = (ListPointer *)&(heading->obsoleted_IPMs),
                plist->current = plist->first, count = 0,
                baseval = curval = nextval = desc->attr_value;
                plist->current != NULL; plist->current = plist->current->next) {
                /* Create and fill in the Obsoleted IPMs sub-object */
                if ((status = mhp_p_im_id(plist->current->data,&subobject)) != OM_SUCCESS)
                return(status);
        
                /* Create a value structure for the subobject and link it */
                if ((status = putvalue(desc,&nextval,IM_OBSOLETED_IPMS,OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        
                append_struct(&baseval,&curval,nextval,&count);
            } /* end for */
            desc->attr_value = baseval;
            desc->count = count;        
            } /* end if */
            break;
        case IM_ORIGINATOR:
            if (heading->m & Heading_originator) {
            if ((status = mhp_p_im_or_desc(heading->originator,&subobject)) != OM_SUCCESS)
                return(status);
        
            if ((status = putvalue(desc,&desc->attr_value,IM_ORIGINATOR,OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
            } /* end if */
            break;
        case IM_PRIMARY_RECIPIENTS:
            if (heading->m & Heading_primary_recipients) {
            for (plist = (ListPointer *)&(heading->primary_recipients),
                plist->current = plist->first, count = 0,
                baseval = curval = nextval = desc->attr_value;
                plist->current != NULL; plist->current = plist->current->next) {
                /* Create and fill in the Primary Recipient sub-object */
                if ((status = mhp_p_im_recip(plist->current->data,&subobject)) != OM_SUCCESS)
                return(status);
        
                /* Create a value structure for the subobject and link it */
                if ((status = putvalue(desc,&nextval,IM_PRIMARY_RECIPIENTS,OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        
                append_struct(&baseval,&curval,nextval,&count);
            } /* end for */
            desc->attr_value = baseval;
            desc->count = count;        
            } /* end if */
            break;
        case IM_RELATED_IPMS:
            if (heading->m & Heading_related_IPMs) {
            for (plist = (ListPointer *)&(heading->related_IPMs),
                plist->current = plist->first, count = 0,
                baseval = curval = nextval = desc->attr_value;
                plist->current != NULL; plist->current = plist->current->next) {
                /* Create and fill in the Related IPMs sub-object */
                if ((status = mhp_p_im_id(plist->current->data,&subobject)) != OM_SUCCESS)
                return(status);
        
                /* Create a value structure for the subobject and link it */
                if ((status = putvalue(desc,&nextval,IM_RELATED_IPMS,OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        
                append_struct(&baseval,&curval,nextval,&count);
            } /* end for */
            desc->attr_value = baseval;
            desc->count = count;        
            } /* end if */
            break;
        case IM_REPLIED_TO_IPM:
            if (heading->m & Heading_replied_to_IPM) {
            if ((status = mhp_p_im_id(heading->replied_to_IPM,&subobject)) != OM_SUCCESS)
                return(status);
        
            if ((status = putvalue(desc,&desc->attr_value,IM_REPLIED_TO_IPM,OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
            } /* end if */
            break;
        case IM_REPLY_RECIPIENTS:
            if (heading->m & Heading_reply_recipients) {
            for (plist = (ListPointer *)&(heading->reply_recipients),
                plist->current = plist->first, count = 0,
                baseval = curval = nextval = desc->attr_value;
                plist->current != NULL; plist->current = plist->current->next) {
                /* Create and fill in the Reply Recipient sub-object */
                if ((status = mhp_p_im_or_desc(plist->current->data,&subobject)) != OM_SUCCESS)
                return(status);
        
                /* Create a value structure for the subobject and link it */
                if ((status = putvalue(desc,&nextval,IM_BLIND_COPY_RECIPIENTS,OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        
                append_struct(&baseval,&curval,nextval,&count);
            } /* end for */
            desc->attr_value = baseval;
            desc->count = count;        
            } /* end if */
            break;
        case IM_REPLY_TIME:
            if (heading->m & Heading_reply_time) {
            if ((buf = (char *)mhp_to_utc(&heading->reply_time)) == NULL)
                return(OM_MEMORY_INSUFFICIENT);
        
            if ((status = putvalue(desc,&desc->attr_value,IM_REPLY_TIME,OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
                return(status);
            } /* end if */
            break;
        case IM_SENSITIVITY:
            if (heading->m & Heading_sensitivity) {
            switch (heading->sensitivity) {
            case SENSITIVITY_personal:
                enumer = IM_PERSONAL;
                break;
            case SENSITIVITY_private:
                enumer = IM_PRIVATE;
                break;
            case SENSITIVITY_company_confidential:
                enumer = IM_COMPANY_CONFIDENTIAL;
                break;
            default:
                enumer = IM_NOT_SENSITIVE;
                break;
            } /* end switch */
            } /* end if */
            else /* if not present, then set default attribute as 
                    required by Heading class definition. KDH */ 
                enumer =  IM_NOT_SENSITIVE;
        
            if ((status = putvalue(desc,&desc->attr_value,IM_SENSITIVITY,OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
                return(status);
            break;

        case IM_SUBJECT:
            if ( heading->m & Heading_subject) {
            if (len = strlen(heading->subject.v.teletex_string)) {
                if ((status = p_strcpy(desc,&desc->attr_value,IM_SUBJECT,OM_S_TELETEX_STRING,
                heading->subject.v.teletex_string,len)) != OM_SUCCESS)
                return(status);
            } /* end if */
            } /* end if */
            break;
        case IM_THIS_IPM:
            /* THIS ID must always be present */
            if ((status = mhp_p_im_id(heading->this_IPM,&subobject)) != OM_SUCCESS)
            return(status);
        
            if ((status = putvalue(desc,&desc->attr_value,IM_THIS_IPM,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
            break;
        } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end i_p_ipm() */




/*
 * mhp_p_im_id(id, object)
 */

static OM_return_code mhp_p_im_id(id, object)
P22IPMIdentifier *id;
OM_int_private_object_header **object;
{
    Uint32 len;
    int i=0;
    int status;
    OM_int_descriptor *desc;
    OM_int_private_object_header *subobject;
    
    if ((status = omp_template(IM_C_IPM_IDENTIFIER,object)) != OM_SUCCESS)
    return(status);
    
    /* Index through IPM Identifier descriptor list sequentially and fill in
     * attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case IM_USER:
        if (id->m & IPMIdentifier_user) {
        if ((status = mhp_p_orname(id->user,&subobject)) != OM_SUCCESS)
            return(status);
    
        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,IM_USER,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case IM_USER_RELATIVE_IDENTIFIER:
        if (id->m & IPMIdentifier_user_relative_identifier) {
        if (len = strlen(id->user_relative_identifier))
            if ((status = p_strcpy(desc,&desc->attr_value,IM_USER_RELATIVE_IDENTIFIER,OM_S_PRINTABLE_STRING,
            id->user_relative_identifier,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_im_id() */




/*
 * mhp_p_im_or_desc(or_desc, object)
 */

static OM_return_code mhp_p_im_or_desc(or_desc, object)
P22ORDescriptor *or_desc;
OM_int_private_object_header **object;
{
    Uint32 len;
    int i=0;
    int status;
    OM_int_descriptor *desc;
    OM_int_private_object_header *subobject;
    
    if ((status = omp_template(IM_C_OR_DESCRIPTOR,object)) != OM_SUCCESS)
    return(status);
    
    /* Index through OR Descriptor descriptor list sequentially and fill in
     * attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case IM_FORMAL_NAME:
        if (or_desc->m & ORDescriptor_formal_name) {
        if ((status = mhp_p_orname(or_desc->formal_name,&subobject)) != OM_SUCCESS)
            return(status);
    
        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,IM_FORMAL_NAME,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case IM_FREE_FORM_NAME:
        if (or_desc->m & ORDescriptor_free_form_name) {
        if (len = strlen(or_desc->free_form_name)) {
            if ((status = p_strcpy(desc,&desc->attr_value,IM_FREE_FORM_NAME,OM_S_TELETEX_STRING,
            or_desc->free_form_name,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    case IM_TELEPHONE_NUMBER:
        if (or_desc->m & ORDescriptor_telephone_number) {
        if (len = strlen(or_desc->telephone_number)) {
            if ((status = p_strcpy(desc,&desc->attr_value,IM_TELEPHONE_NUMBER,OM_S_PRINTABLE_STRING,
            or_desc->telephone_number,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_im_or_desc() */

/*
 * mhp_p_im_recip(recip, object)
 */

static OM_return_code mhp_p_im_recip(recip, object)
P22RecipientSpecifier *recip;
OM_int_private_object_header **object;
{
   int i=0;
   int status;
   OM_boolean flag;
   OM_enumeration enumer;
   OM_int_descriptor *desc;
   OM_int_private_object_header *subobject;
   
   if ((status = omp_template(IM_C_RECIPIENT_SPECIFIER,object)) != OM_SUCCESS)
   return(status);
   
   /* Index through Recipient Specifier descriptor list sequentially and fill
    * in attribute if it exists. */
   for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
   desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
   switch (desc->attr_type) {
   case IM_RECIPIENT:
       if (recip->m & RecipientSpecifier_recipient) {
       if ((status = mhp_p_im_or_desc(recip->recipient,&subobject)) != OM_SUCCESS)
           return(status);
   
       /* Create a value structure for the subobject and link it */
       if ((status = putvalue(desc,&desc->attr_value,IM_RECIPIENT,OM_S_OBJECT,subobject)) != OM_SUCCESS)
           return(status);
       } /* end if */
       break;

   case IM_IPM_RETURN_REQUESTED:
       flag = FALSE;
       if (recip->m & RecipientSpecifier_notification_requests) {
       if (recip->notification_requests & NOTIFICATION_REQUESTS_ipm_return)
           flag = TRUE;
       else
           flag = FALSE;
       } /* end if */

       if ((status = putvalue(desc,&desc->attr_value,IM_IPM_RETURN_REQUESTED,OM_S_BOOLEAN,(void*)flag)) != OM_SUCCESS)
           return(status);
       break;

   case IM_NOTIFICATION_REQUEST:
       enumer = IM_NEVER; /* default */
       if (recip->m & RecipientSpecifier_notification_requests) {
       if (recip->notification_requests & NOTIFICATION_REQUESTS_nrn)
           enumer = IM_NON_RECEIPT;
       else if (recip->notification_requests & NOTIFICATION_REQUESTS_rn)
           enumer = IM_ALWAYS;
       else enumer = IM_NEVER;
       } /* end if */
       if ((status = putvalue(desc,&desc->attr_value,IM_NOTIFICATION_REQUEST,OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
           return(status);
       break;

   case IM_REPLY_REQUESTED:
       flag = FALSE; /* default */
       if (recip->m & RecipientSpecifier_reply_requested) {
       if (recip->reply_requested)
           flag = TRUE;
       else
           flag = FALSE;
       } /* end if */
       if ((status = putvalue(desc,&desc->attr_value,IM_REPLY_REQUESTED,OM_S_BOOLEAN,(void*)flag)) != OM_SUCCESS)
           return(status);
       break;

   } /* end switch */
   } /* end for */
   return(OM_SUCCESS);
} /* end mhp_p_im_recip() */



/*
 * mhp_p_im_ia5(bp, object)
 *
 */

static OM_return_code mhp_p_im_ia5(bp, object)
struct bodypart *bp;
OM_int_private_object_header **object;
{

   P22IA5TextBodyPart *ia5;
   int status;
   OM_enumeration enumer;
   Uint16 i=0;
   OM_int_descriptor *desc;
   
   if ((status = omp_template(IM_C_IA5_TEXT_BD_PRT,object)) != OM_SUCCESS)
   return(status);
   
   ia5 = (P22IA5TextBodyPart *)bp->data;
   
   /* Index through IA5 Body Part descriptor list sequentially and fill
    * in attribute if it exists. */
   for (i = TOP; 
    (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
    i++) 
   {
   
   desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
   switch (desc->attr_type) {
   case IM_REPERTOIRE:
       if ((ia5->m & IA5TextBodyPart_parameters) 
            && (ia5->parameters.m & IA5TextParameters_repertoire)) {
       switch (ia5->parameters.repertoire) {
       case REPERTOIRE_ita2:
           enumer = IM_ITA2;
           break;
       case REPERTOIRE_ia5:
       default:
           enumer = IM_IA5;
           break;
       } /* end switch */
       } /* end if */
       else { /* default if not present */
            enumer = IM_IA5;
       }
   
       if ((status = putvalue(desc,&desc->attr_value,IM_REPERTOIRE,
                    OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
           return(status);
       break;
   
   case IM_TEXT:
       if (ia5->m & IA5TextBodyPart_data) {
       /* This is Bodypart filename (in ia5->data) */
       if ((status = write_str(desc,&desc->attr_value,IM_TEXT,OM_S_IA5_STRING,
                            (char*)ia5->data)) != OM_SUCCESS)
           return(status);
       } /* end if */
       break;
   } /* end switch */
   } /* end for */
   return(OM_SUCCESS);
} /* end mhp_p_im_ia5() */




/*
 * mhp_p_im_binary(bp, object) copies Unidentified Body Part structure
 * into Bilaterally Defined Body Part object.
 * Input:
 *        bp - pointer to bodypart containing P22Unidentified.
 *        object - pointer to variable containing handle to object.
 * Output:
 *        object - now containing handle to filled Bilaterally Defined object.
 * Result:
 *        OM_SUCCESS, or error codes otherwise.
 *
 */

static OM_return_code mhp_p_im_binary(bp, object)
struct bodypart *bp;
OM_int_private_object_header **object;
{
   int status;
   Uint16 i=0;
   OM_int_descriptor *desc;
   
   if ((status = omp_template(IM_C_BILAT_DEF_BD_PRT,object)) != OM_SUCCESS)
   return(status);
   
   /* Index through Bilaterally Defined Body Part descriptor list sequentially
    * and fill in attribute if it exists. */
   for (i = TOP; 
     (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
     i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case IM_BILATERAL_DATA: 
           /* filename pointer is value of bp->data */
        if ((status = write_str(desc,&desc->attr_value,IM_BILATERAL_DATA,
                        OM_S_OCTET_STRING, (char*)*bp->data)) != OM_SUCCESS)
        return(status);
        break;
    } /* end switch */
   } /* end for */
   
   return(OM_SUCCESS);
} /* end mhp_p_im_binary() */


/*
 * mhp_p_im_iso6037bp(bp, object)
 *
 * to do : memory-based
 */

static OM_return_code mhp_p_im_iso6937(bp, object)
struct bodypart *bp;
OM_int_private_object_header **object;
{
   P22ISO6937Text *iso;
   int status;
   OM_enumeration enumer;
   Uint16 i=0;
   OM_int_descriptor *desc;
   
   if ((status = omp_template(IM_C_ISO_6937_TEXT_BD_PRT,object)) != OM_SUCCESS)
       return(status);
   
   iso = (P22ISO6937Text *)bp->data;
   
   /* Index through ISO 6937 Body Part descriptor list sequentially and fill
    * in attribute if it exists. */
   for (i = TOP; 
    (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
    i++) 
   {
   
   desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
   switch (desc->attr_type) {
   case IM_REPERTOIRE:
       if ((iso->m & ISO6937Text_params) 
            && (iso->params.m & ISO6937Text_repertoire)) {
       switch (iso->params.repertoire) {
       case REPERTOIRE_part1and2:
       default:
           enumer = IM_PART1_AND_PART2;
           break;
       } /* end switch */
       } /* end if */

       else { /* default if not present */
            enumer = IM_PART1_AND_PART2;
       }

       if ((status = putvalue(desc,&desc->attr_value,IM_REPERTOIRE,
                            OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
           return(status);
       break;
   
   case IM_TEXT:
       if (iso->m & ISO6937Text_text) {
       /* This is Bodypart filename (in iso->data) */
       if ((status = write_str(desc,&desc->attr_value,IM_TEXT,
                    OM_S_OCTET_STRING,(char*)iso->text)) != OM_SUCCESS)
           return(status);
       } /* end if */
       break;
   } /* end switch */
   } /* end for */
   
   return(OM_SUCCESS);
} /* end mhp_p_im_iso6937bp() */



/*
 * mhp_p_im_msgbp(bp, object)
 *
 */

static OM_return_code mhp_p_im_msgbp(bp, object)
struct bodypart *bp;
OM_int_private_object_header **object;
{

   OM_int_private_object_header *subobject;
   P22MessageBodyPart *msg;
   int status;
   OM_enumeration enumer;
   Uint16 i=0;
   OM_int_descriptor *desc;
   
   if ((status = omp_template(IM_C_MESSAGE_BD_PRT,object)) != OM_SUCCESS)
   return(status);
   
   msg = (P22MessageBodyPart *)bp->data;
   
   /* Index through Message Body Part descriptor list sequentially and fill
    * in attribute if it exists. */
   
   for (i = TOP; 
    (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
    i++) 
   {
   
   desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
   switch (desc->attr_type) {
   case IM_ENVELOPE:
    if (msg->m & MessageBodyPart_parameters) {
        if ((status = mhp_p_im_delenvelop(msg, & subobject)) != OM_SUCCESS)
            return(status);
   
        if ((status = putvalue(desc,&desc->attr_value,IM_ENVELOPE,
                    OM_S_OBJECT, subobject)) != OM_SUCCESS)
            return(status);
       } /* end if */
       break;
   
   case IM_IPM:
       if (msg->m &  MessageBodyPart_data ) {
        /* mhp_p_im_ipm() takes P22IPM which is msg->data */
        if ((status = mhp_p_im_ipm(msg->data, &subobject)) != OM_SUCCESS)
            return(status);
        if ((status = putvalue(desc,&desc->attr_value,IM_IPM,
                    OM_S_OBJECT, subobject)) != OM_SUCCESS)
            return(status);
       } /* end if */
       break;
   default:
    break;
   } /* end switch */
   
   } /* end for */
   
   return(OM_SUCCESS);

} /* end mhp_p_im_msgbp() */


/*
 * mhp_p_im_external(bp, object) copies ExternallyDefined Body Part structure
 * into Externally Defined Body Part object.
 *
 * Input:
 *        bp - pointer to bodypart containing P22ExternallyDefinedBodyPart.
 *        object - pointer to variable containing handle to object.
 * Output:
 *        object - now containing handle to filled Externally Defined object.
 * Result:
 *        OM_SUCCESS, or error codes otherwise.
 *
 * NOTES: The asn1 Parameters external structure has the data in memory 
 *        (parameters is usually short); whereas the asn1 Data structure 
 *        has the data in file. The P22ExternallyDefinedBodyPart struct
 *        imbeds the Parameters ext_type, and, for Data, points to
 *        struct ext_type.
 *
 */

static OM_return_code mhp_p_im_external(bp, object)
struct bodypart *bp;
OM_int_private_object_header **object;
{
   int status;
   Uint16 i=0;
   OM_int_descriptor *desc;
   OM_int_private_object_header *subobject;
   P22ExternallyDefinedBodyPart *extbp;
   
   if ((status = omp_template(IM_C_EXTERN_DEF_BD_PRT,object)) != OM_SUCCESS)
       return(status);

   /* let's extract the realy P22Externally... struct from bp first */
   extbp = (P22ExternallyDefinedBodyPart *)bp->data;
   
   /* Index through Externally Defined Body Part descriptor list sequentially
    * and fill in attribute if it exists. */
   for (i = TOP; 
     (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
     i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {

        case IM_EXTERNAL_DATA:
            if (extbp->m & ExternallyDefinedBodyPart_data)
            {

                /* P22ExternallyDefinedData is same as struct ext_type, and
                ** the real data is in a file 
                */
                if ((status = mhp_p_external( (struct ext_type *)extbp->data,
                                   OM_TRUE, & subobject)) != OM_SUCCESS)
                   return(status);

                if ((status = putvalue(desc, &desc->attr_value, 
                                  IM_EXTERNAL_DATA, OM_S_OBJECT, 
                                  subobject)) != OM_SUCCESS)
                   return(status);
            }

            break;

        case IM_EXTERNAL_PARAMETERS:
            if (extbp->m & ExternallyDefinedBodyPart_parameters)
            {
                /* the real data of Parameters is in memory.  */
                if ((status = mhp_p_external( 
                                    (struct ext_type *) & extbp->parameters,
                                    OM_FALSE, & subobject)) != OM_SUCCESS)
                   return(status);

                if ((status = putvalue(desc, &desc->attr_value, 
                                  IM_EXTERNAL_PARAMETERS, OM_S_OBJECT, 
                                  subobject)) != OM_SUCCESS)
                   return(status);
            }

            break;

        } /* switch */
   } /* for */
   return(OM_SUCCESS);
} /* mhp_p_im_external */


/*
 * mhp_p_im_delenvelop() copies info in P3OtherMessageDeliveryFields and
 * delivery time into the Delivery Envelope object.
 * Input:
 *      msg - pointer to P22MessageBodyPart which contains the DeliveryFields
 *            as well as delivery time.
 *      object - pointer to variable containing handle to object.
 * Output:
 *      object - filled with data
 * Result:
 *      OM_SUCCESS or otherwise
 * Called by:
 *      mhp_p_im_msgbp()
 * Calling:
 *
 */ 

static OM_return_code mhp_p_im_delenvelop(msgbp, object)
P22MessageBodyPart *msgbp;
OM_int_private_object_header **object;
{

OM_int_private_object_header    *subobject;
P3OtherMessageDeliveryFields    *others;
int status;
OM_enumeration enumer;
char *buf;
char *pvalue;
Uint16 i=0;
OM_int_descriptor *desc;
OM_int_value *baseval, *curval, *nextval;
ListPointer *plist;
Uint16 count;
OM_type prev_attribute_type = OM_NO_MORE_TYPES;

others = (P3OtherMessageDeliveryFields*) &msgbp->parameters.delivery_envelope;

if ((status = omp_template(MH_C_DELIVERY_ENVELOPE,object)) != OM_SUCCESS)
    return(status);

/* Index through Delivery Envelope descriptor list sequentially and fill
 * in attribute if it exists. 
*/

   for (i = TOP; 
    (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
    i++) 
   {

    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    if (prev_attribute_type == desc->attr_type) continue;
    prev_attribute_type = desc->attr_type;

    switch (desc->attr_type) {
    case MH_T_ACTUAL_RECIPIENT_NAME:
        if (others->m & OtherMessageDeliveryFields_this_recipient_name)
        {
            if ((status = mhp_p_orname(others->this_recipient_name,&subobject))
                                != OM_SUCCESS)
                return(status);
            if ((status = putvalue(desc,&desc->attr_value,
                        MH_T_ACTUAL_RECIPIENT_NAME,
                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        }
        break;

    case MH_T_CONTENT_TYPE: 
        if (others->m & OtherMessageDeliveryFields_content_type){
        if (others->content_type.t == DeliveredContentType_built_in){
            switch(others->content_type.v.built_in){
            case BUILT_IN_unidentified:
                enumer = MH_CTI_UNIDENTIFIED;
                break;
            case BUILT_IN_interpersonal_messaging_1988:
                enumer = MH_CTI_P2_1988;
                break;
            case BUILT_IN_interpersonal_messaging_1984:
                enumer = MH_CTI_P2_1984;
                break;
            default:
                break;
            } /* switch */
            
            /* in CAE, content type is actually an integer */
            if ((status = putvalue(desc,&desc->attr_value,
                        MH_T_CONTENT_TYPE,
                        OM_S_INTEGER,(void *)enumer)) != OM_SUCCESS)
                return(status);
        } /* end if builtin */
        else { /* 1988 content type object id not yet supported */

        } /* else object ids */

        } /* end if content type */

        break;
    case MH_T_CONVERSION_PROHIBITED: /* optional element */
        if (others->m & OtherMessageDeliveryFields_delivery_flags) {
           if (others->delivery_flags 
                & DELIVERY_FLAGS_implicit_conversion_prohibited) 
           {
               OM_boolean tmp = OM_TRUE;
               if ((status = putvalue(desc, &desc->attr_value,
                          MH_T_CONVERSION_PROHIBITED, OM_S_BOOLEAN,
                          (void *)tmp)) != OM_SUCCESS)
                  return(status);
           }
        } /* end if */
        break;
    case MH_T_CONVERTED_EITS:
        if (others->m 
             & OtherMessageDeliveryFields_converted_encoded_information_types){
            if ((status = mhp_p_eits(others->
                            converted_encoded_information_types, & subobject))
                            != OM_SUCCESS)
                return(status);
            if ((status = putvalue(desc,&desc->attr_value,
                        MH_T_CONVERTED_EITS,
                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
            } /* end if */
        break;

    case MH_T_DELIVERY_TIME:
        /* not part of P3OtherMessageDelvieryFields */
        if ((msgbp->m & MessageBodyPart_parameters)
                && msgbp->parameters.m & MessageParameters_delivery_time){
            if ((buf = (char *)mhp_to_utc(&msgbp->parameters.delivery_time)) 
                                    == NULL)
                return(OM_MEMORY_INSUFFICIENT);
            if ((status=putvalue(desc,&desc->attr_value,MH_T_DELIVERY_TIME,
                        OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
                return(status);
        } /* end if */
        break;

    case MH_T_DISTINGUISHED_RECIP_ADDR:
        /* not part of Message Body Part. No counter part in X.400  */
        break;
    case MH_T_ORIGINALLY_INTENDED_RECIP:
        if (others->m & 
                OtherMessageDeliveryFields_originally_intended_recipient_name){
            if ((status = mhp_p_orname(others->
                            originally_intended_recipient_name, &subobject))
                        != OM_SUCCESS)
                return(status);
            if ((status = putvalue(desc,&desc->attr_value,
                        MH_T_ORIGINALLY_INTENDED_RECIP,
                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        } /* end if */
        break;

    case MH_T_MTS_IDENTIFIER:
        /* this should be absent in MessageBodyPart */
        break;
    case MH_T_ORIGINAL_EITS:
        if (others->m &
                OtherMessageDeliveryFields_original_encoded_information_types){
            if ((status = mhp_p_eits(others->original_encoded_information_types,
                                    &subobject)) != OM_SUCCESS)
                return(status);
            if ((status = putvalue(desc,&desc->attr_value,
                        MH_T_ORIGINAL_EITS,
                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        } /* end if */
        break;

    case MH_T_ORIGINATOR_NAME:
        if (others->m & OtherMessageDeliveryFields_originator_name){
            if ((status = mhp_p_orname(others->originator_name,&subobject))
                        != OM_SUCCESS)
                return(status);
            
            if ((status = putvalue(desc,&desc->attr_value,
                        MH_T_ORIGINATOR_NAME,
                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        }
        break;

    case MH_T_OTHER_RECIPIENT_NAMES:
        if (others->m & OtherMessageDeliveryFields_other_recipient_names) {
        for (plist = (ListPointer *) & others->other_recipient_names,
            plist->current = plist->first, count = 0,
            baseval = curval = nextval = desc->attr_value;
            plist->current != NULL; plist->current = plist->current->next) 
        {
            /* Create and fill in the recipient name sub-object */
            if ((status = mhp_p_orname(plist->current->data,&subobject)) 
                            != OM_SUCCESS)
                return(status);
    
            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_OTHER_RECIPIENT_NAMES,
                            OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
    
            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;        
        } /* end if */
        break;

    case MH_T_PRIORITY:
        if (others->m & OtherMessageDeliveryFields_priority) {
            switch(others->priority){
            case PRIORITY_non_urgent:
                enumer = MH_PTY_LOW;
                break;
            case PRIORITY_normal:
            default:
                enumer = MH_PTY_NORMAL;
                break;
            case PRIORITY_urgent:
                enumer = MH_PTY_URGENT;
                break;
            }
            if ((status = putvalue(desc,&desc->attr_value,
                        MH_T_PRIORITY,
                        OM_S_ENUMERATION,(void *)enumer)) != OM_SUCCESS)
                return(status);
        } /* end if */
        break;

    case MH_T_SUBMISSION_TIME:
        if (  others->m & OtherMessageDeliveryFields_message_submission_time){
            if ((buf = (char *)mhp_to_utc(&others->message_submission_time))
                                                 == NULL)
                return(OM_MEMORY_INSUFFICIENT);
            if ((status=putvalue(desc,&desc->attr_value,MH_T_SUBMISSION_TIME,
                            OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
                return(status);
        } /* end if */
        break;

    case MH_T_PREFERRED_DELIVERY_MODES:
        if ( pvalue = (char *)get_delenvl_extension( msgbp, ETYPE_requested_delivery_method)) {
            int i;   /* Not to be confused with outer loop i */
            P1RequestedDeliveryMethod *prq;

            prq = (P1RequestedDeliveryMethod *)pvalue;

            for (i = 0, baseval = curval = nextval = desc->attr_value, count = 0; i < prq->n ; i++)
            {
                        if ((status = putvalue(desc, &nextval,
                MH_T_PREFERRED_DELIVERY_MODES, OM_S_ENUMERATION,
                (void *) prq->v[i]) != OM_SUCCESS))
                    return(status);
            append_struct(&baseval,&curval,nextval,&count);

            }

            desc->attr_value = baseval;
            desc->count = count;
        }
        else {
                        if ((status = putvalue(desc, &desc->attr_value,
                MH_T_PREFERRED_DELIVERY_MODES, OM_S_ENUMERATION,
                (void *) MH_DM_ANY) != OM_SUCCESS))
                    return(status);
        }
    break;
    default:
        /* other 1988 fields not supported. */
        break;
    } /* end switch */

   } /* end for */

return(OM_SUCCESS);

} /* mhp_p_im_delenvelop() */


/*
 * mhp_p_im_ttx(bp, object) copies Teletex Body Part structure
 * into Teletex Body Part object.
 * Input:
 *        bp - pointer to bodypart containing P22TeletexBodyPart.
 *        object - pointer to variable containing handle to object.
 * Output:
 *        object - now containing handle to filled Teletex Body Part object.
 * Result:
 *        OM_SUCCESS, or error codes otherwise.
 *
 */

static OM_return_code mhp_p_im_ttx(bp, object)
struct bodypart *bp;
OM_int_private_object_header **object;
{
P22TeletexBodyPart *ttxbp;
int status;
Uint16 i=0;
OM_int_descriptor *desc;
OM_int_private_object_header *subobject;

ttxbp = (P22TeletexBodyPart *)bp->data;
if ((status = omp_template(IM_C_TELETEX_BD_PRT,object)) != OM_SUCCESS)
    return(status);

/* Index through Teletex Body Part descriptor list sequentially
 * and fill in attribute if it exists. */
  for (i = TOP; 
    (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
    i++) 
  {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case IM_TELETEX_DOCUMENT: 
        if (ttxbp->m & TeletexBodyPart_data) {
            /* filename pointer is value of bp->data */
        if (ttxbp->data != NULL) { /* file */
        /* need to split file into teletex pages. Each page, delimited in
           the file by Ctrl-L, a value. */
    /*
        if ((status = write_str(desc,&desc->attr_value,IM_TELETEX_DOCUMENT,OM_S_TELETEX_STRING, ttxbp->data)) != OM_SUCCESS)
    */
            return(status);
        } /* end if */
        } /* end if */
        break;
    case IM_TELETEX_NBPS:
        if ((ttxbp->m & TeletexBodyPart_parameters) &&
                (ttxbp->parameters.m & TeletexParameters_non_basic_paramters)){
            if ((status = mhp_p_teletex_nbps(
                        ttxbp->parameters.non_basic_paramters,&subobject))
                        != OM_SUCCESS)
                return(status);
            if ((status = putvalue(desc,&desc->attr_value,IM_TELETEX_NBPS,OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        } /* end if */
        break;
    case IM_TELEX_COMPATIBLE:
        if ((ttxbp->m & TeletexBodyPart_parameters) &&
                (ttxbp->parameters.m & TeletexParameters_telex_compatible)){
            if ((status = putvalue(desc,&desc->attr_value,IM_TELEX_COMPATIBLE, 
                 OM_S_BOOLEAN,
                 (void*)(ttxbp->parameters.telex_compatible?OM_TRUE:OM_FALSE))) 
                 != OM_SUCCESS)
                    return(status);
        } /* end if */
        
        break;
    default:
        break;
    } /* end switch */
  } /* end for */

return(OM_SUCCESS);

} /* end mhp_p_im_ttxbp() */



/*
 * mhp_p_im_ipn(ipn, object)
 */

static OM_return_code mhp_p_im_ipn(ipn, object)
P22IPN *ipn;
OM_int_private_object_header **object;
{
OM_int_private_object_header *subobject;
P22ReceiptFields *rf;
P22NonReceiptFields *nrf;
char *buf;
Uint32 len;
Uint16 i=0, count;
int status;
OM_boolean flag;
OM_enumeration enumer;
OM_int_descriptor *desc;
OM_int_value *baseval, *curval, *nextval;
ListPointer *plist;

/*
TRACE2("Start mhp_p_im_ipn in file %s\n", __FILE__);
*/

if (ipn->choice.t == IPN_receipt_fields) { /* Receipt Notification */
    if ((status = omp_template(IM_C_RECEIPT_NOTIF,object)) !=OM_SUCCESS)
        return(status);
    rf = & ipn->choice.v.receipt_fields;
}
else { /* Non-Receipt Notification */
    if ((status = omp_template(IM_C_NON_RECEIPT_NOTIF,object)) !=OM_SUCCESS)
        return(status);
    nrf = & ipn->choice.v.non_receipt_fields;
}

/* Index through IPN descriptor list sequentially and fill in attribute
 * if it exists. */
    for (i = TOP; 
        (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
        i++) 
  {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case IM_CONVERSION_EITS:
        if (ipn->m & IPN_conversion_eits) {
        /* Create and fill in Original EITs sub-object */
        if ((status = mhp_p_eits(ipn->conversion_eits,&subobject))
            != OM_SUCCESS)
            return(status);
    
        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_ORIGINAL_EITS,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case IM_IPM_INTENDED_RECIPIENT:
        if (ipn->m & IPN_ipm_preferred_recipient) {
        if ((status = mhp_p_im_or_desc(ipn->ipm_preferred_recipient,&subobject)) != OM_SUCCESS)
            return(status);
    
        if ((status = putvalue(desc,&desc->attr_value,IM_IPM_INTENDED_RECIPIENT,OM_S_OBJECT,subobject))
            != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case IM_IPN_ORIGINATOR:
        if (ipn->m & IPN_ipn_originator) {
        if ((status = mhp_p_im_or_desc(ipn->ipn_originator,&subobject)) != OM_SUCCESS)
            return(status);
    
        if ((status = putvalue(desc,&desc->attr_value,IM_IPN_ORIGINATOR,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case IM_SUBJECT_IPM:
        if (ipn->m & IPN_subject_IPM) {
        if ((status = mhp_p_im_id(ipn->subject_IPM, &subobject)) != OM_SUCCESS)
            return(status);
        
        if ((status = putvalue(desc,&desc->attr_value,IM_SUBJECT_IPM,OM_S_OBJECT,
            subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
            
    /* non-receipt fields */
    case IM_AUTO_FORWARD_COMMENT:
        if (nrf->m & NonReceiptFields_auto_forward_comment) {
        if (len = strlen(nrf->auto_forward_comment)) { 
            if ((status = p_strcpy(desc,&desc->attr_value,IM_AUTO_FORWARD_COMMENT,OM_S_PRINTABLE_STRING,nrf->auto_forward_comment,len)) != OM_SUCCESS)
                return(status);
        } /* end if */  
        } /* end if */
        break;

    case IM_DISCARD_REASON:
        if (nrf->m & NonReceiptFields_discard_reason) {
            if ((status = putvalue(desc,&desc->attr_value,IM_DISCARD_REASON,OM_S_ENUMERATION,(void*)nrf->discard_reason)) != OM_SUCCESS)
                return(status); 
        } /* end if */
        else  /* return -1 for no discard */
            if ((status = putvalue(desc,&desc->attr_value,IM_DISCARD_REASON,OM_S_ENUMERATION,(void*)-1)) != OM_SUCCESS)
                return(status);
        break;

    case IM_NON_RECEIPT_REASON:
        if (nrf-> m & NonReceiptFields_non_receipt_reason) {
            /* this field must be present.So default should never be used.KDH*/
            if ((status = putvalue(desc,&desc->attr_value,IM_NON_RECEIPT_REASON,OM_S_ENUMERATION,(void*)nrf->non_receipt_reason)) != OM_SUCCESS)
                return(status);
        } /* end if */
        else { /* default is ipm-discarded. */
            if ((status = putvalue(desc,&desc->attr_value,IM_NON_RECEIPT_REASON,OM_S_ENUMERATION,(void*)IM_IPM_DISCARDED)) != OM_SUCCESS)
                return(status);
        }
        break;

    case IM_RETURNED_IPM:
        if (nrf-> m & NonReceiptFields_returned_IPM) {
            if ((status=mhp_parse_ipm(nrf->returned_IPM,&subobject))!=OM_SUCCESS){
                return(status);
            } /* end if */
            if ((status = putvalue(desc,&desc->attr_value,IM_RETURNED_IPM,OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        } /* end if */
        break;
    
    /* receipt fields */
    case IM_ACKNOWLEDGEMENT_MODE:
        if (rf->m & ReceiptFields_acknowledgement_mode) {
            if ((status = putvalue(desc,&desc->attr_value,IM_ACKNOWLEDGEMENT_MODE,OM_S_ENUMERATION,(void*)rf->acknowledgement_mode)) != OM_SUCCESS)
                return(status);
        } /* end if */
        else { /* default is manual */
            if ((status = putvalue(desc,&desc->attr_value,IM_ACKNOWLEDGEMENT_MODE,OM_S_ENUMERATION,(void*)IM_MANUAL)) != OM_SUCCESS)
                return(status);
        }
        break;

    case IM_RECEIPT_TIME:
        if (rf->m & ReceiptFields_receipt_time) {
            if ((buf = (char *)mhp_to_utc(&rf->receipt_time)) == NULL)
                return(OM_MEMORY_INSUFFICIENT);
            if ((status=putvalue(desc,&desc->attr_value,IM_RECEIPT_TIME,OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
                return(status);
        } /* end if */
        break;
    case IM_SUPPLEMENTARY_RECEIPT_INFO:
        if (rf->m & ReceiptFields_suppl_receipt_info) {
        if (len = strlen(rf->suppl_receipt_info)) {
            if ((status = p_strcpy(desc,&desc->attr_value,IM_SUPPLEMENTARY_RECEIPT_INFO,OM_S_PRINTABLE_STRING,rf->suppl_receipt_info,len)) != OM_SUCCESS)
                return(status);
        } /* end if */
        } /* end if */
        break;
    default:
        break;
    } /* end switch */
  } /* end for */
return(OM_SUCCESS);
} /* end mhp_p_im_ipn() */
