// FILE:  platfile.C

#include	<sys/types.h>
#include	<sys/unistd.h>
#include	<dirent.h>
#include	<fstream.h>
#include	<string.h>
#include	<assert.h>
#include	"../../../include/platform.h"
#include	"../../../include/platfile.h"


//
// PUBLIC:  read the file identified by 'file_name' into 'contents'.  To 
// read the file, the size of the file is calculated and 'file_size' is set.
//

STATUS PlatformFile::read_file() {
	ifstream	ifile ( full_file_name, ios::in );
 
	
	if ( !ifile ) {
#ifdef	DEBUG
		cout << "Open file error on file '" << full_file_name <<
			"'" << endl;
#endif
		return ERROR;		// File open error.
	}
	
	set_filesize( &ifile );
	contents = new char[ file_size+1 ];
	assert( contents != 0 );
	ifile.read( contents, (int) file_size );
	contents[ file_size ] = '\0';

	return SUCCESS;
}


//
// PUBLIC:  call the UNIX function readdir() to read a file name from the
// directory identified by 'dirp'.  The readdir() function sets 'dirent'.
// The value of the file name field of the dirent structure is appended to 
// the directory path (held in 'dirname') to form the complete file name.  
//

STATUS PlatformFile::read_file_directory() { 

	do {
		dirent = readdir( dirp );
		if ( dirent == NULL )
			return DIRECTORY_EMPTY; // may be an error.
	}
	//
	// All UNIX file directories contain '.' and '..' files.
	// These files are ignored.
	//
	while ( ( !strcmp( dirent->d_name, "." )) ||
		( !strcmp( dirent->d_name, ".." )) );

	strcpy( file_name, dirent->d_name );
	strcpy( full_file_name, dirname );
	strcat( full_file_name, "/" );
	strcat( full_file_name, file_name );
	
	return SUCCESS;
}


//
// PRIVATE:  compute the size of a file by seeking to the end and 
// determining the offset from the beginning.
//

void PlatformFile::set_filesize( ifstream *file ) {
	streampos	mark;


	mark = file->tellg();		// Mark current position.
	file->seekg( 0 );		// Move to file start.
	file->seekg( 0, ios::end );	// Seek to EOF.
	file_size = file->tellg();	// Get offset from file start.
	file->seekg( mark );		// Return to file mark.
}


//
// PUBLIC:  move a file to a new directory, modifying the 'filename' variable 
// to reflect the change of name.
//

STATUS PlatformFile::move_file( char *new_filedir ) {
	char	new_filename[ MAX_FILENAME ];
	
	strcpy( new_filename, new_filedir );
	strcat( new_filename, "/" );
	strcat( new_filename, file_name );

#ifdef	DEBUG
	cout << "Linking " << full_file_name << " to " << new_filename << endl;
#endif
	
	if ( link( full_file_name, new_filename ) == -1 ) {
		cout << "Link returned non-zero" << endl;
		return ERROR;
	}
	if ( unlink( full_file_name ) == -1 )
		return ERROR;
	strcpy( full_file_name, new_filename );
	return SUCCESS;
}
