// FILE:  platform.C

#include	<sys/types.h>
#include	<sys/unistd.h>
#include	<stdlib.h>
#include	<dirent.h>
#include	<string.h>
#include	<iostream.h>
#include	<fstream.h>
#include	<assert.h>
#include	<xom.h>
#include	<xedip.h>
extern "C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
#include	<stdio.h>
#include	"/usr/lib/x400/gtwapi/include/x4client.h"
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/interchange.h"
#include	"../../../include/edi_user.h"
#include	"../../../include/platio.h"
#include	"../../../include/config.h"
#include	"../../../include/subject.h"
#include	"../../../include/platfile.h"
#include	"../../../include/tpfile.h"
#include	"../../../include/ifile.h"
#include	"../../../include/api_obj.h"
#include	"../../../include/x400api.h"

void main ( int argc, char *argv[] ) {

	Config		config;
//	Log		log( BASIC_LOG, LOG_FILE );
	Log		log( VERBOSE_LOG, LOG_FILE );
	PlatformIO	processing;
	STATUS		status;
	int		level;
	
	//
	// Delete the stop file if it exists.  Ignore any return value
	// from the unlink system call.
	//
	
	unlink( STOP_FILE );

#ifdef	DEBUG
	//
	// This turns on tracing in HP software.  It is not under our control.
	//

	x4_gapi_trace = X4_GAPI_TRACE_ALL;
	if ( ( x4_gapi_trace_fp = fopen( TRACE_FILE, "w" ) ) == NULL) {
		cout << "Cannot open API Trace file" << endl;
		exit( 1 );
	}
#endif

	if ( argc > 1 ) {

		switch ( argc ) {
		
			case 3:
				log.set_logfile( argv[ 2 ] );
				// fall through
			case 2:
				level = *argv[ 1 ] - 0x30;
				if ( level>=NONE_LOG && level<=VERBOSE_LOG )
					log.set_loglevel( ( LOGLEVEL ) level );
				// fall through
			case 1:
				// use constructor values
				break;
			default:
				cout << "USAGE:   edicomp loglevel logfile" 
				     << endl;
				cout << "         ** OR **" << endl;
				cout << "USAGE:   edicomp loglevel" << endl;
				cout << "         ** OR **" << endl;
				cout << "USAGE:   edicomp" << endl;
				exit( 0 );
		}
	}
		
	//
	// Initialize the config object from info in the configuration file.
	//	

	status = config.read_configuration_file();
	if ( ! OK ( status ) )
		exit( 1 );

#ifdef	DEBUG
	config.display_configuration();
#endif
	
	//
	// Initialize the directories from which the platform will read 
	// files containing interchanges (both inbound and outbound).
	//

	InterchangeFile *user_outbound = 
		new InterchangeFile( config.edi_user_directory_name() );
	InterchangeFile *user_inbound = 
		new InterchangeFile( config.transmission_directory_name() );
	assert( user_outbound != 0 );
	assert( user_inbound != 0 );

	X400Message	inbound_msg( &log );

	//
	// Test for input in round-robin fashion until user shuts down.
	//

	while ( OK( status ) ) {
		status = SUCCESS;
		status |= user_outbound->open_file_directory();
		status |= user_inbound->open_file_directory();
		status |= user_outbound->send( &config, &log );
		status |= user_inbound->receive( &config, &log );
		user_outbound->close_file_directory();
		user_inbound->close_file_directory();
		status |= inbound_msg.receive( &status, &config );
		// process the TSDS
		processing.mod_logging( &log );	// has user changed level/file?
		status |= processing.cont();	// has user requested shutdown?
	}

	delete user_outbound;
	delete user_inbound;
}
