// FILE:  tpfile.C

#include	<sys/types.h>
#include	<sys/unistd.h>
#include	<dirent.h>
#include	<fstream.h>
#include	<assert.h>
#include	<string.h>
#include	<stdlib.h>
#include	<xom.h>
extern "C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
#include	"/usr/lib/x400/gtwapi/include/xedip.h"   
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"../../../include/platform.h"
#include	"../../../include/platfile.h"
#include	"../../../include/log.h"
#include	"../../../include/or_name.h"
#include	"../../../include/tpfile.h"


STATUS TradingPartnerFile::read_communications_attributes( char *user_code,
							   char *user_qual,
							   char *tp_code,
							   char *tp_qual ) {

	STATUS	status = SUCCESS;

	//
	// Construct the file directory name for the Local EDI User.
	//

	strcpy( full_file_name, TP_DIR );
	strcat( full_file_name, "/" );
	strcat( full_file_name, user_code );
	strcat( full_file_name, user_qual );
	set_dir_name( full_file_name );

	//
	// Confirm that the Local EDI User file directory exists.
	//

	if ( open_file_directory() == ERROR ) {
		log->log_event( "read_communication_attributes", 
				NO_SUCH_LOCAL_USER,
				MOVING_INTERCHANGE );
#ifdef	DEBUG
		cout << "Open file directory error on directory '" << 
			full_file_name << "'" << endl;
#endif
		status = ERROR;
	}
	else {
		close_file_directory();

		//
		// Construct the file name for the Trading Partner, and read
		// the file.
		//
	
		strcpy( tp_filename, full_file_name );
		strcat( tp_filename, "/" );
		strcat( tp_filename, tp_code );
		strcat( tp_filename, tp_qual );
		status = read_file();
	}

	return status;
}


STATUS TradingPartnerFile::read_file() {

	ifstream	ifile ( tp_filename, ios::in );
	char		protocol_id_string[ MAX_PROT_ID_STRING ];
	char		buf[ MAX_GETLINE ];
 
	
	if ( !ifile ) {
		log->log_event( "read_file", 
				NO_SUCH_TRADING_PARTNER,
				MOVING_INTERCHANGE );
#ifdef	DEBUG
		cout << "Open file error on file '" << tp_filename <<
			"'" << endl;
#endif
		return ERROR;		// File open error.
	}

	//
	// Parse the protocol identifier from the file.  It is located on the
	// first line.
	//

	ifile.getline( protocol_id_string, MAX_PROT_ID_STRING );

	if ( !strcmp( protocol_id_string, PEDI_SERVICE ))
		read_pedi_tp( &ifile );
	else if ( !strcmp( protocol_id_string, P2_SERVICE )) {
		protocol_id = MHS_IPM;
		// read the P2 transmission attributes
		ORName *recip = new ORName;
		assert( recip != 0 );
		recip->read_x400_address( &ifile );
		ifile.getline( buf, MAX_GETLINE );
		orig_security =  ( SECURITY ) atoi( buf );
		ifile.getline( buf, MAX_GETLINE );
		prior = atoi( buf );
	}
	else if ( !strcmp( protocol_id_string, FTAM_SERVICE )) {
		protocol_id = FTAM;
		// read the FTAM transmission attributes
		ifile.getline( name, MAX_LINE );
		ifile.getline( password, MAX_LINE );
		ifile.getline( filedir_name, MAX_LINE );
		ifile.getline( x500_name, MAX_LINE );
	}
	else if ( !strcmp( protocol_id_string, DIALUP_SERVICE )) {
		protocol_id = DIAL_UP;
		// read the DIALUP transmission attributes
		ifile.getline( vanname, MAX_LINE );
	}
	else {
		log->log_event( "read_file", 
				NO_SUCH_PROTOCOL_ID,
				MOVING_INTERCHANGE );
#ifdef	DEBUG
		cout << "Unidentified Protocol Id'" << protocol_id << 
		 "'" << endl;
#endif
		return ERROR;
	}

#ifdef	DEBUG
	display_tp_attributes();
#endif

	return SUCCESS;
}


BOOL TradingPartnerFile::validate_partner( ORName *tp_or_addr ) {

	//
	// Construct the file name for the O/R Address file.
	//

	strcpy( tp_filename, full_file_name );
	strcat( tp_filename, "/" );
	strcat( tp_filename, X400_ADDR_FILE );

	//
	// Open and read the X.400 Address file.
	//

	ifstream	ifile ( tp_filename, ios::in );
	
	if ( ! ifile ) {
#ifdef	DEBUG
		cout << "Open file error on file '" << tp_filename <<
			"'" << endl;
#endif
		return FALSE;
	}
	else {
		ORName	or_name;

		while ( ! ifile.eof() ) {
			or_name.read_x400_address( &ifile );
			if ( tp_or_addr->compare_or_name( &or_name ) )
				return TRUE;
		}
		return FALSE;
	}
}

BOOL TradingPartnerFile::validate_partner( char *tp_code, char *tp_qual ) {

	//
	// Construct the file name for the Trading Partner.
	//
	
	strcpy( tp_filename, full_file_name );
	strcat( tp_filename, "/" );
	strcat( tp_filename, tp_code );
	strcat( tp_filename, tp_qual );

	//
	// If the file can be opened, the trading partner is valid.
	//

	ifstream	ifile ( tp_filename, ios::in );
	
	if ( ! ifile )
		return FALSE;
	else
		return TRUE;
}


void TradingPartnerFile::read_pedi_tp( ifstream *ifile ) {

	char	buf[ MAX_LINE ];


	protocol_id = MHS_EDI;

	this_recip = new ORName;
	assert( this_recip != 0 );
	this_recip->read_x400_address( ifile );
		
	edin_receiver = new ORName;
	assert( edin_receiver != 0 );
	edin_receiver->read_x400_address( ifile );
	// Delete the EDIN Receiver if not in the file.
	if ( edin_receiver->country_name() == NULL ) {
		delete edin_receiver;
		edin_receiver = NULL;
	}
	
	ifile->getline( buf, MAX_GETLINE );
	positive_notif = atoi( buf );
	ifile->getline( buf, MAX_GETLINE );
	negative_notif = atoi( buf );
	ifile->getline( buf, MAX_GETLINE );
	forwarded_notif = atoi( buf );
	ifile->getline( buf, MAX_GETLINE );
	resp_passing = atoi( buf );
	ifile->getline( buf, MAX_GETLINE );
	prior = atoi( buf );
}

#ifdef	DEBUG
	
void TradingPartnerFile::display_tp_attributes() {	
	
	if ( protocol_id == MHS_EDI ) {
		cout << "Protocol Id:  MHS_EDI" << endl;
		this_recip->display_name();
		edin_receiver->display_name();
		cout << "Originator Security:  " << orig_security << endl;
		cout << "Positive Notification:  " << positive_notif << endl;
		cout << "Negative Notification:  " << negative_notif << endl;
		cout << "Forwarded Notification:  " << forwarded_notif << endl;
		cout << "Proof of Notification:  " << proof_of_notif << endl;
		cout << "Proof of Receipt:  " << proof_of_rcpt << endl;
		cout << "Nonrepudiation of Notification:  " << 
				nonrepud_of_notif << endl;
		cout << "Nonrepudiation of Reception:  " << 
				nonrepud_of_rcpt << endl;
		cout << "Responsibility Passing Allowed:  " << resp_passing 
			<< endl;
		cout << "Priority:  " << prior << endl;
	}
	else if ( protocol_id == MHS_IPM ) {
		cout << "Protocol Id:  MHS_IPM" << endl;
		this_recip->display_name();
		cout << "Originator Security:  " << orig_security << endl;
		cout << "Priority:  " << prior << endl;
	}
	else if ( protocol_id == FTAM ) {
		cout << "Protocol Id:  FTAM" << endl;
		cout << "Login Name:  " << name << endl;
		cout << "Password:  " << password << endl;
		cout << "File Directory Name:  " << filedir_name << endl;
		cout << "X.500 Name:  " << x500_name << endl;
	}
	else {
		cout << "Protocol Id:  DIAL_UP" << endl;
		cout << "VAN Name:  " << vanname << endl;
	}
}

#endif
