// FILE:  x400api.C

#include	<sys/types.h>
#include	<sys/unistd.h>
#include	<dirent.h>
#include	<fstream.h>
#include	<xom.h>
#include	<assert.h>
#include	<xedip.h>
extern "C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/subject.h"
#include	"../../../include/interchange.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/edi_user.h"
#include	"../../../include/config.h"
#include	"../../../include/platfile.h"
#include	"../../../include/tpfile.h"
#include	"../../../include/api_obj.h"
#include	"../../../include/x400api.h"


//
// Open a session with the X.400 MT API service.
//

STATUS X400Message::x400_open() {

	OM_return_code 	om_status;
	OM_string	client_name;

	//
	// Open a session with the API.
	//

#ifdef	DEBUG
	cout << "CLIENT NAME:  " << client << endl;
#endif

	MAKE_OM_STRING( client_name, client, strlen( client ) );
	if ( ( om_status = mt_open( client_name, client_name, feature_list, 
				    &session, &workspace ) ) != OM_SUCCESS ) {
		log->log_event( "mt_open", om_status, EXITING );
		return ERROR;
	}

	//
	// Confirm that the requested features are available.
	//
	
	for ( int i=0; feature_list[i].feature.length!=0; i++ ) {

		if ( feature_list[ i ].activated == OM_FALSE ) {
			log->log_event( "feature unavailable", ( char * ) 
					feature_list[ i ].feature.elements,
					EXITING );
			return ERROR;
		}
	}

#ifdef	DEBUG
	cout << "API Session Opened" << endl;
#endif

	return SUCCESS;
}
		

//
// Close the session opened with the x400_open().
//

STATUS X400Message::x400_close() {	 

	OM_return_code	om_status;
	
	if ( ( om_status = mt_close( session ) ) != OM_SUCCESS ) {
		log->log_event( "mt_close", om_status, EXITING );
		return ERROR;
	}

#ifdef	DEBUG
	cout << "Terminated API Session" << endl;
#endif
	return SUCCESS;
}
	

//
// Convey an outbound X.400 information object to the API service.
//

STATUS X400Message::x400_transfer_out( APIObject *object ) {

	OM_return_code	om_status;
	
	if ( ( om_status = mt_transfer_out( session, object->pri_object() ) ) 
							!= OM_SUCCESS ) {
		log->log_event( "mt_transfer_out", om_status, EXITING );
		return ERROR;
	}

#ifdef	DEBUG
	cout << "Transferred X.400 Message to MTA" << endl;
#endif
	
	return SUCCESS;
}
	

//
// Begin the transfer of an inbound X.400 information object held in the 
// API service queue.
//

STATUS X400Message::x400_start_transfer_in( APIObject *object ) {

	OM_return_code		om_status;
	OM_private_object	priv;
	
	om_status = mt_start_transfer_in( session, &priv, &identifier );
	if ( om_status != OM_SUCCESS ) {
		log->log_event( "mt_start_transfer_in", om_status, EXITING );
		return ERROR;
	}

	object->set_api_private( priv );

#ifdef	DEBUG
	cout << "Transfer of X.400 Message from MTA Started" << endl;
#endif
	
	return SUCCESS;
}
	

//
// Allow the API service to delete its copy of the inbound object.
//

STATUS X400Message::x400_finish_transfer_in( APIObject *object ) {

	OM_return_code		om_status;
	
	om_status = mt_finish_transfer_in(session,object->pri_object(),OM_TRUE);
	if ( om_status != OM_SUCCESS ) {
		log->log_event( "mt_finish_transfer_in", om_status, EXITING );
		return ERROR;
	}

#ifdef	DEBUG
	cout << "Transfer of X.400 Message from MTA Completed" << endl;
#endif
	
	return SUCCESS;
}
	

//
// Wait for a up to specified time period for an inbound object.
//

STATUS X400Message::x400_wait( BOOL *avail ) {

	OM_return_code	om_status;
	OM_boolean	available;
	
	om_status = mt_wait( session, interval, &available );
	if ( om_status != OM_SUCCESS ) {
		log->log_event( "mt_wait", om_status, EXITING );
		return ERROR;
	}

	*avail = ( available == OM_TRUE ) ? TRUE : FALSE;

#ifdef	DEBUG
	cout << "Wait for an Inbound Message From the MTA for " << 
		interval << " milli seconds" << endl;
#endif
	
	return SUCCESS;
}

//
// Use the mt_wait() to suspend platform execution.
//

STATUS X400Message::x400_wait() {

	OM_return_code	om_status;
	OM_boolean	available;
	
	om_status = mt_wait( session, interval, &available );
	if ( om_status != OM_SUCCESS ) {
		log->log_event( "mt_wait", om_status, EXITING );
		return ERROR;
	}

#ifdef	DEBUG
	cout << "Wait for an Inbound Message From the MTA for " << 
		interval << " milli seconds" << endl;
#endif
	
	return SUCCESS;
}

//
// Determine the number of objects in the API queue.
//

STATUS X400Message::x400_size( int *count ) {

	OM_return_code	om_status;
	MH_object_count	number;
	
	om_status = mt_size( session, &number );
	if ( om_status != OM_SUCCESS ) {
		log->log_event( "mt_size", om_status, EXITING );
		return ERROR;
	}

	*count = ( int ) number;

#ifdef	DEBUG
	cout << "MTA Inbound Queue Contains " << *count << " messages" << endl;
#endif
	return SUCCESS;
}

void X400Message::set_user( EDIUser *user ) {

	local_user = user->x400_address();
	strcpy( client, user->id_code() );
	strcat( client, user->id_code_qualifier() );
}


X400Message::X400Message( Log *l ) {

	tp_entry = NULL;
	interchange = NULL;
	log = l;
	log_file = log->get_logfile();
	log_level = log->get_loglevel();
	cur_time[ 0 ] = NULL;

	MAKE_OM_STRING( identifier, id_string, strlen( id_string ) );

	MAKE_OM_STRING( feature_list[ 0 ].feature,
			OMP_O_MH_FE_BASIC_TRANSFER, 6 );
	MAKE_OM_STRING( feature_list[ 1 ].feature,
			OMP_O_MH_FE_TRANSFER_IN, 6 );
	MAKE_OM_STRING( feature_list[ 2 ].feature,
			OMP_O_MH_FE_TRANSFER_OUT, 6 );
	MAKE_OM_STRING( feature_list[ 3 ].feature,
			OMP_O_MH_FE_MH_88, 6 );
	MAKE_OM_STRING( feature_list[ 4 ].feature,
			OMP_O_MH_FE_IM_88, 6 );
	MAKE_OM_STRING( feature_list[ 5 ].feature,
			OMP_O_EDI_PACKAGE, 6 );
	feature_list[ 6 ].feature.length = 0;

	interval = ( MH_interval ) WAIT_TIME;

	//
	// workspace, session, and identifier are initialized 
	// by om_XXXX() functions.
	//
}


X400Message::X400Message( TradingPartnerFile *x500, ORName *user, 
			  Log *l, Interchange *ic ) {

	tp_entry = x500;
	local_user = user;
	interchange = ic;
	log = l;
	log_file = log->get_logfile();
	log_level = log->get_loglevel();
	cur_time[ 0 ] = NULL;

	MAKE_OM_STRING( identifier, id_string, strlen( id_string ) );

	MAKE_OM_STRING( feature_list[ 0 ].feature,
			OMP_O_MH_FE_BASIC_TRANSFER, 6 );
	MAKE_OM_STRING( feature_list[ 1 ].feature,
			OMP_O_MH_FE_TRANSFER_IN, 6 );
	MAKE_OM_STRING( feature_list[ 2 ].feature,
			OMP_O_MH_FE_TRANSFER_OUT, 6 );
	MAKE_OM_STRING( feature_list[ 3 ].feature,
			OMP_O_MH_FE_MH_88, 6 );
	MAKE_OM_STRING( feature_list[ 4 ].feature,
			OMP_O_MH_FE_IM_88, 6 );
	MAKE_OM_STRING( feature_list[ 5 ].feature,
			OMP_O_EDI_PACKAGE, 6 );	
	feature_list[ 6 ].feature.length = 0;

	interval = ( MH_interval ) WAIT_TIME;

	//
	// Concatenate the EDI Sender ID Qualifier to the EDI
	// Sender ID Code to form the client name.
	// 

	strcpy( client, interchange->get_sender_ID_code() );
	strcat( client, interchange->get_sender_ID_qualifier() );

	//
	// workspace, session, and identifier are initialized 
	// by om_XXXX() functions.
	//
}

