// FILE:  x400cntin.C

#include	<fstream.h>
#include	<sys/types.h>
#include	<sys/unistd.h>
#include	<dirent.h>
#include        <time.h>
#include	<xom.h>
#include	<climportedi.h>
#include	<assert.h>
#include	<xedip.h>
extern "C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"/usr/lib/x400/gtwapi/include/ximp.h" 
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/subject.h"
#include	"../../../include/interchange.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/edi_user.h"
#include	"../../../include/config.h"
#include	"../../../include/platfile.h"
#include	"../../../include/tpfile.h"
#include	"../../../include/api_obj.h"
#include	"../../../include/x400api.h"


// #define	PN	1
//#define	NN	1


//
// An EDI message may contain, as its primary bodypart, a forwarded
// EDI message.  A forwarded message comprises the envelope of the subject
// message (if security services were requested or if the message was 
// forwarded for the first time), the header of the subject message, the
// primary bodypart of the subject message (if it was not removed by the 
// forwarding EDI-UA), possible additional bodyparts and supplementary 
// information. To parse a forwarded EDI message, the attributes comprising
// the forwarded message content are copied first, in case security services
// must be validated.  The forwarded envelope is then processed, followed by
// the forwarded content.
//
STATUS X400Message::parse_edim_bodypart( APIObject *edim_bodypart,
					 EDIUser   *edi_user )  {

	static OM_type	types[ 4 ] = { EDI_HEADING, EDI_PRIMARY_BODYPART,
				       EDI_ADD_BODYPARTS, OM_NO_MORE_TYPES };
	static OM_type	deliv_env_types[ 1 ] = { EDI_DELIV_ENVELOPE }; 
	static OM_type	forward_types[ 5 ] = { EDI_HEADING, 
					       EDI_PRIMARY_BODYPART,
					       EDI_ADD_BODYPARTS, 
					       EDI_SUPPLEMENTARY_INFO,
					       OM_NO_MORE_TYPES };
        APIObject		     edi_heading( edim_bodypart );
        APIObject		     edi_prim_bodypart( edim_bodypart );
        APIObject		     edi_add_bodypart( edim_bodypart );
        OM_value                     value;
	OM_exclusions   exclusions = OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES;
	OM_exclusions   forward_exclusions = OM_EXCLUDE_SUBOBJECTS |
				     OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES;
        STATUS			     status = SUCCESS;

	//
        // Locate the content object for the forwarded message. 
	//
	// Keep for security validation (furture).
        // status |= edim_bodypart->api_get( exclusions, types );
	// if ( ! OK( status ) )
	//  	return status;
	// Public object for setting the original content. 
        // subject_message.set_orig_content( edim_bodypart->api_public() );
        
	// Process the forwarded envelope.
        status |= edim_bodypart->api_get( forward_exclusions, deliv_env_types );
	if ( ! OK( status ) )
	  	return status;

//        parse_edi_object( edim_bodypart, edi_user );

	// Process the remainder of the forwarded message.
	status |= edim_bodypart->api_get( forward_exclusions, forward_types ); 

        while( edim_bodypart->more_types() ) {
	
	  value = edim_bodypart->value();
	  switch( edim_bodypart->type() ) {

		case EDI_HEADING:
		     edi_heading.set_api_private( 
				(OM_private_object) value.object.object );
		     parse_edi_heading( &edi_heading, edi_user );
		     break;

                case EDI_PRIMARY_BODYPART:
		     edi_prim_bodypart.set_api_private( 
				(OM_private_object) value.object.object );
		     parse_edi_primary_bodypart( &edi_prim_bodypart, edi_user );
		     break;

		case EDI_ADD_BODYPARTS: 
		     edi_add_bodypart.set_api_private( 
				(OM_private_object) value.object.object );
		     parse_edi_additional_bodyparts( &edi_add_bodypart, 
						     edi_user );
		     break;

                case EDI_SUPPLEMENTARY_INFO:
		     // log the attrubute value.
		     // value.elements << outfile << endl;
		     break;

                default:
		     break;
	  }
        edim_bodypart->get_next_type();
       }

       return( status );

}
	 
// 
// The EDI bodypart contains the interchange to be conveyed to the user.
// To parse the interchange, the attribute identifying the bodypart is
// copied from the private object argument.  The interchange is presented
// to the user by writing the interchange to a specific file directory or
// the TSDS.
//

STATUS X400Message::parse_edi_bodypart( APIObject *prim_bodypart,
					EDIUser   *edi_user ) {

	static OM_type	types[ 1 ] = { EDI_BODY };
	OM_exclusions   exclusions = OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES;
	OM_value	value;
        ofstream        *inbound_interchange_file;
	char            inbound_file[ MAX_FILENAME ];
	char            filename[ MAX_TIME_BUF ];
	STATUS		status = SUCCESS;

#ifdef	DEBUG
	cout << "In parse_edi_bodypart()" << endl;
#endif
	//
        // Copy the interchange from the message.
	//
//        status |= prim_bodypart->api_get( exclusions, types );
//	if ( ! OK( status ) )
//	  	return status;

	while ( prim_bodypart->type() != EDI_BODY )
		prim_bodypart->get_next_type();

	// Set the interchange equal to the public object attribute value.
	value = prim_bodypart->value();
        
        // Write the interchange to either a file directory or the TSDS queue.
	if ( edi_user->tsds_queue() )
           ;  // Write interchange to TSDS input queue.

	else { // Write interchange to file in local input-directory
           strcpy( inbound_file, edi_user->input_directory() );
	   strcat( inbound_file, "/" );
           // Create a file (using a system call which returns the time of
	   // day and append it onto the full path name. TIMESTAMP error ??
	   timestamp( filename );
	   strcat( inbound_file, filename );
#ifdef	DEBUG
	   cout << "writing to file " << inbound_file << endl;
#endif
         // Open the file to contain the incoming interchange.
	 inbound_interchange_file = new ofstream( inbound_file, ios::out );
	 // Make sure the file was properly opened.
	 if ( !inbound_interchange_file ) {
	    log->log_event( "parse_edi_bodypart", FILE_NOT_CREATED, EXITING );
	    return( status |= ERROR );
         }
#ifdef DEBUG
	cout << "Inbound interchange file: " << inbound_file << endl;
#endif
	 // Write the value to the interchange file and close it.
	 *inbound_interchange_file << ( char * ) value.string.elements;
	 inbound_interchange_file->close();
      }

  return ( status );

}

//
// the primary bodypart of an EDI message contains either the interchange
// to be conveyed to the user or a forwarded EDI message.  To parse the
// primary bodypart, the attribute identifying the object's class
// is copied from the private object argument.
//
STATUS X400Message::parse_edi_primary_bodypart( APIObject *prim_bodypart,
						EDIUser   *edi_user ) {

	static OM_type	types[ 1 ] = { OM_CLASS };
	OM_exclusions    exclusions = OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES;
	STATUS		status = SUCCESS;
	
#ifdef	DEBUG
	cout << "In parse_edi_primary_bodypart()" << endl;
#endif
        //
        // Determine whether primary bodypart identifies an interchange
	// or a forwarded EDI message.
	//

	//
        // Copy private object to public object. 
	//
//        status |= prim_bodypart->api_get( exclusions, types );
//	if ( ! OK( status ) )
//	 	return status;

	if ( prim_bodypart->is_class( OMP_O_EDI_C_EDI_BODYPART ) ) {
		status |= parse_edi_bodypart( prim_bodypart, edi_user );
	}
	else {
	   	// EDI_C_EDIM_BODYPART
		status |= parse_edim_bodypart( prim_bodypart, edi_user);
	}
	
	return status;
}

//
// Additional EDI bodyparts contain information to supplement the 
// interchange bodypart.  Additional bodyparts cannot be or contain
// interchanges.  To parse and additional bodypart, the private object
// argument is copied to a public object, without exclusions.  The 
// hierarchical structure of the public object (the 'EDIM externally defined
// bodypart' object contains an 'externally defined bodypart' subobject,
// which contains an 'external' subobject) is parsed using programming
// constructs.  The additional bodypart data is obtained from one of the
// encoding attributes of the 'external' object.  Once obtained, the data
// is written to a file.  Additional bodyparts are never conveyed to the TSDS.
//
STATUS X400Message::parse_edi_additional_bodyparts( APIObject *bodypart,
						    EDIUser   *edi_user ) {

	OM_exclusions   exclusions = OM_NO_EXCLUSIONS;
	OM_value	value;
	char		*body;
	ofstream	*inbound_add_file;
	char		add_file[ MAX_FILENAME ];
	char		filename[ MAX_TIME_BUF ];
	STATUS		status = SUCCESS;
	
	//
        // Copy private object to public object. 
	//
//        status |= bodypart->api_get( exclusions );
//	if ( ! OK( status ) )
//	 	return status;

 	while ( bodypart->more_types() ) {
	  if ( bodypart->type() == EDI_EX_BODYPART ) {
	    value = bodypart->value();
	    bodypart->set_api_public( value.object.object );
 	      while ( bodypart->more_types() ) {
		if ( bodypart->type() == IM_EXTERNAL_DATA ) {
		  value = bodypart->value();
		  bodypart->set_api_public( value.object.object );
 		  while ( bodypart->more_types() ) {
	    	    value = bodypart->value();
        	    switch( bodypart->type() ) {
	   	      case OM_ARBITRARY_ENCODING: 
	   	      case OM_OCTET_ALIGNED_ENCODING:
	   	      case OM_ASN1_ENCODING:
			body = ( char * ) value.string.elements;
			break;
		    } // switch
	 	    bodypart->get_next_type();
		  } // while
		} // if
	        bodypart->get_next_type();
	      } // while
	    } // if
	    bodypart->get_next_type();
	  } // while
   
   // Write the additional bodypart to the appropriate file directory.
   strcpy( add_file, edi_user->additional_directory() );
   strcat( add_file, "/" );
   // Create a file (using a system call which returns the time of
   // day and append it onto the full path name. TIMESTAMP error???
   timestamp( filename );
   strcat( add_file, filename );
#ifdef	DEBUG
   cout << "writing to file " << add_file << endl;
#endif
   // Open the file to contain the incoming interchange.
   inbound_add_file = new ofstream( add_file, ios::out );
   // Make sure the file was properly opened.
   if ( !inbound_add_file ) {
      log->log_event( "parse_edi_bodypart", FILE_NOT_CREATED, EXITING );
      return( status |= ERROR );
   }
#ifdef DEBUG
   cout << "Inbound add file: " << add_file << endl;
#endif
   // Write the value to the interchange file and close it.
   *inbound_add_file << ( char * ) value.string.elements;
   inbound_add_file->close();

   return status;
}


STATUS X400Message::parse_edi_notif_requests( OM_public_object notif_req,
					      APIObject *recipient ) {

        APIObject	edi_notif_req( recipient );
	OM_value	value;
	STATUS		status = SUCCESS;

        edi_notif_req.set_api_public( notif_req );

        while ( edi_notif_req.more_types() ) {

	   //  attribute value.
           value = edi_notif_req.value();

           switch( edi_notif_req.type() ) {

	     case EDI_POSITIVE_NOTIF:
		subject_message.set_edi_pos_notif( value.boolean );
		break;

	     case EDI_NEGATIVE_NOTIF:
		subject_message.set_edi_neg_notif( value.boolean );
		break;

	     case EDI_FORWARD_NOTIF:
		subject_message.set_edi_forw_notif( value.boolean );
		break;

	     case EDI_PROOF_OF_RECEPTION:
	     case EDI_NON_REPUD_OF_RECEPTION:
		subject_message.set_reason_code(EDI_NUMB_CANNOT_DELIV_TO_USER);
		subject_message.set_diagnostic( EDI_NUMD_UNABLE_TO_SIGN_NOTIF );
		status |= MESSAGE_NOT_ACCEPTED;
		break;
           }
        // Increment to next type
        edi_notif_req.get_next_type();
        }

	return status;
}


//
// The EDI extension provides a mechanism fo inclusion of future extensions
// to the EDI messaging standard.  The EDI-UA does not support any
// extensions currently.  If an extension is received and marked as critical
// for acceptance of responsibility, the message must be rejected.  Otherwise,
// the extension is ignored.
//
STATUS X400Message::parse_edi_extension( APIObject *extension ) {

        OM_exclusions	exclusions = OM_NO_EXCLUSIONS;

	static OM_type	types[ 2 ] = { // EDI_CRITICALITY,
					OM_NO_MORE_TYPES };
        STATUS          status = SUCCESS;

        while ( extension->more_types() ) {
		  // EDI_CRITICALITY needs to be defined...
              if ( ( extension->type() == 0 /* FIX PH EDI_CRITICALITY */ ) &&
		   ( extension->value().boolean == OM_TRUE ) ) {
		   subject_message.set_reason_code( 
						EDI_NUMB_CANNOT_DELIV_TO_USER );
		   subject_message.set_diagnostic( 
						EDI_NUMD_UNSUPP_EXTENSION );
		   // build_nn( EDI_NUMB_CANNOT_DELIV_TO_USER,
		   //	        EDI_NUMD_UNSUPP_EXTENSION );
		   // Abort message. 
		   return( status |= MESSAGE_NOT_ACCEPTED );
              }
        // Increment to next type
	extension->get_next_type();
	}
  return ( status );
}

//
// The recipient object contains information that may be specified on
// a per-recipient basis, such as requests for EDI notifications.  To
// parse the attributes specific to the local user, the local user must
// be identified from the list of recipients.   If a recipient object
// exists for the local user, all attributes are copied from the recipient
// object. Although only certain attributes are processed, copying all
// the attributes permits logging of the values received.
//
STATUS X400Message::parse_recipients( APIObject * recipient,
				      EDIUser   *edi_user ) {
	STATUS			status = SUCCESS;
	APIObject 	        int_recip( recipient );
	APIObject 	        recip_ext( recipient );
	APIObject 	        recipients( recipient );
	OM_value        	value;
	OM_exclusions  	        exclusions = OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES;
	static OM_type		types[ 6 ] = { EDI_EDI_NOTIF_REQUESTS,
					       EDI_RECIP,
					       EDI_INTER_RECIP,
				      	       EDI_RECIP_EXTENSIONS,
				    	       EDI_ACT_REQUEST,
				       	       OM_NO_MORE_TYPES };

#ifdef DEBUG
	cout << "in parse_recipients()" << endl;
#endif

	// Copy all attributes from the recipient object for the local user.
//        status |= recipient->api_get( exclusions, types );
//	if ( ! OK( status ) )
//	 	return status;
        
	while ( recipient->more_types() ) {

	   //  attribute value.
           value = recipient->value();

#ifdef DEBUG
	   cout << "Type = " << recipient->type() << endl;
#endif
           switch( recipient->type() ) {

	     case EDI_EDI_NOTIF_REQUESTS:
		status |= parse_edi_notif_requests( value.object.object, 
						    recipient );
		break;

	     case EDI_INTER_RECIP:
		int_recip.set_api_public( recipient->value().object.object );
		while ( int_recip.more_types() ) {
			if ( int_recip.type() == EDI_IDENTIFICATION_CODE ) {
				if ( strcmp( edi_user->id_code(), ( char * )
						 value.string.elements ) ) {
		   			subject_message.set_reason_code(
						EDI_NUMB_CANNOT_DELIV_TO_USER );
		   			subject_message.set_diagnostic( 
						EDI_NUMD_UNSUPP_EXTENSION );
					// build_nn( EDI_NUMB_CANNOT_DELIV_TO_USER,
			      		//	EDI_NUMD_EDIM_RECIP_AMBIGUOUS );
                    			// Abort message. 
	            			return(status |= MESSAGE_NOT_ACCEPTED);
                		}
			}
			if ( int_recip.type() == EDI_IDENTIFICATION_CODE_Q ) {
				if ( strcmp( edi_user->id_code_qualifier(), 
					( char * ) value.string.elements ) ) {
		   			subject_message.set_reason_code( 
						EDI_NUMB_CANNOT_DELIV_TO_USER );
		   			subject_message.set_diagnostic( 
						EDI_NUMD_UNSUPP_EXTENSION );
					// build_nn( EDI_NUMB_CANNOT_DELIV_TO_USER,
			      		//	EDI_NUMD_EDIM_RECIP_AMBIGUOUS );
                    			// Abort message. 
	            			return(status |= MESSAGE_NOT_ACCEPTED);
                		}
			}
			int_recip.get_next_type();
		}
		break;

             case EDI_RECIP_EXTENSIONS:
		recip_ext.set_api_public( value.object.object );
		status |= parse_edi_extension( &recip_ext );
		break;

             case EDI_ACT_REQUEST:
		if ( edi_user->reject_copy() ) { 
		   if( memcmp( value.string.elements, 
		   	       OMP_O_EDI_ARO_FOR_ACTION, 5 ) ) {
		   	subject_message.set_reason_code( 
						EDI_NUMB_CANNOT_DELIV_TO_USER );
		   	subject_message.set_diagnostic( 
						EDI_NUMD_UNSUPP_EXTENSION );
                      // build_nn( EDI_NUMB_CANNOT_DELIV_TO_USER,
		      //	        EDI_NUMD_ACT_REQUEST_NOT_SUPP );
                      // Abort message. 
	              return( status |= MESSAGE_NOT_ACCEPTED );
                   }
                }
	   }
        // Increment to next type
	recipient->get_next_type();
        }

	return( status );
}

//
// EDI application security elements allow the exchange of security
// elements having end-to-end significance.  The EDI-UA does not
// currently recognize any security elements implemented within the
// EDI application.  If any application security elements are labeled as
// critical for acceptance of responsibility, the message must be rejected.
//
STATUS X400Message::parse_edi_application_security_elements(
               				    APIObject *security ) {

	APIObject	extension( security );
	STATUS		status = SUCCESS;

        while ( security->more_types() ) {
              if ( security->type() == EDI_APP_SECUR_EXTENSIONS ) {
		extension.set_api_public( security->value().object.object );
	      	status |= parse_edi_extension( &extension );
              }
        // Increment to next type.
	security->get_next_type();
	}
   return( status );
 }

//
// Parse the syntax id for the syntax id string and version.
//  
void X400Message::parse_edi_syntax_id( APIObject *syntax_id,
				     char *syntax_id_string,
				     char *syntax_version    ) {

        while ( syntax_id->more_types() ) {
		switch( syntax_id->type() ) {
			case EDI_SYNTAX_ID_STRING:
			   strcpy( syntax_id_string, 
				   (char *) syntax_id->value().string.elements);
			   break;

			case EDI_SYNTAX_VERSION:
			   strcpy( syntax_version, 
				   (char *) syntax_id->value().string.elements);
                           break;
      
			default:
			   break;
		}
        // Increment to next type.
	syntax_id->get_next_type();
        }	
 }


//
// The EDI message header contains a structured representation of info 
// about the message.  This information may determine whether the message
// is made available to the user or rejected.  The first step in processing
// the message header is to parse the recipient attribute values for the
// local user.  The recipient object contains the notifications requested,
// which are required when parsing other header attributes.  After the
// recipient attributes are parsed, all header attributes are copied from
// the private object argument.  Although only certain attributes are
// processed, copying all the attributes permits logging of the values received.
//
STATUS X400Message::parse_edi_heading( APIObject *heading,
				       EDIUser   *edi_user ) {

        char  		id_code[ RECIP_ID_CODE_SIZE ];
        char  		id_qual[ ID_QUALIFIER_SIZE ];
        APIObject	extension( heading );
        APIObject	security( heading );
        APIObject	syntax_id( heading );
        APIObject	recipients( heading );
        OM_exclusions	exclusions = OM_NO_EXCLUSIONS;
        OM_exclusions	recip_exclusions = OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES;
	static	OM_type recip_types[ 2 ] = { EDI_RECIPS, OM_NO_MORE_TYPES };       
	OM_value	value;
	STATUS		status = SUCCESS;
	char		*bp_type;
	char		syntax_id_string[ SYNTAX_ID_SIZE ];
	char		syntax_version[ SYNTAX_VERSION_SIZE ];
	time_t          totsecs;

#ifdef	DEBUG
	cout << "In parse_edi_heading()" << endl;
#endif
//	status |= heading->api_get( recip_exclusions, recip_types );
//	if ( ! OK( status ) )
//	 	return status;
	//
	// Process the recipient object for the local user.
	// to get the notifications requested which are required 
	// when parsing the other header attributes. 
	//
	heading->save_index();
	while ( heading->more_types() ) {
		if ( heading->type() == EDI_RECIPS ) {
			recipients.set_api_public(
					heading->value().object.object );
			status |= parse_recipients( &recipients, edi_user );
			break;
		}
		heading->get_next_type();
	}
	heading->reset_index();
	
	//
        // Copy all attributes from the message header.
	//
//        status |= heading->api_get( exclusions );
//	if ( ! OK( status ) )
//	 	return status;

        while ( heading->more_types() ) {

	   //  attribute value.
           value = heading->value();
#ifdef DEBUG
		cout << "Type = " << heading->type() << endl;
#endif
           switch( heading->type() ) {

             case EDI_THIS_EDIM:
		   // save the object this edim attribute value.
		   subject_message.set_this_edim_id( value.object.object );
		   break;

             case EDI_ORIG:
		   // save the object originator attribute value.
		   subject_message.set_edi_originator( value.object.object );
		   break;

             case EDI_EDIN_RECEIVER:
		   // Save the object edin receiver. 
		   subject_message.set_edin_receiver( value.object.object );
		   break;

             case EDI_RESPONS_FORWARD:
		   // Save the boolean edi response forward. 
		   subject_message.set_edi_resp_forwarded( value.boolean );
		   break;

             case EDI_EDI_BODYPART_TYPE:
		   // Make sure the bodytype is acceptable.  If not, the
		   // EDI-UA will build a neg notif and reject the message.
                   // 
                   if ( edi_user->test_edi_standards() ) {
			bp_type = ( char * ) value.string.elements;
			if ( ! edi_user->edi_standard( bp_type[4] )) {
		   		subject_message.set_reason_code( 
					EDI_NUB_INVALID_BODYPART_TYPE );
		  		subject_message.set_diagnostic( 
					EDI_NUMD_INVALID_BODYPART_TYPE );
//		       build_nn( EDI_NUMB_CANNOT_DELIV_TO_USER,
//			         EDI_NUMD_INVALID_BODYPART_TYPE );
			}
                       // abort message 
		       return( status |= MESSAGE_NOT_ACCEPTED );
		   }
		   break;

             case EDI_INC_COPY: 
		   if ( edi_user->reject_incomplete_copy() ) {
		      if ( value.boolean == OM_TRUE ) {
		   		subject_message.set_reason_code( 
						EDI_NUMB_CANNOT_DELIV_TO_USER );
		   		subject_message.set_diagnostic( 
						EDI_NUMD_UNSUPP_EXTENSION );
		         // build_nn( EDI_NUMB_CANNOT_DELIV_TO_USER,
			 //           EDI_NUMD_INC_COPY_REJECTED );
                         // abort message 
		         return( status |= MESSAGE_NOT_ACCEPTED );
                      }
                   }
		   break;

	     case EDI_EXPIRY_TIME: 
		   // If attrubute value is less than current time the
		   // message has expired. Build a neg notif and abort the
		   // message.
		   totsecs = time( NULL );
		   if ( memcmp ( value.string.elements, 
		   	         asctime( localtime( &totsecs ) ),
				 ( int ) value.string.length ) < 0 )
		   {
		   	subject_message.set_reason_code( 
						EDI_NUMB_CANNOT_DELIV_TO_USER );
		   	subject_message.set_diagnostic( 
						EDI_NUMD_EDIM_EXPIRED );
		       // build_nn( EDI_NUMB_CANNOT_DELIV_TO_USER,
		       //          EDI_NUMD_EDIM_EXPIRED);
                       // abort message
		       return( status |= MESSAGE_NOT_ACCEPTED );
		   }
		   break;

             // Related EDI Messages.
//	     case EDI_RELATED_EDI_MSGS:

	     // Related non-EDI Messages.
//	     case EDI_RELATED_NON_EDI_MSGS:

	     // Obsoleted EDIM.
//	     case EDI_OBSOLETED_EDIMS:

             case EDI_APP_SECUR_ELEMENTS:
		   security.set_api_public( value.object.object );
		   status |= parse_edi_application_security_elements(
					    &security );
		   if ( ! OK( status ) )
		      return status;
		   break;

             // Cross Referencing Information
//	     case EDI_CROSS_REFERENCING_INFO:

//             case EDI_EDI_MSG_TYPE: 

             // Service String advice.
//	     case EDI_SERVICE_STRING_ADVICE:

             case EDI_SYNTAX_ID: 
		   // Make sure the syntax id is acceptable.  If not, the
		   // EDI-UA will build a neg notif and reject the message.
		   // local syntaxes are of type queue.
		   syntax_id.set_api_public( value.object.object );
		   parse_edi_syntax_id( &syntax_id, syntax_id_string, 
                                                   syntax_version );
                   if ( ! edi_user->validate_syntax_id( syntax_id_string,
						      syntax_version ) ) { 
		   	subject_message.set_reason_code( 
						EDI_NUMB_CANNOT_DELIV_TO_USER );
		   	subject_message.set_diagnostic( 
						EDI_NUMD_UNSUPP_EXTENSION );
		       // build_nn( EDI_NUMB_CANNOT_DELIV_TO_USER,
		       //          EDI_NUMD_INVALID_SYNTAX_ID );
                       // abort message. 
		       return( status |= MESSAGE_NOT_ACCEPTED );
		   }
		   break;

	     case EDI_INTER_SENDER:
		   // Save the edin receiver. 
		   subject_message.set_interchange_sender( value.object.object );
		   break;

             // Date and Time of preparation. 
	     case EDI_DATE_AND_TIME_OF_PREP:
		   break;

	     // Application Reference
	     case EDI_APP_REFERENCE:
		   break;

             case EDI_HEADING_EXTENSIONS:
		   extension.set_api_public( value.object.object );
		   parse_edi_extension( &extension );
		   break;

             default:
		   break;

	  }
         // Increment to next type
	 heading->get_next_type();
	}
	// Test the validity of the originating EDI user, if requested by 
        // the local user.
 	if ( edi_user->test_sender() ) {       	
           if ( subject_message.interchange_sender() != NULL )
	   // Now compare the x.400 sender ID code and qualifier just filled
	   // using the directory with the sender info off of the heading.
	     if ( ! tp_entry->validate_partner( id_code, id_qual ) ) {
		   subject_message.set_reason_code( 
						EDI_NUMB_CANNOT_DELIV_TO_USER );
		   subject_message.set_diagnostic( 
						EDI_NUMD_UNSUPP_EXTENSION );
                // build_nn( EDI_NUMB_CANNOT_DELIV_TO_USER,
		//	  EDI_NUMD_INTER_SENDER_UNKNOWN );
		// abort message.
	        return( status |= MESSAGE_NOT_ACCEPTED );
             } 
        }

#ifdef	DEBUG
	cout << "edi_heading status = " << status << endl;
#endif
  return status;
}	

//
// An EDI message comprises an EDI message header, a primary bodypart
// and optionally one or more additional bodyparts.  The primary bodypart
// contains the interchange to be conveyed to the user, or a forwarded
// EDI message.  To parse the EDI message, the attributes identifying the
// message components are copied from the private object argument.
//

STATUS X400Message::parse_edi_message (APIObject  *edim,
				       EDIUser    *edi_user ) {
        OM_value    value;
        APIObject   heading( edim );
	APIObject   primary_bodypart( edim );
	APIObject   add_bodyparts( edim );
        STATUS      status = SUCCESS;

	OM_exclusions    exclusions = OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES
				      | OM_EXCLUDE_SUBOBJECTS;
        static OM_type   types[ 12 ] = 
	{ OM_CLASS, EDI_HEADING, EDI_PRIMARY_BODYPART, EDI_ADD_BODYPARTS, 
	  EDI_SUBJECT_EDIM, EDI_EDIN_ORIG, EDI_FIRST_RECIP, EDI_NOTIF_TIME,
	  EDI_EDIN_INITIATOR, EDI_SUPPLEMENTARY_INFO, EDI_NN_REASON_CODE,
	  OM_NO_MORE_TYPES };

        // Copy private-object to public-object.
        // Exclude subobjects AND exlude all but certain types.
//        status |= edim->api_get( exclusions, types );
//	if ( ! OK( status ) )
//		return status;

subject_message.set_reason_code(NO_REASON);
subject_message.set_diagnostic( NO_DIAGNOSTIC );
subject_message.set_edi_pos_notif( OM_FALSE );
subject_message.set_edi_neg_notif( OM_FALSE );

        while ( edim->more_types() ) {
#ifdef	DEBUG
	   cout << "EDIM/EDIN Type = " << edim->type() << endl;
#endif
	   value = edim->value();
           
	   switch( edim->type() ) {

	      case OM_CLASS:
	      	if (! memcmp( value.string.elements, 
			      OMP_O_EDI_C_NEGATIVE_NOTIF, 7 ) || 
	      	    ! memcmp( value.string.elements, 
			      OMP_O_EDI_C_POSITIVE_NOTIF, 7 ) ) {
			// This is a notification, all attributes have already
			// been logged.  Just return.
			return status;
		}
	        break;

	      case EDI_HEADING:
		   heading.set_api_public( 
				( OM_public_object ) value.object.object );
		   status |= parse_edi_heading( &heading, edi_user );
		   if ( ! OK( status ) )
		      return status;
		   if ( status & MESSAGE_NOT_ACCEPTED )
		      return status;
		   break;

	      case EDI_PRIMARY_BODYPART:
		   primary_bodypart.set_api_public( 
				( OM_public_object ) value.object.object );
		   status |= parse_edi_primary_bodypart( &primary_bodypart, edi_user );
		   if ( ! OK( status ) )
		      return status;
		   break;
  
	      case EDI_ADD_BODYPARTS:      
		   add_bodyparts.set_api_public( 
				( OM_public_object ) value.object.object );
		   status |= parse_edi_additional_bodyparts( 
				&add_bodyparts, edi_user );
		   if ( ! OK( status ) )
		      return status;
		   break;
	  }
         // Increment to next type
	 edim->get_next_type();
	}

#ifdef	PN
subject_message.set_reason_code(NO_REASON);
subject_message.set_diagnostic( NO_DIAGNOSTIC );
subject_message.set_edi_pos_notif( OM_TRUE );
#endif
#ifdef	NN
subject_message.set_reason_code(EDI_NUB_INVALID_BODYPART_TYPE);
subject_message.set_diagnostic( EDI_NUMD_INVALID_BODYPART_TYPE );
subject_message.set_edi_neg_notif( OM_TRUE );
#endif
  
  return status;

}
