// FILE:  x400cntout.cc

#include	<fstream.h>
#include	<sys/types.h>
#include	<sys/unistd.h>
#include	<dirent.h>
#include	<xom.h>
#include	<sys/time.h>
#include	<string.h>
#include	<assert.h>
#include	<xedip.h>
extern "C" {
#include        "/usr/lib/x400/gtwapi/include/nist_xmh.h"
}
#include        "/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/subject.h"
#include	"../../../include/interchange.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/edi_user.h"
#include	"../../../include/config.h"
#include	"../../../include/platfile.h"
#include	"../../../include/tpfile.h"
#include	"../../../include/api_obj.h"
#include	"../../../include/x400api.h"

//
//  Concatenate the preparation date and time from the 
//  interchange and return the pointer to the whole UTC string.
//

inline utc_time X400Message::build_prep_date_and_time(Interchange *interchange) {
    return( (utc_time) strcat( (char *) interchange->get_date_preparation(), 
                               (char *) interchange->get_time_preparation() ) );
}


void X400Message::build_this_EDIM( APIObject 	*this_edim,
				     APIObject	*originator ) {
 
#ifdef	DEBUG
	cout << "In build_this_EDIM()" << endl;
#endif

     	// 
     	// Build this_EDIM public object
     	//

	this_edim->make_class( OMP_O_EDI_C_EDIM_ID );
	this_edim->put_desc( EDI_USER, originator->pub_object() );
     	timestamp( this_edim->time_buf() );
        // user-relative-id is simply the time stamp because 
        // there will not be more than one EDIM per 1/1000 of a second.
     	this_edim->put_desc( EDI_USER_RELATIVE_ID,
                   	     OM_S_PRINTABLE_STRING, this_edim->time_buf() );
     	this_edim->end_object();

#ifdef	DEBUG
	cout << "leaving EDIM " << endl;
#endif
}


void X400Message::build_edi_notif_requests( APIObject *edi_notif_req ) {

#ifdef	DEBUG
	cout << "In build_edi_notif_requests()" << endl;
#endif

    	//
    	// Build the edi notification requests public object.
    	//

	edi_notif_req->make_class( OMP_O_EDI_C_EDI_NOTIF_REQUESTS );
    	edi_notif_req->put_desc( EDI_POSITIVE_NOTIF, OM_S_BOOLEAN, 
				 tp_entry->positive_notification() );
    	edi_notif_req->put_desc( EDI_NEGATIVE_NOTIF, OM_S_BOOLEAN,
             			 tp_entry->negative_notification() );
    	edi_notif_req->put_desc( EDI_FORWARD_NOTIF, OM_S_BOOLEAN,
             			 tp_entry->forwarded_notification() );
//    	edi_notif_req->put_desc( EDI_PROOF_OF_NOTIF,  //NOTE: No #define exists
//             		 	 OM_S_BOOLEAN, 
//				 tp_entry->proof_of_notification() );
//    	edi_notif_req->put_desc( EDI_NON_REPUD_OF_NOTIF,  //NOTE: No #define
//              			 OM_S_BOOLEAN,
//              			 tp_entry->nonrepudiation_of_notification() );
    	edi_notif_req->put_desc( EDI_PROOF_OF_RECEPTION, OM_S_BOOLEAN,
	      			 tp_entry->proof_of_reception() );
    	edi_notif_req->put_desc( EDI_NON_REPUD_OF_RECEPTION, OM_S_BOOLEAN,
              			 tp_entry->nonrepudiation_of_reception() );
    	edi_notif_req->end_object();

#ifdef	DEBUG
	cout << "leaving edi notif" << endl;
#endif
}


void X400Message::build_interchange_recipient( APIObject *inter_recip ) {
 
#ifdef	DEBUG
	cout << "In build_interchange_recipient()" << endl;
#endif

     	// 
     	// Build the interchange recipient public object
     	//
     
	inter_recip->make_class( OMP_O_EDI_C_INTER_SENDER_RECIP );
     	inter_recip->put_desc( EDI_IDENTIFICATION_CODE, OM_S_TELETEX_STRING, 
              		       interchange->get_recipient_ID_code()  );
        if ( interchange->get_recipient_ID_qualifier()[ 0 ] )
     		inter_recip->put_desc( EDI_IDENTIFICATION_CODE_Q, 
				 OM_S_TELETEX_STRING, 
              		         interchange->get_recipient_ID_qualifier() );
        if ( interchange->get_recipient_routing_addr()[ 0 ] )
     		inter_recip->put_desc( EDI_ROUTING_ADDRESS, OM_S_TELETEX_STRING,
              		         interchange->get_recipient_routing_addr() );
     	inter_recip->end_object();

#ifdef	DEBUG
	cout << "leaving interchange recip" << endl;
#endif
}


void X400Message::build_recipient_ref( APIObject  *recipient_ref ) {
 
#ifdef	DEBUG
	cout << "In build_recipient_ref()" << endl;
#endif
	
	// 
     	// Build Recipient Reference public object
     	//
	
	recipient_ref->make_class( OMP_O_EDI_C_RECIP_REFERENCE );
     	recipient_ref->put_desc( EDI_RECIP_REFERENCE, OM_S_TELETEX_STRING, 
              			 interchange->get_recipient_reference() );
        if ( interchange->get_recipient_reference_qualifier()[ 0 ] )
     		recipient_ref->put_desc( EDI_RECIP_REFERENCE_Q, 
			   OM_S_TELETEX_STRING, 
              		   interchange->get_recipient_reference_qualifier() );
     	recipient_ref->end_object();

#ifdef	DEBUG
	cout << "leaving recipient_ref" << endl;
#endif
}


void X400Message::build_authorization_info( APIObject *auth_info ) {

#ifdef	DEBUG
	cout << "In build_authorization_info()" << endl;
#endif
	
     	// 
     	// Build Authorization Information public object
     	//
     	
	auth_info->make_class( OMP_O_EDI_C_AUTHORIZATION_INFO );
     	auth_info->put_desc( EDI_AUTHORIZATION_INFO, OM_S_TELETEX_STRING, 
              		     interchange->get_authorization_info() );
     	if ( interchange->get_authorization_info_qualifier()[ 0 ] )
		auth_info->put_desc( EDI_AUTHORIZATION_INFO_Q, 
			     OM_S_TELETEX_STRING, 
              		     interchange->get_authorization_info_qualifier() ); 
     	auth_info->end_object();

#ifdef	DEBUG
	cout << "leaving auth info" << endl;
#endif
}


void X400Message::build_recipients( APIObject  *recipients,
				    APIObject  *recip,
     				    APIObject  *edi_notification_requests,
     				    APIObject  *interchange_recipient,
     				    APIObject  *authorization_information,
     				    APIObject  *recipient_reference ) 
{
	OM_boolean	flg;
    
     	// 
     	// Build "recipients" public object
     	//
     	
	recipients->make_class( OMP_O_EDI_C_RECIPS );

     	recipients->put_desc( EDI_RECIP, recip->pub_object() );

     	recipients->put_desc( EDI_ACT_REQUEST, OM_S_OBJECT_IDENTIFIER_STRING,
              		      OMP_O_EDI_ARO_FOR_ACTION );

     	build_edi_notif_requests( edi_notification_requests );
     	recipients->put_desc( EDI_EDI_NOTIF_REQUESTS, 
              		      edi_notification_requests->pub_object() ); 

     	recipients->put_desc( EDI_RESPONS_PASSING_ALLOWED, OM_S_BOOLEAN, 
              		      tp_entry->resp_passing_allowed());

	if ( interchange->get_recipient_ID_code()[ 0 ] ) {
     		build_interchange_recipient( interchange_recipient );
     		recipients->put_desc( EDI_INTER_RECIP, 
              		      	      interchange_recipient->pub_object() );
	}
	if ( interchange->get_recipient_reference()[ 0 ] ) {
	     	build_recipient_ref( recipient_reference );
	     	recipients->put_desc( EDI_RECIP_REFERENCE, 
	              		      recipient_reference->pub_object() );
	}

     	if ( interchange->get_interchange_control_reference()[ 0 ] )
		recipients->put_desc( EDI_INTER_CONTROL_REFERENCE, 
			     OM_S_TELETEX_STRING,
              		     interchange->get_interchange_control_reference() );

	if ( interchange->get_process_priority_code()[ 0 ] )
	     	recipients->put_desc( EDI_PROCESSING_PRIORITY_CODE, 
				      OM_S_TELETEX_STRING,
              		      	      interchange->get_process_priority_code());

        flg = ( interchange->get_acknowledgement_request()[ 0 ] ) ?
		(OM_boolean) interchange->get_acknowledgement_request() : 
		OM_FALSE;
	recipients->put_desc( EDI_ACKNOWLEDGEMENT_REQUEST, OM_S_BOOLEAN, flg );

	if ( interchange->get_comm_agreement_ID()[ 0 ] )
	     	recipients->put_desc( EDI_COMMS_AGREEMENT_ID, 
				      OM_S_TELETEX_STRING, 
              		      	      interchange->get_comm_agreement_ID() );

	flg = ( interchange->get_test_indicator()[ 0 ] ) ?
		(OM_boolean) interchange->get_test_indicator() : OM_FALSE;
	recipients->put_desc( EDI_TEST_INDICATOR, OM_S_BOOLEAN, flg );

     	if ( interchange->get_authorization_info()[ 0 ] ) {
//		build_authorization_info( authorization_information);
//  		recipients->put_desc( EDI_AUTHORIZATION_INFO,
//             		      	      authorization_information->pub_object() );
	}

     	recipients->end_object();

#ifdef	DEBUG
	cout << "leaving recipients" << endl;
#endif
}


//
//  The EDIN receiver attribute specifies the O/R address of a user,
//  different from the originating user, to which EDI notifications are
//  to be sent.  If the local user is to receive the notifications, the 
//  EDIN receiver attribute is not added.  Note that only the O/R name
//  portion of the EDIN receiver field can be present on message origination.
//

void X400Message::build_edin_receiver( APIObject  *edin_receiver,
				       APIObject  *edin_recip ) {
 
	// 
    	// Build edin_receiver public object
     	//

     	edin_receiver->make_class( OMP_O_EDI_C_EDIN_RECEIVER );

     	// NOTE: Only need to get the EDIN Receiver Name (i.e., don't
    	// have to get the "Original EDIM Identifier" or "First Recipient")
     	// from the directory because they are not permitted on origination.

      	or_name( tp_entry->edin_receiver_name(), OMP_O_MH_C_OR_NAME, 
		 edin_recip );
      	edin_receiver->put_desc( EDI_EDIN_RECEIVER_NAME, 
              			 edin_recip->pub_object() );
       	edin_receiver->end_object();

#ifdef	DEBUG
	cout << "leaving edin receiver" << endl;
#endif
}


void X400Message::build_service_string_advice( 
					APIObject *service_string_advice ) {

#ifdef	DEBUG
	cout << "In build_service_string_advice()" << endl;
#endif

     	// 
     	// Build service string advice public object.
     	//
     
	service_string_advice->make_class( OMP_O_EDI_C_SERVICE_STRING_ADVICE );
     	service_string_advice->put_desc( EDI_COMPONENT_DATA_ELEMENT_SEP,
              		       OM_S_OCTET_STRING,
              		       interchange->get_comp_data_element_separator() );
     	service_string_advice->put_desc( EDI_DATA_ELEMENT_SEPARATOR,
              			   OM_S_OCTET_STRING,
              			   interchange->get_data_element_separator() );
     	service_string_advice->put_desc( EDI_DECIMAL_NOTATION,
              				 OM_S_OCTET_STRING,
              				 interchange->get_decimal_notation() );
     	if ( interchange->get_release_indicator()[ 0 ] )
		service_string_advice->put_desc( EDI_RELEASE_INDICATOR, 
					 OM_S_OCTET_STRING,
              				 interchange->get_release_indicator() );
     	if ( interchange->get_reserved()[ 0 ] ) 
		service_string_advice->put_desc( EDI_RESERVED,
              				 OM_S_OCTET_STRING,
              				 interchange->get_reserved() ); 
     	service_string_advice->put_desc( EDI_SEGMENT_TERMINATOR,
              				 OM_S_OCTET_STRING,
              				 interchange->get_segment_terminator());
     	service_string_advice->end_object();

#ifdef	DEBUG
	cout << "leaving service string advice" << endl;
#endif
}


void X400Message::build_syntax_identifier( APIObject *syntax_identifier ) {

     	// 
     	// Build this_EDIM public object
     	//
     	
	syntax_identifier->make_class( OMP_O_EDI_C_SYNTAX_ID );
     	syntax_identifier->put_desc( EDI_SYNTAX_ID_STRING, OM_S_TELETEX_STRING,
              			     interchange->get_syntax_id_string() );
     	syntax_identifier->put_desc( EDI_SYNTAX_VERSION, OM_S_PRINTABLE_STRING,
              			     interchange->get_syntax_version() );
     	syntax_identifier->end_object();
    
#ifdef	DEBUG
	cout << "leaving syntax identifier" << endl;
#endif
}


void X400Message::build_interchange_sender(APIObject  *interchange_sender) {

     	// 
     	// Build this_EDIM public object
     	//
     
	interchange_sender->make_class( OMP_O_EDI_C_INTER_SENDER_RECIP );
     	interchange_sender->put_desc( EDI_IDENTIFICATION_CODE, 
				      OM_S_TELETEX_STRING,
              			      interchange->get_sender_ID_code() );
        if ( interchange->get_sender_ID_qualifier()[ 0 ] )
     		interchange_sender->put_desc( EDI_IDENTIFICATION_CODE_Q,
              			      OM_S_TELETEX_STRING,
              			      interchange->get_sender_ID_qualifier() );
        if ( interchange->get_sender_routing_addr()[ 0 ] )
     	interchange_sender->put_desc( EDI_ROUTING_ADDRESS,
              			      OM_S_TELETEX_STRING,
              			      interchange->get_sender_routing_addr() );
     	interchange_sender->end_object();
    
#ifdef	DEBUG
	cout << "leaving interchange sender" << endl;
#endif
}


void X400Message::build_edi_heading( APIObject *header,
				     APIObject *originator,
				     APIObject *recipient, 
   				     APIObject *this_EDIM,
   				     APIObject *recipients,
   				     APIObject *edin_receiver,
   				     APIObject *edin_recip,
   				     APIObject *service_string_advice,
   				     APIObject *syntax_identifier,
   				     APIObject *interchange_sender, 
   				     APIObject *edi_notification_requests,
   				     APIObject *interchange_recipient,
   				     APIObject *authorization_information,
   				     APIObject *recipient_reference )
{
#ifdef	DEBUG
	cout << "In build_edi_heading()" << endl;
#endif
     	
	//
     	// Build EDI Heading public object.
     	//

     	header->make_class( OMP_O_EDI_C_HEADING );

     	build_this_EDIM( this_EDIM, originator );
     	header->put_desc( EDI_THIS_EDIM, this_EDIM->pub_object() );

     	header->put_desc( EDI_ORIG, originator->pub_object() );

     	build_recipients( recipients, recipient, edi_notification_requests,
			  interchange_recipient, authorization_information,
			  recipient_reference );
     	header->put_desc( EDI_RECIPS, recipients->pub_object() );

     	// Build the EDIN Receiver if one is given.
     	if ( tp_entry->edin_receiver_name() != NULL ) {   
        	build_edin_receiver( edin_receiver, edin_recip );
		header->put_desc( EDI_EDIN_RECEIVER, 
				  edin_receiver->pub_object() );
     	}

     	header->put_desc( EDI_RESPONS_FORWARD, OM_S_BOOLEAN, OM_FALSE);

     	header->put_desc( EDI_EDI_BODYPART_TYPE, OM_S_OBJECT_IDENTIFIER_STRING,
              		  interchange->get_bodypart_type() );

     	header->put_desc( EDI_INC_COPY, OM_S_BOOLEAN, OM_FALSE);

//
// NOTE: Many of the following heading fields which are commented out are
//       for either future use or not supported in this implementation.
//

//  Not used in this implementation.
//     header->put_desc( // Expiry Time
//              EDI_EXPIRY_TIME, 
//              OM_S_UTC_TIME_STRING,
//              ??????? );

//  Not used in this implementation.
//     header->put_desc( // Related EDI Messages
//              EDI_RELATED_MSGS, 
//              OM_S_OBJECT, 
//              Related_EDI_Messages());

//  Not used in this implementation.
//     header->put_desc( // Related non-EDI Messages
//              EDI_RELATED_NON_EDI_MSGS, 
//              OM_S_OBJECT, 
//              Related_non_EDI_Messages());

//  Not used on outbound messages.
//     header->put_desc( // Obsoleted EDIMs
//              EDI_OBSOLETED_EDIMS, 
//              OM_S_OBJECT, 
//              Obsoleted_EDIMs());

//  Not supported in this implementation. (future use)
//     header->put_desc( // EDI Application Security Elements
//              EDI_APP_SECUR_ELEMENTS, 
//              OM_S_OBJECT, 
//              EDI_appl_security_elements());

//     header->put_desc( // Cross Referencing Information
//              OMP_O_EDI_CROSS_REFERENCING_INFO, 
//              OM_S_OBJECT,
//              Cross_ref_info());

//	header->put_desc( EDI_EDI_MSG_TYPE, OM_S_TELETEX_STRING,
//              		  interchange->get_edi_message_type() );

     	build_service_string_advice( service_string_advice );
     	header->put_desc( EDI_SERVICE_STRING_ADVICE, 
              		  service_string_advice->pub_object() );

//     	build_syntax_identifier( syntax_identifier );
//	header->put_desc( EDI_SYNTAX_ID, syntax_identifier->pub_object() );

	if ( interchange->get_sender_ID_code()[ 0 ] ) {
     		build_interchange_sender( interchange_sender );
     		header->put_desc( EDI_INTER_SENDER, 
				  interchange_sender->pub_object() );
	}

//     	if ( interchange->get_date_preparation()[ 0 ] )
//		header->put_desc( EDI_DATE_AND_TIME_OF_PREP, 
//				  OM_S_UTC_TIME_STRING, 
//              		  	  build_prep_date_and_time( interchange ) );

	if ( interchange->get_application_ref()[ 0 ] )
	     	header->put_desc( EDI_APP_REFERENCE, OM_S_TELETEX_STRING,
	              		  interchange->get_application_ref() );

//  Not used in this implementation.
//     header->put_desc( // Heading Extensions
//              EDI_HEADING_EXTENSIONS, 
//              OM_S_OBJECT, 
//              heading_extensions());

     	header->end_object();

#ifdef	DEBUG
	cout << "leaving heading" << endl;
#endif
}

void X400Message::build_primary_bodypart(APIObject *bodypart)  {

#ifdef	DEBUG
	cout << "In build_primary_bodypart()" << endl;
#endif
     
     	//
     	// Build EDI Bodypart public object.
     	//

     	bodypart->make_class( OMP_O_EDI_C_EDI_BODYPART );
     	bodypart->put_desc( EDI_BODY,OM_S_OCTET_STRING,interchange->content() );
     	bodypart->end_object();

#ifdef	DEBUG
	cout << "leaving bodypart" << endl;
#endif
}


void X400Message::build_edi_content( APIObject *content,
				     APIObject *originator,
				     APIObject *recipient, 
   				     APIObject *edi_notification_requests,
				     APIObject *interchange_recipient,
				     APIObject *authorization_information,
   				     APIObject *recipient_reference, 
				     APIObject *this_EDIM,
				     APIObject *recipients, 
				     APIObject *edin_receiver,
   				     APIObject *edin_recip,
				     APIObject *service_string_advice,
				     APIObject *syntax_identifier, 
				     APIObject *interchange_sender,
				     APIObject *header,
   				     APIObject *bodypart )
{
#ifdef	DEBUG
	cout << "In build_edi_content()" << endl;
#endif

	//
     	// Build EDI content public object.
     	//

     	content->make_class( OMP_O_EDI_C_EDIM );

     	build_edi_heading( header, originator, recipient, this_EDIM, recipients,
			   edin_receiver, edin_recip, service_string_advice,
			   syntax_identifier, interchange_sender,
			   edi_notification_requests, interchange_recipient,
			   authorization_information, recipient_reference );
     	content->put_desc( EDI_HEADING, header->pub_object() );

     	build_primary_bodypart(bodypart);
     	content->put_desc( EDI_PRIMARY_BODYPART, bodypart->pub_object() );
     	content->end_object();

#ifdef	DEBUG
	cout << "leaving content" << endl;
#endif
}
