// FILE:  x400in.C

#include	<fstream.h>
#include	<sys/types.h>
#include	<sys/unistd.h>
#include	<dirent.h>
#include	<xom.h>
#include	<assert.h>
#include	<xedip.h>
extern "C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/subject.h"
#include	"../../../include/interchange.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/edi_user.h"
#include	"../../../include/config.h"
#include	"../../../include/platfile.h"
#include	"../../../include/tpfile.h"
#include	"../../../include/api_obj.h"
#include	"../../../include/x400api.h"

//#define	DR	1

//
// Transfer in objects from the API input queue, if available.
//

STATUS X400Message::receive( STATUS *status_arg, Config *config ) {

	int	count;
	STATUS	status = SUCCESS;
	Queue	*users;
	EDIUser	*user;

#ifdef	DEBUG
	cout << "In receive()" << endl;
#endif
 
	users = config->edi_users();
	users->set_search();
	user = ( EDIUser * ) users->current();
	
	while ( user ) {

		if ( user->use_x400() ) {

			set_user( user );
			
			// Open a session with the API service.
			if ( ! OK( x400_open() ) )
				return ERROR;
	
			// Test if any messages are in the API input queue.
			if ( ! OK( x400_size( &count ) ) )
				return ERROR;

#ifdef	DEBUG
			cout << count << " messages in input queue" << endl;
#endif
	
			// Process all the messages waiting.
			if ( count ) {
				*status_arg |= ACTIVITY_OCCURRED;
				for ( int i=0; i<count; i++ )
					status |= 
					    parse_inbound_mhs_object( config );
			}
	
			// If no activity has occurred, use the API wait 
			// function to temporarily suspend execution.
			if ( ! ACTIVITY( *status_arg ) )
				status |= x400_wait();

			// Close the session with the API service.
			status |= x400_close();
		} // if

		user = ( EDIUser * ) users->current();

	} // while

#ifdef	DEBUG
	cout << "X.400 receive status = " << status << endl;
#endif
	return status;
}	


//
// Determine the type of object received (Message vs Report) and parse the
// object.
//

STATUS X400Message::parse_inbound_mhs_object( Config *config ) {

	APIObject	p1_object( workspace, log );
	static OM_type	types[ 2 ] = { OM_CLASS, OM_NO_MORE_TYPES };
	OM_exclusions	exclusions = OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES;
	STATUS		status = SUCCESS;

	*log_file << endl << "**************************************************************" << endl << endl;

	//
	// Read an object from the API input queue.
	//
	
	status |= x400_start_transfer_in( &p1_object );
	if ( ! OK( status ) )
		return ERROR;
		
	//
	// First, get the class to determine whether the inbound object
	// is a message or a report.
	//

	status |= p1_object.api_get( exclusions, types );
	if ( ! OK( status ) )
		return ERROR;
		
	if ( p1_object.is_class( OMP_O_MH_C_MESSAGE ) ) {
		p1_object.api_delete();
#ifdef	DEBUG
		cout << "Inbound Message" << endl;
		log->log_event( "parse_inbound_mhs_object", 
				"INBOUND MESSAGE IN QUEUE",
				INFORMATION );
#endif
		status |= parse_edi_object( &p1_object, config );
	}
	else if ( p1_object.is_class( OMP_O_MH_C_REPORT ) ) {
		p1_object.api_delete();
#ifdef	DEBUG
		cout << "Inbound Report" << endl;
		log->log_event( "parse_inbound_mhs_object", 
				"INBOUND REPORT IN QUEUE",
				INFORMATION );
#endif
		status |= parse_mts_report( &p1_object );
	}
	else
		log->log_event( "parse_inbound_mhs_object", 
				ERR_NOT_EXPECTED_OBJECT,
				INFORMATION );

	status |= x400_finish_transfer_in( &p1_object );

#ifdef	DR
subject_message.set_reason_code(MH_RE_TRANSFER_FAILED);
subject_message.set_diagnostic( MH_DG_OR_NAME_UNRECOGNIZED );
subject_message.set_report_request( MH_RQ_ALWAYS );
#endif

	if ( subject_message.report_request() >= MH_RQ_ALWAYS &&
	     subject_message.reason_code() == NO_REASON ) {
		status |= x400_close();
		build_mts_report( DELIVERY_REPORT );
	}
	else if ( subject_message.report_request() >= MH_RQ_NON_DELIVERY &&
		  subject_message.reason_code() == MH_RE_TRANSFER_FAILED ) {
		status |= x400_close();
		status |= build_mts_report( NONDELIVERY_REPORT,
				  	    subject_message.reason_code(),
				  	    subject_message.diagnostic() );
	}
	if ( subject_message.edi_pos_notif() &&
	     subject_message.reason_code() == NO_REASON ) {
		status |= x400_close();
		status |= build_pn();
	}
	else if ( subject_message.edi_neg_notif() &&
		  subject_message.reason_code() != NO_REASON ) {
		status |= x400_close();
		status |= build_nn();
	}

#ifdef	DEBUG
	cout << "Finished Inbound Object" << endl;
	log->log_event( "parse_inbound_mhs_object", 
			"FINISHED INBOUND OBJECT IN QUEUE",
			INFORMATION );
#endif

	return status;
}


//
// Parse an inbound MTS report.
//

STATUS X400Message::parse_mts_report( APIObject	*p1_object ) {

	STATUS		status = SUCCESS;
//	OM_exclusions	exclusions = OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES;
	OM_exclusions	exclusions = OM_NO_EXCLUSIONS;
	static OM_type	types[ 13 ] = 
	{ MH_T_CONTENT_TYPE, MH_T_ORIGINAL_EITS, 
	  MH_T_SUBJECT_MTS_IDENTIFIER, MH_T_EXTERNAL_TRACE_INFO, 
	  MH_T_INTERNAL_TRACE_INFO, MH_T_MTS_IDENTIFIER, 
	  /*MH_T_PER_RECIPIENT_REPORTS,*/ MH_T_REPORT_ADDITIONAL_INFO, 
	  MH_T_REPORT_DESTINATION, MH_T_SUBJECT_EXT_TRACE_INFO,
	  OM_NO_MORE_TYPES };
	
//	status |= p1_object->api_get( exclusions, types );
	status |= p1_object->api_get( exclusions );
	if ( ! OK( status ) ) 
		return ERROR;
		
	log_object( p1_object, LOG_SUBOBJECTS );
	p1_object->api_delete();

	return SUCCESS;
}


//
// Parse an inbound EDI (X.435 or P2) message.
//

STATUS X400Message::parse_edi_object( APIObject	*p1_object, Config *config ) {

	STATUS		status = SUCCESS;
	APIObject	content( p1_object );
	APIObject	originator( p1_object );
	ORName		orig_name;
	OM_value	value;
	ORName		rcpt_name;
	EDIUser		*local_edi_user = NULL;
	OM_exclusions	exclusions = OM_EXCLUDE_ALL_BUT_CERTAIN_TYPES;
	static OM_type	types[ 20 ] = 
	{ OM_CLASS, MH_T_EXTENSIONS, MH_T_CONTENT, MH_T_CONTENT_CORRELATOR,
	  MH_T_EXTERNAL_TRACE_INFO, MH_T_INTERNAL_TRACE_INFO,
	  MH_T_RECIPIENT_DESCRIPTORS, MH_T_CONFIDENTIALITY_ALGORITHM, 
	  MH_T_CONTENT_IDENTIFIER, MH_T_CONTENT_TYPE, MH_T_EXPANSION_HISTORY, 
	  MH_T_MTS_IDENTIFIER, MH_T_ORIGIN_CHECK, MH_T_ORIGINATOR_CERTIFICATE,
	  MH_T_ORIGINATOR_NAME, MH_T_PRIORITY, MH_T_SECURITY_LABEL, 
	  MH_T_ORIGINAL_EITS, OM_NO_MORE_TYPES };
	
#ifdef	DEBUG
	cout << "In parse_edi_object()" << endl;
#endif

	//
	// Get all requested types.
	//

	status |= p1_object->api_get( exclusions, types );
	if ( ! OK( status ) )
		return status;

	log_object( p1_object, LOG_SUBOBJECTS );

	while ( p1_object->more_types() ) {

#ifdef	DEBUG
		cout << "type = " << p1_object->type() << endl;
#endif
		value = p1_object->value();

		switch ( p1_object->type() ) {
	
			//
			// From the Extensible Object class.
			//
			case MH_T_EXTENSIONS:
				if ( parse_extension( value.object.object )
					== ERROR ) {
					subject_message.set_reason_code( 
					    MH_RE_TRANSFER_FAILED );
					subject_message.set_diagnostic(
					    MH_DG_CRITICAL_FUNC_UNSUPPORTED );
					status |= MESSAGE_NOT_ACCEPTED;
				}
				break;

			//
			// From the Communique class.
			//
			case MH_T_CONTENT_CORRELATOR:
				subject_message.set_content_correlator( 
					value.object.object );
				break;
			case MH_T_CONTENT_IDENTIFIER:
				subject_message.set_content_identifier( 
					value.string.elements );
				break;
			case MH_T_CONTENT_TYPE:
				subject_message.set_content_type(
					value.integer );
				// Confirm content type is supported.
				if ( value.integer != EDI_CONTENT_TYPE &&
				     value.integer != MH_CTI_P2_1984 &&
				     value.integer != MH_CTI_P2_1988 ) {
					subject_message.set_reason_code( 
					    MH_RE_TRANSFER_FAILED );
					subject_message.set_diagnostic(
					    MH_DG_CONTENT_TYPE_UNSUPPORTED );
					status |= MESSAGE_NOT_ACCEPTED;
				}
				break;
			case MH_T_EXPANSION_HISTORY:
				parse_expansion_history( value.object.object );
				break;
			case MH_T_EXTERNAL_TRACE_INFO:
				parse_external_trace( value.object.object );
				break;
			case MH_T_MTS_IDENTIFIER:
				subject_message.set_mts_identifier( 
					value.object.object );
				break;
			case MH_T_ORIGINATOR_NAME:
				subject_message.set_originator( 
					value.object.object );
				break;
			case MH_T_RECIPIENT_DESCRIPTORS:
				// Only parse the RD if the correct local user
				// has not not yet been identified.
				if ( local_edi_user == NULL )
					local_edi_user = parse_rd( 
							  value.object.object,
							  &rcpt_name, config );
				break;
			//
			// From the Message Class.
			//
			case MH_T_CONTENT:
				content.set_api_public( value.object.object );
				break;
			case MH_T_PRIORITY:
				subject_message.set_priority( value.integer );
				break;

			case MH_T_CONFIDENTIALITY_ALGORITHM:
			case MH_T_ORIGIN_CHECK:
			case MH_T_ORIGINATOR_CERTIFICATE:
			case MH_T_SECURITY_LABEL:
				// Secure messaging is not supported currently.
				subject_message.set_reason_code( 
					    MH_RE_TRANSFER_FAILED );
				subject_message.set_diagnostic(
					    MH_DG_SECURE_MESSAGING_ERROR );
				status |= MESSAGE_NOT_ACCEPTED;
				break;
		}

		p1_object->get_next_type();
	}

	//
	// If required, validate the trading partner based on the originator's
	// X.400 address.
	//

	if ( local_edi_user->test_sender() ) {
		originator.set_api_public( subject_message.originator() );
		public_to_or_name( &originator, &orig_name );
		if ( ! tp_entry->validate_partner( &orig_name ) ) {
			subject_message.set_reason_code(MH_RE_TRANSFER_FAILED);
			subject_message.set_diagnostic(
						MH_DG_NO_BILATERAL_AGREEMENT ); 
			status |= MESSAGE_NOT_ACCEPTED;
		}
	}
	
	if ( status & MESSAGE_NOT_ACCEPTED )
		return status;

	if ( subject_message.mta_responsibility() != OM_TRUE )
		// Simply discard this message
		return status;
	if ( subject_message.content_type() == EDI_CONTENT_TYPE ) 
		status |= parse_edi_message(&content,local_edi_user);
	else {
#ifdef	DEBUG
		cout << "LOCAL USER: " << local_edi_user->id_code()
		     << local_edi_user->id_code_qualifier() << endl;
#endif
		status |= parse_ipm_content( &content, local_edi_user );
	}

	p1_object->api_delete();

#ifdef	DEBUG
	cout << "status = " << status << endl;
#endif
	
	return status;
}
			

//
// If an extension is marked CRITICAL_FOR_DELIVERY, return an error since 
// the EDI-UA does not support any extensions.  All other extension 
// attributes are ignored.
//

STATUS X400Message::parse_extension( OM_public_object pub ) {

	APIObject	extension( workspace, log );

	extension.set_api_public( pub );
	while ( extension.more_types() ) {
		if ( extension.type() == MH_T_CRITICAL_FOR_DELIVERY &&
		     extension.value().boolean == OM_TRUE )
			return ERROR;
		extension.get_next_type();
	}
	return SUCCESS;
}			 


//
// Parse the last O/R Name from the expansion history.  The O/R Name, if 
// present, is used as the destination for a report, if one need be generated.
//

void X400Message::parse_expansion_history( OM_public_object pub ) {

	APIObject	expansion_history( workspace, log );

	expansion_history.set_api_public( pub );	

	while ( expansion_history.more_types() ) {
		if ( expansion_history.type() == MH_T_NAME )
			subject_message.set_last_expansion( 
				expansion_history.value().object.object );
		expansion_history.get_next_type();
	}
}			 


//
// Save the external trace object, and parse the arrival time from it.
//

void X400Message::parse_external_trace( OM_public_object pub ) {

	APIObject	ext_trace( workspace, log );

	ext_trace.set_api_public( pub );	
	subject_message.set_external_trace_info( pub );

	while ( ext_trace.more_types() ) {
		if ( ext_trace.type() == MH_T_ARRIVAL_TIME ) {
			subject_message.set_arrival_time( 
				ext_trace.value().string.elements );
			break;
		}
		ext_trace.get_next_type();
	}
}			 


//
// Save the redirection history object, and parse the intended recipient name.
//

void X400Message::parse_redirection_history( OM_public_object pub ) {

	APIObject	redir( workspace, log );

	redir.set_api_public( pub );	
	subject_message.set_redirection_history( pub );

	while ( redir.more_types() ) {
		if ( redir.type() == MH_T_INTENDED_RECIPIENT_NAME ) {
			subject_message.set_intended_recipient( 
				redir.value().object.object );
			break;
		}
		redir.get_next_type();
	}
}			 


//
// To determine if this RD is for the local user, 2 criteria must be met:
// 	1. The O/R address of the recipient must match the local user's.
//	2. The local user's id_code and id_code_qualifier must match the
//	   client name for this API session.
//

EDIUser *X400Message::parse_rd( OM_public_object 	pub, 
			        ORName 			*rcpt_name,
			    	Config			*config ) {

	APIObject	rd( workspace, log );
	APIObject	recipient( workspace, log );
	EDIUser		*local_user = NULL; 
	char		client_name[ MAX_CLIENT ];
	OM_value	value;	

#ifdef	DEBUG
	cout << "In parse_rd()" << endl;
#endif

	rd.set_api_public( pub );

	while ( rd.more_types() ) {

		value = rd.value();

#ifdef	DEBUG
		cout << "type = " << rd.type() << endl;
#endif

		switch ( rd.type() ) {

			case MH_T_RECIPIENT_NAME:
				recipient.set_api_public( (OM_public_object )
							value.object.object );
				// Find local user with matching O/R name.
				public_to_or_name( &recipient, rcpt_name );
				local_user = config->get_local_user(rcpt_name);
				if ( local_user == NULL ) 
					return NULL;
				strcpy( client_name, local_user->id_code() );
				strcat( client_name, 
					local_user->id_code_qualifier() );
				// Did this local user initiate the API session?
				if ( strcmp( client, client_name ) )
					return NULL;
				subject_message.set_actual_recipient( 
					value.object.object );
				break;
			case MH_T_RECIPIENT_NUMBER:
				subject_message.set_recipient_number( 
					value.integer );
				break;
			case MH_T_MTA_REPORT_REQUEST:
				subject_message.set_report_request( 
					value.enumeration );
				break;
			case MH_T_REDIRECTION_HISTORY:
				parse_redirection_history(value.object.object);
				break;
			case MH_T_MTA_RESPONSIBILITY:
				subject_message.set_mta_responsibility(
					value.boolean );
				break;
		}

		rd.get_next_type();
	}

	return local_user;
}
