// FILE:  x400log.C

#include	<fstream.h>
#include	<sys/types.h>
#include	<sys/unistd.h>
#include	<dirent.h>
#include	<xom.h>
#include	<assert.h>
#include	<xedip.h>
extern "C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"/usr/lib/x400/gtwapi/include/ximp.h"
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/subject.h"
#include	"../../../include/interchange.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/edi_user.h"
#include	"../../../include/config.h"
#include	"../../../include/platfile.h"
#include	"../../../include/tpfile.h"
#include	"../../../include/api_obj.h"
#include	"../../../include/x400api.h"


#define	LOG_BOOLEAN(b)	(b==OM_TRUE) ? 				\
				*log_file<<"TRUE"<<endl : 	\
				*log_file<<"FALSE"<<endl

//
// PUBLIC:  log an OM public object.  
//

void X400Message::log_object( APIObject	*object,
			      int 	subflag ) {

	OM_descriptor	*desc = object->pub_object();
	
	while ( desc->type != OM_NO_MORE_TYPES ) {
		if ( desc->type == OM_CLASS )
			log_class( desc, "", subflag );
		desc++;
	}
}


//
// PRIVATE:  log the class of a public object.  No abstract classes are 
// listed in this function.
//

void X400Message::log_class( OM_descriptor 	*desc,
			     char		*prefix,
			     int		subflag ) {

	char	*class_oid =  ( char * ) desc->value.string.elements;
	int	last_digits = class_oid[ 6 ];

	if ( ! strncmp( class_oid, "\x56\x06\x01\x02\x05\x09", 6 ) ) {

		//
		// These are for IP Messages.
		//
		
		if ( log_level >= BASIC_LOG && last_digits == 0x06 ) 
			log_attributes( desc, prefix, "IPM MESSAGE", subflag );
		
		if ( log_level >= TRACE_LOG ) {

			if ( last_digits == 0x08 )
				log_attributes( desc, prefix, 
						"IPM IDENTIFIER", subflag );
			else if ( last_digits == 0x05 )
				log_attributes( desc, prefix, 
						"IA5 BODYPART", subflag );
			else if ( last_digits == 0x0E )
				log_attributes( desc, prefix, 
						"O/R DESCRIPTOR", subflag );
			else if ( last_digits == 0x10 ) 
				log_attributes( desc, prefix, 
						"RECIPIENT SPECIFIER",subflag );
		}	
		return;
	}
	
	if ( ! strncmp( class_oid, "\x56\x06\x01\x02\x05\x1E", 6 ) ) {

		//
		// These are for EDI Messages.
		//
		
		if ( log_level >= BASIC_LOG ) {

			switch ( last_digits ) {

				case 0x06:
					log_attributes( desc, prefix,
						"EDI MESSAGE", subflag );
					break;
				case 0x09:
					log_attributes( desc, prefix,
						"EDIM ID", subflag );
					break;
				case 0x10:
					log_attributes( desc, prefix,
						"FORWARDED NOTIF", subflag );
					break;
				case 0x12:
					log_attributes( desc, prefix,
						"EDI HEADING", subflag );
					break;
				case 0x13:
					log_attributes( desc, prefix,
						"INTERCHANGE SENDER/RECIP", 
						subflag );
					break;
				case 0x14:
					log_attributes( desc, prefix,
						"NEGATIVE NOTIF", subflag );
					break;
				case 0x19:
					log_attributes( desc, prefix,
						"POSITIVE NOTIF", subflag );
					break;
				case 0x1D:
					log_attributes( desc, prefix,
						"RECIPIENTS", subflag );
					break;
			}
		}
		if ( log_level >= TRACE_LOG ) {
	
			switch ( last_digits ) {

				case 0x05:
					log_attributes( desc, prefix,
						"EDI NOTIF REQUESTS", subflag );
					break;
				case 0x0A:
					log_attributes( desc, prefix,
						"EDIN RECEIVER", subflag );
					break;
				case 0x0C:
					log_attributes( desc, prefix,
						"FN PDAU REASON CODE",subflag );
					break;
				case 0x0E:
					log_attributes( desc, prefix,
						"FN UAMS REASON CODE",subflag );
					break;
				case 0x0F:
					log_attributes( desc, prefix,
						"FN USER REASON CODE",subflag );
					break;
				case 0x15:
					log_attributes( desc, prefix,
						"NN PDAU REASON CODE",subflag );
					break;
				case 0x17:
					log_attributes( desc, prefix,
						"NN UAMS REASON CODE",subflag );
					break;
				case 0x18:
					log_attributes( desc, prefix,
						"NN USER REASON CODE",subflag );
					break;
			}	
		}
		if ( log_level >= INTERCHANGE_LOG ) {
	
			switch ( last_digits ) {

				case 0x00:
					log_attributes( desc, prefix,
						"AUTHORIZATION INFO", subflag );
					break;
				case 0x1C:
					log_attributes( desc, prefix,
						"RECIPIENT REFERENCE",subflag );
					break;
				case 0x1F:
					log_attributes( desc, prefix,
						"SERVICE STRING ADV", subflag );
					break;
				case 0x20:
					log_attributes( desc, prefix,
						"SYNTAX ID", subflag );
					break;
			}	
		}
		if ( log_level >= VERBOSE_LOG ) {
	
			switch ( last_digits ) {

				case 0x01:
					log_attributes( desc, prefix,
						"CROSS REF INFO", subflag );
					break;
				case 0x02:
					log_attributes( desc, prefix,
						"EDI APP SECURITY", subflag );
					break;
				case 0x03:
					log_attributes( desc, prefix,
						"EDI BODYPART", subflag );
					break;
				case 0x0B:
					log_attributes( desc, prefix,
						"EDI EXTENSIONS", subflag );
					break;
				case 0x1E:
					log_attributes( desc, prefix,
						"SECURITY ELEMENTS", subflag );
					break;
			}
		}
	
		return;
	}	
	
	if ( log_level >= BASIC_LOG ) {

		switch ( last_digits ) {
			
			case 0x0C:
				log_attributes( desc, prefix, 
						"DELIVERY ENVELOPE", subflag );
				break;
			case 0x0F:
				log_attributes( desc, prefix, 
						"EXTENSIBLE OBJECT", subflag );
				break;
			case 0x19:
				log_attributes( desc, prefix, "MESSAGE", 
						subflag );
				break;
			case 0x1A:
				log_attributes( desc, prefix, "MESSAGE RD", 
						subflag );
				break;
			case 0x1B:
				log_attributes( desc, prefix, "MTS IDENTIFIER", 
						subflag );
				break;
			case 0x1C:
				log_attributes( desc, prefix, "O/R ADDRESS", 
						subflag );
				break;
			case 0x1D:
				log_attributes( desc, prefix, "O/R NAME", 
						subflag );
				break;
			case 0x25:
				log_attributes( desc, prefix, "REPORT", 
						subflag );
				break;
			case 0x2B:
				log_attributes( desc, prefix, 
						"SUBMITTED MESSAGE RD",subflag);
				break;
			case 0x2D:
				log_attributes( desc, prefix, 
						"SUBMITTED PROBE RD", subflag );
				break;
		}
	}
	
	if ( log_level >= TRACE_LOG ) {

		switch ( last_digits ) {
	
			case 0x17:
				log_attributes( desc, prefix, "LOCAL NDR", 
						subflag );
				break;
			case 0x18:
				log_attributes( desc, prefix, 
						"LOCAL PER RECIP NDR",subflag );
				break;
			case 0x0E:
				log_attributes( desc, prefix, 
						"EXPANSION RECORD", subflag );
				break;
			case 0x11:
				log_attributes( desc, prefix, 
						"EXTERNAL TRACE ENTRY",subflag);
				break;
			case 0x14:
				log_attributes( desc, prefix, 
						"INTERNAL TRACE ENTRY",subflag);
				break;
			case 0x1E:
				log_attributes( desc, prefix, "PER RECIP DR", 
						subflag );
				break;
			case 0x1F:
				log_attributes( desc, prefix, "PER RECIP NDR", 
						subflag );
				break;
			case 0x24:
				log_attributes( desc, prefix, 
						"REDIRECTION RECORD", subflag );
				break;
		}
	}
	
	if ( log_level >= VERBOSE_LOG ) {

		switch ( last_digits ) {
	
			case 0x03:
				log_attributes( desc, prefix, "BILATERAL INFO", 
						subflag );
				break;
			case 0x0D:
				log_attributes( desc, prefix, "EITS", subflag );
				break;
			case 0x10:
				log_attributes( desc, prefix, "EXTENSION", 
						subflag );
				break;
			case 0x27:
				log_attributes( desc, prefix, "SESSION", 
						subflag );
				break;
		}
	}
	
	*log_file << endl;

	return;
}


//
// PRIVATE:  Parse the OM_descriptor.  This function access the OM_object
// directly, since values must be extracted.
//

void X400Message::log_desc( OM_descriptor *desc, char *prefix ) {

	OM_value	value = desc->value;
	if ( desc->syntax != OM_S_BOOLEAN && desc->syntax != OM_S_INTEGER &&
	     desc->syntax != OM_S_ENUMERATION && desc->syntax != OM_S_OBJECT )
		// The SYNTAX is some type of string.
		TERMINATE( value.string );
	*log_file << prefix;

	if ( desc->type == OM_CLASS && 
	     desc->syntax == OM_S_OBJECT_IDENTIFIER_STRING )
		return;

	switch ( desc->type ) {

#ifdef	POSTAL
		case MH_T_FORWARDING_ADDRESS:
			log_desc( value.object.object );
			break;
		case MH_T_ORIGINATOR_RETURN_ADDRESS:
			log_desc( value.object.object );
			break;
		case MH_T_POSTAL_ADDRESS_DETAILS:
			*log_file << "POSTAL ADDRESS DETAILS:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_ADDRESS_IN_FULL:
			*log_file << "POSTAL ADDRESS IN FULL:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_ADDRESS_IN_LINES:
			*log_file << "POSTAL ADDRESS IN LINES:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_CODE:
			*log_file << "POSTAL CODE:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_COUNTRY_NAME:
			*log_file << "POSTAL COUNTRY NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_DELIVERY_POINT_NAME:
			*log_file << "POSTAL DELIVERY POINT NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_DELIV_SYSTEM_NAME:
			*log_file << "POSTAL DELIVERY SYSTEM NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_GENERAL_DELIV_ADDR:
			*log_file << "POSTAL GENERAL DELIVERY ADDRESS:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_LOCALE:
			*log_file << "POSTAL LOCALE:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_MODE:
			*log_file << "POSTAL MODE:  ";
			log_postal_mode( value.enumeration );
			break;
		case MH_T_POSTAL_OFFICE_BOX_NUMBER:
			*log_file << "POSTAL OFFICE BOX NUMBER:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_OFFICE_NAME:
			*log_file << "POSTAL OFFICE NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_OFFICE_NUMBER:
			*log_file << "POSTAL OFFICE NUMBER:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_ORGANIZATION_NAME:
			*log_file << "POSTAL ORGANIZATION NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_PATRON_DETAILS:
			*log_file << "POSTAL PATRON DETAILS:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_PATRON_NAME:
			*log_file << "POSTAL PATRON NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_POSTAL_REPORT:
			*log_file << "POSTAL REPORT:  ";
			log_postal_report( value.enumeration );
			break;
		case MH_T_POSTAL_STREET_ADDRESS:
			*log_file << "POSTAL STREET ADDRESS:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_RECIP_NUMBER_FOR_ADVICE:
			*log_file << "RECIPIENT NUMBER FOR ADVICE:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_REGISTRATION:
			*log_file << "REGISTRATION:  ";
			log_registration( value.enumeration );
			break;
		case MH_T_RENDITION_ATTRIBUTES:
			*log_file << "RENDITION ATTRIBUTES:  "
			log_rendition_attributes( ( char * ) value.string.elements );
			break;
#endif	// POSTAL

#ifdef	PROBE
		case MH_T_CONTENT_LENGTH:
			*log_file << "CONTENT LENGTH:  ";
			*log_file << value.integer << endl;
			break;
#endif	// PROBE

#ifdef	MA_INTERFACE
		case MH_T_DELIVERY_CONFIRMS:
			log_desc( value.object.object );
			break;
		case MH_T_ENVELOPES:
			log_desc( value.object.object );
			break;
		case MH_T_TEMPORARY:
			*log_file << "TEMPORARY:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
#endif	// MA_INTERFACE

		//
		// Only check log level for select types.  The remaining
		// should be logged for BASIC_LOG, or are unique to a class
		// for which logging would have already been tested.
		//
		case EDI_AUTHORIZATION_INFO:		// Authorization Info
			*log_file << "EDI AUTHORIZATION INFO:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_AUTHORIZATION_INFO_Q:		// Authorization Info
			*log_file << "EDI AUTHORIZATION INFO QUAL:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_APP_CROSS_REFERENCE:		// Cross Ref
			*log_file << "EDI APP CROSS REF:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_MSG_REFERENCE:			// Cross Ref
			*log_file << "EDI MSG REFERENCE" << endl;
			break;
		case EDI_BODYPART_REFERENCE:		// Cross Ref
			*log_file << "EDI BODYPART REFERENCE:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_APP_SECUR_ELEMENT:		// EDI App Secur Ele
			*log_file << "EDI APP SECUR ELEMENT:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_ENCRYPTED_PRIMARY_BODYPART:   // EDI App Secur Ele
			*log_file << "EDI ENCRYPTED PRIMARY BODYPART:  ";
			LOG_BOOLEAN( value.boolean );
			break;
		case EDI_APP_SECUR_EXTENSIONS:		// EDI App Secur Ele
			*log_file << "EDI APP SECUR EXTENSIONS" << endl;
			break;
		case EDI_BODY:				// EDI Body Part
			*log_file << "EDI BODY:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_THIS_EDIM:			// Heading
			*log_file << "THIS EDIM" << endl;
			break;
		case EDI_ORIG:				// Heading
			*log_file << "ORIGINATOR" << endl;
			break;
		case EDI_RECIPS:			// Heading
			*log_file << "RECIPIENT INFO" << endl;
			break;
		case EDI_EDIN_RECEIVER:			// Heading
			*log_file << "EDIN RECEIVER" << endl;
			break;
		case EDI_RESPONS_FORWARD:		// Heading
			*log_file << "EDI RESPONSIBILITY FORWARDED:  ";
			LOG_BOOLEAN( value.boolean );
			break;
		case EDI_EDI_BODYPART_TYPE:		// Heading
			*log_file << "EDI BODYPART TYPE:  ";
			log_bodypart_type( ( char * ) value.string.elements );
			break;
		case EDI_INC_COPY:			// Heading
			*log_file << "INCOMPLETE COPY:  ";
			LOG_BOOLEAN( value.boolean );
			break;
		case EDI_EXPIRY_TIME:			// Heading
			*log_file << "EDI EXPIRY TIME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_RELATED_EDI_MSGS:		// Heading
			*log_file << "RELATED EDI MESSAGES" << endl;
			break;
		case EDI_RELATED_NON_EDI_MSGS:		// Heading
			*log_file << "RELATED NON EDI MESSAGES" << endl;
			break;
		case EDI_OBSOLETED_EDIMS:		// Heading
			*log_file << "OBSOLETED EDIMS" << endl;
			break;
		case EDI_APP_SECUR_ELEMENTS:		// Heading
			*log_file << "EDI APP SECURITY ELEMENTS" << endl;
			break;
		case EDI_CROSS_REFERENCING_INFO:	// Heading
			*log_file << "CROSS REFERENCING INFO" << endl;
			break;
		case EDI_EDI_MSG_TYPE:			// Heading
			*log_file << "EDI MESSAGE TYPE:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_SERVICE_STRING_ADVICE:		// Heading
			*log_file << "SERVICE STRING ADVICE" << endl;
			break;
		case EDI_SYNTAX_ID:			// Heading
			*log_file << "SYNTAX IDENTIFIER" << endl;
			break;
		case EDI_INTER_SENDER:			// Heading
			*log_file << "INTERCHANGE SENDER" << endl;
			break;
		case EDI_DATE_AND_TIME_OF_PREP:		// Heading
			*log_file << "DATE AND TIME OF PREP:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_APP_REFERENCE:			// Heading
			*log_file << "APPLICATION REFERENCE:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_HEADING_EXTENSIONS:		// Heading
			*log_file << "HEADING EXTENSIONS" << endl;
			break;
		case EDI_SUBJECT_EDIM:			// EDI Notif
			*log_file << "SUBJECT EDIM" << endl;
			break;
		case EDI_EDIN_ORIG:			// EDI Notif
			*log_file << "EDIN ORIGINATOR" << endl;
			break;
		case EDI_FIRST_RECIP:			// EDI Notif
			*log_file << "FIRST RECIPIENT" << endl;
			break;
		case EDI_NOTIF_TIME:			// EDI Notif
			*log_file << "NOTIFICATION TIME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_SECUR_ELEMENTS:		// EDI Notif
			*log_file << "EDI SECURITY ELEMENTS" << endl;
			break;
		case EDI_EDIN_INITIATOR:		// EDI Notif
			*log_file << "EDIN INITIATOR:  ";
			log_edin_initiator( value.enumeration );
			break;
		case EDI_NOTIF_EXTENSIONS:		// EDI Notif
			*log_file << "EDI NOTIFICATION EXTENSIONS" << endl;
			break;
		case EDI_SUPPLEMENTARY_INFO:		// EDI Notif
			*log_file << "SUPLLEMENTARY INFORMATION:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_POSITIVE_NOTIF:		// EDI Notif Req
			*log_file << "EDI POSITIVE NOTIF REQ:  ";
			LOG_BOOLEAN( value.boolean );
			break;
		case EDI_NEGATIVE_NOTIF:		// EDI Notif Req
			*log_file << "EDI NEGATIVE NOTIF REQ:  ";
			LOG_BOOLEAN( value.boolean );
			break;
		case EDI_FORWARD_NOTIF:			// EDI Notif Req
			*log_file << "EDI FORWARD NOTIF REQ:  ";
			LOG_BOOLEAN( value.boolean );
			break;
		case EDI_PROOF_OF_RECEPTION:		// EDI Notif Req
			*log_file << "EDI PROOF OF RECEPTION REQ:  ";
			LOG_BOOLEAN( value.boolean );
			break;
		case EDI_NON_REPUD_OF_RECEPTION:	// EDI Notif Req
			*log_file << "EDI NON REPUD OF RECEPTION REQ:  ";
			LOG_BOOLEAN( value.boolean );
			break;
		case EDI_HEADING:			// EDIM
			*log_file << "EDI HEADING" << endl;
			break;
		case EDI_PRIMARY_BODYPART:		// EDIM
			*log_file << "EDI PRIMARY BODYPART" << endl;
			break;
		case EDI_ADD_BODYPARTS:			// EDIM
			*log_file << "EDI ADDITIONAL BODYPARTS" << endl;
			break;
		case EDI_EX_BODYPART:			// EDIM Ex Def BP
			*log_file << "EDI EXTERNAL BODYPART" << endl;
			break;
		case EDI_USER:				// EDIM ID
			*log_file << "EDI USER" << endl;
			break;
		case EDI_USER_RELATIVE_ID:		// EDIM ID
			*log_file << "EDI USER RELATIVE ID:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_EDIN_RECEIVER_NAME:		// EDIN Receiver
			*log_file << "EDIN RECEIVER NAME" << endl;
			break;
		case EDI_ORIGINAL_EDIM_ID:		// EDIN Receiver
			*log_file << "ORIGINAL EDIM ID" << endl;
			break;
		case EDI_FN_SECUR_CHECK:		// FN UAMS Reason Code
			*log_file << "FN SECURITY CHECK:  ";
			LOG_BOOLEAN( value.boolean );
			break;
		case EDI_FORWARD_TO:			// FN
			*log_file << "FORWARDED TO" << endl;
			break;
		case EDI_FN_REASON_CODE:		// FN
			*log_file << "FN REASON CODE" << endl;
			break;
		case EDI_IDENTIFICATION_CODE:		// Inter Sender/Recip
			*log_file << "EDI IDENTIFICATION CODE:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_IDENTIFICATION_CODE_Q:		// Inter Sender/Recip
			*log_file << "EDI IDENTIFICATION CODE QUAL:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_ROUTING_ADDRESS:		// Inter Sender/Recip
			*log_file << "EDI ROUTING ADDRESS:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_NN_REASON_CODE:		// NN
			*log_file << "NN REASON CODE" << endl;
			break;
		case EDI_BASIC_CODE:			// Reason Code
			*log_file << "EDI BASIC CODE:  ";
			*log_file << value.integer << endl;
			break;
		case EDI_DIAGNOSTIC:			// Reason Code
			*log_file << "EDI DIAGNOSTIC:  ";
			*log_file << value.integer << endl;
			break;
		case EDI_RECIP_REFERENCE:		// Recip Ref
			*log_file << "RECIPIENT REFERENCE:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_RECIP_REFERENCE_Q:		// Recip Ref
			*log_file << "RECIPIENT REFERENCE QUAL:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_RECIP:				// Recips
			*log_file << "RECIPIENT" << endl;
			break;
		case EDI_ACT_REQUEST:			// Recips
			*log_file << "ACTION_REQ:  ";
			log_act_request ( ( char * ) value.string.elements );
			break;
		case EDI_EDI_NOTIF_REQUESTS:		// Recips
			*log_file << "EDI NOTIFICATION REQUESTS" << endl;
			break;
		case EDI_RESPONS_PASSING_ALLOWED:	// Recips
			*log_file << "RESPONSIBILITY PASSING ALLOWED:  ";
			LOG_BOOLEAN( value.boolean );
			break;
		case EDI_INTER_RECIP:			// Recips
			*log_file << "INTERCHANGE RECIPIENT" << endl;
			break;
		case EDI_INTER_CONTROL_REFERENCE:	// Recips
			*log_file << "INTERCHANGE CONTROL REF:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_PROCESSING_PRIORITY_CODE:	// Recips
			*log_file << "PROCESSING PRIORITY CODE:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_ACKNOWLEDGEMENT_REQUEST:	// Recips
			*log_file << "ACKNOWLEDGEMENT REQUEST:  ";
			LOG_BOOLEAN( value.boolean );
			break;
		case EDI_COMMS_AGREEMENT_ID:		// Recips
			*log_file << "EDI COMMS AGREEMENT ID:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_TEST_INDICATOR:		// Recips
			*log_file << "TEST INDICATOR:  ";
			LOG_BOOLEAN( value.boolean );
			break;
		case EDI_RECIP_EXTENSIONS:		// Recips
			*log_file << "RECIPIENT EXTENSIONS" << endl;
			break;
		case EDI_ORIGINAL_CONTENT:		// Secur Ele
			*log_file << "ORIGIANL CONTENT" << endl;
			break;
		case EDI_ORIGINAL_CONTENT_INTEG_CK:	// Secur Ele
			*log_file << "ORIGIANL CONTENT INTEGRITY CK" << endl;
			break;
		case EDI_COMPONENT_DATA_ELEMENT_SEP:	// Serv Str Adv
			*log_file << "COMPONENT DATA ELEMENT SEP:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_DATA_ELEMENT_SEPARATOR:	// Serv Str Adv
			*log_file << "DATA ELEMENT SEPARATOR:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_DECIMAL_NOTATION:		// Serv Str Adv
			*log_file << "DECIMAL NOTATION:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_RELEASE_INDICATOR:		// Serv Str Adv
			*log_file << "RELEASE_INDICATOR:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_RESERVED:			// Serv Str Adv
			*log_file << "RESERVED:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_SEGMENT_TERMINATOR:		// Serv Str Adv
			*log_file << "SEGMENT_TERMINATOR:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_SYNTAX_ID_STRING:		// Syntax Id
			*log_file << "SYNTAX ID STRING:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case EDI_SYNTAX_VERSION:		// Syntax Id
			*log_file << "SYNTAX VERSION:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_ACTION:			// External Trace
			*log_file << "ACTION:  ";
			log_action( value.enumeration );
			break;
		case MH_T_ACTUAL_RECIPIENT_NAME:	// Delivery Envelope
			*log_file << "ACTUAL RECIPIENT NAME:" << endl;
			break;
		case MH_T_ADMD_NAME:			// O/R Address
			*log_file << "ADMD NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_ALTERNATE_RECIP_ALLOWED:	// Communique
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "ALTERNATE RECIPIENT ALLOWED:  ";
				LOG_BOOLEAN( value.boolean );
			}
			break;			
		case MH_T_ALTERNATE_RECIPIENT_NAME:	// Submitted Probe RD
			*log_file << "ALTERNATE RECIPIENT NAME:"<<endl;
			break;
		case MH_T_ARRIVAL_TIME:			// External Trace
			*log_file << "ARRIVAL TIME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_ATTEMPTED_ADMD_NAME:		// External Trace
			*log_file << "ATTEMPTED ADMD NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_ATTEMPTED_COUNTRY_NAME:	// External Trace
			*log_file << "ATTEMPTED COUNTRY NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_ATTEMPTED_MTA_NAME:		// Internal Trace
			*log_file << "ATTEMPTED MTA NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_ATTEMPTED_PRMD_IDENTIFIER:	// External Trace
			*log_file << "ATTEMPTED PRMD IDENTIFIER:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_BILATERAL_INFO:		// Communique
			*log_file << "BILATERAL INFO" << endl;
			break;
		case MH_T_BUILTIN_EITS:			// EITs
			*log_file << "BUILTIN EITS:  ";
			log_builtin_eits( value.enumeration );
			break;
		case MH_T_BUREAU_FAX_DELIVERY:		// Submitted Message RD
			*log_file << "BUREAU FAX DELIVERY:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
		case MH_T_COMMON_NAME:			// O/R Address
			*log_file << "COMMON NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_CONTENT:			// Message
			*log_file << "CONTENT" << endl;
			break;
		case MH_T_CONTENT_CORRELATOR:		// Communique
			if ( log_level >= TRACE_LOG ) {
				*log_file << "CONTENT CORRELATOR:  ";
				log_any_syntax( desc->syntax, value );
			}
			break;
		case MH_T_CONTENT_EXTENSIONS:		// Delivery Report
			*log_file << "CONTENT_EXTENSIONS" << endl;
			break;
		case MH_T_CONTENT_IDENTIFIER:		// Communique
			if ( log_level >= TRACE_LOG ) {
				*log_file << "CONTENT IDENTIFIER:  ";
				*log_file << ( char * ) value.string.elements 
					  << endl;
			}
			break;
		case MH_T_CONTENT_RETURN_REQUESTED:	// Message
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "CONTENT RETURN REQUESTED:  ";
				LOG_BOOLEAN( value.boolean );
			}
			break;			
		case MH_T_CONTENT_TYPE:			// Communique
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "CONTENT TYPE:  ";
				if ( desc->syntax == OM_S_INTEGER )
				       log_content_type( value.integer );
				else
				       log_content_type( ( char * ) 
						       value.string.elements );
			}
			break;
		case MH_T_CONVERSION_LOSS_PROHIBITED:	// Communique
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "CONVERSION LOSS PROHIBITED:  ";
				LOG_BOOLEAN( value.boolean );
			}
			break;			
		case MH_T_CONVERSION_PROHIBITED:	// Communique
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "CONVERSION PROHIBITED:  ";
				LOG_BOOLEAN( value.boolean );
			}
			break;			
		case MH_T_CONVERTED_EITS:		// Per Recip Report
			*log_file << "CONVERTED EITS" << endl;
			break;
		case MH_T_COUNTRY_NAME:			// O/R Address
			*log_file << "COUNTRY NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_CRITICAL_FOR_DELIVERY:	// Extensible Object
			*log_file << "CRITICAL FOR DELIVERY:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
		case MH_T_CRITICAL_FOR_SUBMISSION:	// Extensible Object
			*log_file << "CRITICAL FOR SUBMISSION:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
		case MH_T_CRITICAL_FOR_TRANSFER:	// Extensible Object
			*log_file << "CRITICAL FOR TRANSFER:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
		case MH_T_DEFERRED_DELIVERY_TIME:	// Message
			if ( log_level >= TRACE_LOG ) {
				*log_file << "DEFERRED DELIVERY TIME:  ";
				*log_file << ( char * ) value.string.elements 
					  << endl;
			}
			break;
		case MH_T_DEFERRED_TIME:		// External trace
			*log_file << "DEFERRED TIME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_DELIVERY_POINT:		// Per-Recip DR
			*log_file << "DELIVERY POINT:  ";
			log_delivery_point( value.enumeration );
			break;
		case MH_T_DELIVERY_TIME:		// Delivery Envelope
			if ( log_level >= TRACE_LOG ) {
				*log_file << "DELIVERY TIME:  ";
				*log_file << ( char * ) value.string.elements 
					  << endl;
			}
			break;
		case MH_T_DIRECTORY_NAME:		// O/R Name
			*log_file << "DIRECTORY NAME:" << endl;
			break;
		case MH_T_DISCLOSURE_ALLOWED:		// Message
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "DISCLOSURE ALLOWED:  ";
				LOG_BOOLEAN( value.boolean );
			}
			break;			
		case MH_T_DISTINGUISHED_RECIP_ADDR:	// Delivery Envelope
			*log_file << "DISTINGUISHED RECIP ADDR" << endl;
			break;
		case MH_T_DOMAIN_TYPE_1:		// O/R Address
			*log_file << "DOMAIN TYPE 1:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_DOMAIN_TYPE_2:		// O/R Address
			*log_file << "DOMAIN TYPE 2:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_DOMAIN_TYPE_3:		// O/R Address
			*log_file << "DOMAIN TYPE 3:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_DOMAIN_TYPE_4:		// O/R Address
			*log_file << "DOMAIN TYPE 4:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_DOMAIN_VALUE_1:		// O/R Address
			*log_file << "DOMAIN VALUE 1:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_DOMAIN_VALUE_2:		// O/R Address
			*log_file << "DOMAIN VALUE 2:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_DOMAIN_VALUE_3:		// O/R Address
			*log_file << "DOMAIN VALUE 3:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_DOMAIN_VALUE_4:		// O/R Address
			*log_file << "DOMAIN VALUE 4:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_EVENT_HANDLE:			// Session
			*log_file << "EVENT HANDLE:  ";
			*log_file << value.integer << endl;
			break;
		case MH_T_EXPANSION_HISTORY:		// Comminuque
			*log_file << "EXPANSION HISTORY" << endl;
			break;
		case MH_T_EXPANSION_PROHIBITED:		// Communique
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "EXPANSION PROHIBITED:  ";
				LOG_BOOLEAN( value.boolean );
			}
			break;			
		case MH_T_EXPLICIT_CONVERSION:		// Submitted Probe RD
			*log_file << "EXPLICIT CONVERSION:  ";
			log_explicit_conversion( value.enumeration );
			break;
		case MH_T_EXTENSION_TYPE:		// Extension
			*log_file << "EXTENSION TYPE:  ";
			if ( desc->syntax == OM_S_INTEGER )
				*log_file << value.integer << endl;
			else
				*log_file << ( char * ) value.string.elements 
					  << endl;
			break;
		case MH_T_EXTENSION_VALUE:		// Extension
			*log_file << "EXTENSION VALUE:  ";
			log_any_syntax( desc->syntax, value );
			break;
		case MH_T_EXTENSIONS:			// Extensible Object
			*log_file << "EXTENSIONS" << endl;
			break;
		case MH_T_EXTERNAL_EITS:		// EITs
			*log_file << "EXTERNAL EITS:  ";
			log_external_eits( ( char * ) value.string.elements );
			break;
		case MH_T_EXTERNAL_TRACE_INFO:		// Communique
			*log_file << "EXTERNAL TRACE INFO" << endl;
			break;
		case MH_T_FORWARDING_ADDR_REQUESTED:	// Submitted Message RD
			*log_file << "FORWARDING ADDRESS REQUESTED:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
		case MH_T_FORWARDING_PROHIBITED:	// Submitted Message RD
			*log_file << "FORWARDING PROHIBITED:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
		case MH_T_GENERATION:			// O/R Address
			*log_file << "GENERATION:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_GIVEN_NAME:			// O/R Address
			*log_file << "GIVEN NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_INFORMATION:			// Bilateral Info
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "INFORMATION:  ";
				*log_file << ( char * ) value.string.elements 
					  << endl;
			}
			break;
		case MH_T_INITIALS:			// O/R Address
			*log_file << "INITIALS:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_INTENDED_RECIPIENT_NAME:	// Redir Record
			*log_file << "INTENDED RECIPIENT NAME" << endl;
			break;
		case MH_T_INTENDED_RECIPIENT_NUMBER:	// Per Recip Report
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "INTENDED RECIPIENT NUMBER:  ";
				*log_file << value.integer << endl;
			}
			break;
		case MH_T_INTERNAL_TRACE_INFO:		// Communique
			*log_file << "INTERNAL TRACE INFO" << endl;
			break;
		case MH_T_LATEST_DELIVERY_TIME:		// Message
			if ( log_level >= TRACE_LOG ) {
				*log_file << "LATEST DELIVERY TIME:  ";
				*log_file << ( char * ) value.string.elements 
					  << endl;
			}
			break;
		case MH_T_LOCAL_IDENTIFIER:		// MTS Identifier
			*log_file << "LOCAL IDENTIFIER:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_MTA_NAME:			// Internal Trace
			*log_file << "MTA NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_MTA_REPORT_REQUEST:		// Message RD
			*log_file << "MTA REPORT REQUEST:  ";
			log_report_request( value.enumeration );
			break;
		case MH_T_MTA_RESPONSIBILITY:		// Message RD
			*log_file << "MTA RESPONSIBILITY:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
		case MH_T_MTS_IDENTIFIER:		// Communique
			*log_file << "MTS IDENTIFIER" << endl;
			break;
		case MH_T_NAME:				// Expansion Record
			*log_file << "NAME:" << endl; 
			break;
		case MH_T_NON_DELIVERY_DIAGNOSTIC:	// Per Recip NDR
			*log_file << "NON-DELIVERY DIAGNOSTIC:  ";
			log_diagnostic( value.enumeration );
			break;
		case MH_T_NON_DELIVERY_REASON:		// Per Recip NDR
			*log_file << "NON-DELIVERY REASON:  ";
			log_reason( value.enumeration );
			break;
		case MH_T_ORGANIZATION_NAME:		// O/R Address
			*log_file << "ORGANIZATION NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_ORGANIZATIONAL_UNIT_NAME_1:	// O/R Address
			*log_file << "ORGANIZATIONAL UNIT NAME 1:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_ORGANIZATIONAL_UNIT_NAME_2:	// O/R Address
			*log_file << "ORGANIZATIONAL UNIT NAME 2:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_ORGANIZATIONAL_UNIT_NAME_3:	// O/R Address
			*log_file << "ORGANIZATIONAL UNIT NAME 3:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_ORGANIZATIONAL_UNIT_NAME_4:	// O/R Address
			*log_file << "ORGANIZATIONAL UNIT NAME 4:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_ORIG_AND_EXPANSION_HISTORY:	// Delivery Report
			*log_file << "ORIG AND EXPANSION HISTORY"<<endl;
			break;
		case MH_T_ORIGINAL_EITS:		// Communique
			*log_file << "ORIGINAL EITS" << endl;
			break;
		case MH_T_ORIGINALLY_INTENDED_RECIP: 	// Per Recip Report
			*log_file << "ORIGINALL INTENDED RECIP" << endl;
			break;
		case MH_T_ORIGINATOR_NAME:		// Communique
			*log_file << "ORIGINATOR NAME" << endl;
			break;
		case MH_T_ORIGINATOR_REPORT_REQUEST:	// Per Recip Report
			if ( log_level >= TRACE_LOG ) {
				*log_file << "ORIGINATOR REPORT REQUEST:  ";
				log_report_request( value.enumeration );
			}
			break;
		case MH_T_OTHER_RECIPIENT_NAMES:	// Delivery Envelope
			if ( log_level >= TRACE_LOG )
				*log_file << "OTHER RECIPIENT NAMES" << endl;
			break;
		case MH_T_PER_RECIPIENT_REPORTS:	// Report
			*log_file << "PER RECIPIENT REPORTS" << endl;
			break;
		case MH_T_PREFERRED_DELIVERY_MODES:	// Submiited Probe RD
			*log_file << "PREFERRED DELIVERY MODES:  ";
			log_delivery_mode( value.enumeration );
			break;
		case MH_T_PRIORITY:			// Message
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "PRIORITY:  ";
				log_priority( value.enumeration );
			}
			break;
		case MH_T_PRMD_IDENTIFIER:		// MTS Identifier
			*log_file << "PRMD IDENTIFIER:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_PRMD_NAME:			// O/R Address
			*log_file << "PRMD NAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_PROOF_OF_DELIV_REQUESTED:	// Submitted Msg RD
			*log_file << "PROOF OF DELIVERY REQUESTED:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
		case MH_T_REASON:			// Redirection Record
			if ( log_level >= TRACE_LOG ) {
				*log_file << "REDIRETION REASON:  ";
				log_redirection_reason( value.enumeration );
			}
			break;
		case MH_T_REASSIGNMENT_PROHIBITED:	// Communique
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "REASSIGNMENT PROHIBITED:  ";
				LOG_BOOLEAN( value.boolean );
			}
			break;			
		case MH_T_RECIPIENT_DESCRIPTORS:	// Communique
			*log_file << "RECIPIENT DESCRIPTORS" << endl;
			break;
		case MH_T_RECIPIENT_NAME:		// Submitted Probe RD
			*log_file << "RECIPIENT NAME" << endl;
			break;
		case MH_T_RECIPIENT_NUMBER:		// Message RD
			*log_file << "RECIPIENT NUMBER:  ";
			*log_file << value.integer << endl;
			break;
		case MH_T_REDIRECTION_HISTORY:		// Message RD
			*log_file << "REDIRECTION HISTORY" << endl;
			break;
		case MH_T_REPORT_ADDITIONAL_INFO:	// Report
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "REPORT ADDITIONAL INFO:  ";
				*log_file << ( char * ) value.string.elements 
					  << endl;
			}
			break;
		case MH_T_REPORT_DESTINATION:		// Report
			*log_file << "REPORT DESTINATION" << endl;
			break;
		case MH_T_REPORTING_DL_NAME:		// Delivery Report
			*log_file << "REPORTING DL NAME" << endl;
			break;
		case MH_T_SUBJECT_EXT_TRACE_INFO:	// Report
			*log_file << "SUBJECT EXT TRACE INFO" << endl;
			break;
		case MH_T_SUBJECT_MTS_IDENTIFIER:	// Delivery Report
			*log_file << "SUBJECT MTS IDENTIFIER" << endl;
			break;
		case MH_T_SUBMISSION_TIME:		// Delivery Envelope
			if ( log_level >= TRACE_LOG ) {
				*log_file << "SUBMISSION TIME:  ";
				*log_file << ( char * ) value.string.elements 
					  << endl;
			}
			break;
		case MH_T_SUPPLEMENTARY_INFO:		// Per Recip Report
			if ( log_level >= VERBOSE_LOG ) {
				*log_file << "SUPPLEMENTARY INFO:  ";
				*log_file << ( char * ) value.string.elements 
					  << endl;
			}
			break;
		case MH_T_SURNAME:			// O/R Address
			*log_file << "SURNAME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case MH_T_TIME:				// Expansion Record
			*log_file << "TIME:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case IM_AUTO_FORWARDED:
			*log_file << "AUTO FORWARDED:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
		case IM_BODY:
			*log_file << "BODY:  " << endl;
			break;
		case IM_FORMAL_NAME:
			*log_file << "FORMAL NAME:  " << endl;
			break;
		case IM_IMPORTANCE:
			*log_file << "IMPORTANCE:  ";
			if ( value.enumeration == IM_LOW )
				*log_file << "LOW" << endl;
			else if ( value.enumeration == IM_ROUTINE )
				*log_file << "ROUTINE" << endl;
			else if ( value.enumeration == IM_HIGH )
				*log_file << "HIGH" << endl;
			else
				*log_file << "UNRECOGNIZED" << 
						value.enumeration << endl;
			break;
		case IM_IPM_RETURN_REQUESTED:
			*log_file << "IPM RETURN REQUESTED:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
		case IM_INCOMPLETE_COPY:
			*log_file << "INCOMPLETE COPY:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
		case IM_NOTIFICATION_REQUEST:
			*log_file << "NOTIFICATION REQUEST:  ";
			if ( value.enumeration == IM_NEVER )
				*log_file << "NEVER" << endl;
			else if ( value.enumeration == IM_NON_RECEIPT )
				*log_file << "NON RECEIPT" << endl;
			else if ( value.enumeration == IM_ALWAYS )
				*log_file << "ALWAYS" << endl;
			else
				*log_file << "UNRECOGNIZED" << 
						value.enumeration << endl;
			break;
		case IM_ORIGINATOR:
			*log_file << "IM ORIGINATOR" << endl;
			break;
		case IM_PRIMARY_RECIPIENTS:
			*log_file << "PRIMARY RECIPIENTS:  " << endl;
			break;
		case IM_RECIPIENT:
			*log_file << "IM RECIPIENT:  " << endl;
			break;
		case IM_REPERTOIRE:
			*log_file << "REPERTOIRE:  ";
			if ( value.enumeration == IM_ITA2 )
				*log_file << "ITA2" << endl;
			else if ( value.enumeration == IM_IA5 )
				*log_file << "IA5" << endl;
			else
				*log_file << "UNRECOGNIZED" << 
						value.enumeration << endl;
			break;
		case IM_REPLY_REQUESTED:
			*log_file << "REPLY_REQUESTED:  ";
			LOG_BOOLEAN( value.boolean );
			break;			
		case IM_SENSITIVITY:
			*log_file << "SENSITIVITY:  ";
			if ( value.enumeration == IM_NOT_SENSITIVE )
				*log_file << "NOT SENSITIVE" << endl;
			else if ( value.enumeration == IM_PERSONAL )
				*log_file << "PERSONAL" << endl;
			else if ( value.enumeration == IM_PRIVATE )
				*log_file << "PRIVATE" << endl;
			else if ( value.enumeration == IM_COMPANY_CONFIDENTIAL )
				*log_file << "COMPANY CONFIDENTIAL" << endl;
			else
				*log_file << "UNRECOGNIZED" << 
						value.enumeration << endl;
			break;
		case IM_SUBJECT:
			*log_file << "SUBJECT:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case IM_TEXT:
			*log_file << "TEXT:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		case IM_THIS_IPM:
			*log_file << "THIS IPM:  " << endl;
			break;
		case IM_USER:
			*log_file << "IM USER:  " << endl;
			break;
		case IM_USER_RELATIVE_IDENTIFIER:
			*log_file << "USER RELATIVE ID:  ";
			*log_file << ( char * ) value.string.elements << endl;
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << desc->type << endl;
			break;
	}
}


void X400Message::log_any_syntax( OM_syntax syntax, OM_value value ) {

	switch ( syntax ) {

		case OM_S_BOOLEAN:
			LOG_BOOLEAN( value.boolean );
			break;
		case OM_S_ENUMERATION:
			*log_file << value.enumeration << endl;
			break;
		case OM_S_INTEGER:
			*log_file << value.integer << endl;
			break;
		case OM_S_BIT_STRING:
		case OM_S_ENCODING:
		case OM_S_GENERAL_STRING:
		case OM_S_GENERALISED_TIME_STRING:
		case OM_S_GRAPHIC_STRING:
		case OM_S_IA5_STRING:
		case OM_S_NUMERIC_STRING:
		case OM_S_OBJECT_DESCRIPTOR_STRING:
		case OM_S_OBJECT_IDENTIFIER_STRING:
		case OM_S_OCTET_STRING:
		case OM_S_PRINTABLE_STRING:
		case OM_S_TELETEX_STRING:
		case OM_S_UTC_TIME_STRING:
		case OM_S_VIDEOTEX_STRING:
		case OM_S_VISIBLE_STRING:
			*log_file << ( char * ) value.string.elements << endl;
			break;
		default:
			*log_file << "ERROR: Unrecognized Syntax" << endl;
			break;
	} // switch
}


void X400Message::log_act_request( char *req ) {

	int	last_digits = req[ 4 ];

	if ( ! strncmp( req, "\x56\x06\x07\x0D", 4 ) ) {

		switch ( last_digits ) {

			case 0x00:
				*log_file << "For Action";
				break;
			case 0x01:
				*log_file << "For Copy";
				break;
			default:
				*log_file << "UNRECOGNIZED:  " << req;
				break;
		}
	}
	else
		*log_file << "UNRECOGNIZED:  " << req;

	*log_file << endl;
}
		
void X400Message::log_action( OM_enumeration action ) {

	switch ( action ) {

		case MH_AC_EXPANDED:
			*log_file << "EXPANDED";
			break;
		case MH_AC_REDIRECTED:
			*log_file << "REDIRECTED";
			break;
		case MH_AC_RELAYED:
			*log_file << "RELAYED";
			break;
		case MH_AC_REROUTED:
			*log_file << "REROUTED";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << action;
	}

	*log_file << endl;
}

void X400Message::log_bodypart_type( char *bp ) {
	int	last_digits = bp[ 4 ];

	if ( ! strncmp( bp, "\x56\x06\x07\x0B", 4 ) ) {

		switch ( last_digits ) {

			case 0x00:
				*log_file << "EDIFACT ISO646";
				break;
			case 0x01:
				*log_file << "EDIFACT T61";
				break;
			case 0x02:
				*log_file << "EDIFACT OCTET";
				break;
			case 0x03:
				*log_file << "X12 ISO646";
				break;
			case 0x04:
				*log_file << "X12 T61";
				break;
			case 0x05:
				*log_file << "X12 OCTET";
				break;
			case 0x06:
				*log_file << "X12 EBCDIC";
				break;
			case 0x07:
				*log_file << "UNTDI ISO646";
				break;
			case 0x08:
				*log_file << "UNTDI T61";
				break;
			case 0x09:
				*log_file << "UNTDI OCTET";
				break;
			case 0x0A:
				*log_file << "PRIVATE OCTET";
				break;
			case 0x0B:
				*log_file << "UNDEFINED OCTET";
				break;
			default:
				*log_file << "UNRECOGNIZED OID:  " << bp;
				break;
		}
	}
	else
		*log_file << "UNRECOGNIZED VALUE:  " << bp;

	*log_file << endl;
}

void X400Message::log_builtin_eits( OM_enumeration eit ) {

	switch ( eit ) {

		case MH_BE_UNDEFINED:
			*log_file << "UNDEFINED";
			break;
		case MH_BE_TELEX:
			*log_file << "TELEX";
			break;
		case MH_BE_IA5_TEXT:
			*log_file << "IA5_TEXT";
			break;
		case MH_BE_G3_FAX:
			*log_file << "G3_FAX";
			break;
		case MH_BE_G4_CLASS1:
			*log_file << "G4_CLASS1";
			break;
		case MH_BE_TELETEX:
			*log_file << "TELETEX";
			break;
		case MH_BE_VIDEOTEX:
			*log_file << "VIDEOTEX";
			break;
		case MH_BE_MIXED_MODE:
			*log_file << "MIXED_MODE";
			break;
		case MH_BE_ODA:
			*log_file << "ODA";
			break;
		case MH_BE_ISO_6937_TEXT:
			*log_file << "6937_TEXT";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << eit;
			break;
	}

	*log_file << endl;
}

void X400Message::log_delivery_mode( OM_enumeration mode ) {

	switch ( mode ) {
	
		case MH_DM_ANY:
			*log_file << "ANY";
			break;
		case MH_DM_MTS:
			*log_file << "MTS";
			break;
		case MH_DM_PDS:
			*log_file << "PDS";
			break;
		case MH_DM_TELEX:
			*log_file << "TELEX";
			break;
		case MH_DM_TELETEX:
			*log_file << "TELETEX";
			break;
		case MH_DM_G3_FAX:
			*log_file << "G3_FAX";
			break;
		case MH_DM_G4_FAX:
			*log_file << "G4_FAX";
			break;
		case MH_DM_IA5_TERMINAL:
			*log_file << "IA5_TERMINAL";
			break;
		case MH_DM_VIDEOTEX:
			*log_file << "VIDEOTEX";
			break;
		case MH_DM_TELEPHONE:
			*log_file << "TELEPHONE";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << mode;
			break;
	}

	*log_file << endl;
}

void X400Message::log_delivery_point( OM_enumeration point ) {

	switch ( point ) {
	
		case MH_DP_PUBLIC_UA:
			*log_file << "PUBLIC_UA";
			break;
		case MH_DP_PRIVATE_UA:
			*log_file << "PRIVATE_UA";
			break;
		case MH_DP_MS:
			*log_file << "MS";
			break;
		case MH_DP_DL:
			*log_file << "DL";
			break;
		case MH_DP_PDAU:
			*log_file << "PDAU";
			break;
		case MH_DP_PDS_PATRON:
			*log_file << "PATRON";
			break;
		case MH_DP_OTHER_AU:
			*log_file << "OTHER_AU";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << point;
			break;
	}

	*log_file << endl;
}


void X400Message::log_diagnostic( OM_enumeration diag ) {

	switch ( diag ) {
	
		case MH_DG_NO_DIAGNOSTIC:
			*log_file << "NO DIAGNOSTIC";
			break;
		case MH_DG_OR_NAME_UNRECOGNIZED:
			*log_file << "OR NAME UNRECOGNIZED";
			break;
		case MH_DG_OR_NAME_AMBIGUOUS:
			*log_file << "OR NAME AMBIGUOUS";
			break;
		case MH_DG_MTS_CONGESTED:
			*log_file << "MTS CONGESTED";
			break;
		case MH_DG_LOOP_DETECTED:
			*log_file << "LOOP DETECTED";
			break;
		case MH_DG_RECIPIENT_UNAVAILABLE:
			*log_file << "RECIPIENT UNAVAILABLE";
			break;
		case MH_DG_MAXIMUM_TIME_EXPIRED:
			*log_file << "MAXIMUM TIME EXPIRED";
			break;
		case MH_DG_EITS_UNSUPPORTED:
			*log_file << "EITS UNSUPPORTED";
			break;
		case MH_DG_CONTENT_TOO_LONG:
			*log_file << "CONTENT TOO LONG";
			break;
		case MH_DG_IMPRACTICAL_TO_CONVERT:
			*log_file << "IMPRACTICAL TO CONVERT";
			break;
		case MH_DG_PROHIBITED_TO_CONVERT:
			*log_file << "PROHIBITED TO CONVERT";
			break;
		case MH_DG_CONVERSION_UNSUBSCRIBED:
			*log_file << "CONVERSION UNSUBSCRIBED";
			break;
		case MH_DG_PARAMETERS_INVALID:
			*log_file << "PARAMETERS INVALID";
			break;
		case MH_DG_CONTENT_SYNTAX_IN_ERROR:
			*log_file << "CONTENT SYNTAX IN ERROR";
			break;
		case MH_DG_LENGTH_CONSTRAINT_VIOLATD:
			*log_file << "LENGTH CONSTRAINT VIOLATED";
			break;
		case MH_DG_NUMBER_CONSTRAINT_VIOLATD:
			*log_file << "NUMBER_CONSTRAINT_VIOLATED";
			break;
		case MH_DG_CONTENT_TYPE_UNSUPPORTED:
			*log_file << "CONTENT TYPE UNSUPPORTED";
			break;
		case MH_DG_TOO_MANY_RECIPIENTS:
			*log_file << "TOO MANY RECIPIENTS";
			break;
		case MH_DG_NO_BILATERAL_AGREEMENT:
			*log_file << "NO BILATERAL AGREEMENT";
			break;
		case MH_DG_CRITICAL_FUNC_UNSUPPORTED:
			*log_file << "CRITICAL FUNC UNSUPPORTED";
			break;
		case MH_DG_CONVERSION_LOSS_PROHIB:
			*log_file << "CONVERSION LOSS PROHIB";
			break;
		case MH_DG_LINE_TOO_LONG:
			*log_file << "LINE TOO LONG";
			break;
		case MH_DG_PAGE_TOO_LONG:
			*log_file << "PAGE TOO LONG";
			break;
		case MH_DG_PICTORIAL_SYMBOL_LOST:
			*log_file << "PICTORIAL SYMBOL LOST";
			break;
		case MH_DG_PUNCTUATION_SYMBOL_LOST:
			*log_file << "PUNCTUATION SYMBOL LOST";
			break;
		case MH_DG_ALPHABETIC_CHARACTER_LOST:
			*log_file << "ALPHABETIC CHARACTER LOST";
			break;
		case MH_DG_MULTIPLE_INFO_LOSSES:
			*log_file << "MULTIPLE INFO LOSSES";
			break;
		case MH_DG_REASSIGNMENT_PROHIBITED:
			*log_file << "REASSIGNMENT PROHIBITED";
			break;
		case MH_DG_REDIRECTION_LOOP_DETECTED:
			*log_file << "REDIRECTION LOOP DETECTED";
			break;
		case MH_DG_EXPANSION_PROHIBITED:
			*log_file << "EXPANSION PROHIBITED";
			break;
		case MH_DG_SUBMISSION_PROHIBITED:
			*log_file << "SUBMISSION PROHIBITED";
			break;
		case MH_DG_EXPANSION_FAILED:
			*log_file << "EXPANSION FAILED";
			break;
		case MH_DG_RENDITION_UNSUPPORTED:
			*log_file << "RENDITION UNSUPPORTED";
			break;
		case MH_DG_MAIL_ADDRESS_INCORRECT:
			*log_file << "MAIL ADDRESS INCORRECT";
			break;
		case MH_DG_MAIL_OFFICE_INCOR_OR_INVD:
			*log_file << "MAIL OFFICE INCORRECT OR INVALID";
			break;
		case MH_DG_MAIL_ADDRESS_INCOMPLETE:
			*log_file << "MAIL ADDRESS INCOMPLETE";
			break;
		case MH_DG_MAIL_RECIPIENT_UNKNOWN:
			*log_file << "MAIL RECIPIENT UNKNOWN";
			break;
		case MH_DG_MAIL_RECIPIENT_DECEASED:
			*log_file << "MAIL RECIPIENT DECEASED";
			break;
		case MH_DG_MAIL_ORGANIZATION_EXPIRED:
			*log_file << "ORGANIZATION EXPIRED";
			break;
		case MH_DG_MAIL_REFUSED:
			*log_file << "REFUSED";
			break;
		case MH_DG_MAIL_UNCLAIMED:
			*log_file << "UNCLAIMED";
			break;
		case MH_DG_MAIL_RECIPIENT_MOVED:
			*log_file << "RECIPIENT MOVED";
			break;
		case MH_DG_MAIL_RECIPIENT_TRAVELLING:
			*log_file << "RECIPIENT TRAVELLING";
			break;
		case MH_DG_MAIL_RECIPIENT_DEPARTED:
			*log_file << "RECIPIENT DEPARTED";
			break;
		case MH_DG_MAIL_NEW_ADDRESS_UNKNOWN:
			*log_file << "NEW ADDRESS UNKNOWN";
			break;
		case MH_DG_MAIL_FORWARDING_UNWANTED:
			*log_file << "FORWARDING UNWANTED";
			break;
		case MH_DG_MAIL_FORWARDING_PROHIB:
			*log_file << "MAIL FORWARDING PROHIB";
			break;
		case MH_DG_SECURE_MESSAGING_ERROR:
			*log_file << "SECURE MESSAGING ERROR";
			break;
		case MH_DG_DOWNGRADING_IMPOSSIBLE:
			*log_file << "DOWNGRADING IMPOSSIBLE";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << diag;
			break;
	}

	*log_file << endl;
}

		
void X400Message::log_edin_initiator( OM_enumeration init ) {

	switch ( init ) {
	
		case EDI_EI_INTERNAL_UA:
			*log_file << "INTERNAL UA";
			break;
		case EDI_EI_INTERNAL_MS:
			*log_file << "INTERNAL MS";
			break;
		case EDI_EI_EX_UA:
			*log_file << "EXTERNAL UA";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << init;
			break;
	}

	*log_file << endl;
}


void X400Message::log_explicit_conversion( OM_enumeration conv ) {

	switch ( conv ) {
	
		case MH_EC_NO_CONVERSION:
			*log_file << "NO CONVERSION";
			break;
		case MH_EC_IA5_TEXT_TO_TELETEX:
			*log_file << "IA5 TEXT TO TELETEX";
			break;
		case MH_EC_TELETEX_TO_TELEX:
			*log_file << "TELETEX TO TELEX";
			break;
		case MH_EC_TELEX_TO_IA5_TEXT:
			*log_file << "TELEX TO IA5 TEXT";
			break;
		case MH_EC_TELEX_TO_TELETEX:
			*log_file << "TELEX TO TELETEX";
			break;
		case MH_EC_TELEX_TO_G4_CLASS1:
			*log_file << "TELEX TO G4 CLASS 1";
			break;
		case MH_EC_TELEX_TO_VIDEOTEX:
			*log_file << "TELEX TO VIDEOTEX";
			break;
		case MH_EC_IA5_TEXT_TO_TELEX:
			*log_file << "IA5 TEXT TO TELEX";
			break;
		case MH_EC_TELEX_TO_G3_FAX:
			*log_file << "TELEX TO G3 FAX";
			break;
		case MH_EC_IA5_TEXT_TO_G4_CLASS1:
			*log_file << "IA5 TEXT TO G4 CLASS 1";
			break;
		case MH_EC_IA5_TEXT_TO_VIDEOTEX:
			*log_file << "IA5 TEXT TO VIDEOTEX";
			break;
		case MH_EC_TELETEX_TO_IA5_TEXT:
			*log_file << "TELETEX TO IA5 TEXT";
			break;
		case MH_EC_TELETEX_TO_G3_FAX:
			*log_file << "TELETEX TO G3 FAX";
			break;
		case MH_EC_TELETEX_TO_G4_CLASS1:
			*log_file << "TELETEX TO G4 CLASS 1";
			break;
		case MH_EC_TELETEX_TO_VIDEOTEX:
			*log_file << "TELETEX TO VIDEOTEX";
			break;
		case MH_EC_VIDEOTEX_TO_TELEX:
			*log_file << "VIDEOTEX TO TELEX";
			break;
		case MH_EC_VIDEOTEX_TO_IA5_TEXT:
			*log_file << "VIDEOTEX_TO_IA5_TEXT:";
			break;
		case MH_EC_VIDEOTEX_TO_TELETEX:
			*log_file << "VIDEOTEX TO TELETEX";
			break;
		case MH_EC_IA5_TEXT_TO_G3_FAX:
			*log_file << "IA5 TEXT TO G3 FAX";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << conv;
			break;
	}

	*log_file << endl;
}

#ifdef	POSTAL

void X400Message::log_postal_mode( OM_enumeration mode ) {

	switch ( mode ) {
	
		case MH_PM_ORDINARY_MAIL:
			*log_file << "ORDINARY MAIL";
			break;
		case MH_PM_SPECIAL_DELIVERY:
			*log_file << "SPECIAL DELIVERY";
			break;
		case MH_PM_EXPRESS_MAIL:
			*log_file << "EXPRESS MAIL";
			break;
		case MH_PM_CC:
			*log_file << "CC";
			break;
		case MH_PM_CC_WITH_TELEPHONE_ADVICE:
			*log_file << "CC WITH TELEPHONE ADVICE";
			break;
		case MH_PM_CC_WITH_TELEX_ADVICE:
			*log_file << "CC WITH TELEX ADVICE";
			break;
		case MH_PM_CC_WITH_TELETEX_ADVICE:
			*log_file << "CC WITH TELETEX ADVICE";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << mode;
			break;
	}

	*log_file << endl;
}


void X400Message::log_postal_report( OM_enumeration rep ) {

	switch ( rep ) {
	
		case MH_PR_UNDELIVBLE_MAIL_VIA_PDS:
			*log_file << "UNDELIVERABLE MAIL VIA PDS";
			break;
		case MH_PR_NOTIFICN_VIA_PDS:
			*log_file << "NOTIFICATION VIA PDS";
			break;
		case MH_PR_NOTIFICN_VIA_MTS:
			*log_file << "NOTIFICATION VIA MTS";
			break;
		case MH_PR_NOTIFICN_VIA_MTS_AND_PDS:
			*log_file << "NOTIFICATION VIA MTS AND PDS";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << rep;
			break;
	}

	*log_file << endl;
}


void X400Message::log_registration( OM_enumeration reg ) {

	switch ( reg ) {
	
		case MH_RG_REGISTERED_MAIL:
			*log_file << "REGISTERED_MAIL";
			break;
		case MH_RG_UNREGISTERED_MAIL_DELIVERY:
			*log_file << "UNREGISTERED MAIL";
			break;
		case MH_RG_REGISTERED_MAIL_IN_PERSON:
			*log_file << "REGISTERED MAIL IN PERSON";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << reg;
			break;
	}

	*log_file << endl;
}


void X400Message::log_rendition_attributes( char *attr ) {

	if ( ! strcmp( attr, OMP_O_MH_RA_BASIC_RENDITION ) )
		*log_file << "BASIC RENDITION";
	else
		*log_file << "UNRECOGNIZED:  " << attr;

	*log_file << endl;
}

#endif	// POSTAL


void X400Message::log_priority( OM_enumeration priority ) {

	switch ( priority ) {
	
		case MH_PTY_NORMAL:
			*log_file << "NORMAL";
			break;
		case MH_PTY_LOW:
			*log_file << "LOW";
			break;
		case MH_PTY_URGENT:
			*log_file << "URGENT";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << priority;
			break;
	}

	*log_file << endl;
}


void X400Message::log_reason( OM_enumeration reason ) {

	switch ( reason ) {
	
		case MH_RE_TRANSFER_FAILED:
			*log_file << "TRANSFER FAILED";
			break;
		case MH_RE_TRANSFER_IMPOSSIBLE:
			*log_file << "TRANSFER IMPOSSIBLE";
			break;
		case MH_RE_CONVERSION_NOT_PERFORMED:
			*log_file << "CONVERSION NOT PERFORMED";
			break;
		case MH_RE_PHYSICAL_RENDITN_NOT_DONE:
			*log_file << "RENDITION NOT DONE";
			break;
		case MH_RE_PHYSICAL_DELIV_NOT_DONE:
			*log_file << "PHYSICAL DELIVERY NOT DONE";
			break;
		case MH_RE_RESTRICTED_DELIVERY:
			*log_file << "RESTRICTED DELIVERY";
			break;
		case MH_RE_DIRECTORY_OPERATN_FAILED:
			*log_file << "DIRECTORY OPERATION FAILED";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << reason;
			break;
	}

	*log_file << endl;
}


void X400Message::log_redirection_reason( OM_enumeration reason ) {

	switch ( reason ) {
	
		case MH_RR_RECIPIENT_ASSIGNED:
			*log_file << "RECIPIENT_ASSIGNED";
			break;
		case MH_RR_ORIGINATOR_REQUESTED:
			*log_file << "ORIGINATOR_REQUESTED:";
			break;
		case MH_RR_RECIPIENT_DOMAIN_ASSIGNED:
			*log_file << "RECIPIENT_DOMAIN_ASSIGNED";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << reason;
			break;
	}

	*log_file << endl;
}


void X400Message::log_report_request( OM_enumeration req ) {

	switch ( req ) {
	
		case MH_RQ_NEVER:
			*log_file << "NEVER";
			break;
		case MH_RQ_NON_DELIVERY:
			*log_file << "NON-DELIVERY";
			break;
		case MH_RQ_ALWAYS:
			*log_file << "ALWAYS";
			break;
		case MH_RQ_ALWAYS_AUDITED:
			*log_file << "ALWAYS AUDITED";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << req;
			break;
	}

	*log_file << endl;
}

#ifdef	SECURITY

void X400Message::log_security_classification( OM_enumeration class ) {

	switch ( class ) {
	
		case MH_SC_UNMARKED:
			*log_file << "UNMARKED";
			break;
		case MH_SC_UNCLASSIFIED:
			*log_file << "UNCLASSIFIED";
			break;
		case MH_SC_RESTRICTED:
			*log_file << "RESTRICTED";
			break;
		case MH_SC_CONFIDENTIAL:
			*log_file << "CONFIDENTIAL";
			break;
		case MH_SC_SECRET:
			*log_file << "SECRET";
			break;
		case MH_SC_TOP_SECRET:
			*log_file << "TOP_SECRET";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << class;
			break;
	}

	*log_file << endl;
}

#endif	// SECURITY

#ifdef	ADDRESS_FORMS

void X400Message::log_terminal_type( OM_enumeration type ) {

	switch ( type ) {
	
		case MH_TT_TELEX:
			*log_file << "TELEX";
			break;
		case MH_TT_TELETEX:
			*log_file << "TELETEX";
			break;
		case MH_TT_G3_FAX:
			*log_file << "G3_FAX";
			break;
		case MH_TT_G4_FAX:
			*log_file << "G4_FAX";
			break;
		case MH_TT_IA5_TERMINAL:
			*log_file << "IA5_TERMINAL";
			break;
		case MH_TT_VIDEOTEX:
			*log_file << "VIDEOTEX";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << type;
			break;
	}

	*log_file << endl;
}

#endif	// ADDRESS_FORMS

void X400Message::log_content_type( OM_integer type ) {

	switch ( type ) {
	
		case MH_CTI_UNIDENTIFIED:
			*log_file << "UNIDENTIFIED";
			break;
		case MH_CTI_EXTERNAL:
			*log_file << "EXTERNAL";
			break;
		case MH_CTI_P2_1984:
			*log_file << "P2_1984";
			break;
		case MH_CTI_P2_1988:
			*log_file << "P2_1988";
			break;
		case 35:
			*log_file << "EDI";
			break;
		default:
			*log_file << "UNRECOGNIZED:  " << type;
			break;
	}

	*log_file << endl;
}


void X400Message::log_content_type( char *type ) {

	if ( ! strcmp( type, OMP_O_MH_CTO_INNER_MESSAGE ) )
		*log_file << "INNER MESSAGE";
	else if ( ! strcmp( type, OMP_O_MH_CTO_UNIDENTIFIED ) )
		*log_file << "UNIDENTIFIED";
	else 
		*log_file << "UNRECOGNIZED:  " << type;

	*log_file << endl;
}


void X400Message::log_external_eits( char *eits ) {

	int	last_digits;
	
	if ( strncmp( eits, "\x56\x06\x04", 3 ) ) {
		
		if ( strncmp( eits, "\x56\x06\x07\x0B", 4 ) ) {
			*log_file << "UNRECOGNIZED:  " << eits << endl;
			return;
		}

		last_digits = eits[ 4 ];

		switch ( last_digits ) {
	
			case 0x00:
				*log_file << "EDIFACT ISO646";
				break;
			case 0x01:
				*log_file << "EDIFACT T61";
				break;
			case 0x02:
				*log_file << "EDIFACT OCTET";
				break;
			case 0x03:
				*log_file << "ANSI X12 ISO646";
				break;
			case 0x04:
				*log_file << "ANSI X12 T61";
				break;
			case 0x05:
				*log_file << "ANSI X12 OCTET";
				break;
			case 0x06:
				*log_file << "ANSI X12 BCDIC";
				break;
			case 0x07:
				*log_file << "UN/TDI ISO646";
				break;
			case 0x08:
				*log_file << "UN/TDI T61";
				break;
			case 0x09:
				*log_file << "UN/TDI OCTET";
				break;
			case 0x0A:
				*log_file << "PRIVATE OCTET";
				break;
			case 0x0B:
				*log_file << "UNDEFINED OCTET";
				break;
			default:
				*log_file << "UNRECOGNIZED:  " << last_digits;
				break;
		}
	}
	else {

		last_digits = eits[ 3 ];

		switch ( last_digits ) {
	
			case 0x00:
				*log_file << "UNDEFINED";
				break;
			case 0x01:
				*log_file << "TELEX";
				break;
			case 0x02:
				*log_file << "IA5 TEXT";
				break;
			case 0x03:
				*log_file << "G3 FAX";
				break;
			case 0x04:
				*log_file << "G4 CLASS 1";
				break;
			case 0x05:
				*log_file << "TELETEX";
				break;
			case 0x06:
				*log_file << "VIDEOTEX";
				break;
			case 0x09:
				*log_file << "MIXED MODE";
				break;
			default:
				*log_file << "UNRECOGNIZED:  " << last_digits;
				break;
		}
	}
	*log_file << endl;
}


void X400Message::log_attributes( OM_descriptor *desc, 
				  char 		*prefix,
				  char		*class_id,
				  int		subflag ) {
	char	new_prefix[5];
	
	strcpy( new_prefix, prefix );
	strcat( new_prefix, "    " );
	desc++;		// bypass the class attribute
	*log_file << prefix << "BEGIN:  " << class_id << endl;
	while ( desc->type != OM_NO_MORE_TYPES ) {
		log_desc( desc, new_prefix );
		if ( desc->syntax == OM_S_OBJECT && subflag == LOG_SUBOBJECTS ) 
			log_class( desc->value.object.object, new_prefix,
				   LOG_SUBOBJECTS );
		desc++;
	}
	*log_file << prefix << "END:    " << class_id << endl;
}

