// FILE:  x400notf.C

#include	<fstream.h>
#include	<sys/types.h>
#include	<sys/unistd.h>
#include	<dirent.h>
#include	<xom.h>
#include	<assert.h>
#include	<xedip.h>
extern "C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/subject.h"
#include	"../../../include/interchange.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/edi_user.h"
#include	"../../../include/config.h"
#include	"../../../include/platfile.h"
#include	"../../../include/tpfile.h"
#include	"../../../include/api_obj.h"
#include	"../../../include/x400api.h"


//
// PRIVATE:  build an EDI Negative Notification.
//

STATUS X400Message::build_nn() {

	STATUS		status = SUCCESS;

#ifdef	DEBUG
	cout << "In build_nn()" << endl;
#endif


	status |= x400_open();
	if ( ! OK ( status ) )
		return ERROR;

	// Declare APIObjects after x400_open() so workspace is set correctly.
	APIObject	nn_content( workspace, log );
	APIObject	reason( workspace, log );
	
	status |= build_notif_content( &nn_content,
				       OMP_O_EDI_C_NEGATIVE_NOTIF,
				       &reason );
	if ( OK( status ) )
		status |= build_message( EDI_NOTIFICATION, &nn_content,
					 (APIObject *)0, (APIObject *)0 );
	nn_content.api_delete();
	reason.api_delete();

#ifdef	DEBUG
	cout << "nn status = " << status << endl;
#endif

	return status;
}


//
// PRIVATE:  build an EDI Positive Notification.
//

STATUS X400Message::build_pn() {

	STATUS		status = SUCCESS;

#ifdef	DEBUG
	cout << "In build_pn()" << endl;
#endif

	status |= x400_open();
	if ( ! OK ( status ) )
		return ERROR;

	// Declare APIObject after x400_open() so workspace is set correctly.
	APIObject	pn_content( workspace, log );
	
	status |= build_notif_content( &pn_content,
				       OMP_O_EDI_C_POSITIVE_NOTIF,
				       (APIObject *)0 );
	if ( OK( status ) )
		status |= build_message( EDI_NOTIFICATION, &pn_content,
					 (APIObject *)0, (APIObject *)0 );
	pn_content.api_delete();

#ifdef	DEBUG
	cout << "pn status = " << status << endl;
#endif

	return status;
}


//
// PRIVATE: build the content for the Notification.
//

STATUS X400Message::build_notif_content( APIObject 	*notif_content,
				      	 char		*notif_class,
					 APIObject	*reason ) {

	STATUS		status = SUCCESS;
	OM_descriptor	*desc;

#ifdef	DEBUG
	cout << "In build_notif_content()" << endl;
#endif
	
	//
	// Create the private object for the notification content.
	//

//	status |= notif_content->api_create( notif_class );

	//
	// Build the notification public object.
	//
	
	notif_content->make_class( notif_class );

	current_time( notif_content->time_buf() ); 
	notif_content->put_desc( EDI_NOTIF_TIME, 
				 OM_S_UTC_TIME_STRING, 
				 notif_content->time_buf() );

	notif_content->put_desc( EDI_EDIN_INITIATOR, 
			      	 OM_S_ENUMERATION, 
			         EDI_EI_INTERNAL_UA );

	if ( ! strcmp( notif_class, OMP_O_EDI_C_NEGATIVE_NOTIF ) )
		status |= nn_reason_code( notif_content, reason );

	//
	// Add to the notification attributes from the subject message:
	// First, add the message recipient as the EDIN originator.
	//

	notif_content->put_desc( EDI_EDIN_ORIG, 
				 subject_message.actual_recipient() );

	//
	// Include the First Recipient if it is present in the message.
	//

	if ( subject_message.first_recipient() != NULL )
		notif_content->put_desc( EDI_FIRST_RECIP,
					 subject_message.first_recipient() );

	//
	// The notification ID is copied from the EDIN Receiver, if present,
	// otherwise it is set to the ID of the subject message.
	//

	desc = ( subject_message.original_edim_id() != NULL ) ?
			subject_message.original_edim_id() : 
			subject_message.this_edim_id();
	notif_content->put_desc( EDI_SUBJECT_EDIM, desc );

	notif_content->end_object();

	//
	// Copy the public object to a private object.
	//
	
//	status |= notif_content->api_put();

#ifdef	DEBUG
	cout << "notif content status = " << status << endl;
#endif

	return status;
}

//
// PRIVATE:  build the negative notification reason code.
//

STATUS X400Message::nn_reason_code( APIObject	*nn_content,
				    APIObject	*reason ) {

	STATUS	status = SUCCESS;

#ifdef	DEBUG
	cout << "In nn_reason_code()" << endl;
#endif
	
	//
	// Create the private object for the UAMS Reason Code.
	//

//	status |= reason->api_create( OMP_O_EDI_C_NN_UAMS_REASON_CODE );

	//
	// Build the Reason Code public object.
	//
	
	reason->make_class( OMP_O_EDI_C_NN_UAMS_REASON_CODE );
	reason->put_desc( EDI_BASIC_CODE, OM_S_INTEGER, 
			  subject_message.reason_code() );
	reason->put_desc( EDI_DIAGNOSTIC, OM_S_INTEGER, 
			  subject_message.diagnostic() );
	reason->end_object();
	
	//
	// Copy the public object to the private object.
	//

//	status |= reason->api_put();

	//
	// Add the UAMS Reason Code to the Notification Content.
	//

	nn_content->put_desc( EDI_NN_REASON_CODE, reason->pub_object() );

#ifdef	DEBUG
	cout << "nn code status = " << status << " diag = " << 
		subject_message.diagnostic() << endl;
#endif

	return status;
}
