// FILE:  x400out.C

#include	<fstream.h>
#include	<sys/types.h>
#include	<sys/unistd.h>
#include	<dirent.h>
#include	<xom.h>
#include	<assert.h>
#include	<xedip.h>
extern "C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/subject.h"
#include	"../../../include/interchange.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/edi_user.h"
#include	"../../../include/config.h"
#include	"../../../include/platfile.h"
#include	"../../../include/tpfile.h"
#include	"../../../include/api_obj.h"
#include	"../../../include/x400api.h"

void X400Message::session_attributes( APIObject *obj ) {

	//
	// Build the Session Attributes of Country Name, ADMD Name, and PRMD 
	// Identifier.  These attributes are required by the MTS Id and the 
	// External Trace Info classes.
	//
	
	obj->put_desc(  MH_T_COUNTRY_NAME, 		  // Add country
		  	OM_S_PRINTABLE_STRING,
		  	local_user->country_name() );

	obj->put_desc( 	MH_T_ADMD_NAME, 		  // Add ADMD
			OM_S_PRINTABLE_STRING, 
			local_user->admd_name() );

	if ( local_user->prmd_name() )
		obj->put_desc( 	MH_T_PRMD_IDENTIFIER,     // Add PRMD
				OM_S_PRINTABLE_STRING, 
				local_user->prmd_name() );
}


void X400Message::external_trace( APIObject 	*mhs_message,
				  APIObject	*ext_trace ) {

#ifdef	DEBUG
	cout << "In external_trace()" << endl;
#endif
	
	//
	// Build the External Trace Entry public object.
	//

	ext_trace->make_class( OMP_O_MH_C_EXTERNAL_TRACE_ENTRY );
	ext_trace->put_desc( MH_T_ACTION, OM_S_ENUMERATION, MH_AC_RELAYED );
	session_attributes( ext_trace );
	current_time( ext_trace->time_buf() );
	ext_trace->put_desc( MH_T_ARRIVAL_TIME, OM_S_UTC_TIME_STRING,
			     ext_trace->time_buf() );
	ext_trace->end_object();

#ifdef	DEBUG
	cout << "leaving ext_trace" << endl;
#endif

	//
	// Add the External Trace Entry to the MHS Message.
	//

	mhs_message->put_desc(MH_T_EXTERNAL_TRACE_INFO,ext_trace->pub_object());
}


void X400Message::mts_identifier( APIObject *mhs_message, 
				  APIObject *mts_id ) {

#ifdef	DEBUG
	cout << "In mts_identifier()" << endl;
#endif
	
	//
	// Build the MTS Identifier public object.
	//
	
	mts_id->make_class( OMP_O_MH_C_MTS_IDENTIFIER );
	session_attributes( mts_id );
	timestamp( mts_id->time_buf() );
	mts_id->put_desc( MH_T_LOCAL_IDENTIFIER, OM_S_IA5_STRING, 
			  mts_id->time_buf() );
	mts_id->end_object();
	
#ifdef	DEBUG
	cout << "leaving mts id" << endl;
#endif

	//
	// Add the MTS Identifier to the MHS Message.
	//

	mhs_message->put_desc( MH_T_MTS_IDENTIFIER, mts_id->pub_object() );
}


void X400Message::original_eits( APIObject	*mhs_message,
				 APIObject 	*orig_eits,
				 int		message_type ) {

#ifdef	DEBUG
	cout << "In original_eits()" << endl;
#endif
	
	//
	// Build the EITS public object.
	//
	
	orig_eits->make_class( OMP_O_MH_C_EITS );
	if ( message_type == EDI_MESSAGE )
		orig_eits->put_desc( MH_T_EXTERNAL_EITS, 
		  	             OM_S_OBJECT_IDENTIFIER_STRING, 
		  	             interchange->get_bodypart_type() );
	else
		orig_eits->put_desc( MH_T_BUILTIN_EITS,
				     OM_S_ENUMERATION,
				     MH_BE_IA5_TEXT );
	orig_eits->end_object();

#ifdef	DEBUG
	cout << "leaving orig_eits" << endl;
#endif

	//
	// Add the Original EITs to the MHS Message.
	//
 
	mhs_message->put_desc( MH_T_ORIGINAL_EITS, orig_eits->pub_object() );
}


void X400Message::rd( APIObject	*mhs_message,
		      APIObject	*recip_desc, 
		      APIObject *recipient,
		      int	information_type ) {

#ifdef	DEBUG
	cout << "In rd()" << endl;
#endif
	
	OM_descriptor	*rcpt;
	
	//
	// Build the RD public object.  For a message, the recipient name
	// is obtained from the Directory.
	//
	
	recip_desc->make_class( OMP_O_MH_C_MESSAGE_RD );
	recip_desc->put_desc( MH_T_EXPLICIT_CONVERSION, OM_S_ENUMERATION,
			      OM_FALSE );
	recip_desc->put_desc( MH_T_BUREAU_FAX_DELIVERY, OM_S_BOOLEAN, 
			      OM_FALSE );
	recip_desc->put_desc( MH_T_FORWARDING_ADDR_REQUESTED, OM_S_BOOLEAN, 
			      OM_FALSE );
	recip_desc->put_desc( MH_T_FORWARDING_PROHIBITED, OM_S_BOOLEAN, 
			      OM_FALSE );
	recip_desc->put_desc( MH_T_PROOF_OF_DELIV_REQUESTED, OM_S_BOOLEAN,
			      OM_FALSE );
	recip_desc->put_desc( MH_T_PREFERRED_DELIVERY_MODES, OM_S_ENUMERATION,
			      MH_DM_ANY );
	recip_desc->put_desc( MH_T_ORIGINATOR_REPORT_REQUEST, OM_S_ENUMERATION,
			      MH_RQ_NON_DELIVERY );
	recip_desc->put_desc( MH_T_RECIPIENT_NUMBER, OM_S_INTEGER, 1 );
	recip_desc->put_desc( MH_T_MTA_REPORT_REQUEST, OM_S_ENUMERATION, 
			      MH_RQ_NON_DELIVERY );
	recip_desc->put_desc( MH_T_MTA_RESPONSIBILITY, OM_S_BOOLEAN, OM_TRUE );
	if ( information_type != EDI_NOTIFICATION ) {
		recip_desc->put_desc( MH_T_RECIPIENT_NAME, 
				      recipient->pub_object() );
	}
	else {
		rcpt = ( subject_message.edin_receiver_name() != NULL ) ?
			subject_message.edin_receiver_name() :
			subject_message.originator();
		recip_desc->put_desc( MH_T_RECIPIENT_NAME, rcpt );
	}
	recip_desc->end_object();

#ifdef	DEBUG
	cout << "leaving recip_desc" << endl;
#endif

	mhs_message->put_desc( MH_T_RECIPIENT_DESCRIPTORS, 
			       recip_desc->pub_object() );
}


STATUS X400Message::build_message( int 		information_type, 
				   APIObject	*mhs_content,
				   APIObject	*originator,
				   APIObject	*recipient ) {

#ifdef	DEBUG
	cout << "In build_message()" << endl;
#endif
	
	STATUS		status = SUCCESS;
	APIObject	mhs_message( mhs_content );
	APIObject	ext_trace( mhs_content );
	APIObject	mts_id( mhs_content );
	APIObject	orig_eits( mhs_content );
	APIObject	recip_desc( mhs_content );
	OM_integer	priority;

	//
	// Create a private object for the MHS message.
	//
	
	if ( ! OK ( mhs_message.api_create( OMP_O_MH_C_MESSAGE ) ) )
		return ERROR;
		
	//
	// Build the message public object.
	//
	
	mhs_message.make_class( OMP_O_MH_C_MESSAGE );

	priority = ( information_type != EDI_NOTIFICATION ) ? 
		       tp_entry->priority() : subject_message.priority();
	mhs_message.put_desc( MH_T_PRIORITY, OM_S_ENUMERATION, priority );
	
 	if ( information_type == EDI_MESSAGE || 
	     information_type == EDI_NOTIFICATION)
		mhs_message.put_desc( MH_T_CONTENT_TYPE, 
				      OM_S_INTEGER, 
			              EDI_CONTENT_TYPE );
 	else
		mhs_message.put_desc( MH_T_CONTENT_TYPE, 
				      OM_S_INTEGER, 
			              MH_CTI_P2_1988 );

// 	mhs_message.put_desc( MH_T_CONVERSION_PROHIBITED, 
//			      OM_S_BOOLEAN, 
//			      OM_TRUE );
	
	//
	// NOTE: Conversion Loss Prohibited is a 1988 feature!.
	//

//	mhs_message.put_desc( MH_T_CONVERSION_LOSS_PROHIBITED, 
//			      OM_S_BOOLEAN, 
//			      OM_TRUE );

	//
	// NOTE: Expansion Prohibited is a 1988 feature!.
	//

//	mhs_message.put_desc( MH_T_EXPANSION_PROHIBITED,
//			      OM_S_BOOLEAN, 
//			      OM_TRUE);

 	external_trace( &mhs_message, &ext_trace );
	mts_identifier( &mhs_message, &mts_id );
	rd( &mhs_message, &recip_desc, recipient, information_type );

	if ( information_type != EDI_NOTIFICATION )
		original_eits( &mhs_message, &orig_eits, information_type );

	if ( information_type == EDI_NOTIFICATION )
		mhs_message.put_desc( MH_T_ORIGINATOR_NAME, 
					subject_message.actual_recipient() );
	else {
		mhs_message.put_desc( MH_T_ORIGINATOR_NAME, 
					originator->pub_object() );
	}

	mhs_message.put_desc( MH_T_CONTENT, mhs_content->pub_object() );

	mhs_message.end_object();
	
	log_object( &mhs_message, LOG_SUBOBJECTS );

	//
	// Copy the public object to the private object.
	//

	status |= mhs_message.api_put();

#ifdef	DEBUG
	cout << "mhs_message status = " << status << endl;
#endif

	//
	// Submit the message to the API service.
	//
	
	status |= x400_transfer_out( &mhs_message );
	
	ext_trace.api_delete();
	mts_id.api_delete();
	orig_eits.api_delete();
	recip_desc.api_delete();

	return status;
}


STATUS X400Message::build_edi_message( int message_type ) {

	STATUS		status = SUCCESS;

#ifdef	DEBUG
	cout << "In build_edi_message()" << endl;
#endif

	if ( ! OK( x400_open() ) )
		return ERROR;
	
	//
	// Declare these after x400_open so that workspace has a value.
	//
	
	APIObject	mhs_content( workspace, log );
	APIObject	originator( workspace, log );
	APIObject	recipient( workspace, log );
	APIObject	edi_notification_requests( workspace, log );
	APIObject	interchange_recipient( workspace, log );
	APIObject	authorization_information( workspace, log );
	APIObject	recipient_reference( workspace, log );
	APIObject	this_EDIM( workspace, log );
	APIObject	recipients( workspace, log );
	APIObject	edin_receiver( workspace, log );
	APIObject	edin_recip( workspace, log );
	APIObject	service_string_advice( workspace, log );
	APIObject	syntax_identifier( workspace, log );
	APIObject	interchange_sender( workspace, log );
	APIObject	header( workspace, log );
	APIObject	bodypart( workspace, log );
	APIObject	this_ipm( workspace, log );
	APIObject	ipm_originator( workspace, log );
	APIObject	ipm_recipient( workspace, log );
	APIObject	rcpt_desc( workspace, log );
	
	or_name( tp_entry->this_recipient(), OMP_O_MH_C_OR_NAME, &recipient );
	or_name( local_user, OMP_O_MH_C_OR_NAME, &originator );
	
	if ( message_type == IPM_MESSAGE )
		build_ipm_content( &mhs_content, &originator, &recipient, 
				   &this_ipm, &bodypart, &ipm_originator, 
				   &ipm_recipient, &rcpt_desc );
	else
		build_edi_content( &mhs_content, &originator, &recipient,
				   &edi_notification_requests, 
				   &interchange_recipient,
				   &authorization_information, 
				   &recipient_reference, &this_EDIM, 
				   &recipients, &edin_receiver, &edin_recip, 
				   &service_string_advice, &syntax_identifier, 
				   &interchange_sender, &header, &bodypart );
	
	status |= build_message( message_type , &mhs_content,
				 &originator, &recipient );
	
	mhs_content.api_delete();
	recipient.api_delete();
	originator.api_delete();
	edi_notification_requests.api_delete();
	interchange_recipient.api_delete();
	authorization_information.api_delete();
	recipient_reference.api_delete();
	this_EDIM.api_delete();
	recipients.api_delete();
	edin_receiver.api_delete();
	edin_recip.api_delete();
	service_string_advice.api_delete();
	syntax_identifier.api_delete();
	interchange_sender.api_delete();
	header.api_delete();
	bodypart.api_delete();
	this_ipm.api_delete();
	ipm_originator.api_delete();
	ipm_recipient.api_delete();
	rcpt_desc.api_delete();
	
	status |= x400_close();

#ifdef	DEBUG
	cout << "build_edi_message() returning " << status << endl;
#endif

	return status;
}
