// FILE:  x400rep.C

#include	<fstream.h>
#include	<sys/types.h>
#include	<sys/unistd.h>
#include	<dirent.h>
#include	<xom.h>
#include	<assert.h>
#include	<xedip.h>
extern "C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/subject.h"
#include	"../../../include/interchange.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/edi_user.h"
#include	"../../../include/config.h"
#include	"../../../include/platfile.h"
#include	"../../../include/tpfile.h"
#include	"../../../include/api_obj.h"
#include	"../../../include/x400api.h"


//
// PRIVATE:  Construct a report for a subject message.
//

STATUS X400Message::build_mts_report( int		report_type,
				      OM_enumeration	reason,
				      OM_enumeration	diag ) {

	APIObject	mts_report( workspace, log );
	APIObject	ext_trace( workspace, log );
	APIObject	per_rcpt( workspace, log );
	STATUS		status = SUCCESS;

#ifdef	DEBUG
	cout << "In build_mts_report()" << endl;
#endif

	status |= x400_open();
	if ( ! OK ( status ) )
		return ERROR;

	//
	// Create the private object for the MTS Report.
	//

	status |= mts_report.api_create( OMP_O_MH_C_REPORT );

	//
	// Build the Report public object.
	//

	status |= build_report_public( &mts_report, &ext_trace, &per_rcpt, 
				       report_type, reason, diag );

	//
	// Copy the public object to the private object.
	//

	status |= mts_report.api_put();

	//
	// Log the report, then delete all private objects.
	//
	
	log_object( &mts_report, LOG_SUBOBJECTS );
	status |= x400_transfer_out( &mts_report );
	
	mts_report.api_delete();
	ext_trace.api_delete();
	per_rcpt.api_delete();

#ifdef	DEBUG
	cout << "mts_report status = " << status << endl;
#endif

	return status;
}


//
// PRIVATE:  Build the public object for the MTS Report.
//

STATUS X400Message::build_report_public( APIObject	*mts_report,
					 APIObject	*ext_trace,
					 APIObject	*per_rcpt,
					 int		report_type,
					 OM_enumeration	reason,
					 OM_enumeration	diag ) {
	STATUS	status = SUCCESS;
	
#ifdef	DEBUG
	cout << "In build__report_public()" << endl;
#endif

	mts_report->make_class( OMP_O_MH_C_REPORT );

	external_trace( mts_report, ext_trace );

	( report_type == DELIVERY_REPORT ) ?
		build_recipient_public( mts_report, per_rcpt, 
					NO_REASON, NO_DIAGNOSTIC ) :
	    	build_recipient_public( mts_report, per_rcpt, reason, diag );

	( subject_message.last_expansion() != NULL ) ?
		mts_report->put_desc( MH_T_REPORT_DESTINATION,
			              subject_message.last_expansion() ) :
		mts_report->put_desc( MH_T_REPORT_DESTINATION,
			              subject_message.originator() ); 
	
	mts_report->put_desc( MH_T_SUBJECT_EXT_TRACE_INFO,
			      subject_message.external_trace_info() );
	
	if ( subject_message.content_correlator() != NULL )
		mts_report->put_desc( MH_T_CONTENT_CORRELATOR,
				      subject_message.content_correlator() );
	
	if ( subject_message.content_identifier() != NULL )
		mts_report->put_desc( MH_T_CONTENT_IDENTIFIER, 
				      OM_S_PRINTABLE_STRING, ( char * ) 
				      subject_message.content_identifier() );

	mts_report->put_desc( MH_T_CONTENT_TYPE, OM_S_INTEGER,
			      subject_message.content_type() );

	mts_report->put_desc( MH_T_SUBJECT_MTS_IDENTIFIER,
			      subject_message.mts_identifier() );

	mts_report->end_object();

#ifdef	DEBUG
	cout << "report_public status = " << status << endl;
#endif

	return status;
}


//
// PRIVATE:  Build the public object for the per-recipient information.
//

void X400Message::build_recipient_public( APIObject		*mts_report,
					  APIObject 		*rcpt, 
					  OM_enumeration	reason,
					  OM_enumeration	diag ) {

#ifdef	DEBUG
	cout << "In build_recipient_public()" << endl;
#endif

	if ( reason == NO_REASON ) {	// Delivery Report
		rcpt->make_class( OMP_O_MH_C_PER_RECIP_DR );
		rcpt->put_desc( MH_T_DELIVERY_TIME, OM_S_UTC_TIME_STRING,
				( char * ) subject_message.arrival_time() );
	}
	else {				// Non-delivery Report
		rcpt->make_class( OMP_O_MH_C_PER_RECIP_NDR );
		rcpt->put_desc( MH_T_NON_DELIVERY_REASON, 
				OM_S_ENUMERATION, reason );
		rcpt->put_desc( MH_T_NON_DELIVERY_DIAGNOSTIC, 
				OM_S_ENUMERATION, diag );
	}

	rcpt->put_desc( MH_T_ARRIVAL_TIME, OM_S_UTC_TIME_STRING,
			( char * ) subject_message.arrival_time() );

	rcpt->put_desc( MH_T_INTENDED_RECIPIENT_NUMBER, OM_S_INTEGER, 
			subject_message.recipient_number() );
	
	rcpt->put_desc( MH_T_ORIGINATOR_REPORT_REQUEST, OM_S_ENUMERATION, 
			subject_message.report_request() );
cout << "Actual in rep = " << subject_message.actual_recipient() << endl;	
	rcpt->put_desc( MH_T_ACTUAL_RECIPIENT_NAME, 
			subject_message.actual_recipient() );
	
	if ( subject_message.intended_recipient() != NULL ) 
		rcpt->put_desc( MH_T_ORIGINALLY_INTENDED_RECIP,   
			        subject_message.intended_recipient() );
	
	if ( subject_message.redirection_history() != NULL ) 
		rcpt->put_desc( MH_T_REDIRECTION_HISTORY,
			        subject_message.redirection_history() );
	
	rcpt->end_object();

	mts_report->put_desc( MH_T_PER_RECIPIENT_REPORTS, rcpt->pub_object() );
}


