// FILE:  x400util.C

#include	<fstream.h>
#include	<sys/types.h>
#include	<sys/unistd.h>
#include	<dirent.h>
#include	<xom.h>
#include	<time.h>
#include	<assert.h>
#include	<xedip.h>
extern "C" {
#include	"/usr/lib/x400/gtwapi/include/nist_xmh.h"
}
#include	"/usr/lib/x400/gtwapi/include/xmhp.h"
#include	"/usr/lib/x400/gtwapi/include/ximp.h"
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/subject.h"
#include	"../../../include/interchange.h"
#include	"../../../include/linklist.h"
#include	"../../../include/or_name.h"
#include	"../../../include/edi_user.h"
#include	"../../../include/config.h"
#include	"../../../include/api_obj.h"
#include	"../../../include/platfile.h"
#include	"../../../include/tpfile.h"
#include	"../../../include/x400api.h"

//
// Build an O/R Name OM_PUBLIC_OBJECT from an ORName object.
//

void X400Message::or_name( ORName	*addr,
			   void		*class_id,
			   APIObject	*name ) {

#ifdef	DEBUG
	cout << "In or_name()" << endl;
#endif

	//
	// Build the public object for the O/R Name/Address.
	//
	
	name->make_class( class_id );
	add_or_attributes( name, addr );
	name->end_object();
}


void X400Message::add_or_attributes( APIObject	*name,
				     ORName	*addr ) {

	if ( addr->directory_name() )
		name->put_desc( MH_T_DIRECTORY_NAME,
		  	  	OM_S_PRINTABLE_STRING,
		  	  	addr->directory_name() );
	
	if ( addr->country_name() )
		name->put_desc( MH_T_COUNTRY_NAME,
		  	  	OM_S_PRINTABLE_STRING,
		  	  	addr->country_name() );

	if ( addr->admd_name() )
		name->put_desc( MH_T_ADMD_NAME,
			  	OM_S_PRINTABLE_STRING,
		  	  	addr->admd_name() );

	if ( addr->prmd_name() )
		name->put_desc( MH_T_PRMD_NAME,
		  	  	OM_S_PRINTABLE_STRING,
		  	  	addr->prmd_name() );

	if ( addr->common_name() )
		name->put_desc( MH_T_COMMON_NAME,
		  	  	OM_S_PRINTABLE_STRING,
		  	  	addr->common_name() );

	if ( addr->org_name() )
		name->put_desc( MH_T_ORGANIZATION_NAME,
		  	  	OM_S_PRINTABLE_STRING,
		  	  	addr->org_name() );

	if ( addr->org_unit1() ) {
		name->put_desc( MH_T_ORGANIZATIONAL_UNIT_NAME_1,
		  	  	OM_S_PRINTABLE_STRING,
		  	  	addr->org_unit1() );

		if ( addr->org_unit2() ) {
			name->put_desc( MH_T_ORGANIZATIONAL_UNIT_NAME_2,
		  		  	OM_S_PRINTABLE_STRING,
		  		  	addr->org_unit2() );

			if ( addr->org_unit3() ) {
				name->put_desc( MH_T_ORGANIZATIONAL_UNIT_NAME_3,
		  			  	OM_S_PRINTABLE_STRING,
		  			  	addr->org_unit3() );

				if ( addr->org_unit4() )	
					name->put_desc( 
					      MH_T_ORGANIZATIONAL_UNIT_NAME_4,
		  				  	OM_S_PRINTABLE_STRING,
		  				  	addr->org_unit4() );
			} // org_unit3
		} // org_unit2
	} // org_unit1	
	
	
	if ( addr->surname() ) {
		name->put_desc( MH_T_SURNAME,
		  	  	OM_S_PRINTABLE_STRING,
		  	  	addr->surname() );

		if ( addr->given_name() )
			name->put_desc( MH_T_GIVEN_NAME,
			  		OM_S_PRINTABLE_STRING,
			  		addr->given_name() );

		if ( addr->initials() )	
			name->put_desc( MH_T_INITIALS,
			  		OM_S_PRINTABLE_STRING,
			  		addr->initials() );

		if ( addr->generation() )	
			name->put_desc( MH_T_GENERATION,
			  		OM_S_PRINTABLE_STRING,
			  		addr->generation() );
	} // surname

	if ( addr->dda_type1() ) {	

		name->put_desc( MH_T_DOMAIN_TYPE_1,
			  	OM_S_PRINTABLE_STRING,
			  	addr->dda_type1() );

		name->put_desc( MH_T_DOMAIN_VALUE_1,
			  	OM_S_PRINTABLE_STRING,
			  	addr->dda_value1() );

		if ( addr->dda_type2() ) {	
			
			name->put_desc( MH_T_DOMAIN_TYPE_2,
			  		OM_S_PRINTABLE_STRING,
			  		addr->dda_type2() );

			name->put_desc( MH_T_DOMAIN_VALUE_2,
			  		OM_S_PRINTABLE_STRING,
			  		addr->dda_value2() );

			if ( addr->dda_type3() ) {

				name->put_desc( MH_T_DOMAIN_TYPE_3,
			  			OM_S_PRINTABLE_STRING,
			  			addr->dda_type3() );

				name->put_desc( MH_T_DOMAIN_VALUE_3,
			  			OM_S_PRINTABLE_STRING,
	  		  			addr->dda_value3() );

				if ( addr->dda_type4() ) {

					name->put_desc( MH_T_DOMAIN_TYPE_4,
			  				OM_S_PRINTABLE_STRING,
			  				addr->dda_type4() );

					name->put_desc( MH_T_DOMAIN_VALUE_4,
			  				OM_S_PRINTABLE_STRING,
			  				addr->dda_value4() );
				} // dda4
			} // dda3
		} // dda2
	} // dda1
}


//
// Get the current time as UTC Time.
//

void X400Message::current_time( char *timebuf ) {
	
	if ( cur_time[ 0 ] != NULL )
		strcpy( timebuf, cur_time );
	else {
		time_t	totsecs;
		totsecs = time( NULL );
		strftime( timebuf, 17, "%y%m%d%H%M%S", localtime( &totsecs ) );
		strcpy( cur_time, timebuf );
	}
}


//
// Get the current time in microseconds to use as a unique identifier.
//

void X400Message::timestamp( char *timebuf ) {
	struct	timeval timeval;
	struct	timezone timezone;
	STATUS	status = SUCCESS;

	if ( gettimeofday( &timeval, &timezone ) == -1 ) 
		log->log_event( "timestamp", "system error", INFORMATION );
	else {
		sprintf( timebuf, "%d", timeval.tv_sec );
		sprintf( &timebuf[ strlen(timebuf) ], "%d", timeval.tv_usec );
	}
}


//
// Build an O/R Descriptor.
//

void X400Message::or_descriptor( APIObject	*or_desc,
				 APIObject	*formal_name ) {
	
#ifdef	DEBUG
	cout << "In or_descriptor()" << endl;
#endif

	or_desc->make_class( OMP_O_IM_C_OR_DESCRIPTOR );
	or_desc->put_desc( IM_FORMAL_NAME, formal_name->pub_object() );
	or_desc->end_object();
}


//
// Convert an OM_PUBLIC_OBJECT to an ORName object.
//

void X400Message::public_to_or_name( APIObject *src_name, ORName *dst_name ) {

	OM_value	value;
	
	while ( src_name->more_types() ) {

		value = src_name->value();

		switch ( src_name->type() ) {

			case MH_T_DIRECTORY_NAME:
				dst_name->set_directory_name( ( char * )
					value.object.object );
				break;
			case MH_T_ADMD_NAME:
				dst_name->set_admd_name( ( char * )
					value.object.object );
				break;
			case MH_T_COMMON_NAME:
				dst_name->set_common_name( ( char * )
					value.object.object );
				break;
			case MH_T_COUNTRY_NAME:
				dst_name->set_country_name( ( char * )
					value.object.object );
				break;
			case MH_T_DOMAIN_TYPE_1:
				dst_name->set_dda_type1( ( char * )
					value.object.object );
				break;
			case MH_T_DOMAIN_TYPE_2:
				dst_name->set_dda_type2( ( char * )
					value.object.object );
				break;
			case MH_T_DOMAIN_TYPE_3:
				dst_name->set_dda_type3( ( char * )
					value.object.object );
				break;
			case MH_T_DOMAIN_TYPE_4:
				dst_name->set_dda_type4( ( char * )
					value.object.object );
				break;
			case MH_T_DOMAIN_VALUE_1:
				dst_name->set_dda_value1( ( char * )
					value.object.object );
				break;
			case MH_T_DOMAIN_VALUE_2:
				dst_name->set_dda_value2( ( char * )
					value.object.object );
				break;
			case MH_T_DOMAIN_VALUE_3:
				dst_name->set_dda_value3( ( char * )
					value.object.object );
				break;
			case MH_T_DOMAIN_VALUE_4:
				dst_name->set_dda_value4( ( char * )
					value.object.object );
				break;
			case MH_T_GENERATION:
				dst_name->set_generation( ( char * )
					value.object.object );
				break;
			case MH_T_GIVEN_NAME:
				dst_name->set_given_name( ( char * )
					value.object.object );
				break;
			case MH_T_INITIALS:
				dst_name->set_initials( ( char * )
					value.object.object );
				break;
			case MH_T_ORGANIZATION_NAME:
				dst_name->set_org_name( ( char * )
					value.object.object );
				break;
			case MH_T_ORGANIZATIONAL_UNIT_NAME_1:
				dst_name->set_org_unit1( ( char * )
					value.object.object );
				break;
			case MH_T_ORGANIZATIONAL_UNIT_NAME_2:
				dst_name->set_org_unit2( ( char * )
					value.object.object );
				break;
			case MH_T_ORGANIZATIONAL_UNIT_NAME_3:
				dst_name->set_org_unit3( ( char * )
					value.object.object );
				break;
			case MH_T_ORGANIZATIONAL_UNIT_NAME_4:
				dst_name->set_org_unit4( ( char * )
					value.object.object );
				break;
			case MH_T_PRMD_NAME:
				dst_name->set_prmd_name( ( char * )
					value.object.object );
				break;
			case MH_T_SURNAME:
				dst_name->set_surname( ( char * )
					value.object.object );
				break;
		}

		src_name->get_next_type();
	}
#ifdef	DEBUG
		dst_name->display_name();
#endif
}

