/*				FILE = ft_copy_parms.c			*/

#include	"xmit_file.h"
#include	"ftam_control_defs.h"
#include	"edi_control_defs.h"
#include	"edi_transfer_defs.h"
#include	<osi/map.h>
#include	<osi/mapftam.h>

char	*find_target(), *make_filename();
void	delete_ddn(), print_ddn();
void	delete_rdn(), print_rdn();
void	delete_ava(), print_ava();
void	delete_object_id(), print_object_id();
void	delete_octet_string(), print_octet_string();
void	passwords_in(),print_strings();

extern	char *pfile,*pdir,*gfile,*gdir,*error_dir,*ctrl_file,*lock_file;
extern	int mode,result,delete_files,keep_log;

/*	This function will create and set the values in the structures	*/
/*	necessary to call the ft_fcopy function.  The variables that	*/
/*	will populate the structures come from the control file.  For	*/
/*	values not provided, default values will be supplied as needed.	*/

ft_copy_parms  (argc, argv, source_dirname, source_filename,
		destination_dirname, destination_filename, ae_label,
		return_event_name, input_dcb, inout_dcb)
int	argc;
char	*argv[];
Ae_dir_name		*source_dirname;
Ft_filename		*source_filename;
Ae_dir_name		*destination_dirname;
Ft_filename		*destination_filename;
Ae_label		*ae_label;
Local_event_name	*return_event_name;
struct Ft_fcopy_in_dcb	**input_dcb;
struct Ft_fcopy_out_dcb	**inout_dcb;
{

	Api_rc			outcome;
	struct Ft_diagnostic	*diag = NULL;
	struct Ft_file_passwords	*file_passwords;

	int	i, fd, len, num_strings, defaults = DEFAULTS;
	char	*p, *source_file = 0, *destination_file = 0;
	char	*source_dir = 0, *destination_dir = 0, *unique_id = 0;
	char	buf[MAX_CONTROL_FILE_SIZE], *strings[MAX_CONTROL_STRINGS+1];

	/* parse the program parameters		*/
	for (i = 1; i < argc; i++)
		if (strncmp(argv[i], PFILE_DEF, sizeof(PFILE_DEF) -1) == 0)
			source_file = argv[i] + sizeof(PFILE_DEF) - 1;
		else	if (strncmp(argv[i], CTRL_DEF, sizeof(CTRL_DEF) -1) == 0)
			ctrl_file = argv[i] + sizeof(CTRL_DEF) - 1;
		else	if (strcmp(argv[i],"DELETE_FILES") == 0)
				delete_files = TRUE;
		else	if (strcmp(argv[i],"DELETE_LOG") == 0)
				keep_log = FALSE;
		else	if (strcmp(argv[i],"KEEP_FILES") == 0)
				delete_files = FALSE;
		else	if (strcmp(argv[i],"KEEP_LOG") == 0)
				keep_log = TRUE;
		else	if (strncmp(argv[i], LOCK_DEF, sizeof(LOCK_DEF) -1) == 0)
			lock_file = argv[i] + sizeof(LOCK_DEF) - 1;
		else	if (strncmp(argv[i], UNIQUE_ID_DEF, sizeof(UNIQUE_ID_DEF) - 1) == 0)
			unique_id = argv[i] + sizeof(UNIQUE_ID_DEF) - 1;
		else	if (strncmp(argv[i], DEST_FILE_DEF, sizeof(DEST_FILE_DEF) -1) == 0)
				destination_file = argv[i] + sizeof(DEST_FILE_DEF) - 1;
		else	if (strncmp(argv[i], DEST_DIR_DEF, sizeof(DEST_DIR_DEF) -1) == 0)
				destination_dir = argv[i] + sizeof(DEST_DIR_DEF) - 1;
		else	if (strncmp(argv[i], SOURCE_DIR_DEF, sizeof(SOURCE_DIR_DEF) -1) == 0)
			source_dir = argv[i] + sizeof(SOURCE_DIR_DEF) - 1;
		else	{
			sprintf(log_str,"ft_copy_parms: Warning unrecognized parameter = '%s', skipped\n",argv[i]);
			logit(log_str);
		}

	pfile = source_file;
	if ((fd = open(source_file, O_RDONLY)) < 0)	{
		sprintf (log_str,"error: unable to open transaction file - '%s'\n", source_file);
		logit(log_str);
		return(FALSE);
	}
	close(fd);	/* must exist, don't need open	*/

	/* open and read control file into 'buf'	*/
	if ((fd = open(ctrl_file, O_RDONLY)) < 0)	{
		sprintf(log_str,"error: unable to open control file - '%s'\n", ctrl_file);
		logit(log_str);
		return(FALSE);
	}
	if ((len = read(fd,buf,MAX_CONTROL_FILE_SIZE)) < 0)	{
		logit("error reading control file\n");
		return(FALSE);
	}
	if (len == MAX_CONTROL_FILE_SIZE)	{
		logit("Control file too large\n");
		return(FALSE);
	}

	/* convert '\n' to '\0' and build array of pointers to strings	*/
	if ((num_strings = lines_to_strings(buf,len,strings, MAX_CONTROL_STRINGS)) <= 0)	{
		logit("Error converting control file\n");
		return(FALSE);
	}

	if (strcmp(strings[0],EDI_FTAM_CONTROL_FILE) != 0)	{
		sprintf(log_str,"Control file not in proper format\n	'%s' not first line",strings[0]);
		logit(log_str);
		return(FALSE);
	}

	/* build the Dir_dn structures for the source and destination	*/
	if (source_dir != 0)	p = source_dir;
	else	if ((p = find_target(strings,SOURCE_DIRNAME)) != 0)
			p += (sizeof(SOURCE_DIRNAME) - 1);
		else	if (defaults)
				p = DEF_SRC_DIRNAME;
			else	p = 0;
	if (build_ddn(source_dirname,p) != TRUE)	{
		logit("Unable to build source directory name\n");
		return(FALSE);
	}
print_ddn(source_dirname);
	if (destination_dir != 0)	p = destination_dir;
	else	if ((p = find_target(strings,DESTINATION_DIRNAME)) != 0)
			p += (sizeof(DESTINATION_DIRNAME) - 1);
		else	if (defaults)
				p = DEF_DEST_DIRNAME;
			else	p = 0;
	if (build_ddn(destination_dirname,p) != TRUE)	{
		logit("Unable to build destination directory name\n");
		return(FALSE);
	}
print_ddn(destination_dirname);

	/* initialize the source and destination filenames, if 	*/
	/* source_file is rooted (not relative) use it directly.*/
	if ((source_file != 0) && (*source_file == '/'))
		*source_filename = source_file;
	else	*source_filename =
			make_filename(find_target(strings,
				SOURCE_DIRECTORY),
				sizeof(SOURCE_DIRECTORY) -1, source_file);
	/* use supplied destination_file directly if rooted	*/
	if ((destination_file != 0) && (*destination_file == '/'))
		*destination_filename = destination_file;
	else	*destination_filename = 
			make_filename(find_target(strings,
				DESTINATION_DIRECTORY),
				sizeof(DESTINATION_DIRECTORY) -1,
				destination_file);
sprintf(log_str,"\tsource_filename = '%s'\n\tdestination_filename = '%s'\n", *source_filename, *destination_filename);
logit(log_str);

	/* initialize ae_label					*/
	ae_label = NULL;

	/* initialize the return_event_name			*/
	*return_event_name = SYNCHRONOUS;

	/* initialize the input_dcb				*/
	if (ft_didcb(FTiFCopy,0,(Octet **)input_dcb,&outcome) != SUCCESS)
		error_handler(outcome, diag);

	/* initialize the source_init_id field				*/
	if ((p = find_target(strings,SOURCE_INIT_ID)) != 0)	{
		p += (sizeof(SOURCE_INIT_ID) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_SRC_INIT_ID;
		len = sizeof(DEF_SRC_INIT_ID) -1;
	}
	if (len > 0)	{
sprintf(log_str,"src_init_id = '%s' len = %d.\n",p,len);
logit(log_str);
		(*input_dcb)->source_init_id = malloc(len + 1);
		strcpy((*input_dcb)->source_init_id,p);
	}

	/* initialize the source_filestore_pw field			*/
	if ((p = find_target(strings,SOURCE_FILESTORE_PW)) != 0)	{
		p += (sizeof(SOURCE_FILESTORE_PW) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_SRC_FILESTORE_PW;
		len = sizeof(DEF_SRC_FILESTORE_PW) -1;
	}
	if (len > 0)	{
sprintf(log_str,"src_filestore_pw = '%s' len = %d.\n",p,len);
logit(log_str);
		(*input_dcb)->source_filestore_pw.pointer = (Octet *)malloc (len + 1);
		(*input_dcb)->source_filestore_pw.length = len;
		strcpy((*input_dcb)->source_filestore_pw.pointer,p);
	}

	/* initialize the source_account field				*/
	if ((p = find_target(strings,SOURCE_ACCOUNT)) != 0)	{
		p += (sizeof(SOURCE_ACCOUNT) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_SRC_ACCOUNT;
		len = sizeof(DEF_SRC_ACCOUNT) -1;
	}
	if (len > 0)	{
		(*input_dcb)->source_account = malloc(len + 1);
		strcpy((*input_dcb)->source_account,p);
	}

	/* Initialize the SOURCE Ft_file_passwords structure, use 	*/
	/* defaults where none are supplied. Passwords will be strings.	*/
	file_passwords = &((*input_dcb)->source_file_passwords);
	if ((p = find_target(strings,SOURCE_PW_READ)) != 0)	{
		p += (sizeof(SOURCE_PW_READ) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_SRC_PW_READ;
		len = sizeof(DEF_SRC_PW_READ) -1;
	}
	if (len > 0)	{
		file_passwords->read.length = len;
		file_passwords->read.pointer = (Octet *)malloc(len +1);
		strcpy(file_passwords->read.pointer,p);
	}
	if ((p = find_target(strings,SOURCE_PW_INSERT)) != 0)	{
		p += (sizeof(SOURCE_PW_INSERT) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_SRC_PW_INSERT;
		len = sizeof(DEF_SRC_PW_INSERT) -1;
	}
	if (len > 0)	{
		file_passwords->insert.length = len;
		file_passwords->insert.pointer = (Octet *)malloc(len + 1);
		strcpy(file_passwords->insert.pointer,p);
	}
	if ((p = find_target(strings,SOURCE_PW_REPLACE)) != 0)	{
		p += (sizeof(SOURCE_PW_REPLACE) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_SRC_PW_REPLACE;
		len = sizeof(DEF_SRC_PW_REPLACE) -1;
	}
	if (len > 0)	{
		file_passwords->replace.length = len;
		file_passwords->replace.pointer = (Octet *)malloc(len + 1);
		strcpy(file_passwords->replace.pointer,p);
	}
	if ((p = find_target(strings,SOURCE_PW_EXTEND)) != 0)	{
		p += (sizeof(SOURCE_PW_EXTEND) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_SRC_PW_EXTEND;
		len = sizeof(DEF_SRC_PW_EXTEND) -1;
	}
	if (len > 0)	{
		file_passwords->extend.length = len;
		file_passwords->extend.pointer = (Octet *)malloc(len + 1);
		strcpy(file_passwords->extend.pointer,p);
	}
	if ((p = find_target(strings,SOURCE_PW_ERASE)) != 0)	{
		p += (sizeof(SOURCE_PW_ERASE) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_SRC_PW_ERASE;
		len = sizeof(DEF_SRC_PW_ERASE) -1;
	}
	if (len > 0)	{
		file_passwords->erase.length = len;
		file_passwords->erase.pointer = (Octet *)malloc(len + 1);
		strcpy(file_passwords->erase.pointer,p);
	}
	if ((p = find_target(strings,SOURCE_PW_READ_ATTR)) != 0)	{
		p += (sizeof(SOURCE_PW_READ_ATTR) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_SRC_PW_READ_ATTR;
		len = sizeof(DEF_SRC_PW_READ_ATTR) -1;
	}
	if (len > 0)	{
		file_passwords->read_attribute.length = len;
		file_passwords->read_attribute.pointer = (Octet *)malloc(len + 1);
		strcpy(file_passwords->read_attribute.pointer,p);
	}
	if ((p = find_target(strings,SOURCE_PW_CHANGE_ATTR)) != 0)	{
		p += (sizeof(SOURCE_PW_CHANGE_ATTR) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_SRC_PW_CHANGE_ATTR;
		len = sizeof(DEF_SRC_PW_CHANGE_ATTR) -1;
	}
	if (len > 0)	{
		file_passwords->change_attribute.length = len;
		file_passwords->change_attribute.pointer = (Octet *)malloc (len + 1);
		strcpy(file_passwords->change_attribute.pointer,p);
	}
	if ((p = find_target(strings,SOURCE_PW_DELETE_FILE)) != 0)	{
		p += (sizeof(SOURCE_PW_DELETE_FILE) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_SRC_PW_DELETE_FILE;
		len = sizeof(DEF_SRC_PW_DELETE_FILE) -1;
	}
	if (len > 0)	{
		file_passwords->delete_file.length = len;
		file_passwords->delete_file.pointer = (Octet *)malloc(len+ 1);
		strcpy(file_passwords->delete_file.pointer,p);
	}
	
	/* initialize the dest_init_id field				*/
	if ((p = find_target(strings,DESTINATION_INIT_ID)) != 0)	{
		p += (sizeof(DESTINATION_INIT_ID) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_DEST_INIT_ID;
		len = sizeof(DEF_DEST_INIT_ID) -1;
	}
	if (len > 0)	{
		(*input_dcb)->dest_init_id = malloc(len + 1);
		strcpy((*input_dcb)->dest_init_id,p);
	}

	/* initialize the dest_filestore_pw				*/
	if ((p = find_target(strings,DESTINATION_FILESTORE_PW)) != 0)	{
		p += (sizeof(DESTINATION_FILESTORE_PW) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_DEST_FILESTORE_PW;
		len = sizeof(DEF_DEST_FILESTORE_PW) -1;
	}
	if (len > 0)	{
		(*input_dcb)->dest_filestore_pw.pointer = (Octet *)malloc (len + 1);
		(*input_dcb)->dest_filestore_pw.length = len;
		strcpy((*input_dcb)->dest_filestore_pw.pointer,p);
	}

	/* initialize the dest_account field				*/
	if ((p = find_target(strings,DESTINATION_ACCOUNT)) != 0)	{
		p += (sizeof(DESTINATION_ACCOUNT) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_DEST_ACCOUNT;
		len = sizeof(DEF_DEST_ACCOUNT) - 1;
	}
	if (len > 0)	{
		(*input_dcb)->dest_account = malloc(len + 1);
		strcpy((*input_dcb)->dest_account,p);
	}

	/* Initialize the DESTINATION Ft_file_passwords structure, use 	*/
	/* defaults where none are supplied. Passwords will be strings.	*/
	file_passwords = &((*input_dcb)->dest_file_passwords);
	if ((p = find_target(strings,DESTINATION_PW_READ)) != 0)	{
		p += (sizeof(DESTINATION_PW_READ) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_DEST_PW_READ;
		len = sizeof(DEF_DEST_PW_READ) -1;
	}
	if (len > 0)	{
		file_passwords->read.length = len;
		file_passwords->read.pointer = (Octet *)malloc(len +1);
		strcpy(file_passwords->read.pointer,p);
	}
	if ((p = find_target(strings,DESTINATION_PW_INSERT)) != 0)	{
		p += (sizeof(DESTINATION_PW_INSERT) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_DEST_PW_INSERT;
		len = sizeof(DEF_DEST_PW_INSERT) -1;
	}
	if (len > 0)	{
		file_passwords->insert.length = len;
		file_passwords->insert.pointer = (Octet *)malloc(len + 1);
		strcpy(file_passwords->insert.pointer,p);
	}
	if ((p = find_target(strings,DESTINATION_PW_REPLACE)) != 0)	{
		p += (sizeof(DESTINATION_PW_REPLACE) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_DEST_PW_REPLACE;
		len = sizeof(DEF_DEST_PW_REPLACE) -1;
	}
	if (len > 0)	{
		file_passwords->replace.length = len;
		file_passwords->replace.pointer = (Octet *)malloc(len + 1);
		strcpy(file_passwords->replace.pointer,p);
	}
	if ((p = find_target(strings,DESTINATION_PW_EXTEND)) != 0)	{
		p += (sizeof(DESTINATION_PW_EXTEND) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_DEST_PW_EXTEND;
		len = sizeof(DEF_DEST_PW_EXTEND) -1;
	}
	if (len > 0)	{
		file_passwords->extend.length = len;
		file_passwords->extend.pointer = (Octet *)malloc(len + 1);
		strcpy(file_passwords->extend.pointer,p);
	}
	if ((p = find_target(strings,DESTINATION_PW_ERASE)) != 0)	{
		p += (sizeof(DESTINATION_PW_ERASE) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_DEST_PW_ERASE;
		len = sizeof(DEF_DEST_PW_ERASE) -1;
	}
	if (len > 0)	{
		file_passwords->erase.length = len;
		file_passwords->erase.pointer = (Octet *)malloc(len + 1);
		strcpy(file_passwords->erase.pointer,p);
	}
	if ((p = find_target(strings,DESTINATION_PW_READ_ATTR)) != 0)	{
		p += (sizeof(DESTINATION_PW_READ_ATTR) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_DEST_PW_READ_ATTR;
		len = sizeof(DEF_DEST_PW_READ_ATTR) -1;
	}
	if (len > 0)	{
		file_passwords->read_attribute.length = len;
		file_passwords->read_attribute.pointer = (Octet *)malloc(len + 1);
		strcpy(file_passwords->read_attribute.pointer,p);
	}
	if ((p = find_target(strings,DESTINATION_PW_CHANGE_ATTR)) != 0)	{
		p += (sizeof(DESTINATION_PW_CHANGE_ATTR) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_DEST_PW_CHANGE_ATTR;
		len = sizeof(DEF_DEST_PW_CHANGE_ATTR) -1;
	}
	if (len > 0)	{
		file_passwords->change_attribute.length = len;
		file_passwords->change_attribute.pointer = (Octet *)malloc (len + 1);
		strcpy(file_passwords->change_attribute.pointer,p);
	}
	if ((p = find_target(strings,DESTINATION_PW_DELETE_FILE)) != 0)	{
		p += (sizeof(DESTINATION_PW_DELETE_FILE) - 1);
		len = strlen(p);
	}	else	{
		p = DEF_DEST_PW_DELETE_FILE;
		len = sizeof(DEF_DEST_PW_DELETE_FILE) -1;
	}
	if (len > 0)	{
		file_passwords->delete_file.length = len;
		file_passwords->delete_file.pointer = (Octet *)malloc(len + 1);
		strcpy(file_passwords->delete_file.pointer,p);
	}

	/* initialize the create_file_pw, use default if exists		*/
	if ((p = find_target(strings,CREATE_FILE_PW)) != 0)	{
		p += (sizeof(CREATE_FILE_PW) -1);
		len = strlen(p);
	}	else	{
		p = DEF_CREATE_FILE_PW;
		len = sizeof(DEF_CREATE_FILE_PW) -1;
	}
	if (len > 0)	{
		(*input_dcb)->create_file_pw.pointer = (Octet *)malloc(len+ 1);
		(*input_dcb)->create_file_pw.length = len;
		strcpy((*input_dcb)->create_file_pw.pointer,p);
	}

	/* initialize the overwrite field				*/
	(*input_dcb)->overwrite = FT_RECREATE_FILE;

	/* initialize the inout_dcb					*/
	*inout_dcb = NULL;

	return(TRUE);
}

	build_ddn(dir_name, ddn_string)
Ae_dir_name	*dir_name;	/* pointer to resultant Dir_dn struct	*/
char		*ddn_string;	/* /C=us/O=hp/OU=unit/CN=name/CN=resp/	*/
{
	struct	Dir_dn	*ddn;
	struct	Dir_rdn	*rdn;
	struct	Dir_ava	*ava;
	char	*p, *p1, *p2, attr[64];
	int	i = 0, j = 0, k = 0, l = 0, ret = TRUE;

/* debug */ sprintf(log_str,"build_ddn: dir_name = %o, ddn_string = '%s' \n", dir_name, ddn_string);
/* debug */ logit(log_str);
	if (!ddn_string || !(*ddn_string))	{
		/* null ddn => this machine	*/
		*dir_name = 0;
		return(TRUE);
	}
	ddn = (struct Dir_ddn *)malloc(sizeof(struct Dir_dn));
	if (*(ddn_string) != '/')	return(FALSE);
	/* Find the number of Dir_rdn structures.  Each Dir_rdn will	*/
	/* contain one Dir_ava structure.  Allow '/' on end of string.	*/
	for (p = ddn_string; *p != '\0'; p++)
		if ((*p == '/') && (*(p+1) != '\0'))	k++;
	rdn = (struct Dir_rdn *)malloc(k * sizeof(struct Dir_rdn));
	ddn->rdn = rdn;
	ddn->n = 0;	/* count of current rdn structures	*/

	/* parse ddn_string, put each entry in a Dir_ava structure	*/
	/* and each Dir_ava structure in one Dir_rdn structure.		*/
	/*								*/
	/* Starting after first '/', find each attribute name (scan up	*/
	/* to '='), find its value (scan up to '/' or NULL) and build	*/
	/* the structures.						*/
	for (p = ddn_string + 1, l = 0; l < k; l++, rdn++)	{
		p1 = p;
		while (*p && (*p != '='))	p++;
		if (!(*p))	{
			delete_ddn(&ddn);
			ret = FALSE;
			break;
		}
		i = p - p1;	/* length of attribute name	*/
		*p++;
		p2 = p;
		while (*p && (*p != '/'))	p++;
		if (!(*p) && (*(p-1) == '='))	{
			delete_ddn(&ddn);
			ret = FALSE;
			break;
		}
		j = p - p2;	/* length of attribute value	*/
		*p++;

		/* create the Dir_ava structure and add value	*/
		ava = (struct Dir_avs *)malloc(sizeof(struct Dir_ava));
		rdn->n = 1;
		rdn->avas = ava;
		ava->attr_id.length = 1;
		ava->attr_id.element = (Sint32 *)malloc(sizeof(Sint32));
		ava->attr_value.pointer = (Octet *)malloc ((j+1)*sizeof (Octet));
		ava->attr_value.length = j;
		memcpy(ava->attr_value.pointer,p2,j);
		*(ava->attr_value.pointer + j) = '\0';
		ava->syntax_id.length = 0;
		ava->syntax_id.element = 0;
		
		/* find which attribute this is and set element id	*/
		strncpy(attr,p1,i);
		attr[i] = '\0';
		if (strcmp(attr,"C") == 0)	{
			/* country	*/
			ava->attr_id.element[0] = 6;
		} else if(strcmp(attr,"L") == 0)	{
			/* Locality	*/
			ava->attr_id.element[0] = 7;
		} else if(strcmp(attr,"O") == 0)	{
			/* Organization	*/
			ava->attr_id.element[0] = 10;
		} else if(strcmp(attr,"OU") == 0)	{
			/* Organizational Unit	*/
			ava->attr_id.element[0] = 11;
		} else if((strcmp(attr,"CN") == 0) ||
			(strcmp(attr,"AE") == 0) ||
			(strcmp(attr,"AP") == 0))	{
			/* Common Name, Application Entity or	*/
			/* Application Process			*/
			ava->attr_id.element[0] = 3;
		} else	{
			/* Unsupported		*/
			ava->attr_id.element[0] = 9999;
		}
		ddn->n++;	/* rdn successfully added	*/
	}
	*dir_name = ddn;	/* delete_ddn sets parm to zero	*/
	return(ret);
}

void	delete_ddn(ddnp)
struct	Dir_dn	**ddnp;
{
	int	i, n;
	struct	Dir_rdn	*rdn;

	if ((ddnp == 0) || (*ddnp == 0))	return;
	for (i = 0, n = (*ddnp)->n, rdn = (*ddnp)->rdn; i < n; rdn++, i++)
		delete_rdn(&rdn);
	free(*ddnp);
	*ddnp = 0;
	return;
}

void	delete_rdn(rdnp)
struct	Dir_rdn	*rdnp;
{
	struct	Dir_ava	*ava;
	int	i, n;
	if (rdnp == 0)	return;
	for (i =0, n = rdnp->n, ava = rdnp->avas; i < n; ava++, i++)
		delete_ava(ava);
	free(rdnp);
	return;
}

void	delete_ava(avap)
struct	Dir_ava	*avap;
{
	if (avap == 0)	return;
	delete_object_id(avap->attr_id);
	delete_octet_string(avap->attr_value);
	delete_object_id(avap->syntax_id);
	free(avap);
	return;
}

void	delete_object_id(idp)
struct	Object_id	*idp;
{
	if ((idp != 0) && (idp->element != 0))	{
		free(idp->element);
		idp->element = 0;
	}
	return;
}

void	delete_octet_string(octp)
struct	Octet_string	*octp;
{
	if ((octp != 0) & (octp->pointer != 0))	{
		free(octp->pointer);
		octp->pointer = 0;
	}
	return;
}

void	print_ddn(ddnp)
struct	Dir_dn	**ddnp;
{
	int	i, n;
	struct	Dir_rdn	*rdn;

	if ((ddnp == 0) || (*ddnp == 0))	return;
	sprintf(log_str,"Dir_ddn: ddnp = %o *ddnp = %o n = %d rdnp = %o\n", ddnp, *ddnp, (*ddnp)->n, (*ddnp)->rdn);
	logit(log_str);
	for (i = 0, n = (*ddnp)->n, rdn = (*ddnp)->rdn; i < n; rdn++, i++)
		print_rdn(rdn);
logit("print_ddn: finished\n");
	return;
}

void	print_rdn(rdnp)
struct	Dir_rdn	*rdnp;
{
	struct	Dir_ava	*ava;
	int	i, n;
	if (rdnp == 0)	return;
	sprintf(log_str,"Dir_rdn: n = %d.\n",rdnp->n);
	logit(log_str);
	for (i =0, n = rdnp->n, ava = rdnp->avas; i < n; ava++, i++)
		print_ava(ava);
	return;
}

void	print_ava(avap)
struct	Dir_ava	*avap;
{
	if (avap == 0)	return;
	logit("Dir_ava:\n");
	print_object_id(&avap->attr_id,"attr_id");
	print_octet_string(&avap->attr_value,"attr_value");
	print_object_id(&avap->syntax_id,"syntax_id");
	return;
}

void	print_object_id(idp,type)
struct	Object_id	*idp;
char	*type;
{
	int	i;
	if ((idp != 0) && (idp->element != 0))	{
		sprintf(log_str,"object_id(%s): length = %d. element = %d.",
			type,idp->length,idp->element[0]);
		logit(log_str);
/*		for (i = 0; i < idp->length; i++)	{	*/
/*			printf("%o ",idp->element[i]);		*/
/*			logit(log_str);				*/
/*		}						*/
		logit("\n");
	}
	return;
}

void	print_octet_string(octp,type)
struct	Octet_string	*octp;
char	*type;
{
	int	i;
	if ((octp != 0) & (octp->pointer != 0))	{
		sprintf(log_str,"Octet_string(%s): length = %d. Octet = %s",
			type,octp->length,octp->pointer);
		logit(log_str);
/*		for (i = 0; i < octp->length; i++)			*/
/*			sprintf(log_str,"%o ",octp->pointer[i]);	*/
/*			logit(log_str);					*/
/*		}							*/
		logit("\n");
	}
	return;
}

char	*make_filename(dir,len,fname)
char	*dir;
int	len;
char	*fname;
{
	char	*p,*base;
	long	ltime,pid;

	base = malloc(256);
	if ((dir == 0) || (*dir == 0))	p = base;
	else	{
		strcpy(base, dir + len);
		p = base + strlen(base);
		if (*(p - 1) != '/')	*p++ = '/';
	}
	if (fname)	strcpy(p,fname);
	else	{
		ltime = time(0);
		ltime &= 0x00ffffff;
		pid = getpid();
		sprintf(p,"%s%ld%ld","NIST_",(long)ltime,(long)pid);
	}
	return(base);
}
