// FILE:  platio.cc

#include	<fstream.h>
#include	<xom.h>
#include	"../../../include/platform.h"
#include	"../../../include/log.h"
#include	"../../../include/platio.h"


//
// cont()
//
// Called By:      public
//
// This method tests for the existence of the a file named STOP_FILE (see 
// platform.h).  If this file exists, the platform will exit gracefully.
//

STATUS PlatformIO::cont() {
	ifstream	stop_file( STOP_FILE, ios::in );

	if ( stop_file ) {
#ifdef	DEBUG
		cout << "cont() status is SHUTDOWN" << endl;
#endif
		return ERROR;
	}
	else {
#ifdef	DEBUG
		cout << "cont() status is CONTINUE" << endl;
#endif
		return SUCCESS;
	}
}


//
// halt()
//
// Called By:      public
//
// This method creates the STOP_FILE to shutdown the platform gracefully.  The 
// method is invoked by a process independent of the platform process.
//

STATUS PlatformIO::halt() {
	ofstream	stop_file( STOP_FILE, ios::out );

	if ( stop_file )
		return ERROR;
	else
		return SUCCESS;

}


//
// mod_logging():
//
// Called By:      public
//
// If the user requested to modify the log level or log file, make the    
// change here.
//

void PlatformIO::mod_logging( Log *log ) {
	ifstream	modfile( LOG_MOD_FILE, ios::in );
	
	if ( ! modfile )
		// No logging modifications currently requested
		return;
	
	char	new_loglevel;
	char	new_logfile[ MAX_FILENAME ];

	modfile.get( new_loglevel );
	if ( new_loglevel != NO_LOGLEVEL_MOD )
		log->set_loglevel( ( LOGLEVEL ) ( new_loglevel - 0x30 ) );

	modfile.getline( new_logfile, MAX_FILENAME - 1 );
	if ( new_logfile[ 0 ] )
		log->set_logfile( new_logfile );

	// unlink LOG_MOD_FILE
}
