/*				FILE = bld_host.c			*/

/*	This function will read commands from a file, from 'start' to	*/
/*	'end', and build the array of 'host' structures.		*/

#include	<string.h>
#include	"xmit_file.h"
#include	"edi_control_defs.h"
#include	"van_control_defs.h"

struct	host	*host_merge();

	FUNCTION bld_host(fd,start,end,buf,strings,number)
int	fd,number;
long	start,end;
char	*buf,**strings;
{
	int	skip,len,num_lines,i,j,k,index,ret = 1,defaults,remote_echo;
	int	rec_len;
	char	*host_id, *def_flags, *pdir, *gdir, *separator, *terminator;
	char	*null_file, *telnbr, *p1, *p2;
	struct	host *p, *host_ptr;

	if ((start <= 0) || (end <= 0) || ((end - start) <= 0))
		return(0);
	lseek(fd,start,0);
	if ((len = read(fd,buf,(end - start))) < 0)	{
		logit("bld_hosts: Unable to read HOST definitions\n");
		return(0);
	}
	if ((num_lines = lines_to_strings(buf,len,strings,number)) == 0) {
		logit("bld-hosts: No HOST definitions\n");
		return(0);
	}
	if ((i = count_target(strings,HOST_BEGIN_DEFINITION)) < 1)	{
		logit("bld_host: No HOSTs defined\n");
		return(0);
	}
	if((host_ptr = (struct host *)calloc((i+1),sizeof(struct host))) <= 0) {
		logit("bld_host: unable to get memory for HOST table\n");
		return(-1);
	}
	for (p = host_ptr, j = 0, index = 0; j < i; j++)	{
/* sprintf(log_str,"bld_host: processing line [%d.] = '%s'\n",index, strings[index]); */
/* logit(log_str); */
		if (strncmp(HOST_BEGIN_DEFINITION,strings[index], sizeof(HOST_BEGIN_DEFINITION) -1) != 0)	{
			logit("bld_host: HOST_BEGIN_DEFINITION not first entry - all entries skipped\n");
			return(0);
		}
		skip = FALSE;
		/* set defaults for a host struct, the rest are zero	*/
		host_id = strings[index]+sizeof(HOST_BEGIN_DEFINITION)-1;
		if (*host_id == '\0')	{
			sprintf(log_str,"bld_host: no HOST specified, skipped\n");
			logit(log_str);
			skip = TRUE;
		}
		def_flags = 0;
		remote_echo = NOT_KNOWN;
		pdir = DEFAULT_PDIR;
		gdir = DEFAULT_GDIR;
		separator = DEFAULT_SEPARATOR;
		terminator = DEFAULT_TERMINATOR;
		null_file = 0;
		rec_len = 0;
		defaults = DEFAULT_DEFAULTS;
		telnbr = 0;
		while (1)	{
			if (++index >= num_lines)	{
				logit("bld_host: warning: command ended without HOST_END_DEFINITION\n");
				break;
			}
/*sprintf(log_str,"bld_host: processing line [%d.] = '%s'\n",index, strings[index]);*/
/*logit(log_str);*/
			if (strncmp(strings[index],HOST_PDIR, sizeof(HOST_PDIR) -1) == 0)	{
				pdir = strings[index]+sizeof(HOST_PDIR)-1;
				continue;
			}
			if (strncmp(strings[index],HOST_GDIR, sizeof(HOST_GDIR) -1) == 0)	{
				gdir = strings[index]+sizeof(HOST_GDIR)-1;
				continue;
			}
			if (strncmp(strings[index],HOST_SEPARATOR, sizeof(HOST_SEPARATOR) -1 ) == 0)	{
				separator = strings[index] + sizeof(HOST_SEPARATOR) -1;
				continue;
			}
			if (strncmp(strings[index],HOST_TERMINATOR, sizeof(HOST_TERMINATOR) -1) == 0)	{
				terminator = strings[index] + sizeof(HOST_TERMINATOR) -1;
				continue;
			}
			if (strncmp(strings[index],HOST_NULL_FILE, sizeof(HOST_NULL_FILE) -1) == 0)	{
				null_file = strings[index] + sizeof(HOST_NULL_FILE) -1;
				continue;
			}
			if (strncmp(strings[index],HOST_REC_LEN, sizeof(HOST_REC_LEN) -1) == 0)	{
				rec_len = ctob(strings[index] + sizeof(HOST_REC_LEN) -1);
				continue;
			}
			if (strncmp(strings[index],HOST_DEFAULTS, sizeof(HOST_DEFAULTS) -1) == 0)	{
				defaults = 0;
				p1 = p2 = strings[index] + sizeof(HOST_DEFAULTS) -1;
				while (*p1)	{
					if ((p2 = strchr(p1,'|')) == 0)	{
						if ((k = chk_host_flag (p1)) == 0)
							defaults = DEFAULT_DEFAULTS;
						else	defaults |= k;
						break;
					}
					*p2 = '\0';
					if ((k = chk_host_flag(p1)) == 0) {
						defaults = DEFAULT_DEFAULTS;
						break;
					}
					defaults |= k;
					p1 = p2 + 1;
				}
				continue;
			}
			if (strncmp(strings[index],HOST_TELNBR, sizeof(HOST_TELNBR) -1) == 0)	{
				telnbr = strings[index]+sizeof(HOST_TELNBR) -1;
				continue;
			}
			if (strncmp(strings[index],BEGIN_RESPOND_INLINE, sizeof(BEGIN_RESPOND_INLINE) -1) == 0)	{
				if ((host_id == 0) || (*host_id == 0))	{
					logit("bld_host: host_id must be defined before BEGIN_RESPOND_INLINE\n");
					skip = TRUE;
				}
				if ((k = index_target(&strings[index], END_RESPOND_INLINE)) < 0)	{
					logit("bld_host: no END_RESPOND_INLINE command\n");
					return(0); /* can't find end */
				}
/*sprintf(log_str,"bld_host: index = %d. k = %d. skip = %d.\n",index,k, skip);*/
/*logit(log_str);*/
				if (skip == FALSE)
					if(begin_respond(&strings[index], k+1,p) < 0)
						skip = TRUE;
				index += k;
/*sprintf(log_str,"bld_host: respond_inline, skip %d. commands\n",k);*/
/*logit(log_str);*/
				continue;
			}
			if (strncmp(strings[index],HOST_END_DEFINITION, sizeof(HOST_END_DEFINITION) -1) == 0)	{
				index++;
				break;
			}
			if (strncmp(strings[index],HOST_BEGIN_DEFINITION, sizeof(HOST_BEGIN_DEFINITION) -1) == 0)	{
				logit("bld_host: warning: no HOST_END_DEFINITION, HOST_BEGIN_DEFINITION encountered\n");
				break;
			}
			sprintf(log_str,"bld_host: warning: Unknown string '%s'\n", strings[index]);
			logit(log_str);
		}
		if (skip)	{
			logit("bld_host: skip = TRUE\n");
			continue;
		}
		/* fill host struct with default and supplied values	*/
		if ((host_id == 0) || (*host_id == 0))	{
			logit("bld_host: host_id not specified\n");
		}	else	{
			p->host_id = (char *)malloc(strlen(host_id)+1);
			atos(p->host_id,host_id);
			if ((def_flags != 0) && (*def_flags != 0))	{
				p->def_flags=(char *)malloc(strlen (def_flags)+1);
				atos(p->def_flags,def_flags);
			}
			if ((pdir != 0) && (*pdir != 0))	{
				p->pdir = (char *)malloc(strlen(pdir)+1);
				atos(p->pdir,pdir);
			}
			if ((gdir != 0) && (*gdir != 0))	{
				p->gdir = (char *)malloc(strlen(gdir)+1);
				atos(p->gdir,gdir);
			}
			if ((separator != 0) && (*separator != 0))	{
				p->separator=(char *)malloc(strlen (separator)+1);
				atos(p->separator,separator);
			}
			if ((terminator != 0) && (*terminator != 0))	{
				p->terminator=(char *)malloc(strlen (terminator)+1);
				atos(p->terminator,terminator);
			}
			if ((null_file != 0) && (*null_file != 0))	{
				p->null_file=(char *)malloc(strlen (null_file)+1);
				atos(p->null_file,null_file);
			}
			p->rec_len = rec_len;
			p->connected = 0;
			p->calling = 0;
			p->defaults = defaults;
			p->remote_echo = remote_echo;
			if ((telnbr != 0) && (*telnbr != 0))	{
				p->telnbr=(char *)malloc(strlen(telnbr)+1);
				atos(p->telnbr,telnbr);
			}
			p->tt_index = 0;
/* debug		prt_host(p,FALSE);*/
/* debug 	sprintf(log_str,"bld_host: built %s\n",p->host_id); */
/* debug 	logit(log_str); */
			p++;
		}
	}
	if ((hostp = host_merge(hostp,host_ptr)) == 0)	return(-1);
/*sprintf(log_str,"bld_host: ret = %d.\n",ret);*/
/*logit(log_str);*/
	return(ret);
}


	FUNCTION chk_host_flag(p)
char	*p;
{
	int	value = 0;
	if (strcmp(p,"XMIT_GET") == 0)
			value = XMIT_GET;
	else	if (strcmp(p,"XMIT_PUT") == 0)
			value = XMIT_PUT;
	else	if (strcmp(p,"XMIT_BOTH") == 0)
			value = XMIT_BOTH;
	else	if (strcmp(p,"MOD_G_ST") == 0)
			value = MOD_G_ST;
	else	if (strcmp(p,"MOD_P_ST") == 0)
			value = MOD_P_ST;
	else	if (strcmp(p,"SET_REMOTE_MODE") == 0)
			value = SET_REMOTE_MODE;
	else	if (strcmp(p,"SPLIT_BOTH") == 0)
			value = SPLIT_BOTH;
	else	if (strcmp(p,"P_ASCII") == 0)
			value = P_ASCII;
	else	if (strcmp(p,"P_XMODEM") == 0)
			value = P_XMODEM;
	else	if (strcmp(p,"G_ASCII") == 0)
			value = G_ASCII;
	else	if (strcmp(p,"G_XMODEM") == 0)
			value = G_XMODEM;
	else	if (strcmp(p,"PUT_DEFAULT_FILE") == 0)
			value = PUT_DEFAULT_FILE;
	else	if (strcmp(p,"GET_DEFAULT_FILE") == 0)
			value = GET_DEFAULT_FILE;
	else	if (strcmp(p,"ALL_IN_ONE_FILE") == 0)
			value = ALL_IN_ONE_FILE;
	else	{
		sprintf(log_str,"chk_host_flag: warning unrecognized flag '%s', flags skipped\n",p);
		logit(log_str);
	}
	return(value);
}
