/*				FILE = bld_respond.c			*/

/*	This function will read commands from a file, for 'start' to	*/
/*	'end', and build the 'respond' structures.			*/

#include	"xmit_file.h"
#include	"edi_control_defs.h"
#include	"van_control_defs.h"

	FUNCTION bld_respond(fd,start,end,buf,strings,number)
int	fd,number;
long	start,end;
char	*buf,**strings;
{
	int	len, num_lines, index,  i, j, k, ret = 1;

	len = end - start;
	if ((start <= 0) || (end <= 0) || (len <= 0))
		return(0);
	lseek(fd,start,0);
/* sprintf(log_str,"bld_respond: start = %d. end = %d.\n",start,end); */
/* logit(log_str); */
	if (read(fd,buf,len) != len)	{
		logit("bld_respond: error - unable to read commands from file\n");
		return(-1);
	}
	if ((num_lines = lines_to_strings(buf,len,strings,number)) == 0) {
		logit("bld_respond: no RESPOND commands\n");
		return(0);
	}
	if ((i = count_target(strings,BEGIN_RESPOND_HOST)) < 1)	{
		logit("bld_respond: no RESPONDs defined\n");
		return(0);
	}
	if (hostp == 0)	{
		logit("bld_respond: no HOST structure\n");
		return(0);
	}
	for (j = 0, index = 0; j < i; j++)	{
		if (strncmp(strings[index],BEGIN_RESPOND_HOST,sizeof (BEGIN_RESPOND_HOST) -1) != 0)	{
			sprintf(log_str,"bld_respond: error - BEGIN_RESPOND_HOST not first entry\n	found - '%s'\n",strings[index]);
			logit(log_str);
			return(0);
		}
		if ((k = index_target(&strings[index],END_RESPOND_HOST)) < 0)	{
			logit("bld_respond: error - no END_RESPOND_HOST\n");
			return(-1);
		}
		if (begin_respond(&strings[index],k+1,hostp) < 0)
			return(-1);
		index += (k + 1);
	}
	return(ret);
}

/*	Process the RESPOND commands for one HOST.  Build the		*/
/*	structures and link to the proper HOST entry.			*/
	FUNCTION begin_respond(strings,num,host)
char	*strings[];
int	num;
struct	host	*host;
{
	int	type,i,j,k,respond_type,l1 = -1,l2 = -1,l3 = -1,ret = -1;
	char	*temp, **p, *end, *id, *savep, *ptr1, *ptr2;
	struct	host	*p1;	/* Link RESPOND to this HOST	*/
	struct	respond	*rptr;
	int	flags, time, no_response, alt1_action, alt2_action, attempts, lower;
	char	*cmd_str, *response, *alt1, *alt2, *reresponse;

	p = strings;
	if (strncmp(*p,BEGIN_RESPOND_HOST,sizeof(BEGIN_RESPOND_HOST)-1)== 0)	{
		id = *p + sizeof(BEGIN_RESPOND_HOST) - 1;
		for (p1 = host; (p1 != 0) && (p1->host_id != 0); p1++)
			if (strcmp(p1->host_id,id) == 0)	break;
		if (!(p1) || !(p1->host_id))	{
			sprintf(log_str,"begin_respond: error - unknown HOST_ID - '%s'\n",*p);
			logit(log_str);
			return(-1);
		}
		end = END_RESPOND_HOST;
		type =0;
	}	else	if (strcmp(*p,BEGIN_RESPOND_INLINE) == 0)	{
			p1 = host;
			end = END_RESPOND_INLINE;
			type = 1;
	}	else	{
		sprintf(log_str,"begin_respond: error - unrecognized command '%s'\n",*p);
		logit(log_str);
		return(-1);
	}
	savep = strings[num];	/* make this HOST RESPOND the only one	*/
	strings[num] = 0;
	for (p++, respond_type = 0, i = 1; i < num; i++, p++)	{
		if (strcmp(*p,end) == 0)	{
			if (i != (num -1))	{
				logit("begin_rspond: END_RESPOND_HOST/INLINE not last command, remaining commands skipped\n");
				goto	save;
			}
			if (respond_type == 0)	break;
			l3 = 0;	/* treat as RESPOND_END_DEFINITION	*/
		}
/* debug  sprintf(log_str,"bld_respond: string = %s\n",*p); */
/* debug  logit(log_str); */
		if (respond_type == 0)	{
			if (strncmp(*p,RESPOND_BEGIN_DEFINITION, sizeof(RESPOND_BEGIN_DEFINITION) - 1) != 0)	{
				sprintf(log_str,"begin_respond: error - RESPOND_BEGIN_DEFINITION command expected, got '%s' instead\n",*p);
				logit(log_str);
				goto	save;
			}
			/* find the # of respond structs for this type */
			if ((j = index_target(&strings[i], RESPOND_END_DEFINITION)) == 0)	{
				logit("begin_respond: error - no RESPOND_END_DEFINITION\n");
				goto	save;
			}
			/* make RESPOND_END_DEFINITION end of strings,	*/
			/* count # of structs needed and restore strings*/
			temp = strings[i + j];
			strings[i + j] = 0;
			k = count_target(&strings[i], RESPOND_NEXT_DEFINITION);
			rptr = (struct respond *)calloc(sizeof (struct respond),k+2);
			strings[i + j] = temp;
			id = *p + sizeof(RESPOND_BEGIN_DEFINITION) -1;
/* sprintf(log_str,"begin_respond: j = %d. k = %d. id = '%s'\n",j,k,id); */
/* logit(log_str); */
			if (strcmp(id,LOGIN_TYPE) == 0)	{
				respond_type = RESPOND_LOGIN;
				if (p1->login)	{
					sprintf(log_str,"begin_respond: warning LOGIN exists (host = %s) - replaced\n",p1->host_id);
					logit(log_str);
				}
			/*	logit("begin_respond: building LOGIN\n");*/
				p1->login = rptr;
			}	else if (strcmp(id,START_TYPE) == 0)	{
				respond_type = RESPOND_START;
				if (p1->start)	{
					sprintf(log_str,"begin_respond: warning START exists (host = %s) - replaced\n",p1->host_id);
					logit(log_str);
				}
			/*	logit("begin_respond: building START\n");*/
				p1->start = rptr;
			}	else if (strcmp(id,PUT_START_TYPE) == 0) {
				respond_type = RESPOND_PUT_START;
				if (p1->p_st)	{
					sprintf(log_str,"begin_respond: warning P_ST exists (host = %s) - replaced\n",p1->host_id);
					logit(log_str);
				}
			/*	logit("begin_respond: building P_ST\n"); */
				p1->p_st = rptr;
			}	else if (strcmp(id,PUT_BETWEEN_TYPE) == 0) {
				respond_type = RESPOND_PUT_BTWN;
				if (p1->p_btwn)	{
					sprintf(log_str,"begin_respond: warning P_BTWN exists (host = %s) - replaced\n",p1->host_id);
					logit(log_str);
				}
			/*	logit("begin_respond: building P_BTWN\n");*/
				p1->p_btwn = rptr;
			}	else if (strcmp(id,PUT_END_TYPE) == 0)	{
				respond_type = RESPOND_PUT_END;
				if (p1->p_end)	{
					sprintf(log_str,"begin_respond: warning P_END exists (host = %s) - replaced\n",p1->host_id);
					logit(log_str);
				}
			/*	logit("begin_respond: building P_END\n");*/
				p1->p_end = rptr;
			}	else if (strcmp(id,GET_START_TYPE) == 0) {
				respond_type = RESPOND_GET_START;
				if (p1->g_st)	{
					sprintf(log_str,"begin_respond: warning G_ST exists (host = %s) - replaced\n",p1->host_id);
					logit(log_str);
				}
			/*	logit("begin_respond: building G_ST\n");*/
				p1->g_st = rptr;
			}	else if (strcmp(id,GET_BETWEEN_TYPE) == 0) {
				respond_type = RESPOND_GET_BTWN;
				if (p1->g_btwn)	{
					sprintf(log_str,"begin_respond: warning G_BTWN exists (host = %s) - replaced\n",p1->host_id);
					logit(log_str);
				}
			/*	logit("begin_respond: building G_BTWN\n");*/
				p1->g_btwn = rptr;
			}	else if (strcmp(id,GET_END_TYPE) == 0)	{
				respond_type = RESPOND_GET_END;
				if (p1->g_end)	{
					sprintf(log_str,"begin_respond: warning G_END exists (host = %s) - replaced\n",p1->host_id);
					logit(log_str);
				}
			/*	logit("begin_respond: building G_END\n");*/
				p1->g_end = rptr;
			}	else if (strcmp(id,END_TYPE) == 0)	{
				respond_type = RESPOND_END;
				if (p1->end)	{
					sprintf(log_str,"begin_respond: warning END exists (host = %s) - replaced\n",p1->host_id);
					logit(log_str);
				}
			/*	logit("begin_respond: building END\n");*/
				p1->end = rptr;
			}	else if (strcmp(id,EXIT_TYPE) == 0)	{
				respond_type = RESPOND_EXIT;
				if (p1->exit)	{
					sprintf(log_str,"begin_respond: warning EXIT exists (host = %s) - replaced\n",p1->host_id);
					logit(log_str);
				}
			/*	logit("begin_respond: building EXIT\n");*/
				p1->exit = rptr;
			}	else	{
				sprintf(log_str,"begin_respond: error - unknown respond type - '%s'\n",*p);
				logit(log_str);
				goto	save;
			}
			cmd_str = 0;
			flags = 0;
			time = 0;
			no_response = 0;
			response = 0;
			alt1 = 0;
			alt1_action = 0;
			alt2 = 0;
			alt2_action = 0;
			reresponse = 0;
			attempts = 0;
			lower = 0;
			continue;
		}
		l1 = strcmp(*p,RESPOND_NEXT_DEFINITION);
		l2 = strcmp(*p,RESPOND_END_DEFINITION);
/*sprintf(log_str,"begin_respond: l1 = %d. l2 = %d. l3 = %d.\n",l1,l2,l3);*/
/*logit(log_str);*/
		if ((l1 == 0) || (l2 == 0) || (l3 == 0))	{
			/* save this instance of this RESPOND type	*/
			if (cmd_str)	{
				rptr->cmd_str = (char *)malloc(strlen (cmd_str) +1);
				atos(rptr->cmd_str,cmd_str);
			}
			rptr->respond_flags = flags;
			rptr->respond_time = time;
			rptr->no_response = no_response;
			if (response)	{
				rptr->response = (char *)malloc (strlen(response)+1);
				atos(rptr->response,response);
			}
			if (alt1)	{
				rptr->alternate1 = (char *)malloc (strlen(alt1)+1);
				atos(rptr->alternate1,alt1);
			}
			rptr->alt1_action = alt1_action;
			if (alt2)	{
				rptr->alternate2 = (char *)malloc (strlen(alt2)+1);
				atos(rptr->alternate2,alt2);
			}
			rptr->alt2_action = alt2_action;
			if (reresponse)	{
				rptr->reresponse = (char *)malloc (strlen(reresponse) +1);
				atos(rptr->reresponse,reresponse);
			}
			rptr->attempts = attempts;
			rptr->lower = lower;
			/* reset for next RESPOND	*/
			rptr++;
			cmd_str = 0;
			flags = 0;
			time = 0;
			no_response = 0;
			response = 0;
			alt1 = 0;
			alt1_action = 0;
			alt2 = 0;
			alt2_action = 0;
			reresponse = 0;
			attempts = 0;
			lower = 0;
			if ((l2 == 0) || (l3 == 0))	{
				/* end-of-definition	*/
				respond_type = 0;
				rptr = 0;
			}
			l1 = l2 = -1;
			continue;
		}
		if (strncmp(*p,RESPOND_CMD_STRING,sizeof(RESPOND_CMD_STRING) -1) == 0)	{
			cmd_str = *p + sizeof(RESPOND_CMD_STRING) -1;
			continue;
		}
		if (strncmp(*p,RESPOND_FLAGS,sizeof(RESPOND_FLAGS) -1) == 0)	{
			ptr1 = ptr2 = *p + sizeof(RESPOND_FLAGS) -1;
			while (*ptr1)	{
				if ((ptr2 = strchr(ptr1,'|')) == 0)
					if ((j = chk_respond_flag(ptr1)) < 0)	{
						ret = FALSE;
						goto	save;
					}	else	{
						flags  |= j;
						break;
					}
				*ptr2 = '\0';
				if ((j = chk_respond_flag(ptr1)) < 0)	{
					ret = FALSE;
					goto	save;
				}
				flags |= j;
				*ptr2 = '|';
				ptr1 = ptr2 +1;
			}
			continue;
		}
		if (strncmp(*p,RESPOND_TIME,sizeof(RESPOND_TIME) -1) == 0)	{
			time = ctob(*p+sizeof(RESPOND_TIME)-1);
			continue;
		}
		if (strncmp(*p,RESPOND_NO_RESPONSE, sizeof(RESPOND_NO_RESPONSE) -1) == 0)	{
			if ((no_response = chk_respond_action(*p+ sizeof(RESPOND_NO_RESPONSE) -1)) < 0)
				goto	save;
			continue;
		}
		if (strncmp(*p,RESPOND_RESPONSE,sizeof(RESPOND_RESPONSE) -1) == 0)	{
			response = *p + sizeof(RESPOND_RESPONSE) -1;
			continue;
		}
		if (strncmp(*p,RESPOND_ALT1,sizeof(RESPOND_ALT1) -1) == 0)	{
			alt1 = *p + sizeof(RESPOND_ALT1) - 1;
			continue;
		}
		if (strncmp(*p,RESPOND_ALT1_ACTION, sizeof(RESPOND_ALT1_ACTION) -1) == 0)	{
			if ((alt1_action = chk_respond_action(*p+ sizeof(RESPOND_ALT2_ACTION)-1)) < 0)
				goto	save;
			continue;
		}
		if (strncmp(*p,RESPOND_ALT2,sizeof(RESPOND_ALT2) -1) == 0)	{
			alt2 = *p + sizeof(RESPOND_ALT2) -1;
			continue;
		}
		if (strncmp(*p,RESPOND_ALT2_ACTION, sizeof(RESPOND_ALT2_ACTION) -1) == 0)	{
			if ((alt2_action = chk_respond_action(*p+ sizeof(RESPOND_ALT2_ACTION)-1)) < 0)
				goto	save;
			continue;
		}
		if (strncmp(*p,RESPOND_RERESPONSE, sizeof(RESPOND_RERESPONSE) -1) == 0)	{
			reresponse = *p + sizeof(RESPOND_RERESPONSE) -1;
			continue;
		}
		if (strncmp(*p,RESPOND_ATTEMPTS,sizeof(RESPOND_ATTEMPTS) -1) == 0)	{
			attempts = ctob(*p+sizeof(RESPOND_ATTEMPTS)-1);
			continue;
		}
		if (strncmp(*p,RESPOND_LOWER,sizeof(RESPOND_LOWER) -1) == 0)	{
			if (*(*p + sizeof(RESPOND_LOWER)-1) == 'T')
				lower = TRUE;
			else	if (*(*p + sizeof(RESPOND_LOWER) -1) == 'F')
				lower = FALSE;
			else	{
				sprintf(log_str,"begin_respond: warning - unknown option skipped - '%s'\n",*p);
				logit(log_str);
			}	
			continue;
		}
		sprintf(log_str,"begin_respond: unknown RESPOND command = '%s' - skipped\n",*p);
		logit(log_str);
	}
	ret = TRUE;
save:	strings[num] = savep;
/*sprintf(log_str,"begin_respond: ret = %d. i = %d. type = %d.\n",ret,i, respond_type);*/
/*logit(log_str);*/
/*	if (ret < 0)	logit(strings[i]);*/
	return(ret);
}

	FUNCTION chk_respond_flag(p)
char	*p;
{
	int	flag = -1;
	if (strcmp(p,"REMOTE_ECHO") == 0)		flag = REMOTE_ECHO;
	else	if(strcmp(p,"NO_FLUSH") == 0)		flag = NO_FLUSH;
	else	if (strcmp(p,"QUIET_LINE") == 0)	flag = QUIET_LINE;
	else	if (strcmp(p,"MOD_CMD") == 0)		flag = MOD_CMD;
	else	{
		sprintf(log_str,"chk_respond_flag: warning - unknown flag ignored - '%s'\n",p);
		logit(log_str);
	}
	return(flag);
}

	FUNCTION chk_respond_action(p)
char	*p;
{
	int	action = -1;
	if (strcmp(p,"ACTION_OK") == 0)
		action = ACTION_OK;
	else	if (strcmp(p,"ACTION_EXIT") == 0)
		action = ACTION_EXIT;
	else	if (strcmp(p,"ACTION_DONE") == 0)
		action = ACTION_DONE;
	else	if (strcmp(p,"ACTION_IGNORE") == 0)
		action = ACTION_IGNORE;
	else	if (strcmp(p,"ACTION_RESET_TIME") == 0)
		action = ACTION_RESET_TIME;
	else	if (strcmp(p,"ACTION_RESTART") == 0)
		action = ACTION_RESTART;
	else	if (strcmp(p,"ACTION_SKIP") == 0)
		action = ACTION_SKIP;
	else	if (strcmp(p,"ACTION_ASCII_DONE") == 0)
		action = ACTION_ASCII_DONE;
	else	if (strcmp(p,"ACTION_ASCII_IGNORE") == 0)
		action = ACTION_ASCII_IGNORE;
	else	if (strcmp(p,"ACTION_ASCII_RESET") == 0)
		action = ACTION_ASCII_RESET;
	else	{
		action = ACTION_OK;
		sprintf(log_str,"chk_respond_action: warning - unrecognized option skipped - '%s'\n",p);
		logit(log_str);
	}
	return(action);
}
