/*				FILE = bld_structs.c			*/

/*	This function will read commands from a file and build the	*/
/*	'host', 'respond' and 'tt_struct' structures for the program	*/
/*	that sends EDI transactions to a van.				*/

#include	"xmit_file.h"
#include	"dial.h"
#include	"edi_control_defs.h"
#include	"van_control_defs.h"

	FUNCTION bld_structs(filename,host_id)
char	*filename, **host_id;
{
	char	*strings[MAX_CONTROL_STRINGS], *buf, *p, *ttname;
	int	len, num, i, j, k, fd, num_hosts;
	struct	tt_struct *tt;
	long	bchd = -1, bthd = -1, bhd = -1, ehd = -1;
	long	brd = -1, erd = -1, btd = -1, etd = -1;

	if ((fd = open(filename, O_RDONLY)) < 0)	{
		sprintf(log_str,"bld_structs: unable to open control file - '%s'\n",filename);
		logit(log_str);
		sprintf(log_str,"	error = '%s'\n",sys_errlist[errno]);
		logit(log_str);
		return(-1);
	}

	/* check the syntax of the structure command file	*/
	if (ck_cmd_syntax(fd,&bchd,&bthd,&bhd,&ehd,&brd,&erd,&btd,&etd)<0) {
		logit("bld_structs: syntax error\n");
		return(-1);
	}

	/* get large enough 'buf' array to build any structure	*/
	len = ehd - bhd;
	if ((i = (erd - brd)) > len)	len = i;
	if ((i = (etd - btd)) > len)	len = i;
	buf = (char *)malloc(len+1);

	if (bld_host(fd,bhd,ehd,buf,strings,MAX_CONTROL_STRINGS) < 0)	{
		logit("bld_structs: bld_host failed\n");
		free(hostp);
		return(-1);
	}
	if (bld_respond(fd,brd,erd,buf,strings,MAX_CONTROL_STRINGS) < 0)	{
		logit("bld_structs: bld_respond failed\n");
		free(hostp);	/* need to free respond structs also	*/
		return(-1);
	}
	if (bld_tt(fd,btd,etd,buf,strings,MAX_CONTROL_STRINGS) < 0)	{
		logit("bld_structs: bld_tt failed\n");
		free(hostp);	/* need to free respond structs also	*/
		if (ttp)	free(ttp);
		return(-1);
	}
	if (bld_thid(fd,bthd) < 0)	{
		logit("bld_structs: bld_thid failed\n");
		free(hostp);	/* need to free respond structs also	*/
		if (ttp)	free(ttp);
		return(-1);
	}
	if (link_structs() < 0)	{
		logit("bld_structs: link_structs failed\n");
		free(hostp);	/* need to free respond structs also	*/
		if (ttp)	free(ttp);
		return(-1);
	}
	if (bld_called(fd,bchd,host_id) < 0)	{
		logit("bld_structs: bld_called failed\n");
		free(hostp);
		if (ttp)	free(ttp);
		return(-1);
	}
/*	prt_host(hostp,TRUE);	*/
	return(1);
}

	FUNCTION getline(fd,buf)
int	fd;
char	*buf;
{
	char	*p = buf;
	int	ret = -1;
	while (read(fd,p,1) == 1)
			if (*p == '\n')	{
				*p = '\0';
				ret = p - buf + 1;
				break;
			}	else	p++;
	return(ret);
}

	FUNCTION ck_cmd_syntax(fd,st,st0,st1,end1,st2,end2,st3,end3)
int	fd;
long	*st,*st0,*st1,*end1,*st2,*end2,*st3,*end3;
{
	int	len, ret = 0, save_type, type = TYPE_NONE;
	char	buf[256];

buf[0] = '\0';
	/* check if XMODEM control file		*/
	if (((len = getline(fd,buf)) <= 0) && 
			(strcmp(buf,EDI_XMODEM_CONTROL_FILE) != 0))
		return(-1);
	while ((len = getline(fd,buf)) >= 0)	{
		if (buf[0] == '#')	{
			continue;
		}	else
		if (strcmp(buf,END_COMMENT_SECTION) == 0)	{
			if (type != TYPE_COMMENT)	{
				logit("ck_cmd_syntax: end_comment_section found embedded in another type -error\n");
				return(-1);
			}
			type = save_type;
		}	else
		if (type == TYPE_COMMENT)	{
			continue;
		}	else
		if (strncmp(buf,CALLED_HOST_DEFINITION, sizeof(CALLED_HOST_DEFINITION) - 1) == 0)	{
			if (type != TYPE_NONE)	{
				logit("ck_cmd_syntax: begin_called_host_definition found embedded in another type -error\n");
				return(-1);
			}
			*st = lseek(fd,0l,1) - strlen(buf) - 1;
		}	else
		if (strncmp(buf,THIS_HOST_DEFINITION, sizeof(THIS_HOST_DEFINITION) - 1) == 0)	{
			if (type != TYPE_NONE)	{
				logit("ck_cmd_syntax: begin_this_host_definition found embedded in another type -error\n");
				return(-1);
			}
			*st0 = lseek(fd,0l,1) - strlen(buf) - 1;
		}	else
		if (strcmp(buf,BEGIN_HOST_DEFINITION) == 0)	{
			if (type != TYPE_NONE)	{
				logit("ck_cmd_syntax: begin_host_definition found embedded in another type -error\n");
				return(-1);
			}
			type = TYPE_HOST;
			*st1 = lseek(fd,0l,1);
		}	else
		if (strcmp(buf,BEGIN_RESPOND_DEFINITION) == 0)	{
			if (type != TYPE_NONE)	{
				logit("ck_cmd_syntax: begin_respond_definition found embedded in another type -error\n");
				return(-1);
			}
			type = TYPE_RESPOND;
			*st2 = lseek(fd,0l,1);
		}	else
		if (strcmp(buf,BEGIN_RESPOND_INLINE) == 0)	{
			if (type != TYPE_HOST)	{
				logit("ck_cmd_syntax: begin_respond_inline found not embedded in HOST type -error\n");
				return(-1);
			}
			type = TYPE_INLINE;
		}	else
		if (strcmp(buf,BEGIN_TT_DEFINITION) == 0)	{
			if (type != TYPE_NONE)	{
				logit("ck_cmd_syntax: begin_tt_definition found embedded in another type -error\n");
				return(-1);
			}
			type = TYPE_TT;
			*st3 = lseek(fd,0l,1);
		}	else
		if (strcmp(buf,END_HOST_DEFINITION) == 0)	{
			if (type != TYPE_HOST)	{
				logit("ck_cmd_syntax: end_host_definition found embedded in another type -error\n");
				return(-1);
			}
			type = TYPE_NONE;
			*end1 = lseek(fd,0l,1);
		}	else
		if (strcmp(buf,END_RESPOND_DEFINITION) == 0)	{
			if (type != TYPE_RESPOND)	{
				logit("ck_cmd_syntax: end_respond_definition found embedded in another type -error\n");
				return(-1);
			}
			type = TYPE_NONE;
			*end2 = lseek(fd,0l,1);
		}	else
		if (strcmp(buf,END_RESPOND_INLINE) == 0)	{
			if (type != TYPE_INLINE)	{
				logit("ck_cmd_syntax: end_respond_inline found embedded in another type -error\n");
				return(-1);
			}
			type = TYPE_HOST;
		}	else
		if (strcmp(buf,END_TT_DEFINITION) == 0)	{
			if (type != TYPE_TT)	{
				logit("ck_cmd_syntax: end_tt_definition found embedded in another type -error\n");
				return(-1);
			}
			type = TYPE_NONE;
			*end3 = lseek(fd,0l,1);
		}	else
		if (strcmp(buf,BEGIN_COMMENT_SECTION) == 0)	{
			/* no nested comments	*/
			if (type == TYPE_COMMENT)	{
				logit("ck_cmd_syntax: nested comment sections not allowed - error");
				return(-1);
			}
			save_type = type;
			type = TYPE_COMMENT;
		}	else
		if (type == TYPE_TT)	{
			if (strncmp(buf,TT_PREAMBLE,sizeof(TT_PREAMBLE) -1) != 0)	{
				logit("ck_cmd_syntax: unallowed command in TT command sequence - error\n");
				return(-1);
			}
		}	else
		if ((type == TYPE_RESPOND) || (type == TYPE_INLINE))	{
			if (strncmp(buf,RESPOND_PREAMBLE,sizeof (RESPOND_PREAMBLE) -1) != 0)	{
				sprintf(log_str,"ck_cmd_syntax: error - unallowed command in RESPOND command sequence - \n'%s'\n",buf);
				logit(log_str);
				return(-1);
			}
		}	else
		if (type == TYPE_HOST)	{
			if (strncmp(buf,HOST_PREAMBLE,sizeof(HOST_PREAMBLE) -1) != 0)	{
				sprintf(logit,"ck_cmd_syntax: error - unallowed command in HOST command sequence\n		cmd = '%s'\n",buf);
				logit(log_str);
				return(-1);
			}
		}	else	{
			sprintf(log_str,"ck_cmd_syntax : error - unknown command = '%s'\n",buf);
			logit(log_str);
			return(-1);
		}
	}
	if (type != TYPE_NONE)	{
		sprintf(log_str,"ck_cmd_syntax: control file not properly terminated - type = %d.\n",type);
		logit(log_str);
		return(-1);
	}
	return(ret);
}

	FUNCTION bld_thid(fd,start)
int	fd;
long	start;
{
	int	i;
	char	buf[256];
	struct	host	*p;

	if (hostp && (start >= 0))	{
		lseek(fd,start,0);
		if (getline(fd,buf) >= 0)	{
			for (i = 0, p = hostp; p->host_id; p++, i++)
				if (strcmp(buf+sizeof (THIS_HOST_DEFINITION)-1, p->host_id) == 0) break;
			if (p->host_id != 0)	this_host_index = i;
		}
	}
	return(1);
}

	FUNCTION bld_called(fd,start,host_id)
int	fd;
long	start;
char	**host_id;
{
	static	char	buf[256];
	struct	host	*p;

	if (start >= 0)	{
		lseek(fd,start,0);
		if (getline(fd,buf) >= 0)
			*host_id = buf+sizeof(CALLED_HOST_DEFINITION)-1;
		else	{
			logit("bld_called: unable to get called host name\n");
			return(-1);
		}
	}
	return(1);
}

/*	For each TT_HOST_ID command, link the TT and associated HOST	*/
/*	structure.							*/
	FUNCTION link_structs()
{
	int	i, ret = 1;
	struct	tt_struct	*p;
	struct	host		*host;

	for (p = ttp, i = 0; p->ttname != 0; p++, i++)
		if (p->host_id != 0)	{
			for (host = hostp; host->host_id != 0; host++)
				if (strcmp(p->host_id,host->host_id) == 0)
					break;
			if (host->host_id != 0)	{
				host->tt_index = i + 1;
				p->in_use = TRUE;
			}	else	{
				sprintf(log_str,"link_structs: unable to link tty - '%s' - to specified host '%s'\n",p->ttname,p->host_id);
				logit(log_str);
				ret = 0;
			}
		}
	return(ret);
}
