/*				FILE = dial.c				*/

#include	"xmit_file.h"
#include	"dial.h"
#include	"dial_allocate.h"
#include	"xmodem.h"

char *dialv = "Dial Command, V2.0(007) 28 Jun 85";

/*  C K U D I A  --  Dialing program for connection to remote system */

/*
 This module should work under all versions of Unix.  It calls externally
 defined system-depended functions for i/o, but depends upon the existence
 of various modem control functions.

 Author: Herm Fischer (HFISCHER@USC-ECLB)
 Contributed to Columbia University for inclusion in C-Kermit.
 Copyright (C) 1985, Herman Fischer, 16400 Ventura Blvd #201, Encino CA 91436
 Permission is granted to any individual or institution to use, copy, or
 redistribute this software so long as it is not sold for profit, provided this
 copyright notice is retained. 
*/

/*
 * Modifications:
 *	20-Jan-88	Modified telephone number logging to accomodate
 *			"1" prefix on long distance dialing.
 *
 *	28-Jun-85	Fixed bug with defaulting the modem-failure message
 *			in LBUF.
 *							-- Dan Schullman
 *
 *	27-Jun-85	Merged in code from Joe Orost at Berkeley for
 *			supporting the US Robotics modem, which included
 *			changing the single characters in MDMINF into
 *			multi-character strings and modifying waitFor.
 *							-- Dan Schullman
 *
 *	26-Jun-85	Allow interrupts to be used to abort dialing,
 *			and ring the bell when a connection is made.
 *			Reorganized some of the failure paths to use the
 *			same code, and now close the line on failures.
 *			Allow use of stored numbers with the DF100 and
 *			DF200 modems.  Handlers now declared after the
 *			call to setjmp.
 *							-- Dan Schullman
 *
 *	24-May-85	DF03, DF100-series, DF200-series, and "unknown" modem
 *			support added.  Also restructured the various data
 *			tables, fixed some bugs related to missing data and
 *			missing case labels, and modified the failure message
 *			to display the "reason" given by the modem.
 *							-- Dan Schullman
 */

/*
 * To add support for another modem, do the following:
 *
 *	Define a modem number symbol (n_XXX) for it, keeping the list
 *	in alphabetical and numerical order, and renumbering the values
 *	as necessary.
 *
 *	Create a MDMINF structure for it, again keeping the list alphabetical
 *	for sanity's sake.
 *
 *	Add the address of the MDMINF structure to the ptrtab array, again
 *	in alphabetical and numerical order.
 *
 *	Add the "user visible" modem name and corresponding modem number to
 *	the mdmtab array, again in alphabetical order.
 *
 *	Read through the code and add modem-specific sections as necessary.
 */

/*
 * The intent of the "unknown" modem is hopefully to allow KERMIT to support
 * unknown modems by having the user type the entire autodial sequence
 * (possibly including control characters, etc.) as the "phone number".
 * The only reason that the CONNECT command cannot be used to do this is
 * that a remote line cannot normally be opened unless carrier is present.
 *
 * The protocol and other characteristics of this modem are unknown, with
 * some "reasonable" values being chosen for some of them.  The only way to
 * detect if a connection is made is to look for carrier present.
 *
 * SUPPORT IS CURRENTLY ONLY PARTIALLY SKETCHED OUT FOR THIS.  ALSO, IT
 * SHOULD PERHAPS BE HANDLED MUCH EARLIER, SIMPLY READING USER INPUT AND
 * SENDING IT TO THE MODEM AND ECHOING MODEM RESPONSES BACK TO THE USER,
 * ALL THE TIME LOOKING FOR CARRIER.  OF COURSE, THE PROBLEM THEN BECOMES
 * ONE OF ALLOWING THE USER TO ABORT THE DIALING.  WE COULD CHOOSE SOME
 * PHRASE THAT WOULD PRESUMABLY NEVER BE A PART OF A VALID AUTODIAL SEQUENCE
 * (E.G., "QUIT" and "quit"). -- DS
 */

static	int tries = 0;

#define	LBUFL	100
static	char lbuf[LBUFL];
static	char *user;

dialtime() {			/* timer interrupt handler */
    longjmp(sjbuf, F_time);
}

dialint()	{		/* user-interrupt handler */
    longjmp(sjbuf, F_int);
}

ttolSlow(tt,s,millisec)
struct	tt_struct	*tt;
char	*s;
int	millisec;
{  /* output s-l-o-w-l-y */
	if (DEBUGGING)	{
		sprintf(log_str,"ttolSlow: called with tt = %o, millisec = %d. s = '%s'\n",
			tt,millisec,s);
		logit(log_str);
	}
	for (; *s; s++)	{
		sendbyte(tt,*s);
		napms(millisec);
	}
}

/*
 * Wait for a string of characters.
 *
 * The characters are waited for individually, and other characters may
 * be received "in between".  This merely guarantees that the characters
 * ARE received, and in the order specified.
 */
waitFor(tt,s)
struct	tt_struct	*tt;
char	*s;
{
	int	i;
	char	c;
	if (DEBUGGING)	{
		sprintf(log_str,"waitFor: called with tt = %o, s = '%s'\n", tt,s);
		logit(log_str);
	}
	while (*s)	{	/* until end of string */
		while (TRUE)	{
			if ((i = readbyte(tt,-1)) == ERROR)	continue;
			c = i & 0377;
			if (c == *s)	break;
		}
		s++;
	}
}

static
didWeGet(s,r)
char *s, *r;
{	/* Looks in string s for response r */
	int lr = strlen(r);	/*  0 means not found, 1 means found it */
	int i;
	for (i = strlen(s)-lr; i >= 0; i--)
		if (s[i] == r[0]) if (!strncmp(s+i,r,lr)) return(1);
	return(0);
}


/* R E S E T -- Reset alarms, etc. on exit. */
static
reset ()
{
	alarm(0);
	signal(SIGALRM,SIG_DFL);	/* restore alarm handler */
	signal(SIGINT, SIG_DFL);	/* restore interrupt handler */
}



/*  D I A L  --  Dial up the remote system */

dial(host)
struct	host	*host;
{
	struct	tt_struct	*tt;
	char	**ptr;
	char	c;
	char	*i, *j, *telnbr;
	int	waitct, status, mdmtyp, n;
	MDMINF	*pmdminf;	/* pointer to modem-specific info */
	char	*pc;		/* pointer to a character */

	if (!host->tt_index)
		return(error("dial: No tt_struct\n.",FALSE,0));
	telnbr = host->telnbr;
	tt = ttp + (host->tt_index -1);
	mdmtyp = tt->mdm_index;
	if (!mdmtyp) mdmtyp = n_HAYES;
	pmdminf = ptrtab[mdmtyp-1];	/* set pointer to modem info */ 
					/* interdigit waits for tone dial */
	if (DEBUGGING)	prt_mdminf(pmdminf);

	waitct = 1*strlen(telnbr);	/* compute time to dial worst case */
	waitct += pmdminf->dial_time;	/* dialtone + completion wait times */
	for (i=telnbr; *i; i++) 	/* add in pause characters time */
		for (j=pmdminf->pause_chars; *j; j++)
			if (*i == *j)	{
				waitct += pmdminf->pause_time; 
				break;
			}
	if (DEBUGGING)	{
		sprintf(log_str,"dial: telnbr = '%s'\n",telnbr);
		logit(log_str);
	}

	/* check valid phone numbers */
	switch (strlen(telnbr)) {
		case 7:	/* local number - okay */
			break;
		case 10:
			if ((strncmp("703",telnbr,3) == 0) ||
				(strncmp("202",telnbr,3) == 0) || 
				(strncmp("301",telnbr,3) == 0))
				break;
			return(error("Only toll-free numbers are permitted\n",FALSE,tt));
		case 11: /* only toll-free are okay */
			if (strncmp("1800",telnbr,4) == 0)
				break;
			/* allow long distance exceptions	*/
			for (ptr = &exempt_numbers[0]; *ptr !=0; ptr++)
				if (strcmp(*ptr,telnbr) == 0)
					goto	good;
			return(error("Only toll-free numbers are permitted\n",FALSE,tt));
		default:
			return(error("Invalid phone number\n",FALSE,tt));
good:		break;
	} /* end switch */


	/* Condition communication line if necessary	*/	    
	if (setmodes(tt,FLAG_DIALING) != TRUE)
		return(error("dial aborted\n",FALSE,tt));

/*
 * INTERRUPT HANDLERS
 */

    if (n = setjmp(sjbuf))	{  /* if a "failure jump" was taken... */
/* debug */	logit("dial: processing interrupt\n");
	flush(tt);			/* flush input buffer */
	if (mdmtyp == n_HAYES)	ttolSlow(tt,hayes_reset,0);
	switch (n)	{		/* type of failure */
		case F_time:		/* timed out */
			error("No connection made within the allotted time.\n",FALSE,tt);
			break;
		case F_int:		/* dialing interrupted */
			error("Dialing interrupted.\n",FALSE,tt);
			break;
		case F_modem:		/* modem detected a failure */
			for (pc=lbuf; *pc; pc++)
		    	    if (!isprint(*pc))
				*pc = ' '; /* display printable reason */
			*pc++ = '\n';
			*pc = '\0';
			error(lbuf,FALSE,tt);
			break;
		case F_minit:		/* cannot initialize modem */
			error("Cannot initialize modem.\n",FALSE,tt);
			break;
	}
	reset();			/* reset alarms, etc. */
	return(ERROR);
    }
    signal(SIGALRM,dialtime);
    signal(SIGINT, dialint);

/*
 * PUT MODEM IN COMMAND MODE.
*/

	/* Hang up the modem (in case it wasn't "on hook")	*/
	if (hangup(host) < 0 )
		return(error("Sorry, Can't hang up tty line\n",FALSE,tt));

	alarm(20);
	flush(tt);		/* flush input buffer if any */

	/*
	 *  Output wake string and wait for response if any expected
	 */
	ttolSlow(tt,pmdminf->wake_str, pmdminf->wake_rate);
	if (pmdminf->wake_prompt) waitFor(tt,pmdminf->wake_prompt);

	/*	Turn off alarm	*/
	alarm(0);

	/* DIAL THE MODEM */

	flush(tt);		/* flush input buffer if any */
	alarm(10);		/* allow modem 10 secs. to accept request */
	ttolSlow(tt,pmdminf->dmode_str, pmdminf->dial_rate);
	if (pmdminf->dmode_prompt)	/* wait for prompt, if any expected */
		waitFor(tt,pmdminf->dmode_prompt);

	alarm(0);		/* turn off alarm on dialing prompts */
	alarm(waitct);		/* time to allow for connecting */
	flush(tt);		/* clear out stuff from waking modem up */
	sprintf(lbuf,pmdminf->dial_str,telnbr); /* form dialing string */
	ttolSlow(tt,lbuf,pmdminf->dial_rate);	/* send dialing string */

	if (mdmtyp == n_RACAL) {
		/* acknowledge printout of dialing string */
		sleep(3);
		flush(tt);
		sendbyte(tt,'\r');
	}


    /* CHECK DIALING STATUS
     *
     * I believe we also need to look for carrier in order to determine if a
     * connection has been made.  In fact, for many we may only want to look for
     * the "failure" responses in order to short-circuit the timeout, and let
     * carrier be the determination of whether a connection has been made. -- DS
     */

	status = 0;
	strcpy(lbuf,"No Connection");	/* default failure reason */
	while (status == 0) {
		switch (mdmtyp) {
#ifdef D642
		case n_RV831:
			c = ttrv831(telnbr);
			if (c) 
				status = FAILED;
			else
			status = CONNECTED;
			flush(tt);	/* flush input buffer */
			    break;
#endif
		case n_DF03:	/* because response lacks CR or NL */
			c = readbyte(tt,-1) & 0177;
			if ( c == 'A' ) status = CONNECTED;
			if ( c == 'B' ) status = FAILED;
			break;
		case n_HAYES:
		case n_HAYESNV:
			c = readbyte(tt,-1) & 0177;	/* Get HAYES status response */
			switch (c) {	/* Switch on status response */
			case '0':
				status = CONNECTED;
				strcpy(lbuf,"Connected at unknown speed");
				break;
	    		case '1':
				status = CONNECTED;
				c = readbyte(tt,-1) & 0177;
				if (c == '0')	{
					tt->real_speed = B2400;
					strcpy(lbuf,"Connected at 2400");
				}	else	{
					tt->real_speed = B300;
					strcpy(lbuf,"Connected 300");
				}
				break;
			case '2':
				status = FAILED;
				strcpy(lbuf,"Invlaid modem status: 2");
				break;
			case '3':
				status = FAILED;
				strcpy(lbuf,"No carrier");
				break;
			case '4':
				status = FAILED;
				strcpy(lbuf,"Error");
				break;
			case '5':
				status = CONNECTED;
				tt->real_speed = B1200;
				strcpy(lbuf,"Connected 1200");
				break;
			case '6':
				status = FAILED;
				strcpy(lbuf,"No dial tone");
				break;
			case '7':
				status = FAILED;
				strcpy(lbuf,"Busy");
				break;
			case '8':
				status = FAILED;
				strcpy(lbuf,"No answer");
				break;
			}	/* End switch on HAYES status */

		case n_UNKNOWN:
			/** SHOULD WAIT FOR CARRIER OR TIMEOUT -- DS **/
			break;
		default:
		    for (n=0; n < LBUFL; n++) {	/* accumulate response */
			lbuf[n] = (readbyte(tt,-1) & 0177); 
			if ( lbuf[n] == '\r' || lbuf[n] == '\n' ) break;
		    }
		    lbuf[n] = '\0';	/* terminate response from modem */
		    if (n) {	/* if one or more characters present */
			switch (mdmtyp) {
			case n_CERMETEK:
				if (didWeGet(lbuf,"\016A")) {	
					status = CONNECTED;
					ttolSlow(tt,"\016U 1\r",200);	/* make transparent*/
				}
			    break;
			case n_DF100:	/* DF100 won't generate some of these */
			case n_DF200:
				if (didWeGet(lbuf,"Attached")) status = CONNECTED;
			/*
			 * The DF100 will respond with "Attached" even if DTR
			 * and/or carrier are not present.  Another reason to
			 * (also) wait for carrier?
		     	 */

				if (didWeGet(lbuf,"Busy")) status = FAILED;
				if (didWeGet(lbuf,"Disconnected")) status = FAILED;
				if (didWeGet(lbuf,"Error")) status = FAILED;
				if (didWeGet(lbuf,"No answer")) status = FAILED;
				if (didWeGet(lbuf,"No dial tone")) status = FAILED;
				if (didWeGet(lbuf,"Speed:")) status = FAILED;
			/*
			 * It appears that the "Speed:..." response comes after an
			 * "Attached" response, so this is never seen.  HOWEVER,
			 * it would be very handy to detect this and temporarily
			 * reset the speed, since it's a nuiscance otherwise.
			 * If we wait for some more input from the modem, how do
			 * we know if it's from the remote host or the modem?
			 * Carrier reportedly doesn't get set until after the
			 * "Speed:..." response (if any) is sent.  Another reason
			 * to (also) wait for carrier.
			 */
				break;
			case n_GDC:
				if (didWeGet(lbuf,"ON LINE")) status = CONNECTED;
				if (didWeGet(lbuf,"NO CONNECT")) status = FAILED;
				break;
			case n_USROBOT:
				if (didWeGet(lbuf,"CONNECT")) status = CONNECTED;
				if (didWeGet(lbuf,"NO CARRIER")) status = FAILED;
				break;
			case n_PENRIL:
				if (didWeGet(lbuf,"OK")) status = CONNECTED;
				if (didWeGet(lbuf,"BUSY")) status = FAILED;
				if (didWeGet(lbuf,"NO RING")) status = FAILED;
				break;
			case n_RACAL:
				if (didWeGet(lbuf,"ON LINE")) status = CONNECTED;
				if (didWeGet(lbuf,"FAILED CALL")) status = FAILED;
				break;
			case n_VENTEL:
				if (didWeGet(lbuf,"ONLINE!")) status = CONNECTED;
				if (didWeGet(lbuf,"BUSY")) status = FAILED;
				if (didWeGet(lbuf,"DEAD PHONE")) status = FAILED;
				break;
			}		/* if (n)			*/
		    }			/* defualt: switch (mdmtyp)	*/
		}			/* switch (mdmtyp)		*/
	}				/* while (status == 0)		*/

	if (status != CONNECTED)		/* modem-detected failure */
		longjmp(sjbuf, F_modem);	/* exit with reason in LBUF */
	reset();			/* reset alarms, etc. */
	flush(tt);			/* flush input buffer if any */
/*	ttolSlow(tt,hayes_reset,0);	*/
	if (DEBUGGING)	{
		cat_field(log_str,"dial: ",lbuf,"\n");
		logit(log_str);
	}
	host->connected |= CONNECTED_MODEM;
	return(TRUE);			/* return, and presumably connect */
}
