/*
 * Contains system routines to get and put bytes, change tty modes, etc
 * Most of the routines are VERY 4.2BSD Specific!!!
 */

#include "xmodem.h"
#include "xmit_file.h"

/*
 *
 *	Get a byte from the specified file.  Buffer the read so we don't
 *	have to use a system call for each character.
 *
 */
	FUNCTION getbyte(tt,fildes,ch)				/* Buffered disk read */
struct	tt_struct	*tt;
int	fildes;
char	*ch;
{
	static char buf[BUFSIZ];	/* Remember buffer */
	static char *bufp = buf;	/* Remember where we are in buffer */
	if (DEBUGGING)	{
		sprintf(log_str,"getbyte: called for fildes = %d.\n",fildes);
		logit(log_str);
	}
	if (nbchr == 0)	{	/* Buffer exausted; read some more */
		if ((nbchr = read(fildes, buf, BUFSIZ)) < 0)
			error("File Read Error",TRUE,tt);
		bufp = buf;		/* Set pointer to start of array */
 	}
	if (--nbchr >= 0)	{
		*ch = *bufp++;
		return(0);
	}
	else	return(EOF);
}
 
/* get a byte from data stream -- timeout if "seconds" elapses */
/* This routine is VERY 4.2 specific */
int	FUNCTION readbyte(tt,seconds)
struct	tt_struct	*tt;
int	seconds;
{
	int	ret;
	fd_set	readfds;
	char c;
	struct	timeval tmout;
	struct	timeval	*tmptr;

	if (DEBUGGING)	{
		sprintf(log_str,"readbyte: seconds = %d. fd = %d\n", seconds, tt->tty_fd);
		logit(log_str);
	}
	if (seconds < 0)	tmptr = 0;
	else	{
		tmout.tv_sec = seconds;
		tmout.tv_usec = 0;
		tmptr= &tmout;
	}

	FD_ZERO(&readfds);
	FD_SET(tt->tty_fd,&readfds);

	if ((ret = select(FD_SETSIZE, &readfds, 0, 0, tmptr)) == 0)	{
		if (DEBUGGING)	logit("readbyte: timeout\n");
		return(TIMEOUT);
	}	else	if (ret < 0)	{
		perror("readbyte");
		return(error("readbyte: error in select\n",FALSE,tt));
	}

	if ((ret = read(tt->tty_fd, &c, 1)) < 0)	{
		perror("readbyte");
		return(error("readbyte: read error\n",FALSE,tt));
	}	if (ret == 0)
			return(error("readbyte: EOF\n",FALSE,tt));

	if (DEBUGGING)	{
		sprintf(log_str,"readbyte: %02xh\n\t\tchar = '%c'\n", (c&0xff),(c&0177));
		logit(log_str);
	}

	return(c & 0xff);  /* return the char */
}

/* 
 get a buffer (length bufsize) from data stream -- timeout if "seconds" elapses.
 Read bunches of characters to save system overhead;
 Further process data while kernel is reading stream (calculating "checksum").
 Try to nap long enough so kernel collects 100 characters or so until we wake up
*/

/* This routine is VERY 4.2 specific */

int	FUNCTION readbuf(tt,bufsize, seconds, tmode, checksum, bufctr)
struct tt_struct *tt;
int bufsize,	/* number of chars to be read */
seconds, 	/* timeout period for each read */
tmode, 		/* transmission mode: 2 = UNIX text, 1 = text, 0 = binary */
*checksum, 	/* pointer to checksum value */
*bufctr;	/* length of actual data string in buffer */
{
	fd_set	readfds;	/* mask for select call */
	struct	timeval tmout;	/* timeout structure for select */
	struct	timeval *tmptr;
	int	numread;	/* number of chars read */
	int 	left;		/* number of chars left to read */
	int	recfin = 0;	/* flag that EOF read */
	char	inbuf[BBUFSIZ];	/* buffer for incoming packet */
	register unsigned char c;	/* character being processed */
	register unsigned short chksm;	/* working copy of checksum */
	register int bfctr;	/* working copy of bufctr */
	int	j;		/* loop index */


	if (DEBUGGING)	{
		sprintf(log_str,"readbuf: seconds = %d. tmode = %d. fd = %d.\n", seconds,tmode,tt->tty_fd);
		logit(log_str);
	}
	if (seconds < 0)	tmptr = 0;
	else	{
		tmout.tv_sec = seconds;
		tmout.tv_usec = 0;
		tmptr= &tmout;
	}

	FD_ZERO(&readfds);
	FD_SET(tt->tty_fd,&readfds);

	chksm = 0;
	bfctr = 0;

	for (left = bufsize; left > 0;) {
		/* read however many chars are waiting */
		if ((select(FD_SETSIZE, &readfds, 0, 0, tmptr)) == 0)	{
			logit("readbuf: timeout\n");
			return(TIMEOUT);
		}

		numread = read(tt->tty_fd, inbuf, left);
		left -= numread;

		if (DEBUGGING)	{
			sprintf(log_str,"DEBUG: readbuf--read %d characters\n", numread);
			logit(log_str);
		}

		/* now process part of packet we just read */

		for (j =  0; j < numread; j++)	{  
				buff[bfctr] = c = inbuf[j] & 0xff;
				if (CRCMODE)  /* CRC */
					chksm = (chksm<<B) ^ crctab[(chksm>>(W-B)) ^ c];

				else        /* checksum */
		       			chksm = ((chksm+c) & 0xff);

				/* binary mode */
				if (tmode == 0)	{  
					bfctr++;
		       			continue;
		       		}

				/* text mode */
				buff[bfctr] &= 0x7f; /* nuke bit 8 */
				if (tmode == 2)	/* UNIX text */
					if (c == CR)       /* skip CR's */
						continue;
				if (c == CTRLZ)  {	/* CP/M EOF char */
					recfin = TRUE;
		       			continue;
		       		}
		       		if (!recfin)
					bfctr++;
		}

		/* go to sleep to save uneeded system calls while kernel
		   is reading data from serial line; forget this when we
		   running at 9600 bps; also fudge constant from 10000 to
		   9000 to avoid sleeping too long
		*/
		if (ttyspeed < 9600)
			napms ( (left<SLEEPNUM ? left:SLEEPNUM) * 9000/ttyspeed);
	}

	*checksum = chksm;
	*bufctr = bfctr;
	if (DEBUGGING)	{
		sprintf(log_str,"readbuf: bfctr = %d.\n",bfctr);
		logit(log_str);
	}
	return(0);
}

/* send a byte to data stream */
	FUNCTION sendbyte(tt,data)
struct	tt_struct	*tt;
char	data;
{
	if (DEBUGGING)	{
		sprintf(log_str, "DEBUG: sendbyte %02xh\n", data & 0xff);
		logit(log_str);
	}

	/* write the byte (assume it goes NOW; no flushing needed) */
	if (write(tt->tty_fd, &data, 1) != 1)	{
		sprintf(log_str,"sendbyte: %s\n",sys_errlist[errno]);
		logit(log_str);
		error("Write error on stream",TRUE,tt);
	}
	return;
}


/* send a buffer to data stream */
	FUNCTION writebuf(tt,buffer, nbytes)
struct	tt_struct	*tt;
char	*buffer;
int	nbytes;
{
	if (DEBUGGING)	{
		sprintf(log_str,"DEBUG: writebuf (%d bytes)\n",nbytes);
		logit(log_str);
	}
	/* write the buffer (assume no TIOCFLUSH needed) */
	if (write(tt->tty_fd, buffer, nbytes) != nbytes)	{
		sprintf(log_str,"writebuf: %s\n",sys_errlist[errno]);
		logit(log_str);
		error("Write error on stream",TRUE,tt);
	}
	return;
}


/*
 * "nap" for specified time -- VERY 4.2BSD specific
 */
	FUNCTION napms (milliseconds)
int	milliseconds;
{
	fd_set	readfds;
	struct	timeval	timeout;

	if (milliseconds == 0)
		return;
	if (DEBUGGING)	{
		sprintf(log_str,"DEBUG: napping for %d ms\n", milliseconds);
		logit(log_str);
	}

	timeout.tv_sec = 0;
	timeout.tv_usec = milliseconds * 1000;

	FD_ZERO(&readfds);

	select(FD_SETSIZE, &readfds, 0, 0, &timeout);
}
 
/* set and restore tty modes for XMODEM transfers */
/* These routines are 4.2/v7(?) specific */
	FUNCTION setmodes(tt,flag)
struct	tt_struct	*tt;
int	flag;
{
	char *ttyname();
	extern onintr();

	if (DEBUGGING)	{
		sprintf(log_str,"setmodes: called tt = %o flag = %d\n",tt,flag);
		logit(log_str);
		sprintf(log_str,"DIALING = %d XMIT = %d XMODEM = %d INIT = %d\n", FLAG_DIALING,FLAG_XMIT,FLAG_XMODEM,FLAG_INIT);
		logit(log_str);
	}
	if (tt->tty_modes == flag)	return(TRUE);
	sleep(1);			/* let the output appear */
	if (get_modes(tt) != TRUE)	return(ERROR);

	if (flag == FLAG_XMODEM)	{
		/* original code was equivalent to the following:	*/
		/* tt->tty_new.sg_flags |= (RAW|ANYP);			*/
		/* tt->tty_new.sg_flags &= ~(ECHO|XTABS|LCASE|NL3);	*/
		/* tt->tty_new.sg_flags &= ~(TAB2|CR3|FF1|BS1|TANDEM);	*/

		tt->tty_new.c_iflag &= IXANY;
		tt->tty_new.c_oflag &= ~(OPOST|TAB3|TAB2|BS1|VT1|NL1| OLCUC);
		tt->tty_new.c_lflag &= ~(XCASE | ECHO | ISIG | ICANON);
		tt->tty_new.c_cflag &= ~(PARENB | PARODD);
		tt->tty_new.c_cflag |= CS8;
	}
	else	if (flag == FLAG_DIALING)	{
			/* old code was equivalent to the following:	*/
			/* tt->tty_new.sg_flags |= RAW;			*/
			/* tt->tty_new.sg_flags &= ~(ECHO | CRMOD);	*/

			tt->tty_new.c_iflag &= (IXANY | IXOFF);
			tt->tty_new.c_oflag &= ~(OPOST | ONLCR);
			tt->tty_new.c_lflag &= ~(ECHO|XCASE | ISIG|ICANON);
			tt->tty_new.c_cflag &= ~(PARENB | PARODD);
			tt->tty_new.c_cflag |= CS8;
		}
		else	if (flag == FLAG_XMIT)	{
				/* old code equivalent to the following	*/
				/*tt->tty_new.sg_flags |= (RAW|ANYP|LNOHANG); */
				/*tt->tty_new.sg_flags &= ~(ECHO|CRMOD|LCASE|XTABS); */

				tt->tty_new.c_iflag &= (IXANY | IXOFF);
				tt->tty_new.c_oflag &= ~(OPOST|ONLCR|OLCUC| TAB3);
				tt->tty_new.c_lflag &= ~(XCASE|ECHO|ISIG| ICANON);
				tt->tty_new.c_cflag &= ~(PARENB | PARODD);
				tt->tty_new.c_cflag |= CS8;
			}
			else	if (flag != FLAG_INIT)
					return(error("setmodes: Bad flag\n", FALSE,0));

	tt->tty_new.c_oflag |= tt->oflags;

	if (tt->real_speed != -1)	{
		tt->tty_new.c_cflag = (tt->tty_new.c_cflag & ~CBAUD) | tt->real_speed;
	}

	/* set new paramters */
	if (ioctl(tt->tty_fd,TCSETAW,&tt->tty_new) < 0)	{
		sprintf(log_str,"setmodes:	errno = %d. '%s'\n		tty_fd = %d. device = '%s'\n",
errno,sys_errlist[errno],tt->tty_fd,ttyname(tt->tty_fd));
		logit(log_str);
		if (fstat(tt->tty_fd,&tt->statbuf) < 0)	{
			sprintf(log_str,"setmodes: fstat failed, errno = %d. '%s'\n",errno,sys_errlist[errno]);
			logit(log_str);
		}
		sprintf(log_str,"setmodes: parms - tty_fd = %d. cmd = %d. arg = %o\n", tt->tty_fd,TCSETAW,&tt->tty_new);
		logit(log_str);
		prt_tt(tt);
		return(error("setmodes: Can't set new TTY Parameters", FALSE,0));
	}
	tt->tty_modes = flag;
	/* set up signal catcher to restore tty state if we are KILLed */
	if (signal(SIGTERM, SIG_IGN) != SIG_IGN)
		signal(SIGTERM, onintr);
	if (DEBUGGING)	logit("setmodes: successful\n");
	return(TRUE);
}

/* restore normal tty modes */
	FUNCTION restoremodes(tt,speed)
struct	tt_struct	*tt;
int	speed;		/* reset tty speed - T/F	*/
{
	if (DEBUGGING)	{
		sprintf(log_str,"restoremodes: called with tt = %o speed = %d\n",tt,speed);
		logit(log_str);
	}
	if (!tt)	return(ERROR);
	if (tt->tty_modes)	{
		if (setmodes(tt,FLAG_INIT) != TRUE)	return(ERROR);
		if (tt->wason && speed)
			if (chmod(tt->ttname,(int)tt->statbuf.st_mode | 022) < 0)
				return(error("Can't change TTY mode", FALSE,tt));
		if (speed)	{
			tt->tty_modes = FALSE;
			tt->wason = FALSE;
			tt->real_speed = -1;
			tt->tty_new.c_cflag = tt->tty_old.c_cflag;
		}
		if (ioctl(tt->tty_fd,TCSETAW,&tt->tty_new) < 0)	{
			sprintf(log_str,"restoremodes: errno = %d. '%s'\n", errno,sys_errlist[errno]);
			logit(log_str);
			return(error("RESET - Can't restore normal TTY Params",FALSE,tt));
		}
	}
	if (signal(SIGTERM, SIG_IGN) != SIG_IGN)
		signal(SIGTERM, SIG_DFL);
	if (DEBUGGING)	logit("restoremodes: successful\n");
	return(TRUE);
}


/* signal catcher */
	FUNCTION onintr()
{	error("Kill signal; bailing out", TRUE, -1);	}

/* Moved to log_ops.c so others could use without the rest of this file	*/
/*									*/
/* create string with a timestamp for log file */
/* char *stamptime()							*/
/*{									*/
/*	char *asctime();		/* stuff to get timestamp */
/*	struct tm *localtime(), *tp;					*/
/*	struct timeval tv;						*/
/*	struct timezone tz;						*/
/*									*/
/*	gettimeofday (&tv, &tz);		/* fill in timestamp */
/*	tp = localtime ((time_t *)&tv.tv_sec);				*/
/*	return(asctime(tp));						*/
/*}									*/


/* get tty speed for time estimates */
	FUNCTION getspeed(tt)
struct	tt_struct	*tt;
{
#ifdef	SUNOS
	static int speedtbl[] = {0, 50, 75, 110, 134, 150, 200, 300, 600,
		1200, 1800, 2400, 4800, 9600, 19200, 38400, 0};
#endif
#ifdef	HPUX
	static int speedtbl[] = {0, 50, 75, 110, 134, 150, 200, 300, 600,
		900, 1200, 1800, 2400, 3600, 4800, 7200, 9600, 19200,
		38400, 57600, 115200, 230400, 460800, 0};
#endif
	if (tt->tty_modes == 0)
		/* get tty structure */
		if (get_modes(tt) != TRUE)
			error("Can't get TTY parameters",TRUE,tt);
	ttyspeed = speedtbl[tt->tty_new.c_cflag & CBAUD];
	if (DEBUGGING)	{
		sprintf(log_str,"Line speed = %d bits per second\n", ttyspeed);
		logit(log_str);
	}
}

