#include "xmodem.h"
#include "xmit_file.h"

extern	void	clean_up_files();

/* get type of transmission requested (text or binary) */
	FUNCTION gettype(ichar)
char ichar;
{
	if (ichar == 't') return(ichar);
	if (ichar == 'b') return(ichar);
	error("Invalid Send/Receive Parameter - not t or b",TRUE,0);
	return(0);
}

/* print error message and abort if requested; restore normal tty modes */
	FUNCTION error(msg,abort,tt)
char *msg;
int abort;
struct	tt_struct	*tt;
{
#define	ERROR_LIMIT	10	/* if overflow probably restoremodes	*/
	static	int aborted = 0;

	if (abort)	sprintf(log_str,"Abort - %s\n",msg);
	else		sprintf(log_str,"Error - %s\n",msg);
	logit(log_str);
	if (aborted > ERROR_LIMIT)	abort = TRUE;
	aborted++;
fini:	if (abort)	{
		if (tt == (struct tt_struct *)(-1))	{
			/* called by onintr - restore all	*/
			for (tt = ttp; tt->ttname; tt++)
				if (tt->tty_modes)
					restoremodes(tt,TRUE);
		}
		else	restoremodes(tt,abort);  /* put back normal tty modes */
		clean_up_files();
		exit(1);
	}
	if (msg)	strcpy(err_msg,msg);
	return(ERROR);
}


/* Construct a proper (i.e. pretty) sector count for messages */

char	FUNCTION *sectdisp(recvsectcnt, bufsize, plus1)
long recvsectcnt;
int bufsize, plus1;
{
	static char string[20];
	if (plus1)
		recvsectcnt += (bufsize == 128) ? 1 : 8;
	if (bufsize == 128 || recvsectcnt == 0)
		sprintf (string, "%d", recvsectcnt);
	else
		sprintf (string, "%d-%d", recvsectcnt-7, recvsectcnt);
	return(string);
}

/* print elapsed time and rate of transfer in logfile */

int quant[] = { 60, 60, 24};	
char *sep[] = { "second", "minute", "hour" };

	FUNCTION prtime (numsect, seconds)
long numsect;
time_t seconds;
{
	register int i;
	register int Seconds;
	int nums[3];
	int rate = 0;

	if (!LOGFLAG)
		return(0);

	Seconds = (int)seconds;

	if (Seconds != 0)
		rate = 128 * numsect/Seconds;

	for (i=0; i<3; i++) {
		nums[i] = (Seconds % quant[i]);
		Seconds /= quant[i];
	}

	sprintf (log_str, "%ld CP/M Sectors Transfered in ", numsect);
	logit(log_str);
	while (--i >= 0)
		if (nums[i])	{
			sprintf(log_str, "%d %s%c ", nums[i], sep[i],
				nums[i] == 1 ? '\0' : 's');
			logit(log_str);
		}
	logit("\n");

	if (rate != 0)	{
		sprintf(log_str,"Transfer Rate = %d Characters per Second\n", rate);
		logit(log_str);
	}

	return(0);
}

/* Print elapsed time estimate */

	FUNCTION projtime (numsect)
long numsect;
{
	register int i;
	register int seconds;
	int nums[3];

	if (numsect == 0)
		return (0);

/* constant below should really be 1280; reduced to 90% to account for time lost in overhead */

	seconds = 1422 * numsect / ttyspeed + 1;

	for (i=0; i<3; i++) {
		nums[i] = (seconds % quant[i]);
		seconds /= quant[i];
	}

	logit("Estimated transmission time ");
	while (--i >= 0)
		if (nums[i])	{
			sprintf(log_str, "%d %s%c ", nums[i], sep[i],
				nums[i] == 1 ? '\0' : 's');
			logit(log_str);
		}
	logit("\n");
	return (0);
}
