#include "xmodem.h"
#include "xmit_file.h"

/**  receive a file  **/

/* returns TRUE if in the midst of a batch transfer	*/
/* returns FALSE if no more files are coming		*/
/* returns ERROR if unrecoverable error occurs		*/
/* returns NULL_FILE if a null file was received	*/

/* This routine is one HUGE do-while loop with far to many indented levels.
 * I chose this route to facilitate error processing and to avoid GOTOs.
 * Given the troubles I've had keeping the nested IF statements straight,
 * I was probably mistaken...
 */
	FUNCTION rfile(tt,name,tfile,null_file,rec_len)
struct	tt_struct	*tt;
char	*name;
int	tfile;	/* in batch mode use name, not name=cpm_unix(buff) - T/F */
char	*null_file; /* string that signifies a null file, delete file	*/
int	rec_len; /* max record length of data in packet, skip remaining	*/
{
    char *sectdisp();
    char *cpm_unix();
    char *strcpy();
    time_t time();

    int null;	/* is file empty - T/F */
    int fd,     /* file descriptor for created file */
    checksum,   /* packet checksum */
    firstchar,  /* first character of a packet */
    sectnum,    /* number of last received packet (modulo 128) */
    sectcurr,   /* second byte of packet--should be packet number (mod 128) */
    sectcomp,   /* third byte of packet--should be complement of sectcurr */
    tmode,      /* text mode if true, binary mode if false */
    errors,     /* running count of errors (reset when 1st packet starts */
    errorflag,  /* set true when packet (or first char of putative packet) is invalid */
    fatalerror, /* set within main "read-packet" Do-While when bad error found */
    inchecksum, /* incoming checksum or CRC */
    expsect,    /* expected number of sectors (YMODEM batch) */
    bufsize;    /* packet size (128 or 1024) */
    long recvsectcnt;   /* running sector count (128 byte sectors) */
    int bufctr; /* number of real chars in read packet */
    unsigned char *nameptr; /* ptr in filename for MODEM7 protocol */
    time_t start;   /* starting time of transfer */
    int openflag = FALSE;   /* is file open for writing? */
	int	len;	/* length of received file	*/
	void	sendtime();

	if (LOGFLAG)	logit("----\nXMODEM File Receive Function\n");
	if (CRCMODE)	logit("CRC mode requested\n");
	if (DEBUGGING)	{
		sprintf(log_str,"rfile: null_file = '%s' rec_len = %d.\n", null_file,rec_len);
		logit(log_str);
	}

    BATCH = FALSE;          /* don't know if really are in batch mode ! */
    fatalerror = FALSE;
    sectnum = errors = recvsectcnt = 0;
    bufsize = 128;

    if (XMITTYPE == 'b')	tmode =0;		/* binary	*/
	else	if (XMITTYPE == 't')	tmode = 1;	/* text		*/
		else	tmode = 2;			/* UNIX text	*/

    sleep(1);       /* wait a second for other side to get ready */
    if (CRCMODE)        /* start up transfer */
        sendbyte(tt,CRCCHR);
    else
        sendbyte(tt,NAK);


    do                  /* start of MAIN Do-While loop to read packets */
        {   
        errorflag = FALSE;

	if (setjmp(sjbuf))	{
		signal(SIGALRM,SIG_DFL);
		return(ERROR);
	}	else	signal(SIGALRM,sendtime);
	alarm(45);	/* make sure other side is xmodem	*/

        do              /* start by reading first byte in packet */
            {
            firstchar = readbyte(tt,6);
            } 
            while ((firstchar != SOH) 
                && (firstchar != STX) 
                && (firstchar != EOT) 
                && (firstchar != ACK || recvsectcnt > 0) 
                && (firstchar != TIMEOUT) 
                && (firstchar != CAN));

	alarm(0);	/* OK - xmodem, cancel alarm and reset signal	*/
	signal(SIGALRM,SIG_DFL);

        if (firstchar == EOT)           /* check for REAL EOT */
            {
            if (readbyte(tt,1) != TIMEOUT)
                {
                firstchar = TIMEOUT;
                errorflag = TRUE;
                logit ("EOT followed by characters; ignored\n");
                }
            }

        if (firstchar == TIMEOUT)	{	/* timeout? */
		sprintf(log_str,"Timeout on Sector %s\n",
			sectdisp(recvsectcnt,bufsize,1));
		logit(log_str);
		errorflag = TRUE;
            }

        if (firstchar == CAN)           /* bailing out? */
            {
            if ((readbyte(tt,3) & 0x7f) == CAN)
                error("Reception canceled at user's request",TRUE,tt);
            else
                {
                errorflag = TRUE;
                logit("Received single CAN character\n");
                }
            }

        if (firstchar == ACK)           /* MODEM7 batch? */
            {
            int i,c; 

            logit("MODEM7 Batch Protocol\n");
            nameptr = buff;
            checksum = 0;

            for (i=0; i<NAMSIZ; i++)
                {
                c = readbyte(tt,3);

                if (c == CAN)
                    {
                    if (readbyte(tt,3) == CAN)
                        error("Program Canceled by User", TRUE,tt);
                    else
                        {
                        logit("Received single CAN character\n");
                        errorflag = TRUE;
                        break;
                        }
                    }

                if (c == EOT && i == 0)
                    {
                    logit("MODEM7 Batch Receive Complete\n");
                    return (FALSE);
                    }

                if (c == TIMEOUT)
                    {
                    logit("Timeout waiting for MODEM7 filename character\n");
                    errorflag = TRUE;
                    break;
                    }

                if (c == BAD_NAME)
                    {
                    logit("Error during MODEM7 filename transfer\n");
                    errorflag = TRUE;
                    break;
                    }

                *nameptr++ = c;
                checksum += c;
                sendbyte(tt,ACK);
                }

            if (!errorflag)
                {
                c = readbyte(tt,3);
                if (c == CTRLZ)     /* OK; end of string found */
                    {
                    sendbyte(tt,checksum + CTRLZ);
                    if (readbyte(tt,3) == ACK)     /* file name found! */
                        {
                        logit("MODEM7 file name OK");
                        *nameptr = '\000';
			/* use cpm_unix if name ~specified or tfile==TRUE */
                        if (!tfile || !name || !(*name))
				name = cpm_unix(buff);
                        BATCH = TRUE;
                        }
                    else
                        {
                        logit("Checksum error in MODEM7 filename\n");
                        errorflag = TRUE;
                        }
                    }
                else
                    {
                    logit("Length error in MODEM7 fielname\n");
                    errorflag = TRUE;
                    }
                }
            }
            

        if (firstchar == SOH || firstchar == STX)  /* start reading packet */
            {
            bufsize = (firstchar == SOH) ? 128 : 1024;

            if (recvsectcnt == 0)           /* 1st data packet, initialize */
                {
                if (bufsize == 1024)
                    logit("1K packet mode chosen\n");
                start = time((time_t *) 0);
                errors = 0;
                }

	    sectcurr = readbyte(tt,3);
	    sectcomp = readbyte(tt,3);
		if ((sectcurr + sectcomp) == 0xff)  /* is packet number checksum correct? */
		    {  
			if (sectcurr == ((sectnum+1) & 0xff))   /* is packet number correct? */
			    {  
			    if (DEBUGGING)	{
			        sprintf(log_str,"DEBUG: packet %d started\n", sectnum);
				logit(log_str);
			     }

			     /* Read, process and calculate checksum for a buffer of data */

			    if (readbuf(tt,bufsize, 3, tmode, &checksum, &bufctr) != TIMEOUT) 
                                {

				/* verify checksum or CRC */

				if (CRCMODE) 
                                    {
				    checksum &= 0xffff;
				    /* get 16-bit CRC */
				    inchecksum = readbyte(tt,3);
				    inchecksum = (inchecksum<<8) | readbyte(tt,3);
				    }
                        
				else
				    inchecksum = readbyte(tt,3);  /* get simple 8-bit checksum */

				if (inchecksum == checksum) /* good checksum, hence good packet */
				    {  
				    if (DEBUGGING)	logit("checksum ok");
				    errors = 0;
				    recvsectcnt += (bufsize == 128) ? 1 : 8;
				    sectnum = sectcurr; 

				    if (!openflag)      /* open output file if necessary */
				    {
					openflag = TRUE;
					/* was 'creat', need to read */
					if ((fd = open(name,O_RDWR | O_CREAT | O_TRUNC,CREATMODE)) < 0)
				        {
					    sendbyte(tt,CAN);
					    sendbyte(tt,CAN);
					    sendbyte(tt,CAN);
					    error("Can't create file for receive", TRUE,tt);
					}
					sprintf(log_str,"File Name: %s\n",
						name);
					logit(log_str);
				    }

				    /* if packet is padded, discard pad	*/
				    if ((rec_len > 0) && (bufctr > rec_len))
					bufctr = rec_len;
				    if (DEBUGGING)	{
					sprintf(log_str,"rfile: rec_len = %d. bufctr = %d.\n",rec_len,bufctr);
					logit(log_str);
				    }
				    if (write(fd, (char *) buff, bufctr) < 0)
					error("File Write Error", TRUE,tt);
				    else
					sendbyte(tt,ACK);      /* ACK the received packet */
				    }

	            /* Start handling various errors and special conditions */

				else        /* bad checksum */
				    {  
				    sprintf(log_str,"Checksum Error on Sector %s\n", sectdisp(recvsectcnt,bufsize,1));
				    logit(log_str);
				    errorflag = TRUE;
				    }
                                }

			    else    /* read timeout */
				{
				sprintf(log_str,"Timeout while reading sector %s\n",sectdisp(recvsectcnt,bufsize,1));
				logit(log_str);
				errorflag = TRUE;
				}
			    }

                        else        /* sector number is wrong OR Ymodem filename */
                            { 
			    if (sectcurr == 0 && recvsectcnt == 0)  /* Ymodem file-name packet */
				{
				logit("YMODEM Batch Protocol\n");

				/* Read and process a file-name packet */

				if (readbuf(tt,bufsize, 3, FALSE, &checksum, &bufctr) != TIMEOUT) 
                                    {

				    /* verify checksum or CRC */

				    if (CRCMODE) 
                                        {
					checksum &= 0xffff;
					inchecksum = readbyte(tt,3);  /* get 16-bit CRC */
					inchecksum = (inchecksum<<8) | readbyte(tt,3);
				        }
                        
				    else
					inchecksum = readbyte(tt,3);  /* get simple 8-bit checksum */

				    if (inchecksum == checksum) /* good checksum, hence good filename */
					{
					logit("checksum ok");
					strcpy(name, (char *)buff);
                                        expsect = ((buff[bufsize-1]<<8) | buff[bufsize-2]);
					sendbyte(tt,ACK);      /* ACK the packet */
					BATCH = TRUE;
					if (strlen(name) == 0)  /* check for no more files */
					    {
					    logit("YMODEM Batch Receive Complete\n");
					    return (FALSE);
					    }
                                        unixify(name);       /* make filename canonical */
					sprintf(log_str,"YMODEM file name: %s\n", name);
					logit(log_str);
					sprintf(log_str,"YMODEM estimated file length %d sectors\n", expsect);
					logit(log_str);
					sprintf(log_str,"YMODEM header info: %s\n", (char *)buff + strlen(name) + 1);
					logit(log_str);
					}

				    else                /* bad filename checksum */
					{
					logit("checksum error on filename sector\n");
					errorflag = TRUE;
					}
				    }
				else
				    {
				    logit("Timeout while reading filename packet\n");
				    errorflag = TRUE;
                                    }
				}

			    else if (sectcurr == sectnum)   /* duplicate sector? */
				{  
				sprintf(log_str,"Duplicate sector %s flushed\n", sectdisp(recvsectcnt,bufsize,0));
				logit(log_str);
				while(readbyte(tt,3) != TIMEOUT)
				    ;
				sendbyte(tt,ACK);
				}
			    else                /* no, real phase error */
				{
				sprintf(log_str,"Phase Error - Expected packet is %s\n", sectdisp(recvsectcnt,bufsize,1));
				logit(log_str);
				errorflag = TRUE;
				fatalerror = TRUE;
				sendbyte(tt,CAN); sendbyte(tt,CAN); sendbyte(tt,CAN);
				}
			    }
		    }

		else        /* bad packet number checksum */
		    {  
		    sprintf(log_str,"Header Sector Number Error on Sector %s\n", sectdisp(recvsectcnt, bufsize,1));
		    logit(log_str);
		    errorflag = TRUE;
		    }

	    }           /* END reading packet loop */
    
	if ((errorflag && !fatalerror) || recvsectcnt == 0) /* check on errors or batch transfers */
	    {  
	    if (errorflag)
	        errors++;
	    if (recvsectcnt != 0)
		while (readbyte(tt,3) != TIMEOUT)  /* wait for line to settle if not beginning */
		    ;

	    if (CRCMODE && recvsectcnt == 0 && errors == CRCSWMAX)
		{
		CRCMODE = FALSE;
		logit("Sender not accepting CRC request, changing to checksum\n");
		sendbyte(tt,NAK);
		}
	    else if (!CRCMODE && recvsectcnt == 0 && errors == CRCSWMAX)
		{
		CRCMODE = TRUE;
		logit("Sender not accepting checksum request, changing to CRC\n");
		sendbyte(tt,CRCCHR);
		}
	    else if (CRCMODE && recvsectcnt == 0)
		sendbyte(tt,CRCCHR);
	    else
		sendbyte(tt,NAK);
	    }
	}
        while ((firstchar != EOT) && (errors < ERRORMAX) && !fatalerror);   /* end of MAIN Do-While */

	if ((firstchar == EOT) && (errors < ERRORMAX))  /* normal exit? */
	    {
	    null = is_null_file(fd,null_file);
	    close(fd);
	    sendbyte(tt,ACK);
	    if (LOGFLAG)	{
		logit("Receive Complete\n");
		prtime (recvsectcnt, time((time_t *) 0) - start);
	    }
	    if (null)	return(NULL_FILE);
	    if (BATCH)          /* send appropriate return code */
		return(TRUE);
	    else
		return(FALSE);
	    }
	else                /* no, error exit */
	    { 
	    if (recvsectcnt != 0)
		sendbyte(tt,CAN); sendbyte(tt,CAN); sendbyte(tt,CAN);
		return(error("ABORTED -- Too Many Errors", FALSE, tt));
	    }
}

	FUNCTION is_null_file(fd,null_file)
int	fd;
char	*null_file;
{
	int len, count,i;
	char buf[65];

	if ((len = lseek(fd,0l,2)) <= 0)	return(TRUE);
	if ((!null_file) || (!(count=strlen(null_file))) || (count>64) || (len<count))
		return(FALSE);
	if ((lseek(fd,0l,0) < 0)||((i = read(fd,&buf[0],count)) != count))
		return(TRUE);
	buf[count] = '\0';
	if (strcmp(buf,null_file) == 0)	return(TRUE);
	return(FALSE);
}
