/*				FILE = tty_ops.c			*/

#include	"xmit_file.h"
#include	"xmodem.h"

char	*ttyname();

	FUNCTION get_modes(tt)
struct	tt_struct	*tt;
{
#define	ROOT 0
	if (DEBUGGING)	{
		sprintf(log_str,"get_modes called: tt = %o\n",tt);
		logit(log_str);
	}
	if (!tt->tty_modes)	{
		if (ioctl(tt->tty_fd,TCGETA,&tt->tty_old) < 0)
			return(error("get_modes: Can't get TTY Parameters", FALSE,0));
		if (stat(tt->ttname, &tt->statbuf) < 0)	/* get tty status */
			return(error("get_modes: Can't get TTY Status", FALSE,0));
		if ((tt->statbuf.st_mode & 022) && ROOT)	{
			sprintf(log_str,"get_modes: chmod for file '%s'\n", tt->ttname);
			logit(log_str);
			/* turn messages off, requires root privleges */
			if (chmod(tt->ttname,(int)tt->statbuf.st_mode & ~022) < 0)
				return(error("get_modes: Can't change TTY mode",FALSE,0));
			else   tt->wason = TRUE;
		}
		tt->tty_modes = FLAG_INIT;
		if (tt->real_speed == -1)
			tt->real_speed = tt->default_speed;
	}
	tt->tty_new.c_iflag = tt->tty_old.c_iflag;
	tt->tty_new.c_oflag = tt->tty_old.c_oflag;
	tt->tty_new.c_cflag = tt->tty_old.c_cflag;
	tt->tty_new.c_lflag = tt->tty_old.c_lflag;
	tt->tty_new.c_line  = tt->tty_old.c_line;
	if (DEBUGGING)	logit("get_modes: successful\n");
	return(TRUE);
}

struct host FUNCTION *get_tty(host_id)
char	*host_id;
{
	struct	termio	temp;
	int	speed = -1, calling = TRUE;
	int	index1,index2;
	struct	tt_struct *tt;
	struct	host	*p;
	if (DEBUGGING)	{
		sprintf(log_str,"get_tty called: host_id = `%s'\n",
			host_id ? host_id : "NULL");
		logit(log_str);
	}
	if (host_id == 0)	{
		calling = FALSE;
		p = hostp + (this_host_index -1);
		index1 = this_host_index;
		host_id = ttyname(0);	/* remote user called us on stdin */
		if (ioctl(0,TCGETA,&temp) < 0)	{
			error("Can't get stdin parameters",FALSE,0);
			return(FALSE);
		}
		speed = temp.c_cflag & CBAUD;
		if (DEBUGGING)	{
			sprintf(log_str,"get_tty: (called) stdin = '%s' speed = %d.\n",host_id,speed);
			logit(log_str);
		}
		goto	found1;
	}
	for (index1 = 1, p = hostp; p->host_id; p++, index1++)
		if (strcmp(host_id,p->host_id) == 0)	break;
	if (p->host_id == 0)	{
		p = 0;
		index1 = 0;
		if (DEBUGGING)	logit("get_tty: host_id not in hosts table\n");
		/* is host_id a ttname, => direct connect or was called	*/
found1:		for (index2 = 1, tt = ttp; tt->ttname; index2++, tt++)
			if (!strcmp(host_id,tt->ttname))	{
				if (tt->in_use)	{
					error("get_tty: tty inuse",FALSE,0);
					return(FALSE);
				}
				if (p || tt->host_index)	break;
				error("get_tty: Requested tty not associated with host",FALSE,0);
				return(FALSE);
			}
		if (!tt->ttname)	{
			error("get_tty: Unknown host_id",FALSE,0);
			return(FALSE);
		}
		if (!index1)	index1 = tt->host_index;
		p = hostp + (index1 -1);
		goto	found2;
	}
/* debug logit("get_tty: found host entry\n"); */
/* debug prt_host(p,FALSE); */
	if ((!p->telnbr) && (!p->tt_index))	{
		error("get_tty: No telephone number or direct connect",FALSE,0);
		return(FALSE);
	}
	if (!p->telnbr)		{
		index2 = p->tt_index;
		tt = ttp + (index2 -1);
/* debug sprintf(log_str,"get_tty: no telnbr - index2 = %d.\n",index2); */
/* debug logit(log_str); */
		goto	found2;
	}
	if (p->tt_index)	{
/* temp 	index2 = p->tt_index;		*/
/* temp 	tt = ttp + (index2 -1);	*/
/* temp 	goto	found2;			*/
		error("get_tty: tty busy",FALSE,0);
		return(FALSE);	/* busy	*/
	}
	for (index2 = 1, tt = ttp; tt->ttname; index2++, tt++)
		if ((!tt->in_use) && (!tt->host_index))	{
found2:			if (DEBUGGING)	{
				sprintf(log_str,"get_tty: before open of file '%s'\n",tt->ttname);
				logit(log_str);
			}
			if ((tt->tty_fd = open(tt->ttname,O_RDWR | O_NDELAY | O_EXCL)) < 0)	{
				sprintf(log_str,"get_tty: %s\n",sys_errlist[errno]);
				logit(log_str);
				error("get_tty: Unable to open tty",FALSE,0);
				return(FALSE);
			}
/* debug sprintf(log_str,"get_tty: fd = %d.\n",tt->tty_fd); */
/* debug logit(log_str); */
			tt->in_use = TRUE;
			break;
		}
	if (!tt->ttname)	{
		error("get_tty: No available tty",FALSE,0);
		return(FALSE);
	}
	tt->host_index = index1;
	p->tt_index = index2;
/* debug sprintf(log_str,"get_tty: index 1 = %d. index2 = %d.\n", index1,index2); */
/* debug logit(log_str); */
	if (speed != -1)	tt->real_speed = speed;
	if (setmodes(tt,FLAG_XMIT) != TRUE)	{
		release_tty(p);
		return(FALSE);
	}
/* debug prt_host(p,TRUE); */
/* debug sprintf(log_str,"get_tty: successful - calling = %d.\n",calling); */
/* debug logit(log_str); */
	p->connected |= CONNECTED_TTY;
	p->calling = calling;
	return(p);
}

	FUNCTION release_tty(host)
struct	host	*host;
{
	struct	tt_struct *tt;
/* debug sprintf(log_str,"release_tty called: host = '%s'\n", */
/* debug host ? host->host_id : "NULL"); */
/* debug logit(log_str); */
	if (!host || !host->tt_index)	return;
	tt = ttp + (host->tt_index - 1);
	restoremodes(tt,TRUE);
	if (tt->tty_fd != -1)	{
		if (close(tt->tty_fd) < 0)	{
			sprintf(log_str,"release_tty: close error - %s\n",sys_errlist[errno]);
			logit(log_str);
		}
		tt->tty_fd = -1;
	}
	if (host->telnbr)	host->tt_index = 0;
	tt->real_speed = -1;
	tt->in_use = FALSE;
	host->connected &= ~CONNECTED_TTY;
	return;
}
