/*				FILE = utils.c				*/

#include	"xmit_file.h"
#include	"xmodem.h"

char	*strcat(), *strcpy();

/*
cat_field(to,str1,str2,str3)
	Concatenate 'str1', 'str2', and 'str3' to 'to'. Don't copy anything
	if 'str2' is null or empty.  The number of bytes copied is returned.
	`str1' is the pre-string and 'str3' is the post-string.
*/
	FUNCTION cat_field(to,str1,str2,str3)
char	*to,*str1,*str2,*str3;
{
	int	i = 0, j = 0;

	if (str2 && (i = strlen(str2)))	{
		if (str1)	{
			j = strlen(str1);
			strcpy(to,str1);
			to += j;
		}
		strcpy(to,str2);
		to += i;
		i += j;
		if (str3)	{
			j = strlen(str3);
			strcpy(to,str3);
			to += j;
		}	else	j = 0;
		i += j;
	}
	*to = '\0';
	return(i);
}

char	FUNCTION *files(dir,except)
char	*dir;
char	*except;	/* skip any file beginning with this string	*/
{
	static	DIR	*dir_p = 0;
	struct	dirent	*p;
	if (!dir)	{
		if (dir_p)	{
			closedir(dir_p);
			dir_p = 0;
		}
		return(0);
	}
	if (!dir_p)	if (!(dir_p = opendir(dir)))	return(0);
loop:	if(!(p = readdir(dir_p)))	return(0);
	if (strcmp(p->d_name,".") && strcmp(p->d_name,"..") && strncmp(p->d_name,except,strlen(except)))
		return(p->d_name);
	goto	loop;
}

	FUNCTION p_dirent(p)
struct	dirent	*p;
{
	printf("d_fileno = %d. d_reclen = %d. d_namlen = %d.\n",
		p->d_fileno,p->d_reclen,p->d_namlen);
	printf("d_name = '%s'\n",p->d_name);
	return(TRUE);
}

/* Wait for a string of characters, exact match required.  Use alarm	*/
/* or will wait forever, readbyte specifies no time limit. No wildcards.*/
	FUNCTION single_match(tt,s,lower)
struct	tt_struct	*tt;	/* where to read from			*/
char	*s;	/* string to match, not changed				*/
int	lower;	/* convert incoming characters to lower case - T/F	*/
{
	char	c, *p = s;
	int	i;

	if (DEBUGGING)	{
		sprintf(log_str,"single_match: called with tt = %o, s = '%s' lower = %d.\n",tt,s,lower);
		logit(log_str);
	}
	while (*p)	{	/* until end of string */
		while (TRUE)	{
			if ((i = readbyte(tt,-1)) == ERROR)	continue;
			c = i & 0177;
			if ((lower ? tolower(c) : c) == *p)	p++;
			else	p = s;	/* failed - reset search string	*/
			break;
		}
	}
}

/* str_match(tt,s1,s2,s3,lower,time)					*/
/*	Match any string exactly.  Wild card ('*' matches zero or more	*/
/*	characters) allowed and first match wins.  A WILD_ONE character	*/
/*	matches any one character.  A null input string (i.e. s1, s2,	*/
/*	or s3) does not match.  Returns 0 if s1, s2, and s3 are null, 1	*/
/*	if s1 matches, 2 if s2, 3 if s3 matches and -1 if read error or	*/
/*	timeout.  Use alarm if time == -1 or will wait forever, call to	*/
/*	readbyte with -1 means no time limit.				*/
	FUNCTION str_match(tt,s1,s2,s3,lower,time)
struct	tt_struct	*tt;
char	*s1;	/* primary string to match, not changed			*/
char	*s2;	/* alternate 1 string to match, not changed		*/
char	*s3;	/* alternate 2 string to match, not changed		*/
int	lower;	/* convert incoming characters to lower case - T/F	*/
int	time;	/* # seconds before readbyte timeout, -1 => forever	*/
{
#define	WILD_ONE	'%'
	char	*p1 = 0, *p2 = 0, *p3 = 0, c;
	int	p1_wild = FALSE, p2_wild = FALSE, p3_wild = FALSE;
	int	i, ret = 0;

	if (DEBUGGING)	{
		sprintf(log_str,"str_match: tt = %0 s1 = '%s'\n	s2 = '%s'\n 	s3 = '%s'\n	lower = %d. time = %d.\n",tt,s1,s2,s3,lower,time);
		logit(log_str);
	}
	/* if all strings are null return zero				*/
	if (s1 && *s1)	p1 = s1;
	if (s2 && *s2)	p2 = s2;
	if (s3 && *s3)	p3 = s2;
	if (!p1 && !p2 && !p3)	goto	fini;

	ret = ERROR;
	/* skip leading wildcards in both strings and set wild variable	*/
	if (p1)
		while (*p1 == '*')	{
			p1++;
			p1_wild = TRUE;
		}
	if (p2)
		while (*p2 == '*')	{
			p2++;
			p2_wild = TRUE;
		}
	if (p3)
		while (*p3 == '*')	{
			p3++;
			p3_wild = TRUE;
		}

	/* read until error or timeout -return(ERROR)- or match 	*/
	/* -return(1 or 2)-						*/
	while ((i = readbyte(tt,time)) != ERROR)	{
		c = i & 0177;		/* no parity bit allowed	*/
		if (lower)	c = tolower(c);
		if (p1)	{
match1:			if ((c == *p1) || (*p1 == WILD_ONE))	{
			/* matched character, increment primary string	*/
				p1++;
				p1_wild = FALSE;
				while (*p1 == '*')	{
					p1++;	/* skip wildcards	*/
					p1_wild = TRUE;
				}
				if (!*p1)	{
					ret = 1;	/* match found	*/
					goto	fini;
				}
			}	else		/* no match		*/
				if (!p1_wild) {	/* if wild, ignore char	*/
					/* else reset search string and	*/
					/* check if first char of string*/
					p1 = s1;
					if (c == *p1)	goto	match1;
				}
		}
		if (p2)	{
match2:			if ((c == *p2) || (*p2 == WILD_ONE))	{
			/* matched character, increment alternate string */
				p2++;
				p2_wild = FALSE;
				while (*p2 == '*')	{
					p2++;	/* skip wildcards	*/
					p2_wild = TRUE;
				}
				if (!*p2)	{
					ret = 2;	/* match found	*/
					goto	fini;
				}
			}	else		/* no match		*/
				if (!p2_wild) {	/* if wild, ignore char	*/
					/* else reset search string and	*/
					/* check if first char of string*/
					p2 = s2;
					if (c == *p2)	goto	match2;
				}
		}
		if (p3)	{
match3:			if ((c == *p3) || (*p3 == WILD_ONE))	{
			/* matched character, increment alternate string */
				p3++;
				p3_wild = FALSE;
				while (*p3 == '*')	{
					p3++;	/* skip wildcards	*/
					p3_wild = TRUE;
				}
				if (!*p3)	{
					ret = 3;	/* match found	*/
					goto	fini;
				}
			}	else		/* no match		*/
				if (!p3_wild) {	/* if wild, ignore char	*/
					/* else reset search string and	*/
					/* check if first char of string*/
					p3 = s3;
					if (c == *p3)	goto	match3;
				}
		}
	}
fini:	if (DEBUGGING)	{
		sprintf(log_str,"str_match: returned - %d.\n",ret);
		logit(log_str);
	}
	return(ret);		/* readbyte returned ERROR	*/
}

/*
ctob(str)
	Convert an ASCII string to a binary integer. The string may be 
	positive or negative and be in decimal, octal or hexadecimal notation.
	Thus 0X25, 045 and 37 are identical. An invalid character stops the
	conversion. If there are no characters a zero is returned.
*/
	FUNCTION ctob(str)
char	*str;
{
	long	total;
	long	digit, base;
	long	sign	= 1;
	total = 0;
	if (*str == '-')	{
		sign = -1;
		str++;
	}	else
		if (*str == '+')	str++;
	if (*str == 'x' || *str == 'X')	goto	hex;
	if (*str < '0' || *str > '9')	goto	done;
	if (*str != '0')	{
		base = 10;
		goto	start;
	}
	++str;
	if (*str != 'x' && *str != 'X')	{
		base = 8;
		goto	start;
	}
hex:	base = 16;
	str++;
start:	while(TRUE)	{
		if (*str <= '9' && *str >= '0')	digit = *str - '0';
		else	/* could be A->F - convert with proper case	*/
			if (*str <= 'Z')	{
				if ((digit = *str - 'A' + 10) < 10)	break;
			}	else
				if ((digit = *str - 'a' + 10) < 10)	break;
		if (digit < 0 || digit >= base)	break;
		total = total * base + digit;
		str++;
	}
done:	total *=sign;
	return ((int)total);
}

/*======================================================================*
* strfit(pattern,sample)						*
* If the two strings fit, return a zero.  Else return a non-zero.	*
*									*
* Algorithm: Determine if all of the letters in 'sample' appear in	*
*	     'pattern' in the same order.  The first letter must match.	*
*									*
* Examples:	WordDump and wdmp	- match				*
*		DisassemblyMem and Dasm	- match				*
*		WordDump and DUMP	- no match, first letters	*
*		WordDUMP and WDpm	- no match, letters out of order*
*									*
* From: David Rankin							*
*	Prime Computer, Framingham, Ma.					*
*									*
* Published:	EDN, April 13, 1989					*
*=======================================================================*/
	FUNCTION strfit(pattern,sample)
char	*pattern,*sample;
{
	/* See if the first letters match				*/
	if (toupper(*pattern) != toupper(*sample))	return(-1);

	/* See if every character in sample is found in pattern		*/
	while (*sample != 0)	{

		/* Look for a character in pattern that matches		*/
		/* the current character in sample			*/
		while (toupper(*pattern) != toupper(*sample))	{

			/* Move to the next character in pattern	*/
			pattern++;

			/* See if we have reached end of pattern	*/
			if (*pattern == 0)	return(-1);
		}
		/* Move to next character in sample			*/
		sample++;
	}

	/* We have successfully matched every character in sample to a	*/
	/* character in pattern						*/
	return(0);
}

/*
atos(to,from)
	Copy an ASCII string to an internal string.  Convert all escaped
	characters (e.g. \r converts to a cariage return character) and
	remove enclosing double quote characters.
*/
	FUNCTION atos(to,from)
char	*to;	/* where to store the internal string		*/
char	*from;	/* where to get the ASCII string to copy	*/
{
	char	*p, *p1;
	int	ret = TRUE;

	p1 = from;
	if (*from == '\"')	{	/* handle quotes	*/
		p = ++from;
		while (*p != '\0')	p++;
		while (((*p != '\"')||(*(p-1) == '\\')) && (from <= p))	p--;
		if (p <= from)	{
			ret = FALSE;
			sprintf(log_str,"warning: no ending quote, assumed - '%s\n",p1);
			logit(log_str);
		}	else	*p ='\0';
	}
	while (*from != '\0')	{
		if (*from == '\\')	{
			from++;
			if (*from == 'b')	*to = '\b';
			else	if (*from == 'f')	*to = '\f';
			else	if (*from == 'n')	*to = '\n';
			else	if (*from == 'r')	*to = '\r';
			else	if (*from == 't')	*to = '\t';
			else	if (*from == 'v')	*to = '\v';
			else	if (*from == '\\')	*to = '\\';
			else	if (*from == '?')	*to = '?';
			else	if (*from == '\'')	*to = '\'';
			else	if (*from == '\"')	*to = '\"';
			/* allow control characters ctrl-C and ctrl-D */
			else	if (*from == 'c')	*to = '\003';
			else	if (*from == 'd')	*to = '\004';
			else	{
				ret = FALSE;
				sprintf(log_str,"Warning: character string contains an illegal character - no conversion - '%s'\n",p1);
				logit(log_str);
				*to++ = '\\';
				*to = *from;
			}
		}	else	*to = *from;
		to++;
		from++;
	}
	*to = '\0';
	return(ret);
}
