/*
 *  XMODEM -- Implements the Christensen XMODEM protocol, 
 *            for packetized file up/downloading.    
 *
 *	This version runs on 4.2/4.3BSD ONLY!  It won't work ANYWHERE else.  
 *
 *	I have tried to keep the 4.2isms (select system call, 4.2BSD/v7 tty
 *	structures, gettimeofday system call, etc.) in the source file
 *	getput.c; but I make no guarantees.  Also, I have made no attempt to
 *	keep variable names under 7 characters (although a cursory check
 *	shows that all variables are unique within 7 first characters).
 *	The program has been successfully run on VAXes (4.2/4.3BSD) and SUNs
 *	(2.0/3.0) against MEX-PC.
 *
 *   -- Based on UMODEM 3.5 by Lauren Weinstein, Richard Conn, and others.
 *
 *  XMODEM Version 1.0  - by Brian Kantor, UCSD (3/84)
 *
 *  Version 2.0 (CRC-16 and Modem7 batch file transfer) -- Steve Grandi, NOAO (5/85)
 *
 *  Version 2.1 (1K packets) -- Steve Grandi, NOAO (7/85)
 *
 *  Version 2.2 (general clean-ups and multi-character read speed-ups) -- Steve Grandi, NOAO (9/85)
 *
 *  Version 2.3 (napping while reading packet; split into several source files) -- Steve Grandi, NOAO (1/86)
 *
 *  Version 3.0 (Ymodem batch receive; associated changes) -- Steve Grandi, NOAO (2/86)
 *
 *  Version 3.1 (Ymodem batch send; associated changes) -- Steve Grandi, NOAO (8/86)
 *
 *  Version 3.2 (general cleanups) -- Steve Grandi, NOAO (9/86)
 *
 *  Please send bug fixes, additions and comments to:
 *	{ihnp4,seismo,hao}!noao!grandi 
 */

#include	"xmit_file.h"
#include	"xmodem.h"
#include	"xmodem_allocate.h"

char	*mktemp();

	FUNCTION xmodem(host,filename,flags)
struct	host	*host;
char	*filename;
char	*flags;
{
	char *getenv();
	char *unix_cpm();
	char *strcpy();
	char *strcat();
	char	*getcwd();
	char	*files();
	
	char *fname = filename;	/* convenient place to stash file names */
	char	*fptr = flags;		/* stash flags			*/
	
	char *stamptime();		/* for timestamp		*/

	struct	tt_struct	*tt;
	struct stat filestatbuf;	/* file status info		*/
	int	value = TRUE;
	char	file_tmp[MAX_FILENAME + 1];
	char	*p, *dir, dir_buf[256];

	char flag;
	long expsect;
	int	i, ret = TRUE;

	/* initialize option flags */

	XMITTYPE = 't';		/* assume text transfer			*/
/*	DEBUGGING = FALSE;	/* Turn on debugging ?			*/
	RECVFLAG = FALSE;	/* not receive				*/
	SENDFLAG = FALSE;	/* not send either			*/
	BATCH = FALSE;		/* nor batch				*/
	CRCMODE = FALSE;	/* use checksums for now		*/
/*	DELFLAG = FALSE;	/* don't delete old log file		*/
	LOGFLAG = FALSE;	/* log xmodem header info ?			*/
	LONGPACK = FALSE; 	/* do not use long packets on transmit	*/
	MDM7BAT = FALSE;	/* no MODEM7 batch mode			*/
	YMDMBAT = FALSE;	/* no YMODEM batch mode			*/
	TFILE = TRUE;		/* use mktemp file in batch mode, receive */

/*	printf("XMODEM Version %d.%d", VERSION/10, VERSION%10);		*/
/*	printf(" -- UNIX-CP/M Remote File Transfer Facility\n");	*/

	tt = ttp + (host->tt_index -1);

	while ((flag = *flags++) != '\0')
	    switch (flag) {
		case '-' : value = FALSE;
			   break;
		case '+' : value = TRUE;
			   break;
		case 'x' : /* test moved to xtransfer.c (main), needed sooner */
/*			   DEBUGGING = value;  /* turn on debugging log */
			   break;
		case 'c' : CRCMODE = value; /* enable CRC on receive */
			   break;
		case 'd' : /* not used	*/
/*			   DELFLAG = value;  /* delete log file */
			   break;
		case 'l' : LOGFLAG = ~value;  /* turn off log  */
			   break;
		case 'B' : MDM7BAT = value;  /* turn on MODEM7 batch protocol */
			   BATCH   = value;
			   break;
		case 'Y' : YMDMBAT = value;  /* turn on YMODEM batch protocol */
			   BATCH   = value;
			   break;
		case 'K' : LONGPACK = value;  /* use 1K packets on transmit */
			   break;
		case 'm' : TFILE = value;	/* use mktemp file	*/
			   break;
		case 'r' : RECVFLAG = value;	/* receive file	*/
			   break;
		case 's' : SENDFLAG = value;	/* send file	*/
			   break;
		case 'u' :			/* UNIX text mode	*/
		case 'b' : 			/* binary mode		*/
		case 't' : XMITTYPE = flag;	/* text mode		*/
			   break;
		default  : sprintf (log_str,"Invalid Flag %c ignored\n", flag);
			   logit(log_str);
			   break;
	   }

	if (DEBUGGING)	LOGFLAG = TRUE;

	if (LOGFLAG)	{
		sprintf(log_str,"\n++++++++  %s", stamptime());
		logit(log_str);
		sprintf(log_str,"XMODEM Version %d.%d\n",
			VERSION/10, VERSION%10);
		logit(log_str);
	}

	getspeed(tt);		/* get tty-speed for time estimates */

	if (RECVFLAG && SENDFLAG)
		return(error("Fatal - Both Send and Receive Functions Specified", FALSE,0));

	if (MDM7BAT && YMDMBAT)
		return(error("Fatal - Both YMODEM and MODEM7 Batch Protocols Specified", FALSE,0));

	if (!RECVFLAG && !SENDFLAG)
		return(error("Fatal - Either Send or Receive Function must be chosen!",FALSE,0));

	if (RECVFLAG)	{
		if(!filename || !*filename)	{
			sprintf(log_str,
				"Batch Receive Started in %s mode\n",
				(XMITTYPE == 'b') ? "binary" : "text");
			logit(log_str);
		}	else	{
			int	fd;
			if((fd = open(filename,0)) != -1) { /* overwriting ? */
				sprintf(log_str,
					"Warning -- Target File Exists and is Being Overwritten\n");
				logit(log_str);
				close(fd);
			}
			sprintf(log_str,"Receiving file = '%s' in %s mode, flags =%s\n",
				filename,(XMITTYPE == 'b')?"binary":"text", fptr);
			logit(log_str);
		}
		setmodes(tt,FLAG_XMODEM); /* set tty modes for transfer */
		while (TRUE)	{	/* receive files */
			if (!filename || !*filename)	{
				strcpy(file_tmp,DEFAULT_XGET_PROTO);
				if ((fname=mktemp(file_tmp)) == 0)
					error("xmodem: Unable to mktemp  filename\n",TRUE,tt);
				else	{
					sprintf(log_str,"xmodem: mktemp file = '%s'\n",fname);
					logit(log_str);
				}
			}
			if ((i = rfile(tt,fname,TFILE,host->null_file, host->rec_len)) == ERROR) {
				ret = ERROR;
				unlink(fname);
				break;
			}
			if (i == NULL_FILE)	{
				error("xmodem: Deleted received null file\n",FALSE,tt);
				unlink(fname);
				break;
			}
			if (!BATCH)	{
				/* responses only between XMODEM single	*/
				/* file non-BATCH retrievals		*/
				if ((i = responses(host,RESPOND_GET_BTWN)) != TRUE)	{
					ret = (i==FALSE?TRUE:ERROR);
					break;
				}
				setmodes(tt,FLAG_XMODEM);
			}	else	/* BATCH, check if finished	*/
				if (i == FALSE)	break;
		}
		setmodes(tt,FLAG_XMIT);	/* set tty modes to xmit	*/
		sleep(2);		/* give other side time to return to terminal mode */
/* debug */	logit("xmodem: receive successful\n");
		return(ret);
	}

	if (SENDFLAG && BATCH)	{
		if (YMDMBAT)	{
			sprintf(log_str,"YMODEM Batch Send Started in %s mode\n", (XMITTYPE == 'b') ? "binary" : "text");
			logit(log_str);
		}	else if (MDM7BAT)	{
			sprintf(log_str,"MODEM7 Batch Send Started in %s mode\n", (XMITTYPE == 'b') ? "binary" : "text");
			logit(log_str);
		}

		if (setmodes(tt,FLAG_XMODEM) != TRUE)
			return(error("xmodem: setmodes failed for BATCH send\n",FALSE,tt));
		if ((filename == 0) || (*filename == 0))	{
			if ((dir = getcwd(dir_buf,256)) == 0)
				error("xmodem: getcwd failed\n",TRUE,tt);
/* debug 	sprintf(log_str,"xmodem: getcwd returned '%s'\n",dir); */
/* debug 	logit(log_str); */
			while (p = files(dir,EXCEPT_LOG))	{
				sprintf(log_str,"files returned '%s'\n",p);
				logit(log_str);
				sfile(tt,p);
			}
		}	else	{
			if (stat(filename, &filestatbuf) < 0) {
				sprintf(log_str,"\nFile %s not found\n",
					filename);
				error(log_str,FALSE,tt);
				goto	fini;
			}
			sfile(tt,filename);
		}
fini:		sfile(tt,"");
		setmodes(tt,FLAG_XMIT);
		logit("Batch Send Complete\n");
		sleep(2);
		return(TRUE);
	}

	if (SENDFLAG && !BATCH)	{
		if ((filename == 0) || (*filename == 0))	{
			if ((dir = getcwd(dir_buf,256)) == 0)
				error("xmodem: getcwd failed\n",TRUE,tt);
/* debug 	sprintf(log_str,"xmodem: getcwd returned '%s'\n",dir); */
/* debug 	logit(log_str); */
			if (!(filename = files(dir,EXCEPT_LOG)))
				return(error("xmodem: send directory empty\n",FALSE,tt));
		}	else	dir = 0;
		if (setmodes(tt,FLAG_XMODEM) != TRUE)
			return(error("xmodem: setmodes failed for send\n", FALSE,tt));
		while (TRUE)	{
			if (stat(filename, &filestatbuf) < 0)
				return(error("Can't find requested file", FALSE,0));
			expsect = (filestatbuf.st_size/128)+1;
			/* sprintf(log_str, */
			/*	"Sending file = '%s` in %s mode, %d bytes\n", */
			/*	filename,(XMITTYPE == 'b')?"binary":"text", */
			/*	filestatbuf.st_size); */
			/* logit(log_str); */
			projtime(expsect);
			if ((ret = sfile(tt,filename)) != TRUE)	break;
			if (!dir || !(filename = files(dir,EXCEPT_LOG))) break;
			if ((i=responses(host,RESPOND_PUT_BTWN)) != TRUE) {
				ret = (i==FALSE?TRUE:ERROR);
				break;
			}	else	setmodes(tt,FLAG_XMODEM);
		}
		setmodes(tt,FLAG_XMIT);
		sleep(2);
		return(ret);
	}
}
