////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//                                                                      
//      FILE NAME : small.c                                              
//                                                                      
//      AUTHOR : John Tebbutt                                           
//                                                                      
//      DESCRIPTION : Test HP's X.500 API stuff.
//                                                                      
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

#include    <iostream.h>
#include    <fstream.h>
#include    <string.h>
#include    <malloc.h>
#include    <assert.h>
#include    <memory.h>
#include    <xom.h>
#include 	<xds.h>                // Directory Service header
extern "C"{
#include 	<xat.h>                // X/Open application tool kit header
}
#include 	<xdsbdcp.h>            // Basic Directory Contents Package header  
#include 	<xdsmdup.h>            // MHS Directory User Package header        
#include 	"x5externs.h"


main()
{
        DS_status           error;  	  // return value from DS functions   
        OM_workspace        workspace;    // workspace for objects            
        OM_private_object   session;      // session for directory operations 
        OM_private_object   result;       // result of read operation         
        OM_sint             invoke_id;    // Invoke-ID of the read operation  
        char                *myname = 		"small";
        int                 i;            // Loop variable 
		OM_object			dn;

    //
	// Set diagnostic print option for XAT debugging library
	print_option= PRINT_FORMAT;

	tellme( myname, "Hello\n" );

    //
    // Construct the purported common name of the entry
    //
	dn = at_string_to_dn( "/C=US/O=NIST" );

    // 
    // variables to extract information from the read result...
    //
	// This is ridiculous. HP C++ compiler would not allow the following
	// initialization statements:
    // OM_type     entry_list[]        = { DS_ENTRY, 0 };
    // OM_type     attributes_list[]   = { DS_ATTRIBUTES, 0 };
    // OM_type     values_list[]       = { DS_ATTRIBUTE_VALUES, 0 };
	//
	// ...so we have to type it all out longhand...
	//
	OM_type     entry_list[2];
	OM_type     attributes_list[2];
	OM_type     values_list[2];
	OM_type     types_list[2];

	entry_list[0] = DS_ENTRY;
	entry_list[1] = 0;

	attributes_list[0] = DS_ATTRIBUTES;
	attributes_list[1] = 0;

	values_list[0] = DS_ATTRIBUTE_VALUES;
	values_list[1] = 0;

	types_list[0] = DS_ATTRIBUTE_TYPE;
	types_list[1] = 0;


    OM_public_object    entry;
    OM_public_object    attributes;
    OM_public_object    thisAttr;     // current attribute 
    OM_value_position   total_number, // see om_get() info below 
						level_2_total;
    OM_object_identifier entryOC; 	  // the object class of the directory entry

    
	tellme( myname, "Calling ds_initialize()\n" );

    if( ( ( workspace = (OM_workspace)ds_initialize() ) == DS_NULL_RESULT ) ){

		cerr << myname << "Workspace initialization failure";
		goto cleanUpAndExit;
	}

	tellme( myname, "Calling ds_bind()\n" );

//    if( ( error = (DS_status)ds_bind( DS_DEFAULT_SESSION, workspace, 
    if( ( error = (DS_status)ds_bind( DS_DEFAULT_SESSION, &session, workspace ) ) 
	
		!= DS_SUCCESS ){
	
		tellme( myname, "ds_bind() failure. Error follows\n" );

		at_print_object( (OM_object)error );
	}

	tellme(myname, "Here's our bound session\n" );

	at_print_object( (OM_object)session );

	tellme( myname, "Calling ds_read()\n" );

    if( ( error = (DS_status)ds_read(	session, 
                            			DS_DEFAULT_CONTEXT, 
                            			dn, 
										DS_SELECT_ALL_TYPES_AND_VALUES,
                            			&result, 
                            			&invoke_id ) ) 
							
			!= DS_SUCCESS ){
		
		tellme( myname, "ds_read() failure. Error follows \n" );

		at_print_object( (OM_object)error );

		tellme( myname, "Calling ds_unbind()\n" );

    	if( ( error = (DS_status)ds_unbind(	session ) ) 
							
			!= DS_SUCCESS ){

				tellme( myname, "Unbind failure\n" );
				at_print_object( (OM_object)error );

			}

		goto cleanUpAndExit;

	}

	tellme( myname, "Calling ds_unbind()\n" );

    if( ( error = (DS_status)ds_unbind(	session ) ) 
							
			!= DS_SUCCESS ){

				tellme( myname, "Unbind failure\n" );
				at_print_object( (OM_object)error );
			}

	//
	// Print out the read result...
	//
	at_print_object( (OM_object)result );


cleanUpAndExit:

    //
    //  We can now safely release all the private objects
    //  and the public objects we no longer need
    //

	tellme( "cleanUpAndExit", "WE ARE OUTTA HERE!!!\n" );
     
    om_delete(session);
    om_delete(result);
    om_delete(entry);
    om_delete(attributes);

	tellme( myname, "Calling ds-shutdown()\n" );

    ds_shutdown() ;

	tellme(myname, "Over and out\n" );

}


//
// the error handling code
//
// NOTE: any errors arising in these functions are ignored.
// NOTE: it is assumed that om_delete() is sufficiently intelligent
// not to try to access any memory pointed to by error without first 
// checking that the value in error is not one of the various object
// constants defined in IEEE 1327.2/D4-1992, section 3.3.3.
//

void handle_om_error(OM_return_code return_code)
{
char * myname = "handle_om_error";

	tellme( myname, "called\n" );
    cerr << "OM error " << return_code << "has occurred\n";

//
//  Keep it (too) simple for the time being...
//

    return;
}
