//////////////////////////////////////////////////////////////////////////////
//
//      FILE NAME : utils.C
//
//      AUTHOR : John Tebbutt
//
//      DESCRIPTION :
//		This file holds general purpose utility routines for use by the 
//		EDI Communications Platform X.500 interface.
//
//      SIDE EFFECTS :
//
//      BUGS, WARNINGS AND SPECIAL COMMENTS :
//
//////////////////////////////////////////////////////////////////////////////

#include    <stream.h>

//
// Routine to output diagnostic messages. Output is conditional on presence
// of compilation token DEBUG. Each parameter is a string. For formatted output,
// in the style of printf(), use the function form() for the message parameter, 
// eg tellme( function_name, form( "I am %d feet tall\n", my_height ) ). Neat!
// Note: caller must provide all newlines, as for printf().
//

void tellme( char *function, char *message )
{

#ifdef LOG

filebuf	f;

if( f.open( "uh.log", output ) == 0 ){

	cerr << "Cannot open log file uh.log\n";
}
ostream to(&f);

	f << function << ": " << message;

#endif

#ifdef DEBUG

	cerr << function << ": " << message;

#endif

}

//
// Utility routine to create a single name string by hyphenating two
// substrings passed as parameters. Need a routine like this because in order
// to do the concatenation we have to allocate memory
//
char *createName( char *s1, char *s2 )
{

int     rlen;
char    *result;
char    *myname = "createName";

		tellme( myname, form( "s1 = %s, s2 = %s\n", s1, s2 ) );

        if( s1 == 0 ){
			tellme( myname, "null s1\n" );
            return( '\0');
        }

        if( s2 == 0 ){
			tellme( myname, "null s2\n" );
            return( '\0' );
        }

        // 
        // The length of our result string is the sum of the lengths of the
        // substrings, plus one for the hyphen, plus one for the NULL
        // terminator...
        //
        rlen = strlen( s1 ) + strlen( s2 ) + 2;

		result = new char[ rlen ];

        strcpy( result, s1 );
        strcat( result, "-" );
        strcat( result, s2 );

        //
        // We are ready to go...
        //

		tellme( myname, form( "Returning << %s >>\n", result ) );

        return( result );
}
