/************
file parse.c 
/**************
#ifndef lint
static char rcsid[]="$Header: parse.c,v 3.47 93/07/27 14:50:24 timble Exp $";
#endif


/* See NIST PM Commented Out Code */

#define	_INCLUDE_POSIX_SOURCE	1
#define	FREE_BODY_CACHE			0x4


#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/conf.h>
#include <p1p2.h>
#include <xom.h>
#include <xds.h>
#include <xdssap.h>
#include <xmhp.h>
#include <ximp.h>
#include <xomhp.h>
#include <x4apifp.h>
#include <climport.h>
#include <x4cat.h> 
#include <xedip.h>
#include <climportedi.h>
/* #include <clexportedi.h> */

/* from intrinsics88/fromfile.c */
extern  int aP1MTS_APDU();
extern  int (*ASNinput)();
extern  int (*ASNseek)();
extern  int os_seek();
extern  int vdgetc();
extern  os_Jmpbuf xenv; 

/* C03_00_06 fix: return fd in setdp() to close file later in mt_fxferin().
 * Needed for multiple xferin's. 
 */
extern  os_FILE *setdp_return_fd(char *filename, int i);

char *mhp_to_utc(struct atime *time);
extern  int mhp_addobj();
/* 
 * this following extern variable is defined for p1p2.c and ext.c to 
 * determine whether it's p2 or p22 (from content type) and used by
 * these two only. However, since 1984 delivery report doesn't have 
 * content type, it's never set. For this release, we are setting this
 * variable to be default to p2. Module p1p2.c will set this appropriately
 * for regular message, but use default value for DR/NDR. In the future, 
 * it might not be needed as there are other variables. Khiem
 */

extern	int	save_ct;
extern	int	x4g_process_generic_content_flag;
extern boolean list_append();
extern boolean list_alloc();
extern boolean mhp_is84MH;
extern boolean mhp_is88MH;

/* declare commonly used functions */
int putvalue(OM_int_descriptor *, OM_int_value **, OM_type, OM_syntax, void *);
int p_strcpy(OM_int_descriptor *, OM_int_value **, OM_type, 
             OM_syntax, void *, Uint32); 
int p_bitcpy(OM_int_descriptor *, OM_int_value **, OM_type,
	     struct bits);
extern  OM_return_code write_str(
     OM_int_descriptor *desc,
     OM_int_value **value,
     Uint16 att_type,
     Uint16 att_syntax,
	  char *filename); 

static	void	get_new_pathname(char *, char *, char *);
char  *get_add_exts(P1ORName *orname, sint16 type); 
char  *get_msg_extension(P1MTS_APDU *mpdu, sint16 type);
char  *get_probe_extension(P1MTS_APDU *mpdu, sint16 type);
char *get_report_content_extension(P1ReportTransferContent *rte , sint16 type);
char *get_report_extension(P1ReportTransferEnvelope *rte , sint16 type);
char *get_per_recip_msg_extension( P1PerRecipientMessageTransferFields *rd,
				   sint16 type);
char *get_perrecp_probe_extension( P1PerRecipientProbeTransferFields *rd,
				   sint16 type);
char *get_per_recip_report_extension(P1PerRecipientReportTransferFields *prr,
                                   sint16 type);



/*
 * mhp_parse(filename, tmpdir, object, parsefd, iq, uq) takes a msg file
 * and parse it 
 * into an appropriate object, message, probe or report. If there's 
 * BAD_MESSAGE error, this function will try to generate  non-delivery
 * report if possible. If so, the message will be deleted from the queue.
 * else the message will be moved to the "uq" queue. For other temporary
 * error, such MEMORY_INSUFFICIENT, those error will be returned without
 * further action.
 *
 * INPUT:   
 *  filename - the full path file name of the ASN1 message, probe or report
 *  to_dir   - directory to put disk-based string value if any.
 *  object   - address for returned handle.
 *  iq       - the input queue to send the non-delivery report if generated.
 *  uq       - the undelivered queue to keep error message.
 *
 * OUTPUT:
 *  object   - handle to the output object, if successful
 *             non-delivery report generated or message move to "uq" 
 *  parsefd  - returned fd of outstanding parsed ASN1 msg, probe, or report.
 * NOTE:
 *  (1) General Binary Content should be limited size, since
 *  it residing in memory (vs BodyPart in file).
 *
 * SEE NOTES in mhp_p_msg() FOR GENERAL ALGORITHM.
 */

int mhp_parse(OM_private_object session,
        char *filename,
        char *to_dir,
        OM_int_private_object_header **object,
	os_FILE **parsefd,
        char *iq,
        char *uq)

{

P1MTS_APDU *mpdu;
extern char *asnmsg;    /* asn1 error message */
extern int  asncode;
extern int  errno;
int status;
int clrmem_flg = 0;

log_error( (OM_return_code) 0, "**** Entry mhp_parse() ****" );
	/* set default as 1984 for delivery report. Used by p1p2.c and ext.c */
	save_ct = BUILT_IN_interpersonal_messaging_1984; 

	x4g_process_generic_content_flag = TRUE; /* to tell ext.c to handle binary content */

	/* _MH_BodyPart_dir is global variable, and set at mt_open(). So do not 
	** need to use this argument for now. Maybe modified in the future
	** to support multiple clients in same process. Khiem
	*/
    /* _MH_BodyPart_dir = to_dir;  /* directory to hold P2 Body Parts files */

    if ((mpdu = (P1MTS_APDU *)os_calloc(sizeof(P1MTS_APDU))) == NULL){
        log_error(OM_MEMORY_INSUFFICIENT, OMP_DF_MEMORY_INSUFFICIENT);
        return(OM_MEMORY_INSUFFICIENT);
    }
log_error( (OM_return_code) 0, "**** Memory OK ****" );

    /* parse the ASN1 message into data structure */
    clrmem_flg |= (FREE_P22 | FREE_BODY_FILES | FREE_BODY_CACHE);
    if (os_setjmp(xenv) == 0) 
    {
	/* setdp() will open file to read, and return fd */
log_error( (OM_return_code) 0, "**** about to call setdp_return_fd ****" );
        *parsefd = setdp_return_fd(filename, (Length) -1); 
log_error( (OM_return_code) 0, "**** after call setdp_return_fd ****" );
        if (aP1MTS_APDU(mpdu) == FALSE) {
log_error( (OM_return_code) 0, "**** aP1MTS_APDU is FALSE ****" );

            /* enddp() in mt_close. Closing unlocks file locked by sxferin*/
            /* enddp(); */

            /* process bad message. No NDReport will be generated as this
               msg just been processed by the MTA. Just move it to undelivered
               queue (actually done back in mt_start_transfer_in() )
            */
            log_error(MH_RC_BAD_MESSAGE, MH_RC_DF_BAD_MESSAGE,
                filename, asnmsg, asncode, MH_INT_DF_INTEND_TO_MOVE, uq, 0);

            clr_mem(mpdu, clrmem_flg);
            os_free(mpdu);
            return(MH_RC_BAD_MESSAGE); /* can't parse mpdu */
        } /* end if */
log_error( (OM_return_code) 0, "**** aP1MTS_APDU is TRUE ****" );
        
        /* enddp() in mt_close. Closing unlocks file locked by sxferin*/
        /* enddp(); /* don't clr_mem here yet */
    } 
    else {
log_error( (OM_return_code) 0, "**** setjmp is bad ****" );
/* goto setjmpok; */
        /* enddp() in mt_close. Closing unlocks file locked by sxferin*/
        /* enddp(); */

        /*the next three lines follow intrinsics88/fromfile.c, common_parser()*/
        if (ASNinput == vdgetc)
            vdclose(1);         /* Free memory only */
        ASNinput = os_fgetc;
        ASNseek = os_seek;

        /* process bad message. No NDReport will be generated, as this msg
           just been processed by local MTA. Just move it over to undelivered
           queue (actually done back in mt_start_transfer_in() )
         */
        log_error(MH_RC_BAD_MESSAGE, MH_RC_DF_BAD_MESSAGE,
            filename, asnmsg, asncode, MH_INT_DF_INTEND_TO_MOVE, uq, 0);

        clr_mem(mpdu, clrmem_flg);
        os_free(mpdu);
        return(MH_RC_BAD_MESSAGE); /* error bad P1 */
    } /* end else */

    /* start creating object: message, probe or report.
       If there's error here, it's temporary. So
       message is put back in the queue by the caller for 
       later processing.
    */
setjmpok:
log_error( (OM_return_code) 0, "**** About to determine type ****" );
    switch (mpdu->t) {
    case MTS_APDU_message:
        if ((status = omp_template(MH_C_MESSAGE, object)) != OM_SUCCESS)
            break;
        status = mhp_p_msg(mpdu, *object);
        break;
    case MTS_APDU_report:
        if ((status = omp_template(MH_C_REPORT, object)) != OM_SUCCESS)
            break;
        status = mhp_p_report(mpdu, *object);
        break;
    case MTS_APDU_probe:
        if ((status = omp_template(MH_C_PROBE, object)) != OM_SUCCESS)
            break;  
        status = mhp_p_probe(mpdu, *object);
        break;
    default: /* this can't really happen */
        log_error(MH_RC_BAD_MESSAGE, MH_RC_DF_BAD_MESSAGE,
            filename, asnmsg, asncode, MH_INT_DF_INTEND_TO_MOVE, uq, 0);

        status = MH_RC_BAD_MESSAGE; 
        break;
    } /* end switch */

    /* start clear memory and tmp body part files */
    clrmem_flg |= (FREE_P22 | FREE_BODY_FILES);
    clr_mem(mpdu, clrmem_flg);
    os_free(mpdu);
    return(status);
} /* end mhp_parse() */



/*
 * get_pure_filename() - Given the pathname for a file, obtains a pointer
 * to the filename portion, minus the directory prefixes.
 *
 * E.g., if pathname is /usr/spool/x400/oq/oq001/Hpdu001, this sets the
 * variable 'file' to point to Hpdu001
 *
 */
static void
get_pure_filename(
	char    *path,
	char    **file)
{
	char    *p;     /* Temp pointer */

	if ((p = strrchr(path, '/')) != NULL)
		p++;
	else
		p = path;
	*file = p;

}       /* get_pure_filename() */


/*
 * get_new_pathname(filename, uq, newname) - Simple function to derive
 * the new pathname for a file that was moved to the UQ. The only reason
 * it exists as a separate function is because it's used several times...
 *
 * INPUT:
 *  filename - original pathname of file in OQ
 *  uq - pathname for UQ directory
 *
 * OUTPUT:
 *  newname - New pathname of file, now that it's in the UQ
 *
 */
static void
get_new_pathname(
	char	*filename,
	char	*uq,
	char	*newname)
{
	char	*purefile;

	(void)get_pure_filename(filename, &purefile);
	sprintf(newname, "%s/%s", uq, purefile);

}	/* get_new_pathname() */


/*
 * mhp_p_msg(mpdu, object) puts the P1 message into Message object.
 * It returns Success or Failure. SEE NOTES BELOW.
 *
 * INPUT:
 *  mpdu    - pointer to the P1MTS_APDU structure.
 *  object  - pointer to the Message object.
 *
 * OUTPUT:
 *  returning OM_SUCCESS or errors.
 *
 * NOTE:
 *    (1) 1988 package contains a number of attributes with more than one
 *    syntax, eg, Content Type or OR Address Attributes. This is represented
 *    in the private object template (ie descriptor_list) as multiple 
 *    attributes, each with a different syntax in "attr_syntax". Only the
 *    FIRST occurrence of the attribute will actually contain the values.
 *    Each value (OM_int_value) also has its own "syntax" field.
 *    The other duplicated attribute in the template are just for checking.
 *
 *    While parsing the msgs, the for-loop will go through the duplicate 
 *    attributes and this would result in duplicated values. This is avoided
 *    by checking the last attribute: if last attribute type is same as this
 *    attribute, skip THIS attribute and go to the next.
 *
 *    (2) The General Algorithm is as follows:
 *        For each attribute 
 *            we detect for all possible values (multiple values of
 *            same syntax, and multiple values of different syntax).
 *        
 *            If single value, and memory is already allocated (os_calloc),   
 *               Use putvalue() with argument &desc->attr_value.
 *            If single value, and copy string is required,
 *               Use p_strcpy() with argument &desc->attr_value. 
 *               Note that p_strcpy() allocates memory.
 *               So, we might need to free SOME buffer as appropriate.
 *            If it's a long string, always set the following by caller:
 *               modifier = on_disk
 *               value.string.length = SIZE of FILE, NOT SIZE OF FILENAME
 *               (as the value.string.elements points to null-terminated
 *                filename). Ie, value.string.length always have the size
 *               of the string value, no matter on disk or in memory. 
 *
 *            If it's multiple values,
 *               Use either p_strcpy() or putvalue() as appropriate, but
 *                  with &NEXTVAL argument.
 *               Then call append_struct() for each value.
 *               Then set desc->attr_value = baseval;
 *                        desc->count = count
 *                    at the end
 *
 */

int mhp_p_msg(mpdu, object)
P1MTS_APDU *mpdu;
OM_int_private_object_header *object;
{
OM_int_private_object_header *subobject;
char *buf;
OM_uint32 len;
OM_uint16 i=0, count;
int status, tmp;
int *evalue;
char *pvalue;
OM_enumeration	enumer;
OM_int_descriptor *desc;
OM_type prev_attribute_type = OM_NO_MORE_TYPES;
OM_int_value *baseval, *curval, *nextval;
ListPointer *bilatlist, *extracelist, *intracelist, *rdlist;
P1InternalTraceInformation *int88list;
P1ExtensionMacroBody *extension;
P1InternalTraceInformationElement *trace_elem;

log_error( (OM_return_code) 0, "**** Entry into mhp_p_msg() ****" );

    /* Index through Message descriptor list sequentially and fill in attribute
     * if it exists. */
    for (i = TOP; 
        object->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
        i++) 
    {
    desc = (OM_int_descriptor_list)&(object->context.descriptor_list[i]);
    baseval = curval = nextval = NULL; count = 0;

/*
 * For attribute with multiple syntaxes, there are duplicated descriptors
 * in the template, but ONLY THE FIRST one contains the values. Hence we
 * need to skip the duplicated descriptors. KDH.
 */
    if (prev_attribute_type == desc->attr_type) continue;
    prev_attribute_type = desc->attr_type;

    switch (desc->attr_type) {
    case MH_T_MTS_IDENTIFIER:
        if (mpdu->v.message.envelope.m & MessageTransferEnvelope_message_identifier) {
        /* Create and fill in MTS Identifier sub-object */
        if ((status = mhp_p_mts_id(mpdu->v.message.envelope.message_identifier,&subobject)) != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_MTS_IDENTIFIER,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_ORIGINATOR_NAME:
        if (mpdu->v.message.envelope.m & MessageTransferEnvelope_originator_name) {
        /* Create and fill in OR Name sub-object */
        if ((status = mhp_p_orname(mpdu->v.message.envelope.originator_name,&subobject)) != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_ORIGINATOR_NAME,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_ORIGINAL_EITS:
        if (mpdu->v.message.envelope.m & MessageTransferEnvelope_original_encoded_information_types) {
        /* Create and fill in Original EITs sub-object */
        if ((status = mhp_p_eits(mpdu->v.message.envelope.original_encoded_information_types,&subobject))
            != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_ORIGINAL_EITS,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_CONTENT_TYPE:
        if (mpdu->v.message.envelope.m & MessageTransferEnvelope_content_type) {
        if (mpdu->v.message.envelope.content_type.t == ContentType_built_in) {
            switch (mpdu->v.message.envelope.content_type.v.built_in) {
            case BUILT_IN_unidentified:
                tmp = MH_CTI_UNIDENTIFIED;
                break;
            case BUILT_IN_external:
                tmp = MH_CTI_EXTERNAL;
                break;
            case BUILT_IN_interpersonal_messaging_1984:
                tmp = MH_CTI_P2_1984;
                break;
            case BUILT_IN_interpersonal_messaging_1988:
                tmp = MH_CTI_P2_1988;
            	break;
/* NIST PM, we added this, fix it */
            case BUILT_IN_edi:
                tmp = 35;
            	break;
            default: /* what ? */
		tmp = mpdu->v.message.envelope.content_type.v.built_in;
		break;
            } /* end switch */

            /* Create value structure and link it */
            if ((status = putvalue(desc,&desc->attr_value,MH_T_CONTENT_TYPE,OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
            return(status);
        } 
        else { /* external not supported in 1984; only in 1988 */
            return (OM_WRONG_VALUE_SYNTAX);
        } /* end else */
        } /* end if */
        break;
    case MH_T_CONTENT_IDENTIFIER:
        if (mpdu->v.message.envelope.m & MessageTransferEnvelope_content_identifier) {
        if (len = strlen(mpdu->v.message.envelope.content_identifier)) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_CONTENT_IDENTIFIER,OM_S_PRINTABLE_STRING,
            mpdu->v.message.envelope.content_identifier,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_PRIORITY:
        if (mpdu->v.message.envelope.m & MessageTransferEnvelope_priority) {
        switch (mpdu->v.message.envelope.priority) {
        case PRIORITY_non_urgent:
            enumer = MH_PTY_LOW;
            break;
        case PRIORITY_urgent:
            enumer = MH_PTY_URGENT;
            break;
        case PRIORITY_normal:
        default:
            enumer = MH_PTY_NORMAL;
            break;
        } /* end switch */
        } /* end if */
        else { /* default is normal, if not present */
            enumer = MH_PTY_NORMAL;
        }
        /* Create value structure and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_PRIORITY,OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
            return(status);
        break;

    case MH_T_ALTERNATE_RECIP_ALLOWED:
        if (mpdu->v.message.envelope.per_message_indicators & PER_MESSAGE_INDICATORS_alternate_recipient_allowed) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_ALTERNATE_RECIP_ALLOWED,OM_S_BOOLEAN,(void*)OM_TRUE)) != OM_SUCCESS)
            return(status);
        } /* end if */
        else { /* default is false */
            if ((status = putvalue(desc,&desc->attr_value,
                    MH_T_ALTERNATE_RECIP_ALLOWED,OM_S_BOOLEAN,(void*)OM_FALSE))
                     != OM_SUCCESS)
            return(status);
        }
        break;

    case MH_T_CONVERSION_PROHIBITED:
        if (mpdu->v.message.envelope.per_message_indicators & PER_MESSAGE_INDICATORS_implicit_conversion_prohibited) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_CONVERSION_PROHIBITED,OM_S_BOOLEAN,(void*)OM_TRUE)) != OM_SUCCESS)
            return(status);
        } /* end if */
        else { /* default is false */
            if ((status = putvalue(desc,&desc->attr_value,
                MH_T_CONVERSION_PROHIBITED,OM_S_BOOLEAN,(void*)OM_FALSE)) 
                != OM_SUCCESS)
            return(status);
        } /* end else */
        break;

    case MH_T_CONTENT_RETURN_REQUESTED:
        if (mpdu->v.message.envelope.per_message_indicators & PER_MESSAGE_INDICATORS_content_return_request) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_CONTENT_RETURN_REQUESTED,OM_S_BOOLEAN,(void*)TRUE)) != OM_SUCCESS)
            return(status);
        } /* end if */

        else { /* default is false */
        if ((status = putvalue(desc,&desc->attr_value,
                MH_T_CONTENT_RETURN_REQUESTED,OM_S_BOOLEAN,(void*)OM_FALSE)) 
                != OM_SUCCESS)
            return(status);
        } /* end else */
        break;

    case MH_T_DISCLOSURE_ALLOWED:
        if (mpdu->v.message.envelope.per_message_indicators & PER_MESSAGE_INDICATORS_disclosure_of_recipients) {
        if ((status = putvalue(desc,&desc->attr_value,
                MH_T_DISCLOSURE_ALLOWED,OM_S_BOOLEAN,(void*)TRUE)) 
                != OM_SUCCESS)
            return(status);
        } /* end if */

        else { /* default is false */
        if ((status = putvalue(desc,&desc->attr_value,
                MH_T_DISCLOSURE_ALLOWED,OM_S_BOOLEAN,(void*)OM_FALSE)) 
                != OM_SUCCESS)
            return(status);
        } /* end else */
        break;

    case MH_T_DEFERRED_DELIVERY_TIME:
        if (mpdu->v.message.envelope.m & MessageTransferEnvelope_deferred_delivery_time) {
        if ((buf = mhp_to_utc(&mpdu->v.message.envelope.deferred_delivery_time)) == NULL)
            return(OM_MEMORY_INSUFFICIENT);

        if ((status = putvalue(desc,&desc->attr_value,MH_T_DEFERRED_DELIVERY_TIME,OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_BILATERAL_INFORMATION:
        /* Bilateral Information: 0-512 occurrences */
        if (mpdu->v.message.envelope.m & MessageTransferEnvelope_per_domain_bilateral_information) {
        for (bilatlist = (ListPointer *)&(mpdu->v.message.envelope.
                                        per_domain_bilateral_information),
                bilatlist->current = bilatlist->first, count = 0,
                baseval = curval = nextval = desc->attr_value;
            bilatlist->current != NULL; 
            bilatlist->current = bilatlist->current->next) 
        {
            /* Create and fill in the Bilateral Info sub-object */
            if ((status = mhp_p_bilat(bilatlist->current->data,&subobject)) 
                                                            != OM_SUCCESS)
                return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_BILATERAL_INFORMATION,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;

    case MH_T_EXTERNAL_TRACE_INFO:
        /* External Trace Information: 0-512 occurrences */
        if (mpdu->v.message.envelope.m & MessageTransferEnvelope_trace_information) {
        for (extracelist = (ListPointer *)&(mpdu->v.message.envelope.trace_information),
            extracelist->current = extracelist->first, count = 0,
            baseval = curval = nextval = desc->attr_value;
            extracelist->current != NULL; extracelist->current = extracelist->current->next) 
        {
            /* Create and fill in the External Trace Info sub-object */
            if ((status = mhp_p_extrace(extracelist->current->data,&subobject))
                                                                 != OM_SUCCESS)
                return(status);
            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_EXTERNAL_TRACE_INFO,
                                    OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;

    case MH_T_INTERNAL_TRACE_INFO:
	if (!mhp_is84MH)
	{ /* if 88MH */
          if (pvalue = (char *)get_msg_extension(mpdu,ETYPE_internal_trace_information))   
	  {
	   P1InternalTraceInformation *temp_p1internaltrace;
	    temp_p1internaltrace = (P1InternalTraceInformation *) pvalue;
	    for ( list_first( temp_p1internaltrace);
			    temp_p1internaltrace->current;
			    list_next( temp_p1internaltrace) ) 
	    { /* for loop */
		trace_elem = (P1InternalTraceInformationElement *) 
					temp_p1internaltrace->current->data;
            /* Create and fill in the Internal Trace Info sub-object */
		if ((status = mhp_p_intrace88 (trace_elem,&subobject)) 
				!= OM_SUCCESS)
		    return(status);

            /* Create a value structure for the subobject and link it */
		if ((status = putvalue(desc, &nextval,MH_T_INTERNAL_TRACE_INFO,
				  OM_S_OBJECT,subobject)) != OM_SUCCESS)
		    return(status);

		append_struct(&baseval,&curval, nextval,&count);
	    } /* for loop */ 

	    desc->attr_value = baseval;
	    desc->count = count;
	  }
        }
	else
        /* Internal Trace Information: 0-512 occurrences */
	{
         if (mpdu->v.message.envelope.m & MessageTransferEnvelope_Internal1984) 
	 {
          for (intracelist = (ListPointer *)&(mpdu->v.message.envelope.internal1984),
            intracelist->current = intracelist->first, count = 0,
            baseval = curval = nextval = desc->attr_value;
            intracelist->current != NULL; 
            intracelist->current = intracelist->current->next) 
          {
            /* Create and fill in the Internal Trace Info sub-object */
            if ((status = mhp_p_intrace(intracelist->current->data,&subobject))                                                     != OM_SUCCESS)
                return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_INTERNAL_TRACE_INFO,
                                    OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
          } /* end for */

          desc->attr_value = baseval;
          desc->count = count;
         } /* end if */
	} /* if 88MH */
        break;

    case MH_T_RECIPIENT_DESCRIPTORS:
        /* Recipient Descriptors: 0-32767 occurrences */
        if (mpdu->v.message.envelope.m & 
                    MessageTransferEnvelope_per_recipient_fields) {
        for (rdlist = (ListPointer *)&(mpdu->v.message.envelope.
                                                    per_recipient_fields),
                    rdlist->current = rdlist->first, count = 0,
                    baseval = curval = nextval = desc->attr_value;
            rdlist->current != NULL; 
            rdlist->current = rdlist->current->next) 
        {
            /* Create and fill in the Recipient Descriptors sub-object */
            if ((status = mhp_p_msg_rd(rdlist->current->data,&subobject))
                                                            != OM_SUCCESS)
                return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_RECIPIENT_DESCRIPTORS,
                                    OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;

    case MH_T_CONTENT: /* Content */
        if (mpdu->v.message.m & Message_content) {
        /* P2 or binary content? */
        switch (mpdu->v.message.envelope.content_type.v.built_in) {
        case BUILT_IN_interpersonal_messaging_1984:
        case BUILT_IN_interpersonal_messaging_1988:
            if ((status = mhp_parse_ipm(mpdu->v.message.content,&subobject)) 
                                            != OM_SUCCESS)
                return(status);
            break;
        case  BUILT_IN_unidentified:
            if ((status = mhp_p_binary(mpdu->v.message.content,&subobject)) 
                                            != OM_SUCCESS)
                return(status);
            break;

        case BUILT_IN_edi:
log_error( (OM_return_code) 0, "**** About to call mhp_p_edi() ****" );
            if ((status = mhp_p_edi(mpdu->v.message.content,&subobject)) 
                                            != OM_SUCCESS)
                return(status);
            break;

        default:  /* case of special content type */
            if ((status = mhp_p_binary(mpdu->v.message.content,&subobject)) 
                                            != OM_SUCCESS)
                return(status);
            break;
        } /* end switch */

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_CONTENT,
                                OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_CONTENT_CORRELATOR: 
        if (pvalue = (char *)get_msg_extension(mpdu,ETYPE_content_correlator)) {
        if (len = ((P1ContentCorrelator *)pvalue)->len) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_CONTENT_CORRELATOR,
            OM_S_OCTET_STRING,((P1ContentCorrelator *)pvalue)->ptr,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_CONVERSION_LOSS_PROHIBITED: 
        /* 
         * We need to ensure that the default value is set in XOM interface
         * as it is required in the package (value-number=1). This would 
         * allow interface conformance and allow loop-back a copy of 
         * the message.
         */

        if (pvalue = (char *)get_msg_extension(mpdu,
                                      ETYPE_conversion_with_loss_prohibited)) 
	{
            if (*(P1ConversionWithLossProhibited *)pvalue) 
	    {
                if ((status = putvalue(desc,&desc->attr_value,
					MH_T_CONVERSION_LOSS_PROHIBITED,
					OM_S_BOOLEAN,(void*)OM_TRUE)) 
					!= OM_SUCCESS)
            			return(status);
            } /* end if */
            else 
	    { /* default is false */
                if ((status = putvalue(desc,&desc->attr_value,
                			MH_T_CONVERSION_LOSS_PROHIBITED,
					OM_S_BOOLEAN,(void*)OM_FALSE)) 
                			!= OM_SUCCESS)
            		return(status);
             } /* end else */
        } /* end if */

        else { /* absence: hence set default values */
            if ((status = putvalue(desc,&desc->attr_value,
                                        MH_T_CONVERSION_LOSS_PROHIBITED,
                                        OM_S_BOOLEAN,(void*)OM_FALSE))
                                        != OM_SUCCESS)
                        return(status);
        } /* end else */

        break;

    case MH_T_EXPANSION_HISTORY: 
        if (pvalue = (char *)get_msg_extension(mpdu,
                                      ETYPE_dl_expansion_history))
        {
	int i;   /* Not to be confused with outer loop i */
        for (i = 0, baseval = curval = nextval = desc->attr_value, count = 0;
            i < ((P1DLExpansionHistory *)pvalue)->n ; i++) 
        {
            /* Create and fill in the Expansion History sub-object */
            if ((status = mhp_p_exphist(((P1DLExpansionHistory *)pvalue)->v[i],
                                        &subobject))
                                        != OM_SUCCESS)
                return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_EXPANSION_HISTORY,
                                   OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;

    case MH_T_EXPANSION_PROHIBITED: 
        /* 
         * We need to ensure that the default value is set in XOM interface
         * as it is required in the package (value-number=1). This would 
         * allow interface conformance and allow loop-back a copy of 
         * the message.
         */
        if (pvalue = (char *)get_msg_extension(mpdu,
                                       ETYPE_dl_expansion_prohibited)) 
	{
        	if (*(P1DLExpansionProhibited *)pvalue) 
		{
        		if ((status = putvalue(desc,&desc->attr_value,
					MH_T_EXPANSION_PROHIBITED,
					OM_S_BOOLEAN,(void*)OM_TRUE)) 
					!= OM_SUCCESS)
            			return(status);
        	} /* end if */
                else 
		{ /* default is false */
            		if ((status = putvalue(desc,&desc->attr_value,
                			MH_T_EXPANSION_PROHIBITED,
					OM_S_BOOLEAN,(void*)OM_FALSE)) 
                			!= OM_SUCCESS)
            		return(status);
                } /* end else */
        } /* end if */

        else { /* absence: hence, set default values */
            if ((status = putvalue(desc,&desc->attr_value,
                                   MH_T_EXPANSION_PROHIBITED,
                                   OM_S_BOOLEAN,(void*)OM_FALSE))
                                   != OM_SUCCESS)
                return(status);
        } /* end else */
        break;

    case MH_T_ORIGIN_CHECK:  
        if (pvalue = (char *)get_msg_extension(mpdu,
                      	ETYPE_message_origin_authentication_check)) 
	{
        /* Create and fill in the message origin check sub-object */
	/* note that we can get away with using 
	(P1ProbeOriginAuthenticationCheck *) because types
	P1ProbeOriginAuthenticationCheck and P1MessageOriginAuthenticationCheck
	are identical. However this is not a good programming style. */
        if ((status = mhp_p_origincheck((P1ProbeOriginAuthenticationCheck *)pvalue,&subobject))
            != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&nextval,MH_T_ORIGIN_CHECK,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_ORIGINATOR_CERTIFICATE: 
        if (pvalue = (char *)get_msg_extension(mpdu,
                      	ETYPE_originator_certificate)) 
	{
        /* Create and fill in the originator certificate sub-object */
        if ((status = mhp_p_orig_cert((P1OriginatorCertificate *)pvalue,&subobject))
            != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&nextval,MH_T_ORIGINATOR_CERTIFICATE,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_REASSIGNMENT_PROHIBITED: 
        /* 
         * set default attribute value for this extension if not
         * present in PDU to conform to XOM interface, and to 
         * avoid object checking when loop-back the message.
         */
        if (pvalue = (char *)get_msg_extension(mpdu,
                            ETYPE_recipient_reassignment_prohibited)) 
	{
        	if (*(P1RecipientReassignmentProhibited *)pvalue) 
		{
        		if ((status = putvalue(desc,&desc->attr_value,
					MH_T_REASSIGNMENT_PROHIBITED,
					OM_S_BOOLEAN,(void*)OM_TRUE)) 
					!= OM_SUCCESS)
            			return(status);
        	} /* end if */
                else 
		{ /* default is false */
            		if ((status = putvalue(desc,&desc->attr_value,
                			MH_T_REASSIGNMENT_PROHIBITED,
					OM_S_BOOLEAN,(void*)OM_FALSE)) 
                			!= OM_SUCCESS)
            		return(status);
                } /* end else */
        } /* end if */

        else { /* absence: hence default values */
            if ((status = putvalue(desc,&desc->attr_value,
                                   MH_T_REASSIGNMENT_PROHIBITED,
                                   OM_S_BOOLEAN,(void*)OM_FALSE))
                                   != OM_SUCCESS)
                return(status);
        } /* end else */

        break;
    case MH_T_SECURITY_LABEL:   
        if (pvalue = (char *)get_msg_extension(mpdu,ETYPE_message_security_label)) {
        /* Create and fill in the Message Security Label sub-object */
        if ((status = mhp_p_seclab((P1SecurityLabel *)pvalue,&subobject))
            != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&nextval,MH_T_SECURITY_LABEL,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_CRITICAL_FOR_DELIVERY:
        {
        P1ExtensionMacro *extensions ;
        aaListType *list;
        P1ExtensionMacroBody *extdata;
        OM_type    criticalityAttrType ;
    
        extensions = &(mpdu->v.message.envelope.extensions);
    
        list_first(extensions); /* set first item on list to current*/
        for (list = (aaListType *) extensions->current;
             list != NULL;
             list = (aaListType *)list_next(extensions))
        {
            extdata = (P1ExtensionMacroBody *)list->data;
            if (extdata->m |= ExtensionMacroBody_criticality) {
                tmp = criticalityAttrType = mhp_get_attr_criticality(extdata, 
                                                    CRITICALITY_for_delivery);

                if (criticalityAttrType != OM_NO_SUCH_TYPE) {

                   /* Create a value structure for the subobject and link it */
                   if ((status = putvalue(desc,&nextval,MH_T_CRITICAL_FOR_DELIVERY,
                                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                       return(status);
    
                   append_struct(&baseval,&curval,nextval,&count);
                }  /* end if */

            } /* end if extdata->m |= ExtensionMacroBody_criticality */

        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
   
        }
        break;

    case MH_T_CRITICAL_FOR_SUBMISSION: 
        {
        P1ExtensionMacro *extensions ;
        aaListType *list;
        P1ExtensionMacroBody *extdata;
        OM_type    criticalityAttrType ;
    
        extensions = &(mpdu->v.message.envelope.extensions);
    
        list_first(extensions); /* set first item on list to current*/
        for (list = (aaListType *) extensions->current;
             list != NULL;
             list = (aaListType *)list_next(extensions))
        {
            extdata = (P1ExtensionMacroBody *)list->data;
            if (extdata->m |= ExtensionMacroBody_criticality) {
                tmp = criticalityAttrType = mhp_get_attr_criticality(extdata, 
                                                    CRITICALITY_for_submission);

                if (criticalityAttrType != OM_NO_SUCH_TYPE) {

                   /* Create a value structure for the subobject and link it */
                   if ((status = putvalue(desc,&nextval,MH_T_CRITICAL_FOR_SUBMISSION,
                                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                       return(status);
    
                   append_struct(&baseval,&curval,nextval,&count);
                }  /* end if */

            } /* end if extdata->m |= ExtensionMacroBody_criticality */

        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
   
        }
        break;

    case MH_T_CRITICAL_FOR_TRANSFER: 
        {
        P1ExtensionMacro *extensions ;
        aaListType *list;
        P1ExtensionMacroBody *extdata;
        OM_type    criticalityAttrType ;
    
        extensions = &(mpdu->v.message.envelope.extensions);
    
        list_first(extensions); /* set first item on list to current*/
        for (list = (aaListType *) extensions->current;
             list != NULL;
             list = (aaListType *)list_next(extensions))
        {
            extdata = (P1ExtensionMacroBody *)list->data;
            if (extdata->m |= ExtensionMacroBody_criticality) {
                tmp = criticalityAttrType = mhp_get_attr_criticality(extdata, 
                                                    CRITICALITY_for_transfer);

                if (criticalityAttrType != OM_NO_SUCH_TYPE) {

                   /* Create a value structure for the subobject and link it */
                   if ((status = putvalue(desc,&nextval,MH_T_CRITICAL_FOR_TRANSFER,
                                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                       return(status);
    
                   append_struct(&baseval,&curval,nextval,&count);
                }  /* end if */

            } /* end if extdata->m |= ExtensionMacroBody_criticality */

        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
   
        }
        break;

    case MH_T_EXTENSIONS: 
        break;

    case MH_T_CONFIDENTIALITY_ALGORITHM:
        break;

    case MH_T_LATEST_DELIVERY_TIME:
        if (pvalue = (char *)get_msg_extension(mpdu, 
                                  ETYPE_latest_delivery_time)) 
	{
            if ((buf = mhp_to_utc((P1LatestDeliveryTime *)pvalue))
                                        == NULL)
                return(OM_MEMORY_INSUFFICIENT);
            if ((status = putvalue(desc,&desc->attr_value,
                     MH_T_LATEST_DELIVERY_TIME,
                     OM_S_UTC_TIME_STRING, buf)) != OM_SUCCESS)
                return(status);
        } /* end if */
        break;

    case MH_T_ORIGINATOR_RETURN_ADDRESS:
        if (evalue = (int *)get_msg_extension(mpdu,ETYPE_originator_return_address)) {
            if ((status = mhp_p_oraddress((P1ORName *)(*evalue),
                                     &subobject))
                        != OM_SUCCESS)
                return(status);

            if ((status = putvalue(desc,&desc->attr_value,
                        MH_T_ORIGINATOR_RETURN_ADDRESS,
                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);
        }
        break;

     default:
        break;

    } /* end switch */
    } /* end for */

    return(OM_SUCCESS);

} /* end mhp_p_msg() */

/*
 * mhp_p_report()
 */

int mhp_p_report(mpdu, object)
P1MTS_APDU *mpdu;
OM_int_private_object_header *object;
{
OM_int_private_object_header *subobject;
char *buf;
Uint32 len;
Uint16 i=0, count;
int status, tmp;
int *evalue;
char *pvalue;
OM_enumeration	enumer;
OM_int_descriptor *desc;
OM_int_value *baseval, *curval, *nextval;
ListPointer *bilatlist, *extracelist, *intracelist, *prlist;
P1InternalTraceInformation *int88trace;
P1ContentCorrelator *cc;
P1InternalTraceInformationElement *trace_elem;
OM_type prev_attribute_type = OM_NO_MORE_TYPES;
P1ReportTransferEnvelope    *env;
P1ReportTransferContent     *drcontent;

   
    env = (P1ReportTransferEnvelope *) &mpdu->v.report.envelope;
    drcontent = (P1ReportTransferContent *) &mpdu->v.report.content;
   

    /* Index through Message descriptor list sequentially and fill in attribute
     * if it exists. 
	*/
    for (i = TOP; 
        object->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
        i++) 
    {
    desc = (OM_int_descriptor_list)&(object->context.descriptor_list[i]);
    baseval = curval = nextval = NULL;  count = 0;

/*
 * For attribute with multiple syntaxes, there are duplicated descriptors
 * in the template, but ONLY THE FIRST one contains the values. Hence we
 * need to skip the duplicated descriptors. KDH.
 */
    if (prev_attribute_type == desc->attr_type) continue;
    prev_attribute_type = desc->attr_type;

    switch (desc->attr_type) {

    case MH_T_MTS_IDENTIFIER:

        if (mpdu->v.report.envelope.m 
                            & ReportTransferEnvelope_report_identifier) {
        /* Create and fill in Report MTS Identifier sub-object */
        if ((status = mhp_p_mts_id(mpdu->v.report.envelope.report_identifier,
                                            &subobject)) != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,
				MH_T_MTS_IDENTIFIER,
				OM_S_OBJECT,subobject))!=OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_REPORT_DESTINATION:   /* the destination of the report */
        if (mpdu->v.report.envelope.m 
                & ReportTransferEnvelope_report_destination_name) 
        {
        /* Create and fill in Report Destination OR Name sub-object */
        if ((status = mhp_p_orname(mpdu->v.report.envelope.
                    report_destination_name,&subobject)) != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_REPORT_DESTINATION,
                                OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_EXTERNAL_TRACE_INFO:
        /* External Trace Information: 0-512 occurrences */
        if (mpdu->v.report.envelope.m & 
                ReportTransferEnvelope_trace_information) 
        {
        for (extracelist = (ListPointer *)
                            &(mpdu->v.report.envelope.trace_information),
                extracelist->current = extracelist->first, count = 0,
                baseval = curval = nextval = desc->attr_value;
            extracelist->current != NULL; 
            extracelist->current = extracelist->current->next) 
        {
            /* Create and fill in the External Trace Info sub-object */
            if ((status = mhp_p_extrace(extracelist->current->data,&subobject))                                                             != OM_SUCCESS)
                return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,
					MH_T_EXTERNAL_TRACE_INFO,
					OM_S_OBJECT,subobject)) 
                    != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;

    case MH_T_SUBJECT_EXT_TRACE_INFO:
        /* External Trace Information: 0-512 occurrences */
        if (mpdu->v.report.content.m 
            & ReportTransferContent_subject_intermediate_trace_information) 
        {
        for (extracelist = (ListPointer *)
               &(mpdu->v.report.content.subject_intermediate_trace_information),
               extracelist->current = extracelist->first, count = 0,
               baseval = curval = nextval = desc->attr_value;
            extracelist->current != NULL; 
            extracelist->current = extracelist->current->next) 
        {
            /* Create and fill in the External Trace Info sub-object */
            if ((status = mhp_p_extrace(extracelist->current->data,&subobject))                                                         != OM_SUCCESS)
                return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval, 
					MH_T_SUBJECT_EXT_TRACE_INFO,
                                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;

    case MH_T_INTERNAL_TRACE_INFO:
        if (!mhp_is84MH)
        {
          if (pvalue = (char *)get_report_extension(&mpdu->v.report.envelope,ETYPE_internal_trace_information))   
	  {
	   P1InternalTraceInformation *temp_p1internaltrace;
	    temp_p1internaltrace = (P1InternalTraceInformation *) pvalue;
	    for ( list_first( temp_p1internaltrace);
			    temp_p1internaltrace->current;
			    list_next( temp_p1internaltrace) ) 
	    { /* for loop */
		trace_elem = (P1InternalTraceInformationElement *) 
					temp_p1internaltrace->current->data;
            /* Create and fill in the Internal Trace Info sub-object */
		if ((status = mhp_p_intrace88 (trace_elem,&subobject)) 
				!= OM_SUCCESS)
		    return(status);

            /* Create a value structure for the subobject and link it */
		if ((status = putvalue(desc, &nextval,MH_T_INTERNAL_TRACE_INFO,
				  OM_S_OBJECT,subobject)) != OM_SUCCESS)
		    return(status);

		append_struct(&baseval,&curval, nextval,&count);
	    } /* for loop */ 

	    desc->attr_value = baseval;
	    desc->count = count;
	  }
        }
	else /* MH 84 */
	{
        /* Internal Trace Information: 0-512 occurrences */
        if (mpdu->v.report.envelope.m & ReportTransferEnvelope_Internal1984) {
        for (intracelist = (ListPointer *)
                    &(mpdu->v.report.envelope.internal1984),
                    intracelist->current = intracelist->first, count = 0,
                    baseval = curval = nextval = desc->attr_value;
            intracelist->current != NULL; 
            intracelist->current = intracelist->current->next) 
        {
            /* Create and fill in the Internal Trace Info sub-object */
            if ((status = mhp_p_intrace(intracelist->current->data,&subobject))                                                         != OM_SUCCESS)
                return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,
					MH_T_INTERNAL_TRACE_INFO,
                                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        } /* MH 84 */
        break;

	/* No original internal trace info returned in a report. KDH 10/17/91 */
	/* COMMENT OUT
    case MH_T_INTERNAL_TRACE_INFO:  /* original msg's subject internal trace
        /* Internal Trace Information: 0-512 occurrences 
        if (mpdu->v.report.content.m & ReportTransferContent_internal1984) {
        for (intracelist=(ListPointer *)&(mpdu->v.report.content.internal1984),
                intracelist->current = intracelist->first, count = 0,
                baseval = curval = nextval = desc->attr_value;
            intracelist->current != NULL; 
            intracelist->current = intracelist->current->next) 
        {
            /* Create and fill in the Internal Trace Info sub-object 
            if ((status = mhp_p_intrace(intracelist->current->data,&subobject))                                                         != OM_SUCCESS)
                return(status);

            /* Create a value structure for the subobject and link it 
            if ((status = putvalue(desc,&nextval,MH_T_INTERNAL_TRACE_INFO,
                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for 

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if 
        break;
	
	COMMENT OUT UP TO HERE. */

    case MH_T_SUBJECT_MTS_IDENTIFIER:

        if (mpdu->v.report.content.m 
                & ReportTransferContent_subject_identifier) {
        /* Create and fill in Subject Identifier sub-object */
        if ((status = mhp_p_mts_id(mpdu->v.report.content.subject_identifier,
                                    &subobject)) != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,
					MH_T_SUBJECT_MTS_IDENTIFIER,
                                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_ORIGINAL_EITS: /* original EITS */
        if (mpdu->v.report.content.m 
                & ReportTransferContent_original_encoded_information_types) {
        /* Create and fill in Original EITs sub-object */
        if ((status = mhp_p_eits(mpdu->v.report.content.
                    original_encoded_information_types,&subobject))
                    != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_ORIGINAL_EITS,
                            OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_CONTENT_IDENTIFIER: /* original content id 1988 */
        if (mpdu->v.report.content.m & 
                    ReportTransferContent_content_identifier) {
        if (len = strlen(mpdu->v.report.content.content_identifier)) {
            if ((status = p_strcpy(desc,&desc->attr_value,
                    MH_T_CONTENT_IDENTIFIER,OM_S_PRINTABLE_STRING,
                    mpdu->v.report.content.content_identifier,len)) 
                    != OM_SUCCESS)
                return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_CONTENT_TYPE:
        if (mpdu->v.report.content.m & ReportTransferContent_content_type) {
        if (mpdu->v.report.content.content_type.t == ContentType_built_in) {
            switch (mpdu->v.report.content.content_type.v.built_in) {
            case BUILT_IN_unidentified:
                tmp = MH_CTI_UNIDENTIFIED;
                break;
            case BUILT_IN_external:
                tmp = MH_CTI_EXTERNAL;
                break;
            case BUILT_IN_interpersonal_messaging_1984:
                tmp = MH_CTI_P2_1984;
                break;
            case BUILT_IN_interpersonal_messaging_1988:
                tmp = MH_CTI_P2_1988;
                break;
            default: /* what ? */
		tmp = mpdu->v.report.content.content_type.v.built_in;
		break;
            } /* end switch */

            /* Create value structure and link it */
            if ((status = putvalue(desc,&desc->attr_value,MH_T_CONTENT_TYPE,
                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                return(status);
        } else { /* 1988 object id not yet supported */
            return (OM_WRONG_VALUE_SYNTAX);
        } /* end else */
        } /* end if */
        break;

    case MH_T_CONTENT: /* returned content */
        /* Content */
        if (mpdu->v.report.content.m & ReportTransferContent_returned_content) 
        {
        /* P2 or binary content? */
	if ( (mpdu->v.report.content.m & ReportTransferContent_content_type)
             && (mpdu->v.report.content.content_type.t == 
		   ContentType_built_in)) {
          switch (mpdu->v.report.content.content_type.v.built_in) {
/*
 * If builtin content type is present then we will switch correctly.
 * However, for 1984, there's no content type in NDR. Current code in p1p2.c
 * will support only parse of return of P2/P22 Content, but NOT return of
 * of Binary Content.
 */
          case BUILT_IN_interpersonal_messaging_1984:
          default: 
            if ((status = mhp_parse_ipm(mpdu->v.report.content.returned_content,                            &subobject)) != OM_SUCCESS)
                return(status);
            break;

          case  BUILT_IN_unidentified:
            if ((status = mhp_p_binary(mpdu->v.report.content.returned_content,
                            &subobject)) != OM_SUCCESS)
            return(status);
            break;
          } /* end switch */
	} /* end if buitin */

/* 
 * If there is no content type, then we will assume that it's p22.
 * This code segment needs to be enhanced to also support ContentType
 * with Object Identifier.
 */
	else {
	  if ((status = mhp_parse_ipm(mpdu->v.report.content.returned_content,
                            &subobject)) != OM_SUCCESS)
               return(status);
        }

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_CONTENT,
                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_REPORT_ADDITIONAL_INFO:
        if (mpdu->v.report.content.m 
                & ReportTransferContent_additional_information) 
        {
        if (len=strlen((char *)mpdu->v.report.content.additional_information))
        {
            if ((status = p_strcpy(desc,&desc->attr_value,
                        MH_T_REPORT_ADDITIONAL_INFO,OM_S_OCTET_STRING,
                        mpdu->v.report.content.additional_information,len)) 
                        != OM_SUCCESS)
                return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_SECURITY_LABEL: /* 1988 feature */
        if (pvalue = (char *)get_report_extension(&mpdu->v.report.envelope,ETYPE_message_security_label)) {
        /* Create and fill in the Message Security Label sub-object */
        if ((status = mhp_p_seclab((P1SecurityLabel *)pvalue,&subobject))
            != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&nextval,MH_T_SECURITY_LABEL,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_ORIG_AND_EXPANSION_HISTORY:   /* 1988 */
        /* Originator and DL Expansion History: 0-512 occurrences */
        if (pvalue = (char *)get_report_extension(&mpdu->v.report.envelope,
               	ETYPE_originator_and_DL_expansion_history)) 
	{
	int i;   /* Not to be confused with outer loop i */
        	for (i = 0, baseval = curval = nextval = desc->attr_value, 
	        	count = 0;
                        i < ((P1OriginatorAndDLExpansionHistory*)pvalue)->n; i++) 
		{
                 /* Create and fill in the Expansion History sub-object */
            		if ((status = mhp_p_origAndexphist((( P1OriginatorAndDLExpansionHistory*)
		                        	pvalue)->v[i],&subobject))
            				!= OM_SUCCESS)
        		    return(status);

            	/* Create a value structure for the subobject and link it */
            		if ((status = putvalue(desc,&nextval,
				MH_T_ORIG_AND_EXPANSION_HISTORY,
				OM_S_OBJECT,subobject)) 
		         	!= OM_SUCCESS)
            			return(status);

            		append_struct(&baseval,&curval,nextval,&count);
        	} /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;

    case MH_T_REPORTING_DL_NAME: /* 1988 */
        if (pvalue = (char *)get_report_extension(&mpdu->v.report.envelope,ETYPE_reporting_DL_name)) {
        /* Create and fill in Reporting DL Name sub-object */
        if ((status = mhp_p_orname((P1ORName *)*pvalue, &subobject)) != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_REPORTING_DL_NAME,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;


    case MH_T_PER_RECIPIENT_REPORTS:
        if (mpdu->v.report.content.m 
                    & ReportTransferContent_per_recipient_fields) 
        {
        for (prlist = (ListPointer *)
                    &(mpdu->v.report.content.per_recipient_fields),
                    prlist->current = prlist->first, count = 0,
                    baseval = curval = nextval = desc->attr_value;
            prlist->current != NULL; 
            prlist->current = prlist->current->next) 
        {
            /* Create and fill in the Per Recipient Report sub-object */
            if ((status = mhp_p_prr(prlist->current->data,&subobject)) 
                                        != OM_SUCCESS)
                return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_PER_RECIPIENT_REPORTS,
                            OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;


    case MH_T_CONTENT_CORRELATOR:        /* 1988 feature */
        if (pvalue = (char *)get_report_content_extension(&mpdu->v.report.content,
                ETYPE_content_correlator))
	{
           cc = (P1ContentCorrelator *) pvalue;
           if (len = cc->len) {
               if ((status = p_strcpy(desc,&desc->attr_value,
                                 MH_T_CONTENT_CORRELATOR,OM_S_ENCODING_STRING,
                                 cc->ptr,len)) != OM_SUCCESS)
                  return(status);
           }
        }
        break ;

    case MH_T_CONTENT_EXTENSIONS:        /* 1988 feature */
        break;

    case MH_T_ORIGIN_CHECK:              /* 1988 feature */
        break;

    case MH_T_REPORTING_MTA_CERTIFICATE: /* 1988 feature */
        break;

    case MH_T_DISTINGUISHED_RECIP_ADDR:  /* 1988 feature */
        break;

    case MH_T_CRITICAL_FOR_DELIVERY:     /* 1988 feature */ 
        {
        P1ExtensionMacro *extensions ;
        aaListType *list;
        P1ExtensionMacroBody *extdata;
        OM_type    criticalityAttrType ;
    
        extensions = &(env->extensions);
    
        list_first(extensions); /* set first item on list to current*/
        for (list = (aaListType *) extensions->current;
             list != NULL;
             list = (aaListType *)list_next(extensions))
        {
            extdata = (P1ExtensionMacroBody *)list->data;
            if (extdata->m |= ExtensionMacroBody_criticality) {
                tmp = criticalityAttrType = mhp_get_attr_criticality(extdata, 
                                                    CRITICALITY_for_delivery);

                if (criticalityAttrType != OM_NO_SUCH_TYPE) {

                   /* Create a value structure for the subobject and link it */
                   if ((status = putvalue(desc,&nextval,MH_T_CRITICAL_FOR_DELIVERY,
                                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                       return(status);
    
                   append_struct(&baseval,&curval,nextval,&count);
                }  /* end if */

            } /* end if extdata->m |= ExtensionMacroBody_criticality */

        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
   
        }
        break;

    case MH_T_CRITICAL_FOR_SUBMISSION:   /* 1988 feature */ 
        {
        P1ExtensionMacro *extensions ;
        aaListType *list;
        P1ExtensionMacroBody *extdata;
        OM_type    criticalityAttrType ;
    
        extensions = &(env->extensions);
    
        list_first(extensions); /* set first item on list to current*/
        for (list = (aaListType *) extensions->current;
             list != NULL;
             list = (aaListType *)list_next(extensions))
        {
            extdata = (P1ExtensionMacroBody *)list->data;
            if (extdata->m |= ExtensionMacroBody_criticality) {
                tmp = criticalityAttrType = mhp_get_attr_criticality(extdata, 
                                                    CRITICALITY_for_submission);

                if (criticalityAttrType != OM_NO_SUCH_TYPE) {

                   /* Create a value structure for the subobject and link it */
                   if ((status = putvalue(desc,&nextval,MH_T_CRITICAL_FOR_SUBMISSION,
                                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                       return(status);
    
                   append_struct(&baseval,&curval,nextval,&count);
                }  /* end if */

            } /* end if extdata->m |= ExtensionMacroBody_criticality */

        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
   
        }
        break;

    case MH_T_CRITICAL_FOR_TRANSFER:     /* 1988 feature */ 
        {
        P1ExtensionMacro *extensions ;
        aaListType *list;
        P1ExtensionMacroBody *extdata;
        OM_type    criticalityAttrType ;
    
        extensions = &(env->extensions);
    
        list_first(extensions); /* set first item on list to current*/
        for (list = (aaListType *) extensions->current;
             list != NULL;
             list = (aaListType *)list_next(extensions))
        {
            extdata = (P1ExtensionMacroBody *)list->data;
            if (extdata->m |= ExtensionMacroBody_criticality) {
                tmp = criticalityAttrType = mhp_get_attr_criticality(extdata, 
                                                    CRITICALITY_for_transfer);

                if (criticalityAttrType != OM_NO_SUCH_TYPE) {

                   /* Create a value structure for the subobject and link it */
                   if ((status = putvalue(desc,&nextval,MH_T_CRITICAL_FOR_TRANSFER,
                                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                       return(status);
    
                   append_struct(&baseval,&curval,nextval,&count);
                }  /* end if */

            } /* end if extdata->m |= ExtensionMacroBody_criticality */

        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
   
        }
        break;

    case MH_T_EXTENSIONS:                /* 1988 feature */ 
        break;


    } /* end switch */
    } /* end for */
	
    return(OM_SUCCESS);
} /* end mhp_p_report() */




/*
 * mhp_p_probe(mpdu, object) puts the P1 probe into Probe object.
 * It returns Success or Failure.
 *
 * INPUT:
 *  mpdu    - pointer to the P1MTS_APDU structure.
 *  object  - pointer to the Message object.
 *
 * OUTPUT:
 *  returning OM_SUCCESS or errors.
 *
 * COMMENTS:
 * NOTE:
 *
 */


int mhp_p_probe(mpdu, object)
P1MTS_APDU *mpdu;
OM_int_private_object_header *object;

{
OM_int_private_object_header *subobject;
char *buf;
Uint32 len;
Uint16 i=0, count;
int status, tmp;
int *evalue;
char *pvalue;
uint16 *criticality;
OM_enumeration	enumer;
OM_boolean  flag;
OM_int_descriptor *desc;
OM_type prev_attribute_type = OM_NO_MORE_TYPES;
OM_int_value *baseval, *curval, *nextval;
ListPointer *bilatlist, *extracelist, *intracelist, *rdlist;
P1InternalTraceInformationElement *trace_elem;


    /* Index through Message descriptor list sequentially and fill in attribute
     * if it exists. */
    for (i = TOP; 
        object->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
        i++) 
    {
    desc = (OM_int_descriptor_list)&(object->context.descriptor_list[i]);
    baseval = curval = nextval = NULL; count = 0;

/*
 * For attribute with multiple syntaxes, there are duplicated descriptors
 * in the template, but ONLY THE FIRST one contains the values. Hence we
 * need to skip the duplicated descriptors. KDH.
 */
    if (prev_attribute_type == desc->attr_type) continue;
    prev_attribute_type = desc->attr_type;


    switch (desc->attr_type) {

    case MH_T_MTS_IDENTIFIER:
        if (mpdu->v.probe.m & Probe_probe_identifier) {
        /* Create and fill in Probe Identifier sub-object */
        if ((status = mhp_p_mts_id(mpdu->v.probe.probe_identifier,&subobject)) 
                                                    != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_MTS_IDENTIFIER,
                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_ORIGINATOR_NAME:
        if (mpdu->v.probe.m & Probe_originator_name) {
        /* Create and fill in OR Name sub-object */
        if ((status = mhp_p_orname(mpdu->v.probe.originator_name,&subobject)) 
                                    != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_ORIGINATOR_NAME,
                            OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_ORIGINAL_EITS:
        if (mpdu->v.probe.m & Probe_original_encoded_information_types) {
        /* Create and fill in Original EITs sub-object */
        if ((status = mhp_p_eits(mpdu->v.probe.
                    original_encoded_information_types,&subobject))
                    != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_ORIGINAL_EITS,
                        OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_CONTENT_TYPE:
        if (mpdu->v.probe.m & Probe_content_type) {
        if (mpdu->v.probe.content_type.t == ContentType_built_in) {
            switch (mpdu->v.probe.content_type.v.built_in) {
            case BUILT_IN_unidentified:
                tmp = MH_UNIDENTIFIED;
                break;
            case BUILT_IN_external:
                tmp = MH_CTI_EXTERNAL;
                break;
            case BUILT_IN_interpersonal_messaging_1984:
                tmp = MH_CTI_P2_1984;
                break;
            case BUILT_IN_interpersonal_messaging_1988:
                tmp = MH_CTI_P2_1988;
                break;
            default: /* what ? */
		tmp = mpdu->v.probe.content_type.v.built_in;
		break;
            } /* end switch */

            /* Create value structure and link it */
            if ((status = putvalue(desc,&desc->attr_value,MH_T_CONTENT_TYPE,
                    OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                return(status);
        } else { /* 1988 object id not supported */
            return (OM_WRONG_VALUE_SYNTAX);
        } /* end else */
        } /* end if */
        break;

    case MH_T_CONTENT_IDENTIFIER:
        if (mpdu->v.probe.m & Probe_content_identifier) {
        if (len = strlen(mpdu->v.probe.content_identifier)) {
            if ((status = p_strcpy(desc,&desc->attr_value,
                    MH_T_CONTENT_IDENTIFIER,OM_S_PRINTABLE_STRING,
                    mpdu->v.probe.content_identifier,len)) != OM_SUCCESS)
                return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_CONTENT_LENGTH:
        if (mpdu->v.probe.m & Probe_content_length) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_CONTENT_LENGTH,
                        OM_S_INTEGER,
                        (void*)mpdu->v.probe.content_length)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_ALTERNATE_RECIP_ALLOWED:
        if (mpdu->v.probe.per_message_indicators 
                & PER_MESSAGE_INDICATORS_alternate_recipient_allowed) {
            flag = OM_TRUE;
        }
        else flag = OM_FALSE;
        if ((status = putvalue(desc,&desc->attr_value,
                MH_T_ALTERNATE_RECIP_ALLOWED,OM_S_BOOLEAN,(void*)flag)) 
                != OM_SUCCESS)
            return(status);
        break;

    case MH_T_CONVERSION_PROHIBITED:
        if (mpdu->v.probe.per_message_indicators 
                    & PER_MESSAGE_INDICATORS_implicit_conversion_prohibited) 
            flag = OM_TRUE;
        else flag = OM_FALSE;
        if ((status = putvalue(desc,&desc->attr_value,
                MH_T_CONVERSION_PROHIBITED,OM_S_BOOLEAN,(void*)flag)) 
                != OM_SUCCESS)
            return(status);
        break;

    case MH_T_CONTENT_RETURN_REQUESTED:
        if (mpdu->v.probe.per_message_indicators 
                    & PER_MESSAGE_INDICATORS_content_return_request) 
            flag = OM_TRUE;
        else flag = OM_FALSE;
        if ((status = putvalue(desc,&desc->attr_value,
                MH_T_CONTENT_RETURN_REQUESTED,OM_S_BOOLEAN,(void*)flag)) 
                != OM_SUCCESS)
            return(status);
        break;

    case MH_T_DISCLOSURE_ALLOWED:
        if (mpdu->v.probe.per_message_indicators 
                    & PER_MESSAGE_INDICATORS_disclosure_of_recipients) 
            flag = OM_TRUE;
        flag = OM_FALSE;
        if ((status = putvalue(desc,&desc->attr_value,
                MH_T_DISCLOSURE_ALLOWED,OM_S_BOOLEAN,(void*)flag)) 
                != OM_SUCCESS)
            return(status);
        break;

    case MH_T_BILATERAL_INFORMATION:
        /* Bilateral Information: 0-512 occurrences */
        if (mpdu->v.probe.m & Probe_per_domain_bilateral_information) {
        for (bilatlist = (ListPointer *)&(mpdu->v.probe.per_domain_bilateral_information),
            bilatlist->current = bilatlist->first, count = 0,
            baseval = curval = nextval = desc->attr_value;
            bilatlist->current != NULL; bilatlist->current = bilatlist->current->next) {
            /* Create and fill in the Bilateral Info sub-object */
            if ((status = mhp_p_bilat(bilatlist->current->data,&subobject)) != OM_SUCCESS)
            return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_BILATERAL_INFORMATION,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;
    case MH_T_EXTERNAL_TRACE_INFO:
        /* External Trace Information: 0-512 occurrences */
        if (mpdu->v.probe.m & Probe_trace_information) {
        for (extracelist = (ListPointer *)&(mpdu->v.probe.trace_information),
            extracelist->current = extracelist->first, count = 0,
            baseval = curval = nextval = desc->attr_value;
            extracelist->current != NULL; extracelist->current = extracelist->current->next) {
            /* Create and fill in the External Trace Info sub-object */
            if ((status = mhp_p_extrace(extracelist->current->data,&subobject)) != OM_SUCCESS)
            return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_EXTERNAL_TRACE_INFO,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;
    case MH_T_INTERNAL_TRACE_INFO:
        if (!mhp_is84MH)
	{ /* if 88MH */
          if (pvalue = (char *)get_probe_extension(mpdu,ETYPE_internal_trace_information))   
	  {
	   P1InternalTraceInformation *temp_p1internaltrace;
	    temp_p1internaltrace = (P1InternalTraceInformation *) pvalue;
	    for ( list_first( temp_p1internaltrace);
			    temp_p1internaltrace->current;
			    list_next( temp_p1internaltrace) ) 
	    { /* for loop */
		trace_elem = (P1InternalTraceInformationElement *) 
					temp_p1internaltrace->current->data;
            /* Create and fill in the Internal Trace Info sub-object */
		if ((status = mhp_p_intrace88 (trace_elem,&subobject)) 
				!= OM_SUCCESS)
		    return(status);

            /* Create a value structure for the subobject and link it */
		if ((status = putvalue(desc, &nextval,MH_T_INTERNAL_TRACE_INFO,
				  OM_S_OBJECT,subobject)) != OM_SUCCESS)
		    return(status);

		append_struct(&baseval,&curval, nextval,&count);
	    } /* for loop */ 

	    desc->attr_value = baseval;
	    desc->count = count;
	  }
	}
	else {

         /* Internal Trace Information: 0-512 occurrences */
         if (mpdu->v.probe.m & Probe_internal1984) {
          for (intracelist = (ListPointer *)&(mpdu->v.probe.internal1984),
            intracelist->current = intracelist->first, count = 0,
            baseval = curval = nextval = desc->attr_value;
            intracelist->current != NULL; intracelist->current = intracelist->current->next) {
            /* Create and fill in the Internal Trace Info sub-object */
            if ((status = mhp_p_intrace(intracelist->current->data,&subobject)) != OM_SUCCESS)
            return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_INTERNAL_TRACE_INFO,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);

            append_struct(&baseval,&curval,nextval,&count);
          } /* end for */

          desc->attr_value = baseval;
          desc->count = count;
         } /* end if */
	}
        break;
    case MH_T_RECIPIENT_DESCRIPTORS:
        /* Recipient Descriptors: 0-32767 occurrences */
        if (mpdu->v.probe.m & Probe_per_recipient_fields) {
        for (rdlist = (ListPointer *)&(mpdu->v.probe.per_recipient_fields),
            rdlist->current = rdlist->first, count = 0,
            baseval = curval = nextval = desc->attr_value;
            rdlist->current != NULL; rdlist->current = rdlist->current->next) {
            /* Create and fill in the Recipient Descriptors sub-object */
            if ((status = mhp_p_probe_rd(rdlist->current->data,&subobject)) != OM_SUCCESS)
            return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_RECIPIENT_DESCRIPTORS,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;

    case MH_T_CONTENT_CORRELATOR: 
        if (pvalue = (char *)get_probe_extension(mpdu,ETYPE_content_correlator)) 
	{
        	if (len = ((P1ContentCorrelator *)pvalue)->len) 
		{
            		if ((status = p_strcpy(desc,&desc->attr_value,
			                       MH_T_CONTENT_CORRELATOR,
                                               OM_S_OCTET_STRING,
					       ((P1ContentCorrelator *)pvalue)->ptr,len)) 
					       != OM_SUCCESS)
            		return(status);
        	} /* end if */
        } /* end if */
        break;
    case MH_T_CONVERSION_LOSS_PROHIBITED: 
        if (pvalue = (char *)get_probe_extension(mpdu,

                                      ETYPE_conversion_with_loss_prohibited)) 
	{
        	if (*(P1ConversionWithLossProhibited *)pvalue) 
		{
        		if ((status = putvalue(desc,&desc->attr_value,
					MH_T_CONVERSION_LOSS_PROHIBITED,
					OM_S_BOOLEAN,(void*)OM_TRUE)) 
					!= OM_SUCCESS)
            			return(status);
        	} /* end if */
                else 
		{ /* default is false */
            		if ((status = putvalue(desc,&desc->attr_value,
                			MH_T_CONVERSION_LOSS_PROHIBITED,
					OM_S_BOOLEAN,(void*)OM_FALSE)) 
                			!= OM_SUCCESS)
            		return(status);
                } /* end else */
        } /* end if */
        break;
    case MH_T_EXPANSION_HISTORY: 
        if (pvalue = (char *)get_probe_extension(mpdu,ETYPE_dl_expansion_history)) 
	{
	int i;   /* Not to be confused with outer loop i */
        for (i = 0, baseval = curval = nextval = desc->attr_value, count = 0;
            i < ((P1DLExpansionHistory *)pvalue)->n; i++) {
            /* Create and fill in the Expansion History sub-object */
            if ((status = mhp_p_exphist(((P1DLExpansionHistory *)pvalue)->v[i],&subobject))
            != OM_SUCCESS)
            return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_EXPANSION_HISTORY,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;
    case MH_T_EXPANSION_PROHIBITED: 
        if (pvalue = (char *)get_probe_extension(mpdu,ETYPE_dl_expansion_prohibited)) 
	{
        	if (*(P1DLExpansionProhibited *)pvalue) 
		{
        		if ((status = putvalue(desc,&desc->attr_value,
					MH_T_EXPANSION_PROHIBITED,
					OM_S_BOOLEAN,(void*)OM_TRUE)) 
					!= OM_SUCCESS)
            			return(status);
        	} /* end if */
                else 
		{ /* default is false */
            		if ((status = putvalue(desc,&desc->attr_value,
                			MH_T_EXPANSION_PROHIBITED,
					OM_S_BOOLEAN,(void*)OM_FALSE)) 
                			!= OM_SUCCESS)
            		return(status);
                } /* end else */
        } /* end if */
        break;
    case MH_T_ORIGIN_CHECK:  
        if (pvalue = (char *)get_probe_extension(mpdu,
                      	ETYPE_probe_origin_authentication_check)) 
	{
        /* Create and fill in the Probe origin check sub-object */
        if ((status = mhp_p_origincheck((P1ProbeOriginAuthenticationCheck *)pvalue,&subobject))
            != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&nextval,MH_T_ORIGIN_CHECK,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_ORIGINATOR_CERTIFICATE: 
        if (pvalue = (char *)get_probe_extension(mpdu,
                      	ETYPE_originator_certificate)) 
	{
        /* Create and fill in the Probe originator certificate sub-object */
        if ((status = mhp_p_orig_cert((P1OriginatorCertificate *)pvalue,&subobject))
            != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&nextval,MH_T_ORIGINATOR_CERTIFICATE,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_REASSIGNMENT_PROHIBITED: 
        if (pvalue = (char *)get_probe_extension(mpdu,
                            ETYPE_recipient_reassignment_prohibited)) 
	{
        	if (*(P1RecipientReassignmentProhibited *)pvalue) 
		{
        		if ((status = putvalue(desc,&desc->attr_value,
					MH_T_REASSIGNMENT_PROHIBITED,
					OM_S_BOOLEAN,(void*)OM_TRUE)) 
					!= OM_SUCCESS)
            			return(status);
        	} /* end if */
                else 
		{ /* default is false */
            		if ((status = putvalue(desc,&desc->attr_value,
                			MH_T_REASSIGNMENT_PROHIBITED,
					OM_S_BOOLEAN,(void*)OM_FALSE)) 
                			!= OM_SUCCESS)
            		return(status);
                } /* end else */
        } /* end if */
        break;
    case MH_T_SECURITY_LABEL:   
        if (pvalue = (char *)get_probe_extension(mpdu,ETYPE_message_security_label)) 
	{
        /* Create and fill in the Probe Security Label sub-object */
        if ((status = mhp_p_seclab((P1SecurityLabel *)pvalue,&subobject))
            != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&nextval,MH_T_SECURITY_LABEL,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_CRITICAL_FOR_DELIVERY: 
        break;
    case MH_T_CRITICAL_FOR_SUBMISSION: 
        break;
    case MH_T_CRITICAL_FOR_TRANSFER: 
        break;
    case MH_T_EXTENSIONS: 
        break;
    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_probe() */




/*
 * mhp_p_mts_id()
 */

int mhp_p_mts_id(id, object)
P1MTSIdentifier *id;
OM_int_private_object_header **object;
{
char *str;
Uint32 len;
Uint16 i;
int status;
OM_int_descriptor *desc;

    /* Create MTS Identifier object */
    if ((status = omp_template(MH_C_MTS_IDENTIFIER,object)) != OM_SUCCESS)
        return(status);

    /* Index through MTS Identifier descriptor list sequentially and fill in
     * attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_COUNTRY_NAME:
        if (id->global_domain_identifier.m 
                & GlobalDomainIdentifier_country_name) 
        {
        if ((status = mhp_p_country(&str,&len,id->global_domain_identifier.
                                country_name)) != OM_SUCCESS)
            return(status);

        if (len)
            if ((status = putvalue(desc,&desc->attr_value,MH_T_COUNTRY_NAME,
                        OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
                return(status);
        } /* end if */
        break;

    case MH_T_ADMD_NAME:
        if (id->global_domain_identifier.m 
                & GlobalDomainIdentifier_administration_domain_name) 
        {
        if ((status = mhp_p_admdcpy(&str,&len,id->
                global_domain_identifier.administration_domain_name))
                != OM_SUCCESS)
            return(status);
        if (len)
            if ((status = putvalue(desc,&desc->attr_value,MH_T_ADMD_NAME,
                    OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
                return(status);
        } /* end if */
        break;

    case MH_T_PRMD_IDENTIFIER:
        if (id->global_domain_identifier.m 
                & GlobalDomainIdentifier_private_domain_identifier) 
        {
        if ((status = mhp_p_prmdcpy(&str,&len,id->
                global_domain_identifier.private_domain_identifier))
                != OM_SUCCESS)
            return(status);
        if (len)
            if ((status = putvalue(desc,&desc->attr_value,MH_T_PRMD_IDENTIFIER,
                    OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_LOCAL_IDENTIFIER:
        if (id->m & MTSIdentifier_local_identifier) {
        if (len = strlen(id->local_identifier)) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_LOCAL_IDENTIFIER,
                    OM_S_IA5_STRING,id->local_identifier,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);

} /* end mhp_p_mts_id() */

/*
 * mhp_p_oraddress(orname, object) creates and fills in ORName object with
 * values from the given P1ORName structure. It returns the object handle
 * to the calling routine for insertion by the calling routine.
 *
 * INPUT:
 *  orname - address of the P1ORName structure
 *  object - address of variable to return object handle
 *
 * OUTPUT:
 *  object - now contains the handle to an OR Adress (or OR Name) if success.
 *
 * NOTE:
 *  For 1984, only the OR Address object is being returned.
 *  For 1988, an OR Name object might be returned.
 *
 */

int mhp_p_oraddress(orname, object)
P1ORName *orname;
OM_int_private_object_header **object;
{
OM_int_private_object_header *subobject;
P1StandardAttributes *std;
P1DomainDefinedAttributes *dda;
P1TeletexDomainDefinedAttributes *ttxDDA = NULL;
P1TeletexPersonalName *ttxPN = NULL;
P1TeletexOrganizationalUnitNames *ttxOU = NULL;
P1ExtendedNetworkAddress *pena;
P1TerminalType ttype;
P1TerminalType *pttype;
char *str, *pvalue;
P1PDSParameter *ppdsparm;
int status;
int *evalue;
Uint16 i, count;
Uint32 len;
OM_int_descriptor *desc;
OM_type prev_attribute_type = OM_NO_MORE_TYPES;
OM_int_value *baseval, *curval, *nextval;

/* Create OR Address object. If this support 1988, need to
 * check whether X.500 Distinguished Name is present. If so
 * OR Name object should be created, instead of OR Address.
 * On second thought, since the class tables always specify 
 * Object(OR_NAME), it's safer to return OR_NAME.
 */
    if ((status = omp_template(MH_C_OR_ADDRESS,object)) != OM_SUCCESS)
    return(status);

    std = (P1StandardAttributes *) orname->standard_attributes; 
    dda = (P1DomainDefinedAttributes *) &orname->domain_defined_attributes;

/*
 * Similar to formatting ORName, there are group of attributes that share 
 * a single extension and data structure: ttxDDA, ttxOU, PN. For ease of
 * algorithm, we'll get these extension and parse them first if present.
 */

    if (pvalue = (char *)get_add_exts(orname,
                  EXTENSION_ATTRIBUTE_TYPE_teletex_personal_name))
        ttxPN = (P1TeletexPersonalName *)pvalue;

    if (pvalue = (char *)get_add_exts(orname,
                  EXTENSION_ATTRIBUTE_TYPE_teletex_organizational_unit_names))
        ttxOU = (P1TeletexOrganizationalUnitNames *)pvalue;

    if (pvalue = (char *)get_add_exts(orname,
                  EXTENSION_ATTRIBUTE_TYPE_teletex_domain_defined_attributes))
        ttxDDA = (P1TeletexDomainDefinedAttributes *)pvalue;


    /* Index through OR Name descriptor list sequentially and fill in
     * attribute if it exists. */
    for (i = TOP; 
        (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
        i++) 
    {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
/*
 * For attribute with multiple syntaxes, there are duplicated descriptors
 * in the template, but ONLY THE FIRST one contains the values. Hence we
 * need to skip the duplicated descriptors. 
 *
 * For every attribute, we need to look at the corresponding class
 * in the package for multiple syntaxes/values: if so, then go get all
 * the present values: eg, ORGNAME might be present as a standard attribute
 * as Printable String, as well as as an extension (Teletex String). This
 * applies to most attributes, in one way or another, except for a few
 * (Country, ADMD, PRMD, etc).
 */
    if (prev_attribute_type == desc->attr_type) continue;
    prev_attribute_type = desc->attr_type;
    baseval = curval = nextval = NULL;
    count = 0;

    switch (desc->attr_type) {
    case MH_T_COUNTRY_NAME:
        if (std->m & StandardAttributes_country_name) {
        if ((status = mhp_p_country(&str,&len,&std->country_name)) != OM_SUCCESS)
            return(status);
        if ((status = putvalue(desc,&desc->attr_value,MH_T_COUNTRY_NAME,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
   case MH_T_ADMD_NAME:
        if (std->m & StandardAttributes_administration_domain_name) {
        if ((status = mhp_p_admdcpy(&str,&len,&std->administration_domain_name)) != OM_SUCCESS)
            return(status);
        if ((status = putvalue(desc,&desc->attr_value,MH_T_ADMD_NAME,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_GENERATION:
        if (std->personal_name.m & PersonalName_generation_qualifier) {
        if (len = strlen(std->personal_name.generation_qualifier)) {
            if ((status = p_strcpy(desc, &nextval,
                          MH_T_GENERATION, OM_S_PRINTABLE_STRING,
                          std->personal_name.generation_qualifier,len)) 
                          != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxPN && (len = strlen(ttxPN->generation_qualifier))) {
            if ((status = p_strcpy(desc, &nextval,
                          MH_T_GENERATION, OM_S_TELETEX_STRING,
                          ttxPN->generation_qualifier,len)) 
                          != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;

        break;

    case MH_T_GIVEN_NAME:
        if (std->personal_name.m & PersonalName_given_name) {
        if (len = strlen(std->personal_name.given_name)) {
            if ((status = p_strcpy(desc,&nextval,
                        MH_T_GIVEN_NAME,OM_S_PRINTABLE_STRING,
                        std->personal_name.given_name,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxPN && (len = strlen(ttxPN->given_name))) {
            if ((status = p_strcpy(desc, &nextval,
                          MH_T_GIVEN_NAME, OM_S_TELETEX_STRING,
                          ttxPN->generation_qualifier,len)) 
                          != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;


        break;
    case MH_T_INITIALS:
        if (std->personal_name.m & PersonalName_initials) {
        if (len = strlen(std->personal_name.initials)) {
            if ((status = p_strcpy(desc,&nextval,
                    MH_T_INITIALS,OM_S_PRINTABLE_STRING,
                    std->personal_name.initials,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxPN && (len = strlen(ttxPN->initials))) {
            if ((status = p_strcpy(desc, &nextval,
                          MH_T_INITIALS, OM_S_TELETEX_STRING,
                          ttxPN->initials,len)) 
                          != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_X121_ADDRESS:
        if (std->m & StandardAttributes_network_address) {
        if (len = strlen(std->network_address)) {
            if ((status = p_strcpy(desc,&desc->attr_value, MH_T_X121_ADDRESS,
                          OM_S_NUMERIC_STRING,
                          std->network_address,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    case MH_T_NUMERIC_USER_IDENTIFIER:
        if (std->m & StandardAttributes_numeric_user_identifier) {
        if (len = strlen(std->numeric_user_identifier)) {
            if ((status = p_strcpy(desc,&desc->attr_value,
                       MH_T_NUMERIC_USER_IDENTIFIER, OM_S_NUMERIC_STRING,
                       std->numeric_user_identifier,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_ORGANIZATION_NAME:
        if (std->m & StandardAttributes_organization_name) {
        if (len = strlen(std->organization_name)) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATION_NAME,OM_S_PRINTABLE_STRING,
                     std->organization_name,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (pvalue = (char *)get_add_exts(orname,
                      EXTENSION_ATTRIBUTE_TYPE_teletex_organization_name)) {
            if (len = strlen(pvalue)) {
                if ((status = p_strcpy(desc, &nextval,
                         MH_T_ORGANIZATION_NAME, OM_S_TELETEX_STRING,
                         pvalue,len)) != OM_SUCCESS)
                    return(status);
                append_struct(&baseval,&curval,nextval,&count);
            }
        }

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_ORGANIZATIONAL_UNIT_NAME_1:

        if (std->m & StandardAttributes_organizational_unit_names) {
        if (len = strlen(std->organizational_unit_names.v[0])) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_1,OM_S_PRINTABLE_STRING,
                     std->organizational_unit_names.v[0],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxOU && (len = strlen(ttxOU->v[0]))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_1,OM_S_TELETEX_STRING,
                     ttxOU->v[0],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_ORGANIZATIONAL_UNIT_NAME_2:

        if (std->m & StandardAttributes_organizational_unit_names) {
        if (len = strlen(std->organizational_unit_names.v[1])) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_2, OM_S_PRINTABLE_STRING,
                     std->organizational_unit_names.v[1],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxOU && (len = strlen(ttxOU->v[1]))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_2,OM_S_TELETEX_STRING,
                     ttxOU->v[1],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_ORGANIZATIONAL_UNIT_NAME_3:
        if (std->m & StandardAttributes_organizational_unit_names) {
        if (len = strlen(std->organizational_unit_names.v[2])) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_3,OM_S_PRINTABLE_STRING,
                     std->organizational_unit_names.v[2],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxOU && (len = strlen(ttxOU->v[2]))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_3,OM_S_TELETEX_STRING,
                     ttxOU->v[2],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_ORGANIZATIONAL_UNIT_NAME_4:
        if (std->m & StandardAttributes_organizational_unit_names) {
        if (len = strlen(std->organizational_unit_names.v[3])) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_4,OM_S_PRINTABLE_STRING,
                     std->organizational_unit_names.v[3],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxOU && (len = strlen(ttxOU->v[3]))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_4,OM_S_TELETEX_STRING,
                     ttxOU->v[3],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_PRMD_NAME:
        if (std->m & StandardAttributes_private_domain_name) {
        if ((status = mhp_p_prmdcpy(&str,&len,&std->private_domain_name)) 
                      != OM_SUCCESS)
            return(status);
        if ((status = putvalue(desc,&desc->attr_value,
                 MH_T_PRMD_NAME,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_SURNAME:
        if ((std->m & StandardAttributes_personal_name) 
             && (std->personal_name.m & PersonalName_surname)) {
        if (len = strlen(std->personal_name.surname)) {
            if ((status = p_strcpy(desc, &nextval,
                       MH_T_SURNAME,OM_S_PRINTABLE_STRING,
                       std->personal_name.surname,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxPN && (len = strlen(ttxPN->surname))) {
            if ((status = p_strcpy(desc, &nextval,
                          MH_T_GENERATION, OM_S_TELETEX_STRING,
                          ttxPN->surname,len)) 
                          != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;

        break;
    case MH_T_TERMINAL_IDENTIFIER:
        if (std->m & StandardAttributes_terminal_identifier) {
        if (len = strlen(std->terminal_identifier)) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_TERMINAL_IDENTIFIER,
            OM_S_PRINTABLE_STRING,std->terminal_identifier,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_DOMAIN_TYPE_1:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[0].m & DomainDefinedAttribute_ddatype) 
             && (len = strlen(dda->v[0].ddatype))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_1,OM_S_PRINTABLE_STRING,
                     dda->v[0].ddatype,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[0].m & TeletexDomainDefinedAttribute_ttxddatype)
            && (len = strlen(ttxDDA->v[0].ttxddatype ))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_1, OM_S_TELETEX_STRING,
                     ttxDDA->v[0].ttxddatype, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_TYPE_2:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[1].m & DomainDefinedAttribute_ddatype) 
            && (len = strlen(dda->v[1].ddatype))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_2,OM_S_PRINTABLE_STRING,
                     dda->v[1].ddatype,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[1].m & TeletexDomainDefinedAttribute_ttxddatype)
            && (len = strlen(ttxDDA->v[1].ttxddatype ))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_2, OM_S_TELETEX_STRING,
                     ttxDDA->v[1].ttxddatype, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_TYPE_3:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[2].m & DomainDefinedAttribute_ddatype) 
             && (len = strlen(dda->v[2].ddatype))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_3,OM_S_PRINTABLE_STRING,
                     dda->v[2].ddatype,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[2].m & TeletexDomainDefinedAttribute_ttxddatype)
            && (len = strlen(ttxDDA->v[2].ttxddatype ))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_3, OM_S_TELETEX_STRING,
                     ttxDDA->v[2].ttxddatype, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_TYPE_4:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[3].m & DomainDefinedAttribute_ddatype) 
             && (len = strlen(dda->v[3].ddatype))) {
            if ((status = p_strcpy(desc, &nextval,
                     MH_T_DOMAIN_TYPE_4,OM_S_PRINTABLE_STRING,
                     dda->v[3].ddatype,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[3].m & TeletexDomainDefinedAttribute_ttxddatype)
            && (len = strlen(ttxDDA->v[3].ttxddatype ))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_4, OM_S_TELETEX_STRING,
                     ttxDDA->v[3].ttxddatype, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_VALUE_1:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[0].m & DomainDefinedAttribute_ddavalue) 
            && (len = strlen(dda->v[0].ddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_1,OM_S_PRINTABLE_STRING,
                     dda->v[0].ddavalue,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[0].m & TeletexDomainDefinedAttribute_ttxddavalue)
            && (len = strlen(ttxDDA->v[0].ttxddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_1, OM_S_TELETEX_STRING,
                     ttxDDA->v[0].ttxddavalue, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_VALUE_2:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[1].m & DomainDefinedAttribute_ddavalue) 
             && (len = strlen(dda->v[1].ddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_2,OM_S_PRINTABLE_STRING,
                     dda->v[1].ddavalue,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[1].m & TeletexDomainDefinedAttribute_ttxddavalue)
            && (len = strlen(ttxDDA->v[1].ttxddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_2, OM_S_TELETEX_STRING,
                     ttxDDA->v[1].ttxddavalue, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_VALUE_3:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[2].m & DomainDefinedAttribute_ddavalue) 
            && (len = strlen(dda->v[2].ddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_3,OM_S_PRINTABLE_STRING,
                     dda->v[2].ddavalue,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[2].m & TeletexDomainDefinedAttribute_ttxddavalue)
            && (len = strlen(ttxDDA->v[2].ttxddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_3, OM_S_TELETEX_STRING,
                     ttxDDA->v[2].ttxddavalue, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_VALUE_4:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[3].m & DomainDefinedAttribute_ddavalue) 
             && (len = strlen(dda->v[3].ddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_4,OM_S_PRINTABLE_STRING,
                     dda->v[3].ddavalue,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[3].m & TeletexDomainDefinedAttribute_ttxddavalue)
            && (len = strlen(ttxDDA->v[3].ttxddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_4, OM_S_TELETEX_STRING,
                     ttxDDA->v[3].ttxddavalue, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;


    case MH_T_COMMON_NAME:
        /* common name takes two extensions, one for Printable and one
         * for Teletex String.
         */

        /* Get Printable Common Name extension, if it exists */
        if (pvalue = (char *)get_add_exts(orname,
                                 EXTENSION_ATTRIBUTE_TYPE_common_name)) {

        /* since P1CommonName is a char[], pvalue is pointer to char */
        if (len = strlen(pvalue)) {
            if ((status = p_strcpy(desc,&nextval,MH_T_COMMON_NAME,
                  OM_S_PRINTABLE_STRING,(void*)pvalue,len)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        /* Get Teletex Common Name extension, if it exists */
        if (pvalue = (char *)get_add_exts(orname,
                               EXTENSION_ATTRIBUTE_TYPE_teletex_common_name)) 
        {
    
        /* since P1TeletexCommonName is char[], pvalue is pointer to char */
        if (len = strlen(pvalue)) {
            if ((status = p_strcpy(desc,&nextval,MH_T_COMMON_NAME,
                         OM_S_TELETEX_STRING,(void*)pvalue,len)) != OM_SUCCESS)
            return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_ADDRESS_DETAILS:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_extension_physical_delivery_address_components)) {

           /* Get Printable Postal Address Details extension, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_ADDRESS_DETAILS,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Postal Address Details extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_ADDRESS_DETAILS,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_ADDRESS_IN_FULL:
/*
 * See comments in MH_T_POSTAL_ADDRESS_IN_LINES. This IN_FULL attribute
 * share the same data structure with IN_LINES attribute.
 *
 * NOTE: because two attributes share the same data structure, should
 * clear extensions ONLY AT THE END OF THE FUNCTION OR AFTER. KDH.
 */
        if (pvalue = (char *)get_add_exts(orname,
                  EXTENSION_ATTRIBUTE_TYPE_unformatted_postal_address)) {
        if (((P1UnformattedPostalAddress *)pvalue)->m 
              & UnformattedPostalAddress_teletex_string) {
            if (len = strlen(((P1UnformattedPostalAddress *)pvalue)->
                               teletex_string)) {
              if ((status = p_strcpy(desc,&nextval,
                       MH_T_POSTAL_ADDRESS_IN_FULL,OM_S_TELETEX_STRING,
                       (void*) ((P1UnformattedPostalAddress *)pvalue)->
                                 teletex_string,len)) != OM_SUCCESS)
                  return(status);

              append_struct(&baseval,&curval,nextval,&count);
            } /* end if */
        } /* end if teletex string */
        } /* end if (pvalue) */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;

    case MH_T_POSTAL_CODE:
        if (pvalue = (char *)get_add_exts(orname,
                       EXTENSION_ATTRIBUTE_TYPE_postal_code)) {
        if (((P1PostalCode *)pvalue)->t == PostalCode_numeric_code) {
            if (len = strlen((char *)((P1PostalCode *)pvalue)->v.
                                       numeric_code)) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_POSTAL_CODE,OM_S_NUMERIC_STRING, 
                     (void*) ((P1PostalCode *)pvalue)->v.numeric_code,len)) 
                     != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
            } /* end if */
        } /* end if numeric code */
        else if (((P1PostalCode *)pvalue)->t == PostalCode_printable_code) {
            if (len = strlen((char *)((P1PostalCode *)pvalue)->v.
                                        printable_code)) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_POSTAL_CODE,OM_S_PRINTABLE_STRING, 
                     (void*) ((P1PostalCode *)pvalue)->v.printable_code,len))
                      != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
            } /* end if */
        } /* end if printable */
        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;

    case MH_T_POSTAL_ADDRESS_IN_LINES:
/* 
 * in asn1 p1p2.h, the P1UnformattedPostalAddress consists of
 * an array of printable strings (corresponding to a sequence of 
 * printablestring in X.402/X.411) and a single teletex string.
 * The printable string corresponding to POSTAL_ADDRESS_IN_LINES attribute.
 * The teletex string corresponds to POSTAL_ADDRESS_IN_FULL attribute.
 * Each line of printables is mapped to an attribute value.
 *
 * NOTE: because two attributes share the same data structure, should
 * clear extensions ONLY AT THE END OF THE FUNCTION OR AFTER. KDH.
 */
        if (pvalue = (char *)get_add_exts(orname,
                  EXTENSION_ATTRIBUTE_TYPE_unformatted_postal_address)) {
        if (((P1UnformattedPostalAddress *)pvalue)->m 
              & UnformattedPostalAddress_printable_address) 
        {
            int i = 0;    /* local index */
            for (i=0; 
                 i <((P1UnformattedPostalAddress *)pvalue)->printable_address.n;
                 i++) {
              if (len = strlen((char *)((P1UnformattedPostalAddress *)pvalue)->
                                printable_address.v[i])) {
                if ((status = p_strcpy(desc, &nextval,
                       MH_T_POSTAL_ADDRESS_IN_LINES, OM_S_PRINTABLE_STRING,
                       (void*) &(((P1UnformattedPostalAddress *)pvalue)->
                                printable_address.v[i][0]),len)) != OM_SUCCESS)
                  return(status);
                append_struct(&baseval,&curval,nextval,&count);
              } /* len > 0 */
            } /* for */
        } /* end if printable string */
        } /* end if extension */

        desc->attr_value = baseval;
        /* desc->count = count; */
	break;

    case MH_T_POSTAL_COUNTRY_NAME:
        if (pvalue = (char *)get_add_exts(orname,
                 EXTENSION_ATTRIBUTE_TYPE_physical_delivery_country_name)) {
        if (((P1PhysicalDeliveryCountryName *)pvalue)->t 
              == PhysicalDeliveryCountryName_x121_dcc_code) {
          if (len = strlen((char *)(
                (P1PhysicalDeliveryCountryName *)pvalue)->v.x121_dcc_code)) {
            if ((status = p_strcpy(desc,&nextval,
                    MH_T_POSTAL_COUNTRY_NAME,OM_S_NUMERIC_STRING, 
                    (void*) ((P1PhysicalDeliveryCountryName *)pvalue)->v.
                              x121_dcc_code,len)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
            } /* end if len */
        } /* end if dcc code */
        else if (((P1PhysicalDeliveryCountryName *)pvalue)->t 
               == PhysicalDeliveryCountryName_iso_3166_alpha2_code) {
          if (len = strlen((char *)((P1PhysicalDeliveryCountryName *)pvalue)->
                                     v.iso_3166_alpha2_code)) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_POSTAL_COUNTRY_NAME,OM_S_PRINTABLE_STRING, 
                     (void*) ((P1PhysicalDeliveryCountryName *)pvalue)->v.
                              iso_3166_alpha2_code,len)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
            } /* end if len */
        } /* end if alpha2 code */
        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */
        break;

    case MH_T_POSTAL_LOCALE:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_unique_postal_name)) {

           /* Get Printable Unique-postal-name extension, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_LOCALE,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Unique-postal-name extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_LOCALE,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_OFFICE_BOX_NUMBER:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_post_office_box_address)) {

           /* Get Printable Post-office-box-address extension, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_OFFICE_BOX_NUMBER,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Post-office-box-address extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_OFFICE_BOX_NUMBER,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;

    case MH_T_POSTAL_OFFICE_NAME:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_physical_delivery_office_name)) {

           /* Get Printable Physical-delivery-office-name, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_OFFICE_NAME,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Physical-delivery-office-name extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_OFFICE_NAME,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;

    case MH_T_POSTAL_OFFICE_NUMBER:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_physical_delivery_office_number)) {

           /* Get Printable Physical-delivery-office-number, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_OFFICE_NUMBER,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Physical-delivery-office-number extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_OFFICE_NUMBER,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;

    case MH_T_POSTAL_ORGANIZATION_NAME:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_physical_delivery_organization_name)) {

           /* Get Printable Physical-delivery-organization-name, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_ORGANIZATION_NAME,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Physical-delivery-organization-name extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_ORGANIZATION_NAME,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;

    case MH_T_POSTAL_STREET_ADDRESS:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_street_address)) {

           /* Get Printable Street-address, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_STREET_ADDRESS,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Street-address extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_STREET_ADDRESS,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_PATRON_NAME:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_physical_delivery_personal_name)) {

           /* Get Printable Physical-delivery-personal-name, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_PATRON_NAME,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Physical-delivery-personal-name, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_PATRON_NAME,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_PATRON_DETAILS:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_extension_OR_address_components)) {

           /* Get Printable Extension-postal-O/R-address-components, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_PATRON_DETAILS,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Extension-postal-O/R-address-components, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_PATRON_DETAILS,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_DELIVERY_POINT_NAME:
        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_local_postal_attributes)) {

           /* Get Printable Postal Delivery Point Name extension, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_DELIVERY_POINT_NAME,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Postal Delivery Point Name extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_DELIVERY_POINT_NAME,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_DELIV_SYSTEM_NAME:
        if (pvalue = (char *)get_add_exts(orname,EXTENSION_ATTRIBUTE_TYPE_pds_name)) {
        if (len = strlen(pvalue)) {
            if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_DELIV_SYSTEM_NAME,
                  OM_S_PRINTABLE_STRING,(void *)pvalue,len)) != OM_SUCCESS)
               return(status);
      
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_GENERAL_DELIV_ADDR:
        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_poste_restante_address)) {

           /* Get Printable Postal General Delivery Address extension, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_GENERAL_DELIV_ADDR,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Postal General Delivery Address extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_GENERAL_DELIV_ADDR,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_TERMINAL_TYPE:
        if (pttype = (P1TerminalType *)get_add_exts(orname,
                      EXTENSION_ATTRIBUTE_TYPE_terminal_type)) {
            ttype = *pttype;
            if ((status = putvalue(desc,&nextval,
                  MH_T_TERMINAL_TYPE,OM_S_ENUMERATION,
                  (void *)ttype))
                 != OM_SUCCESS)
               return(status);
      
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_PRESENTATION_ADDRESS:
        if (pena = (P1ExtendedNetworkAddress *)get_add_exts(orname,
                     EXTENSION_ATTRIBUTE_TYPE_extended_network_address)) {
           if (pena->t == ExtendedNetworkAddress_psap_address) {
           /* Create and fill in Presentation Address Subobject*/
	       if ((status = mhp_p_pres_addr(&pena->v.psap_address,
						 &subobject)) != OM_SUCCESS)
		    return(status);

           /* Create a value structure for the subobject and link it */
               if ((status = putvalue(desc,&desc->attr_value,
                           MH_T_PRESENTATION_ADDRESS,OM_S_OBJECT,subobject)) 
                                  != OM_SUCCESS)
              return(status);
           } /* end if */
        }
        break;


    case MH_T_ISDN_NUMBER:
        if (pena = (P1ExtendedNetworkAddress *)get_add_exts(orname,
                     EXTENSION_ATTRIBUTE_TYPE_extended_network_address)) {
           if (pena->t == ExtendedNetworkAddress_e163_4_address) {
              if (pena->v.e163_4_address.m & ExtendedNetworkAddress_basenumber) {
                 if (len = strlen(pena->v.e163_4_address.basenumber)) {
                    if ((status = p_strcpy(desc,&nextval,MH_T_ISDN_NUMBER,
                         OM_S_NUMERIC_STRING, (void*)pena->v.e163_4_address.basenumber,len))
                           != OM_SUCCESS)
                        return(status);

                    append_struct(&baseval,&curval,nextval,&count);
                 } /* end if len */
              }  /* end if mask */
           }   /* end if tag */
	}   /* end if orname */

        desc->attr_value = baseval;
        /* desc->count = count; */
        break;


    case MH_T_ISDN_SUBADDRESS:

        if (pena = (P1ExtendedNetworkAddress *)get_add_exts(orname,
                     EXTENSION_ATTRIBUTE_TYPE_extended_network_address)) {
           if (pena->t == ExtendedNetworkAddress_e163_4_address) {
              if (pena->v.e163_4_address.m & ExtendedNetworkAddress_sub_address) {
                 if (len = strlen(pena->v.e163_4_address.sub_address)) {
                    if ((status = p_strcpy(desc,&nextval,MH_T_ISDN_SUBADDRESS,
                         OM_S_NUMERIC_STRING, (void*)pena->v.e163_4_address.sub_address,len))
                           != OM_SUCCESS)
                        return(status);

                    append_struct(&baseval,&curval,nextval,&count);
                 } /* end if len */
              } /* end if mask */
           }  /* end if tag */
        }   /* end if orname */

        desc->attr_value = baseval;
        /* desc->count = count; */
        break;

    default:
	break;

    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_oraddress() */

/*
 * mhp_p_dir_name()
 */
int mhp_p_dir_name(dirname, object)
P1Name *dirname;
OM_int_private_object_header **object;
{
OM_int_private_object_header *subobject;
char *str;
Uint32 len;
Uint16 i, count;
int status;
OM_int_descriptor *desc;
OM_type prev_attribute_type = OM_NO_MORE_TYPES;
OM_int_value *baseval, *curval, *nextval;
P1RelativeDistinguishedName *rdn;
aaListPointer list;
aaListType *item;

    /* Create object */
/*    if ((status = omp_template(DS_C_DS_DN, object)) != OM_SUCCESS)
        return(status);
*/
    /* Index through Directory Name descriptor list sequentially 
     * and fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);

    if (prev_attribute_type == desc->attr_type) continue;
    prev_attribute_type = desc->attr_type;
    baseval = curval = nextval = NULL;
    count = 0;

    switch (desc->attr_type) {
       case DS_RDNS:
          if (dirname->t == Name_RDNSequence) {

             list = (aaListPointer) &(dirname->v.rDNSequence);
             list_first(list);

             for (item = list->first;
                  item != NULL; 
                  item = (aaListType *) list_next(list))   {

                  rdn =  (P1RelativeDistinguishedName *)item->data ;
                  if ((status = mhp_p_rdn(rdn ,&subobject))
                                                != OM_SUCCESS)
                     return(status);
    
                  if ((status = putvalue(desc,&nextval,
                            DS_RDNS,OM_S_OBJECT,subobject))
                            != OM_SUCCESS)
                     return(status);

                  append_struct(&baseval,&curval,nextval,&count);  
	       } /* end for */

             desc->attr_value = baseval;
             desc->count = count;
          }
          break;

       default:
          break ;

    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);

} /* mhp_p_dir_name() */

/*
 * mhp_p_rdn()
 */
int mhp_p_rdn(rdn, object)
P1RelativeDistinguishedName *rdn;
OM_int_private_object_header **object;
{
OM_int_private_object_header *subobject;
char *str;
Uint32 len;
Uint16 i, count;
int status;
OM_int_descriptor *desc;
P1AttributeValueAssertion *ava;
OM_type prev_attribute_type = OM_NO_MORE_TYPES;
OM_int_value *baseval, *curval, *nextval;
aaListPointer list;
aaListType *item;


    /* Create object */
/*    if ((status = omp_template(DS_C_DS_RDN, object)) != OM_SUCCESS)
        return(status);
*/
    /* Index through Relative Distinguished Name descriptor list sequentially 
     * and fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);

    if (prev_attribute_type == desc->attr_type) continue;
    prev_attribute_type = desc->attr_type;
    baseval = curval = nextval = NULL;
    count = 0;

    switch (desc->attr_type) {
       case DS_AVAS:
          list = (aaListPointer) rdn;
          list_first(list);

          for (item = list->first;
               item != NULL; 
               item = (aaListType *) list_next(list))   {
              
              ava = (P1AttributeValueAssertion *)item->data ;
              if ((status = mhp_p_ava(ava ,&subobject)) != OM_SUCCESS)
                  return(status);
    
              if ((status = putvalue(desc,&nextval,
                            DS_AVAS,OM_S_OBJECT,subobject))
                            != OM_SUCCESS)
                  return(status);

              append_struct(&baseval,&curval,nextval,&count);
          }
          desc->attr_value = baseval;
          desc->count = count;
          break;

       default:
          break;

    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);

} /* mhp_p_rdn() */

/*
 * mhp_p_ava()
 */
int mhp_p_ava(ava, object)
P1AttributeValueAssertion *ava;
OM_int_private_object_header **object;
{
char *str;
Uint32 len;
Uint16 i, count;
int status;
OM_int_descriptor *desc;
OM_type prev_attribute_type = OM_NO_MORE_TYPES;
OM_int_value *baseval, *curval, *nextval;
OM_string  string;

    /* Create object */
/*    if ((status = omp_template(DS_C_AVA, object)) != OM_SUCCESS)
        return(status);
*/
    /* Index through Attribute Value Assertion descriptor list sequentially 
     * and fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);

    if (prev_attribute_type == desc->attr_type) continue;
    prev_attribute_type = desc->attr_type;
    baseval = curval = nextval = NULL;
    count = 0;

    switch (desc->attr_type) {
       case DS_ATTRIBUTE_TYPE:
          if (ava->m & AttributeValueAssertion_AttributeType) {
             mhp_addobj(&string, &(ava->attributeType));
             if ((status = p_strcpy(desc,&desc->attr_value,
                                 DS_ATTRIBUTE_TYPE,
                                 OM_S_OBJECT_IDENTIFIER_STRING,
                                 (void*) string.elements, string.length)
                                         != OM_SUCCESS))
                 return(status);
          }
          break;

       case DS_ATTRIBUTE_VALUES:
          /* Note: mask is not getting set for required parm, so ignoring it */
          if (len = ava->attributeValue.len) {
              if ((status = p_strcpy(desc,&desc->attr_value,DS_ATTRIBUTE_VALUES,
                OM_S_ENCODING_STRING,ava->attributeValue.ptr,len)) != OM_SUCCESS)
              return(status);
          }
          break;

       default:
          break;

    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* mhp_p_ava() */

/*
 * mhp_p_pres_addr()
 */
int mhp_p_pres_addr(paddr, object)
P1PresentationAddress *paddr;
OM_int_private_object_header **object;
{
char *str;
Uint32 len;
Uint16 i, count;
int status;
OM_int_descriptor *desc;
OM_type prev_attribute_type = OM_NO_MORE_TYPES;
OM_int_value *baseval, *curval, *nextval;

    /* Create object */
/*    if ((status = omp_template(DS_C_PRESENTATION_ADDRESS,
                                             object)) != OM_SUCCESS)
        return(status);
*/
    /* Index through Presentation Address descriptor list sequentially 
     * and fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);

    if (prev_attribute_type == desc->attr_type) continue;
    prev_attribute_type = desc->attr_type;
    baseval = curval = nextval = NULL;
    count = 0;

    switch (desc->attr_type) {
       case DS_N_ADDRESSES:
          if (paddr->m | PresentationAddress_nAddress) {
             int i;
             for (i = 0; i < paddr->nAddress.n; i++)  {
                if (paddr->nAddress.v[i].length) {
                   if ((status = p_strcpy(desc, &nextval,
                          DS_N_ADDRESSES, OM_S_OCTET_STRING,
                          paddr->nAddress.v[i].value,
                          paddr->nAddress.v[i].length))
                             != OM_SUCCESS)
                   return(status);
                }
                append_struct(&baseval,&curval,nextval,&count);
             }
          }
        desc->attr_value = baseval;
        desc->count = count;
        break;

       case DS_P_SELECTOR:
          if (paddr->m | PresentationAddress_pSelector) {
             if (paddr->pSelector.length) {
               if ((status = p_strcpy(desc, &desc->attr_value,
                          DS_P_SELECTOR, OM_S_OCTET_STRING,
                          paddr->pSelector.value,
                          paddr->pSelector.length))
                             != OM_SUCCESS)
                return(status);
             }
          }
          break;

       case DS_S_SELECTOR:
          if (paddr->m | PresentationAddress_sSelector) {
             if (paddr->sSelector.length) {
               if ((status = p_strcpy(desc, &desc->attr_value,
                          DS_S_SELECTOR, OM_S_OCTET_STRING,
                          paddr->sSelector.value,
                          paddr->sSelector.length))
                             != OM_SUCCESS)
                return(status);
             }
          }
          break;

       case DS_T_SELECTOR:
          if (paddr->m | PresentationAddress_tSelector) {
             if (paddr->tSelector.length) {
               if ((status = p_strcpy(desc, &desc->attr_value,
                          DS_T_SELECTOR, OM_S_OCTET_STRING,
                          paddr->tSelector.value,
                          paddr->tSelector.length))
                             != OM_SUCCESS)
                return(status);
             }
          }
          break;

       default:
          break;

    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);

} /* end mhp_p_pres_addr() */



/*
 * mhp_p_orname(orname, object) creates and fills in ORName object with
 * values from the given P1ORName structure. It returns the object handle
 * to the calling routine for insertion by the calling routine.
 *
 * INPUT:
 *  orname - address of the P1ORName structure
 *  object - address of variable to return object handle
 *
 * OUTPUT:
 *  object - now contains the handle to an OR Adress (or OR Name) if success.
 *
 * NOTE:
 *  For 1984, only the OR Address object is being returned.
 *  For 1988, an OR Name object might be returned.
 *
 */

int mhp_p_orname(orname, object)
P1ORName *orname;
OM_int_private_object_header **object;
{
OM_int_private_object_header *subobject;
P1StandardAttributes *std;
P1DomainDefinedAttributes *dda;
P1TeletexDomainDefinedAttributes *ttxDDA = NULL;
P1TeletexPersonalName *ttxPN = NULL;
P1TeletexOrganizationalUnitNames *ttxOU = NULL;
P1ExtendedNetworkAddress *pena;
P1TerminalType ttype;
P1TerminalType *pttype;
char *str, *pvalue;
P1PDSParameter *ppdsparm;
int status;
int *evalue;
Uint16 i, count;
Uint32 len;
OM_int_descriptor *desc;
OM_type prev_attribute_type = OM_NO_MORE_TYPES;
OM_int_value *baseval, *curval, *nextval;

/* Create OR Address object. If this support 1988, need to
 * check whether X.500 Distinguished Name is present. If so
 * OR Name object should be created, instead of OR Address.
 * On second thought, since the class tables always specify 
 * Object(OR_NAME), it's safer to return OR_NAME.
 */
    if ((status = omp_template(MH_C_OR_NAME,object)) != OM_SUCCESS)
    return(status);

    std = (P1StandardAttributes *) orname->standard_attributes; 
    dda = (P1DomainDefinedAttributes *) &orname->domain_defined_attributes;

/*
 * Similar to formatting ORName, there are group of attributes that share 
 * a single extension and data structure: ttxDDA, ttxOU, PN. For ease of
 * algorithm, we'll get these extension and parse them first if present.
 */

    if (pvalue = (char *)get_add_exts(orname,
                  EXTENSION_ATTRIBUTE_TYPE_teletex_personal_name))
        ttxPN = (P1TeletexPersonalName *)pvalue;

    if (pvalue = (char *)get_add_exts(orname,
                  EXTENSION_ATTRIBUTE_TYPE_teletex_organizational_unit_names))
        ttxOU = (P1TeletexOrganizationalUnitNames *)pvalue;

    if (pvalue = (char *)get_add_exts(orname,
                  EXTENSION_ATTRIBUTE_TYPE_teletex_domain_defined_attributes))
        ttxDDA = (P1TeletexDomainDefinedAttributes *)pvalue;


    /* Index through OR Name descriptor list sequentially and fill in
     * attribute if it exists. */
    for (i = TOP; 
        (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
        i++) 
    {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
/*
 * For attribute with multiple syntaxes, there are duplicated descriptors
 * in the template, but ONLY THE FIRST one contains the values. Hence we
 * need to skip the duplicated descriptors. 
 *
 * For every attribute, we need to look at the corresponding class
 * in the package for multiple syntaxes/values: if so, then go get all
 * the present values: eg, ORGNAME might be present as a standard attribute
 * as Printable String, as well as as an extension (Teletex String). This
 * applies to most attributes, in one way or another, except for a few
 * (Country, ADMD, PRMD, etc).
 */
    if (prev_attribute_type == desc->attr_type) continue;
    prev_attribute_type = desc->attr_type;
    baseval = curval = nextval = NULL;
    count = 0;

    switch (desc->attr_type) {
    case MH_T_COUNTRY_NAME:
        if (std->m & StandardAttributes_country_name) {
        if ((status = mhp_p_country(&str,&len,&std->country_name)) != OM_SUCCESS)
            return(status);
        if ((status = putvalue(desc,&desc->attr_value,MH_T_COUNTRY_NAME,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
   case MH_T_ADMD_NAME:
        if (std->m & StandardAttributes_administration_domain_name) {
        if ((status = mhp_p_admdcpy(&str,&len,&std->administration_domain_name)) != OM_SUCCESS)
            return(status);
        if ((status = putvalue(desc,&desc->attr_value,MH_T_ADMD_NAME,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_GENERATION:
        if (std->personal_name.m & PersonalName_generation_qualifier) {
        if (len = strlen(std->personal_name.generation_qualifier)) {
            if ((status = p_strcpy(desc, &nextval,
                          MH_T_GENERATION, OM_S_PRINTABLE_STRING,
                          std->personal_name.generation_qualifier,len)) 
                          != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxPN && (len = strlen(ttxPN->generation_qualifier))) {
            if ((status = p_strcpy(desc, &nextval,
                          MH_T_GENERATION, OM_S_TELETEX_STRING,
                          ttxPN->generation_qualifier,len)) 
                          != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;

        break;

    case MH_T_GIVEN_NAME:
        if (std->personal_name.m & PersonalName_given_name) {
        if (len = strlen(std->personal_name.given_name)) {
            if ((status = p_strcpy(desc,&nextval,
                        MH_T_GIVEN_NAME,OM_S_PRINTABLE_STRING,
                        std->personal_name.given_name,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxPN && (len = strlen(ttxPN->given_name))) {
            if ((status = p_strcpy(desc, &nextval,
                          MH_T_GIVEN_NAME, OM_S_TELETEX_STRING,
                          ttxPN->generation_qualifier,len)) 
                          != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;


        break;
    case MH_T_INITIALS:
        if (std->personal_name.m & PersonalName_initials) {
        if (len = strlen(std->personal_name.initials)) {
            if ((status = p_strcpy(desc,&nextval,
                    MH_T_INITIALS,OM_S_PRINTABLE_STRING,
                    std->personal_name.initials,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxPN && (len = strlen(ttxPN->initials))) {
            if ((status = p_strcpy(desc, &nextval,
                          MH_T_INITIALS, OM_S_TELETEX_STRING,
                          ttxPN->initials,len)) 
                          != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_X121_ADDRESS:
        if (std->m & StandardAttributes_network_address) {
        if (len = strlen(std->network_address)) {
            if ((status = p_strcpy(desc,&desc->attr_value, MH_T_X121_ADDRESS,
                          OM_S_NUMERIC_STRING,
                          std->network_address,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    case MH_T_NUMERIC_USER_IDENTIFIER:
        if (std->m & StandardAttributes_numeric_user_identifier) {
        if (len = strlen(std->numeric_user_identifier)) {
            if ((status = p_strcpy(desc,&desc->attr_value,
                       MH_T_NUMERIC_USER_IDENTIFIER, OM_S_NUMERIC_STRING,
                       std->numeric_user_identifier,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_ORGANIZATION_NAME:
        if (std->m & StandardAttributes_organization_name) {
        if (len = strlen(std->organization_name)) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATION_NAME,OM_S_PRINTABLE_STRING,
                     std->organization_name,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (pvalue = (char *)get_add_exts(orname,
                      EXTENSION_ATTRIBUTE_TYPE_teletex_organization_name)) {
            if (len = strlen(pvalue)) {
                if ((status = p_strcpy(desc, &nextval,
                         MH_T_ORGANIZATION_NAME, OM_S_TELETEX_STRING,
                         pvalue,len)) != OM_SUCCESS)
                    return(status);
                append_struct(&baseval,&curval,nextval,&count);
            }
        }

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_ORGANIZATIONAL_UNIT_NAME_1:

        if (std->m & StandardAttributes_organizational_unit_names) {
        if (len = strlen(std->organizational_unit_names.v[0])) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_1,OM_S_PRINTABLE_STRING,
                     std->organizational_unit_names.v[0],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxOU && (len = strlen(ttxOU->v[0]))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_1,OM_S_TELETEX_STRING,
                     ttxOU->v[0],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_ORGANIZATIONAL_UNIT_NAME_2:

        if (std->m & StandardAttributes_organizational_unit_names) {
        if (len = strlen(std->organizational_unit_names.v[1])) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_2, OM_S_PRINTABLE_STRING,
                     std->organizational_unit_names.v[1],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxOU && (len = strlen(ttxOU->v[1]))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_2,OM_S_TELETEX_STRING,
                     ttxOU->v[1],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_ORGANIZATIONAL_UNIT_NAME_3:
        if (std->m & StandardAttributes_organizational_unit_names) {
        if (len = strlen(std->organizational_unit_names.v[2])) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_3,OM_S_PRINTABLE_STRING,
                     std->organizational_unit_names.v[2],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxOU && (len = strlen(ttxOU->v[2]))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_3,OM_S_TELETEX_STRING,
                     ttxOU->v[2],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_ORGANIZATIONAL_UNIT_NAME_4:
        if (std->m & StandardAttributes_organizational_unit_names) {
        if (len = strlen(std->organizational_unit_names.v[3])) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_4,OM_S_PRINTABLE_STRING,
                     std->organizational_unit_names.v[3],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxOU && (len = strlen(ttxOU->v[3]))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_ORGANIZATIONAL_UNIT_NAME_4,OM_S_TELETEX_STRING,
                     ttxOU->v[3],len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_PRMD_NAME:
        if (std->m & StandardAttributes_private_domain_name) {
        if ((status = mhp_p_prmdcpy(&str,&len,&std->private_domain_name)) 
                      != OM_SUCCESS)
            return(status);
        if ((status = putvalue(desc,&desc->attr_value,
                 MH_T_PRMD_NAME,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_SURNAME:
        if ((std->m & StandardAttributes_personal_name) 
             && (std->personal_name.m & PersonalName_surname)) {
        if (len = strlen(std->personal_name.surname)) {
            if ((status = p_strcpy(desc, &nextval,
                       MH_T_SURNAME,OM_S_PRINTABLE_STRING,
                       std->personal_name.surname,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxPN && (len = strlen(ttxPN->surname))) {
            if ((status = p_strcpy(desc, &nextval,
                          MH_T_GENERATION, OM_S_TELETEX_STRING,
                          ttxPN->surname,len)) 
                          != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;

        break;
    case MH_T_TERMINAL_IDENTIFIER:
        if (std->m & StandardAttributes_terminal_identifier) {
        if (len = strlen(std->terminal_identifier)) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_TERMINAL_IDENTIFIER,
            OM_S_PRINTABLE_STRING,std->terminal_identifier,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_DOMAIN_TYPE_1:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[0].m & DomainDefinedAttribute_ddatype) 
             && (len = strlen(dda->v[0].ddatype))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_1,OM_S_PRINTABLE_STRING,
                     dda->v[0].ddatype,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[0].m & TeletexDomainDefinedAttribute_ttxddatype)
            && (len = strlen(ttxDDA->v[0].ttxddatype ))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_1, OM_S_TELETEX_STRING,
                     ttxDDA->v[0].ttxddatype, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_TYPE_2:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[1].m & DomainDefinedAttribute_ddatype) 
            && (len = strlen(dda->v[1].ddatype))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_2,OM_S_PRINTABLE_STRING,
                     dda->v[1].ddatype,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[1].m & TeletexDomainDefinedAttribute_ttxddatype)
            && (len = strlen(ttxDDA->v[1].ttxddatype ))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_2, OM_S_TELETEX_STRING,
                     ttxDDA->v[1].ttxddatype, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_TYPE_3:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[2].m & DomainDefinedAttribute_ddatype) 
             && (len = strlen(dda->v[2].ddatype))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_3,OM_S_PRINTABLE_STRING,
                     dda->v[2].ddatype,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[2].m & TeletexDomainDefinedAttribute_ttxddatype)
            && (len = strlen(ttxDDA->v[2].ttxddatype ))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_3, OM_S_TELETEX_STRING,
                     ttxDDA->v[2].ttxddatype, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_TYPE_4:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[3].m & DomainDefinedAttribute_ddatype) 
             && (len = strlen(dda->v[3].ddatype))) {
            if ((status = p_strcpy(desc, &nextval,
                     MH_T_DOMAIN_TYPE_4,OM_S_PRINTABLE_STRING,
                     dda->v[3].ddatype,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[3].m & TeletexDomainDefinedAttribute_ttxddatype)
            && (len = strlen(ttxDDA->v[3].ttxddatype ))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_TYPE_4, OM_S_TELETEX_STRING,
                     ttxDDA->v[3].ttxddatype, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_VALUE_1:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[0].m & DomainDefinedAttribute_ddavalue) 
            && (len = strlen(dda->v[0].ddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_1,OM_S_PRINTABLE_STRING,
                     dda->v[0].ddavalue,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[0].m & TeletexDomainDefinedAttribute_ttxddavalue)
            && (len = strlen(ttxDDA->v[0].ttxddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_1, OM_S_TELETEX_STRING,
                     ttxDDA->v[0].ttxddavalue, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_VALUE_2:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[1].m & DomainDefinedAttribute_ddavalue) 
             && (len = strlen(dda->v[1].ddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_2,OM_S_PRINTABLE_STRING,
                     dda->v[1].ddavalue,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[1].m & TeletexDomainDefinedAttribute_ttxddavalue)
            && (len = strlen(ttxDDA->v[1].ttxddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_2, OM_S_TELETEX_STRING,
                     ttxDDA->v[1].ttxddavalue, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_VALUE_3:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[2].m & DomainDefinedAttribute_ddavalue) 
            && (len = strlen(dda->v[2].ddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_3,OM_S_PRINTABLE_STRING,
                     dda->v[2].ddavalue,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[2].m & TeletexDomainDefinedAttribute_ttxddavalue)
            && (len = strlen(ttxDDA->v[2].ttxddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_3, OM_S_TELETEX_STRING,
                     ttxDDA->v[2].ttxddavalue, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;

    case MH_T_DOMAIN_VALUE_4:

        if (orname->m & ORName_domain_defined_attributes) {
        if ((dda->v[3].m & DomainDefinedAttribute_ddavalue) 
             && (len = strlen(dda->v[3].ddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_4,OM_S_PRINTABLE_STRING,
                     dda->v[3].ddavalue,len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        if (ttxDDA 
            && (ttxDDA->v[3].m & TeletexDomainDefinedAttribute_ttxddavalue)
            && (len = strlen(ttxDDA->v[3].ttxddavalue))) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_DOMAIN_VALUE_4, OM_S_TELETEX_STRING,
                     ttxDDA->v[3].ttxddavalue, len)) != OM_SUCCESS)
                return(status);
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        break;


    case MH_T_COMMON_NAME:
        /* common name takes two extensions, one for Printable and one
         * for Teletex String.
         */

        /* Get Printable Common Name extension, if it exists */
        if (pvalue = (char *)get_add_exts(orname,
                                 EXTENSION_ATTRIBUTE_TYPE_common_name)) {

        /* since P1CommonName is a char[], pvalue is pointer to char */
        if (len = strlen(pvalue)) {
            if ((status = p_strcpy(desc,&nextval,MH_T_COMMON_NAME,
                  OM_S_PRINTABLE_STRING,(void*)pvalue,len)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        /* Get Teletex Common Name extension, if it exists */
        if (pvalue = (char *)get_add_exts(orname,
                               EXTENSION_ATTRIBUTE_TYPE_teletex_common_name)) 
        {
    
        /* since P1TeletexCommonName is char[], pvalue is pointer to char */
        if (len = strlen(pvalue)) {
            if ((status = p_strcpy(desc,&nextval,MH_T_COMMON_NAME,
                         OM_S_TELETEX_STRING,(void*)pvalue,len)) != OM_SUCCESS)
            return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_ADDRESS_DETAILS:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_extension_physical_delivery_address_components)) {

           /* Get Printable Postal Address Details extension, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_ADDRESS_DETAILS,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Postal Address Details extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_ADDRESS_DETAILS,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_ADDRESS_IN_FULL:
/*
 * See comments in MH_T_POSTAL_ADDRESS_IN_LINES. This IN_FULL attribute
 * share the same data structure with IN_LINES attribute.
 *
 * NOTE: because two attributes share the same data structure, should
 * clear extensions ONLY AT THE END OF THE FUNCTION OR AFTER. KDH.
 */
        if (pvalue = (char *)get_add_exts(orname,
                  EXTENSION_ATTRIBUTE_TYPE_unformatted_postal_address)) {
        if (((P1UnformattedPostalAddress *)pvalue)->m 
              & UnformattedPostalAddress_teletex_string) {
            if (len = strlen(((P1UnformattedPostalAddress *)pvalue)->
                               teletex_string)) {
              if ((status = p_strcpy(desc,&nextval,
                       MH_T_POSTAL_ADDRESS_IN_FULL,OM_S_TELETEX_STRING,
                       (void*) ((P1UnformattedPostalAddress *)pvalue)->
                                 teletex_string,len)) != OM_SUCCESS)
                  return(status);

              append_struct(&baseval,&curval,nextval,&count);
            } /* end if */
        } /* end if teletex string */
        } /* end if (pvalue) */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;

    case MH_T_POSTAL_CODE:
        if (pvalue = (char *)get_add_exts(orname,
                       EXTENSION_ATTRIBUTE_TYPE_postal_code)) {
        if (((P1PostalCode *)pvalue)->t == PostalCode_numeric_code) {
            if (len = strlen((char *)((P1PostalCode *)pvalue)->v.
                                       numeric_code)) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_POSTAL_CODE,OM_S_NUMERIC_STRING, 
                     (void*) ((P1PostalCode *)pvalue)->v.numeric_code,len)) 
                     != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
            } /* end if */
        } /* end if numeric code */
        else if (((P1PostalCode *)pvalue)->t == PostalCode_printable_code) {
            if (len = strlen((char *)((P1PostalCode *)pvalue)->v.
                                        printable_code)) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_POSTAL_CODE,OM_S_PRINTABLE_STRING, 
                     (void*) ((P1PostalCode *)pvalue)->v.printable_code,len))
                      != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
            } /* end if */
        } /* end if printable */
        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;

    case MH_T_POSTAL_ADDRESS_IN_LINES:
/* 
 * in asn1 p1p2.h, the P1UnformattedPostalAddress consists of
 * an array of printable strings (corresponding to a sequence of 
 * printablestring in X.402/X.411) and a single teletex string.
 * The printable string corresponding to POSTAL_ADDRESS_IN_LINES attribute.
 * The teletex string corresponds to POSTAL_ADDRESS_IN_FULL attribute.
 * Each line of printables is mapped to an attribute value.
 *
 * NOTE: because two attributes share the same data structure, should
 * clear extensions ONLY AT THE END OF THE FUNCTION OR AFTER. KDH.
 */
        if (pvalue = (char *)get_add_exts(orname,
                  EXTENSION_ATTRIBUTE_TYPE_unformatted_postal_address)) {
        if (((P1UnformattedPostalAddress *)pvalue)->m 
              & UnformattedPostalAddress_printable_address) 
        {
            int i = 0;    /* local index */
            for (i=0; 
                 i <((P1UnformattedPostalAddress *)pvalue)->printable_address.n;
                 i++) {
              if (len = strlen((char *)((P1UnformattedPostalAddress *)pvalue)->
                                printable_address.v[i])) {
                if ((status = p_strcpy(desc, &nextval,
                       MH_T_POSTAL_ADDRESS_IN_LINES, OM_S_PRINTABLE_STRING,
                       (void*) &(((P1UnformattedPostalAddress *)pvalue)->
                                printable_address.v[i][0]),len)) != OM_SUCCESS)
                  return(status);
                append_struct(&baseval,&curval,nextval,&count);
              } /* len > 0 */
            } /* for */
        } /* end if printable string */
        } /* end if extension */

        desc->attr_value = baseval;
        /* desc->count = count; */
	break;

    case MH_T_POSTAL_COUNTRY_NAME:
        if (pvalue = (char *)get_add_exts(orname,
                 EXTENSION_ATTRIBUTE_TYPE_physical_delivery_country_name)) {
        if (((P1PhysicalDeliveryCountryName *)pvalue)->t 
              == PhysicalDeliveryCountryName_x121_dcc_code) {
          if (len = strlen((char *)(
                (P1PhysicalDeliveryCountryName *)pvalue)->v.x121_dcc_code)) {
            if ((status = p_strcpy(desc,&nextval,
                    MH_T_POSTAL_COUNTRY_NAME,OM_S_NUMERIC_STRING, 
                    (void*) ((P1PhysicalDeliveryCountryName *)pvalue)->v.
                              x121_dcc_code,len)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
            } /* end if len */
        } /* end if dcc code */
        else if (((P1PhysicalDeliveryCountryName *)pvalue)->t 
               == PhysicalDeliveryCountryName_iso_3166_alpha2_code) {
          if (len = strlen((char *)((P1PhysicalDeliveryCountryName *)pvalue)->
                                     v.iso_3166_alpha2_code)) {
            if ((status = p_strcpy(desc,&nextval,
                     MH_T_POSTAL_COUNTRY_NAME,OM_S_PRINTABLE_STRING, 
                     (void*) ((P1PhysicalDeliveryCountryName *)pvalue)->v.
                              iso_3166_alpha2_code,len)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
            } /* end if len */
        } /* end if alpha2 code */
        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */
        break;

    case MH_T_POSTAL_LOCALE:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_unique_postal_name)) {

           /* Get Printable Unique-postal-name extension, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_LOCALE,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Unique-postal-name extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_LOCALE,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_OFFICE_BOX_NUMBER:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_post_office_box_address)) {

           /* Get Printable Post-office-box-address extension, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_OFFICE_BOX_NUMBER,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Post-office-box-address extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_OFFICE_BOX_NUMBER,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;

    case MH_T_POSTAL_OFFICE_NAME:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_physical_delivery_office_name)) {

           /* Get Printable Physical-delivery-office-name, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_OFFICE_NAME,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Physical-delivery-office-name extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_OFFICE_NAME,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;

    case MH_T_POSTAL_OFFICE_NUMBER:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_physical_delivery_office_number)) {

           /* Get Printable Physical-delivery-office-number, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_OFFICE_NUMBER,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Physical-delivery-office-number extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_OFFICE_NUMBER,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;

    case MH_T_POSTAL_ORGANIZATION_NAME:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_physical_delivery_organization_name)) {

           /* Get Printable Physical-delivery-organization-name, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_ORGANIZATION_NAME,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Physical-delivery-organization-name extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_ORGANIZATION_NAME,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;

    case MH_T_POSTAL_STREET_ADDRESS:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_street_address)) {

           /* Get Printable Street-address, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_STREET_ADDRESS,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Street-address extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_STREET_ADDRESS,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_PATRON_NAME:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_physical_delivery_personal_name)) {

           /* Get Printable Physical-delivery-personal-name, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_PATRON_NAME,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Physical-delivery-personal-name, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_PATRON_NAME,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_PATRON_DETAILS:

        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_extension_OR_address_components)) {

           /* Get Printable Extension-postal-O/R-address-components, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_PATRON_DETAILS,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Extension-postal-O/R-address-components, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_PATRON_DETAILS,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_DELIVERY_POINT_NAME:
        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_local_postal_attributes)) {

           /* Get Printable Postal Delivery Point Name extension, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_DELIVERY_POINT_NAME,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Postal Delivery Point Name extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_DELIVERY_POINT_NAME,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_DELIV_SYSTEM_NAME:
        if (pvalue = (char *)get_add_exts(orname,EXTENSION_ATTRIBUTE_TYPE_pds_name)) {
        if (len = strlen(pvalue)) {
            if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_DELIV_SYSTEM_NAME,
                  OM_S_PRINTABLE_STRING,(void *)pvalue,len)) != OM_SUCCESS)
               return(status);
      
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */
        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_POSTAL_GENERAL_DELIV_ADDR:
        if (ppdsparm = (P1PDSParameter *)get_add_exts(orname,
                EXTENSION_ATTRIBUTE_TYPE_poste_restante_address)) {

           /* Get Printable Postal General Delivery Address extension, if it exists */
           if (ppdsparm->m & PDSParameter_printable_string) {
              if (len = strlen(ppdsparm->printable_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_GENERAL_DELIV_ADDR,
                        OM_S_PRINTABLE_STRING,(void*)ppdsparm->printable_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

           /* Get Teletex Postal General Delivery Address extension, if it exists */
           if (ppdsparm->m & PDSParameter_teletex_string) {
              if (len = strlen(ppdsparm->teletex_string)) {
                  if ((status = p_strcpy(desc,&nextval,MH_T_POSTAL_GENERAL_DELIV_ADDR,
                        OM_S_TELETEX_STRING,(void*)ppdsparm->teletex_string,len)) != OM_SUCCESS)
                      return(status);
      
                  append_struct(&baseval,&curval,nextval,&count);
              } /* end if */
           } /* end if */

        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_TERMINAL_TYPE:
        if (pttype = (P1TerminalType *)get_add_exts(orname,
                      EXTENSION_ATTRIBUTE_TYPE_terminal_type)) {
            ttype = *pttype;
            if ((status = putvalue(desc,&nextval,
                  MH_T_TERMINAL_TYPE,OM_S_ENUMERATION,
                  (void *)ttype))
                 != OM_SUCCESS)
               return(status);
      
            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        /* desc->count = count; */ /*p_strcpy() increments count via putvalue*/
        break;


    case MH_T_PRESENTATION_ADDRESS:
        if (pena = (P1ExtendedNetworkAddress *)get_add_exts(orname,
                     EXTENSION_ATTRIBUTE_TYPE_extended_network_address)) {
           if (pena->t == ExtendedNetworkAddress_psap_address) {
           /* Create and fill in Presentation Address Subobject*/
	       if ((status = mhp_p_pres_addr(&pena->v.psap_address,
						 &subobject)) != OM_SUCCESS)
		    return(status);

           /* Create a value structure for the subobject and link it */
               if ((status = putvalue(desc,&desc->attr_value,
                           MH_T_PRESENTATION_ADDRESS,OM_S_OBJECT,subobject)) 
                                  != OM_SUCCESS)
              return(status);
           } /* end if */
        }
        break;


    case MH_T_ISDN_NUMBER:
        if (pena = (P1ExtendedNetworkAddress *)get_add_exts(orname,
                     EXTENSION_ATTRIBUTE_TYPE_extended_network_address)) {
           if (pena->t == ExtendedNetworkAddress_e163_4_address) {
              if (pena->v.e163_4_address.m & ExtendedNetworkAddress_basenumber) {
                 if (len = strlen(pena->v.e163_4_address.basenumber)) {
                    if ((status = p_strcpy(desc,&nextval,MH_T_ISDN_NUMBER,
                         OM_S_NUMERIC_STRING, (void*)pena->v.e163_4_address.basenumber,len))
                           != OM_SUCCESS)
                        return(status);

                    append_struct(&baseval,&curval,nextval,&count);
                 } /* end if len */
              }  /* end if mask */
           }   /* end if tag */
	}   /* end if orname */

        desc->attr_value = baseval;
        /* desc->count = count; */
        break;


    case MH_T_ISDN_SUBADDRESS:

        if (pena = (P1ExtendedNetworkAddress *)get_add_exts(orname,
                     EXTENSION_ATTRIBUTE_TYPE_extended_network_address)) {
           if (pena->t == ExtendedNetworkAddress_e163_4_address) {
              if (pena->v.e163_4_address.m & ExtendedNetworkAddress_sub_address) {
                 if (len = strlen(pena->v.e163_4_address.sub_address)) {
                    if ((status = p_strcpy(desc,&nextval,MH_T_ISDN_SUBADDRESS,
                         OM_S_NUMERIC_STRING, (void*)pena->v.e163_4_address.sub_address,len))
                           != OM_SUCCESS)
                        return(status);

                    append_struct(&baseval,&curval,nextval,&count);
                 } /* end if len */
              } /* end if mask */
           }  /* end if tag */
        }   /* end if orname */

        desc->attr_value = baseval;
        /* desc->count = count; */
        break;

    case MH_T_DIRECTORY_NAME:
	if (orname->m & ORName_directory_name)
	{
        if ((status = mhp_p_dir_name(orname->directory_name,&subobject)) 
                                                    != OM_SUCCESS)
            return(status);
        if ((status = putvalue(desc,&desc->attr_value,
                        MH_T_DIRECTORY_NAME,OM_S_OBJECT,subobject))
                        != OM_SUCCESS)
            return(status);
	} /* end if */
        break;


    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_orname() */


/*
 * mhp_p_msg_rd() put P1PerRecipientMessageTransferFields structure
 * into MH_C_RECIPIENT_DESCRIPTOR object, and return the object handle.
 *
 * INPUT:
 *  rd  - address of P1PerRecipientMessageTransferFields.
 *
 * OUTPUT:
 *  object  - handle to the returned Recipient Descriptor object.
 */

int mhp_p_msg_rd(rd, object)
P1PerRecipientMessageTransferFields *rd;
OM_int_private_object_header **object;
{
Uint32 len;
int i, status;
Uint16 count;
OM_enumeration	enumer;
OM_boolean flag;
int *evalue;
char *pvalue;
OM_int_descriptor *desc;
OM_int_private_object_header *subobject;
OM_int_value *baseval, *curval, *nextval;
P1ExtensionMacroBody *extension;
OM_string string;
int  tmp;

    if ((status = omp_template(MH_C_MESSAGE_RD,object)) != OM_SUCCESS)
    return(status);

    /* Index through Recipient descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; 
		(*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
		i++) 
	{
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {

    case MH_T_MTA_REPORT_REQUEST:
        if (rd->m & PerRecipientMessageTransferFields_per_recipient_indicators)
		{
        enumer = MH_RQ_NON_DELIVERY;
        if (rd->per_recipient_indicators & 
            	PER_RECIPIENT_INDICATORS_originating_MTA_report) {
            if (rd->per_recipient_indicators & 
                PER_RECIPIENT_INDICATORS_originating_MTA_non_delivery_report) {
                enumer = MH_RQ_ALWAYS_AUDITED;
            }
            else {
                enumer = MH_RQ_ALWAYS;
            }
        }

        if ((status = putvalue(desc,&desc->attr_value,MH_T_MTA_REPORT_REQUEST,
					OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_MTA_RESPONSIBILITY:
        if (rd->m & PerRecipientMessageTransferFields_per_recipient_indicators) {
        if (rd->per_recipient_indicators & PER_RECIPIENT_INDICATORS_responsibility)
            flag = TRUE;
        else
            flag = FALSE;

        if ((status = putvalue(desc,&desc->attr_value,MH_T_MTA_RESPONSIBILITY,
					OM_S_BOOLEAN,(void*)flag)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_RECIPIENT_NUMBER:
        if (rd->m & PerRecipientMessageTransferFields_originally_specified_recipient_number) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_RECIPIENT_NUMBER,
					OM_S_INTEGER, 
					(void*)rd->originally_specified_recipient_number)) 
					!= OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_RECIPIENT_NAME:
        if (rd->m & PerRecipientMessageTransferFields_recipient_name) {
        /* Create and fill in OR Name sub-object */
        if ((status = mhp_p_orname(rd->recipient_name,&subobject))!=OM_SUCCESS)
            return(status);

        if ((status = putvalue(desc,&desc->attr_value,MH_T_RECIPIENT_NAME,
					OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_EXPLICIT_CONVERSION:
        if (rd->m & PerRecipientMessageTransferFields_explicit_conversion)

           switch (rd->explicit_conversion) {
           case EXPLICIT_CONVERSION_ia5_text_to_teletex:
              enumer = MH_EC_IA5_TEXT_TO_TELETEX;
              break;
           
           case EXPLICIT_CONVERSION_teletex_to_telex:
              enumer = MH_EC_TELETEX_TO_TELEX;
              break;
           
           case EXPLICIT_CONVERSION_telex_to_ia5_text:
              enumer = MH_EC_TELEX_TO_IA5_TEXT;
              break;

           case EXPLICIT_CONVERSION_telex_to_teletex:
              enumer = MH_EC_TELEX_TO_TELETEX;
              break;

           case EXPLICIT_CONVERSION_telex_to_g4_class_1:
              enumer = MH_EC_TELEX_TO_G4_CLASS1;
              break;

           case EXPLICIT_CONVERSION_telex_to_videotex:
              enumer = MH_EC_TELEX_TO_VIDEOTEX;
              break;

           case EXPLICIT_CONVERSION_ia5_text_to_telex:
              enumer = MH_EC_IA5_TEXT_TO_TELEX;
              break;
           
           case EXPLICIT_CONVERSION_telex_to_g3_facsimile:
              enumer = MH_EC_TELEX_TO_G3_FAX;
              break;
           
           case EXPLICIT_CONVERSION_ia5_text_to_g3_facsimile:
              enumer = MH_EC_IA5_TEXT_TO_G3_FAX;
              break;
           
           case EXPLICIT_CONVERSION_ia5_text_to_g4_class_1:
              enumer = MH_EC_IA5_TEXT_TO_G4_CLASS1;
              break;
           
           case EXPLICIT_CONVERSION_ia5_text_to_videotex:
              enumer = MH_EC_IA5_TEXT_TO_VIDEOTEX;
              break;
           
           case EXPLICIT_CONVERSION_teletex_to_ia5_text:
              enumer = MH_EC_TELETEX_TO_IA5_TEXT;
              break;
           
           case EXPLICIT_CONVERSION_teletex_to_g3_facsimile:
              enumer = MH_EC_TELETEX_TO_G3_FAX;
              break;
           
           case EXPLICIT_CONVERSION_teletex_to_g4_class_1:
              enumer = MH_EC_TELETEX_TO_G4_CLASS1;
              break;
           
           case EXPLICIT_CONVERSION_teletex_to_videotex:
              enumer = MH_EC_TELETEX_TO_VIDEOTEX;
              break;
           
           case EXPLICIT_CONVERSION_videotex_to_telex:
              enumer = MH_EC_VIDEOTEX_TO_TELEX;
              break;
           
           case EXPLICIT_CONVERSION_videotex_to_ia5_text:
              enumer = MH_EC_VIDEOTEX_TO_IA5_TEXT;
              break;
           
           case EXPLICIT_CONVERSION_videotex_to_teletex:
              enumer = MH_EC_VIDEOTEX_TO_TELETEX;
              break;
           default:
              enumer = MH_EC_NO_CONVERSION;
           }

        else
           enumer = MH_EC_NO_CONVERSION;

        if ((status = putvalue(desc,&desc->attr_value,MH_T_EXPLICIT_CONVERSION,
				OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
        	return(status);
        break;

    case MH_T_ALTERNATE_RECIPIENT_NAME:
        if (pvalue = (char *)get_per_recip_msg_extension(rd,
					ETYPE_originator_requested_alternate_recipient)) {

        /* pvalue is a pointer to an int, which in turn is a pointer
         * to P1ORName.  mhp_p_orname() will assume that the content of 
         * pvalue is a pointer to P1ORName. Create and fill in OR Name 
         * sub-object 
         */
            if ((status = mhp_p_orname( 
                       (*(P1OriginatorRequestedAlternateRecipient *)pvalue),
                       &subobject)) 
                                 != OM_SUCCESS)
                return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&desc->attr_value,
					MH_T_ALTERNATE_RECIPIENT_NAME,OM_S_OBJECT,subobject)) 
					!= OM_SUCCESS)
            	return(status);
        } /* end if */
        break;

    case MH_T_RENDITION_ATTRIBUTES:
        if (pvalue = (char *)get_per_recip_msg_extension(rd,ETYPE_physical_rendition_attributes)) {
        if (len = ((P1PhysicalRenditionAttributes *)pvalue)->length) {
	    mhp_addobj(&string, ((P1PhysicalRenditionAttributes *)pvalue));
            if ((status = p_strcpy(desc,&desc->attr_value,
				MH_T_RENDITION_ATTRIBUTES,
            			OM_S_OBJECT_IDENTIFIER_STRING,
				(void*) string.elements, string.length)
					!= OM_SUCCESS))
            	return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_FORWARDING_ADDR_REQUESTED:
        flag = OM_FALSE;   /* default inital value: required at interface*/
        if (pvalue = (char *)get_per_recip_msg_extension(rd,
                      ETYPE_physical_forwarding_address_request)) {
            if (*(P1PhysicalForwardingAddressRequest *) pvalue)
                flag = OM_TRUE;
        } /* end if */

        if ((status = putvalue(desc,&desc->attr_value,
                MH_T_FORWARDING_ADDR_REQUESTED,OM_S_BOOLEAN,(void*)flag)) 
                != OM_SUCCESS)
            return(status);
        break;

    case MH_T_FORWARDING_PROHIBITED:
        flag = OM_FALSE;
        if (pvalue = (char *)get_per_recip_msg_extension(rd,
                              ETYPE_physical_forwarding_prohibited)) {
            if (* (P1PhysicalForwardingProhibited *) pvalue)
                flag = OM_TRUE;
        } /* end if */

        if ((status = putvalue(desc,&desc->attr_value,
                   MH_T_FORWARDING_PROHIBITED,OM_S_BOOLEAN,(void*)flag)) 
                   != OM_SUCCESS)
            return(status);
        break;

    case MH_T_PROOF_OF_DELIV_REQUESTED:
        flag = OM_FALSE;   /* must set initial attr value at interface */
        if (pvalue = (char *)get_per_recip_msg_extension(rd,
                                    ETYPE_proof_of_delivery_request)) {
            if (* (P1ProofOfDeliveryRequest*)pvalue)
                flag = OM_TRUE;
        } /* end if */

        if ((status = putvalue(desc,&desc->attr_value,
                  MH_T_PROOF_OF_DELIV_REQUESTED,
                  OM_S_BOOLEAN,(void*)flag)) != OM_SUCCESS)
            return(status);
        break;

    case MH_T_BUREAU_FAX_DELIVERY:

        /* only extract the 7-th bit. Other bits in P1PhysicalDeliveryMode
         * go to Postal Mode.
         */
   
        flag = OM_FALSE;   /* must have default initial value at interface */
        if (pvalue = (char *)get_per_recip_msg_extension(rd,
                                ETYPE_physical_delivery_modes)) {
        if ( *(P1PhysicalDeliveryModes *)pvalue 
                & PHYSICALDELIVERYMODES_bureau_fax_delivery)
            flag = OM_TRUE;
        } /* end if */

        if ((status = putvalue(desc,&desc->attr_value,
                    MH_T_BUREAU_FAX_DELIVERY,
                    OM_S_BOOLEAN,(void*)flag)) != OM_SUCCESS)
            return(status);
        break;

    case MH_T_POSTAL_MODE:
        /* no need to generate default values as it's not required */
        if (pvalue = (char *)get_per_recip_msg_extension(rd,
                   ETYPE_physical_delivery_modes)) {
           if (*(P1PhysicalDeliveryModes *)pvalue &  
                        PHYSICALDELIVERYMODES_ordinary_mail)   
               enumer = MH_PM_ORDINARY_MAIL;                                            
           else if (*(P1PhysicalDeliveryModes *)pvalue &      
                        PHYSICALDELIVERYMODES_special_delivery) 
               enumer = MH_PM_SPECIAL_DELIVERY;                                         
           else if (*(P1PhysicalDeliveryModes *)pvalue &       
                        PHYSICALDELIVERYMODES_express_mail)   
               enumer = MH_PM_EXPRESS_MAIL;                                             
           else if (*(P1PhysicalDeliveryModes *)pvalue &     
                        PHYSICALDELIVERYMODES_counter_collection) 
               enumer = MH_PM_CC;                                                       
           else if (*(P1PhysicalDeliveryModes *)pvalue & 
                PHYSICALDELIVERYMODES_counter_collection_with_telephone_advice) 
               enumer = MH_PM_CC_WITH_TELEPHONE_ADVICE;                                 
           else if (*(P1PhysicalDeliveryModes *)pvalue &
                PHYSICALDELIVERYMODES_counter_collection_with_telex_advice)     
               enumer = MH_PM_CC_WITH_TELEX_ADVICE;                                     
           else if (*(P1PhysicalDeliveryModes *)pvalue &  
                PHYSICALDELIVERYMODES_counter_collection_with_teletext_advice)  
               enumer = MH_PM_CC_WITH_TELETEX_ADVICE;    
           else
               continue ;

           if ((status = putvalue(desc,&desc->attr_value,
                         MH_T_POSTAL_MODE,   
                         OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS) 
               return(status); 
        } /* end if */

        break;

    case MH_T_POSTAL_REPORT:
        if (pvalue = (char *)get_per_recip_msg_extension(rd,
                                ETYPE_physical_delivery_report_request)) {
           switch (* (P1PhysicalDeliveryReportRequest *)pvalue) {                        
           case PHYSICALDELIVERYREPORTREQUEST_return_of_undeliverable_mail_by_pds:   
               enumer = MH_PR_UNDELIVBLE_MAIL_VIA_PDS;    
               break;                                                                
           case PHYSICALDELIVERYREPORTREQUEST_return_of_notification_by_PDS:
               enumer = MH_PR_NOTIFICN_VIA_PDS;          
               break;                                                                
           case PHYSICALDELIVERYREPORTREQUEST_return_of_notification_by_MHS:
               enumer = MH_PR_NOTIFICN_VIA_MTS;                            
               break;                                                                
           case PHYSICALDELIVERYREPORTREQUEST_return_of_notification_by_PDS_and_MHS: 
               enumer = MH_PR_NOTIFICN_VIA_MTS_AND_PDS;                   
               break;                                                                
           default:                                                     
               enumer = MH_PR_UNDELIVBLE_MAIL_VIA_PDS;                   
               break;                                                                
           } /* end switch */                                                        

           if ((status = putvalue(desc,&desc->attr_value,MH_T_POSTAL_REPORT,
                                OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
               return(status);
        } /* end if */
        break;

    case MH_T_RECIP_NUMBER_FOR_ADVICE:
        if (pvalue = (char *)get_per_recip_msg_extension(rd,
                               ETYPE_recipient_number_for_advice)) {
           if (len = strlen((char *)pvalue)) {
              if ((status = p_strcpy(desc,&desc->attr_value, MH_T_RECIP_NUMBER_FOR_ADVICE,
                                 OM_S_TELETEX_STRING, (void *)pvalue,len)) != OM_SUCCESS)
              return(status);
           } /* end if */
        } /* end if */
        break;

    case MH_T_REGISTRATION:
        if (pvalue = (char *)get_per_recip_msg_extension(rd,ETYPE_registered_mail_type)) {
        switch (* (P1RegisteredMailType *)pvalue) {
        case REGISTEREDMAILTYPE_non_registered_mail:
        default:
            enumer = MH_RG_UNREGISTERED_MAIL;
            break;
        case REGISTEREDMAILTYPE_registered_mail:
            enumer = MH_RG_REGISTERED_MAIL;
            break;
        case REGISTEREDMAILTYPE_registered_mail_to_addressee_in_person:
            enumer = MH_RG_REGISTERED_MAIL_IN_PERSON;
            break;
        } /* end switch */

        if ((status = putvalue(desc,&desc->attr_value,MH_T_REGISTRATION,
                              OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_REDIRECTION_HISTORY:
        if (pvalue = (char *)get_per_recip_msg_extension(rd,ETYPE_redirection_history)) {
        for (((P1RedirectionHistory *)pvalue)->current = ((P1RedirectionHistory *)pvalue)->first,
            baseval = curval = nextval = NULL, count = 0;
            ((P1RedirectionHistory *)pvalue)->current != NULL;
            ((P1RedirectionHistory *)pvalue)->current = ((P1RedirectionHistory *)pvalue)->current->next) {
            /* Create and fill in the Redirection History sub-object */
            if ((status = mhp_p_redir(((P1RedirectionHistory *)pvalue)->current->data,&subobject))
            != OM_SUCCESS)
            return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_REDIRECTION_HISTORY,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;

    case MH_T_TOKEN:          /* 1988 Security */
        if (pvalue = (char *)get_per_recip_msg_extension(rd,ETYPE_message_token)) {
        } /* end if */
        break;

    case MH_T_INTEGRITY_CHECK: /* 1988 Security */
        if (pvalue = (char *)get_per_recip_msg_extension(rd,ETYPE_content_integrity_check)) {
        } /* end if */
        break;

    case MH_T_PREFERRED_DELIVERY_MODES:
        if (pvalue = (char *)get_per_recip_msg_extension(rd,
                               ETYPE_requested_delivery_method)) 
        {
            int i;   /* Not to be confused with outer loop i */
            P1RequestedDeliveryMethod *prq;

            prq = (P1RequestedDeliveryMethod *)pvalue;

            for (i = 0, 
                   baseval = curval = nextval = desc->attr_value, count = 0; 
                 i < prq->n ; i++)
            {
                if ((status = putvalue(desc, &nextval,
                            MH_T_PREFERRED_DELIVERY_MODES, OM_S_ENUMERATION,
                            (void *) prq->v[i]) != OM_SUCCESS))
                    return(status);
                append_struct(&baseval,&curval,nextval,&count);

            }

            desc->attr_value = baseval;
            desc->count = count;

        } /* end if */

        else /* set default values as required by package */
        {
            if ((status = putvalue(desc, & desc->attr_value, 
                                   MH_T_PREFERRED_DELIVERY_MODES, 
                                   OM_S_ENUMERATION, (void *)MH_DM_ANY))
                                   != OM_SUCCESS)
                return(status);
        } /* else */

        break;



    case MH_T_ORIGINATOR_REPORT_REQUEST:
		enumer = MH_RQ_NON_DELIVERY; /* default */
        if (rd->m & 
            PerRecipientMessageTransferFields_per_recipient_indicators) {
        enumer = MH_RQ_NEVER;
        if (rd->per_recipient_indicators & 
            PER_RECIPIENT_INDICATORS_originator_non_delivery_report)
            enumer = MH_RQ_NON_DELIVERY;
        else if (rd->per_recipient_indicators & 
            PER_RECIPIENT_INDICATORS_originator_report)
            enumer = MH_RQ_ALWAYS;
        } /* end if */

        if ((status = putvalue(desc,&desc->attr_value,
            MH_T_ORIGINATOR_REPORT_REQUEST,OM_S_ENUMERATION,
            (void*)enumer)) != OM_SUCCESS)
            return(status);
        break;


    case MH_T_CRITICAL_FOR_DELIVERY:
        {
        P1ExtensionMacro *extensions ;
        aaListType *list;
        P1ExtensionMacroBody *extdata;
        OM_type    criticalityAttrType ;

        extensions = &(rd->extensions);
    
        list_first(extensions); /* set first item on list to current*/
        for (list = (aaListType *) extensions->current;
             list != NULL;
             list = (aaListType *)list_next(extensions))
        {
            extdata = (P1ExtensionMacroBody *)list->data;
            if (extdata->m |= ExtensionMacroBody_criticality) {
                tmp = criticalityAttrType = mhp_get_attr_criticality(extdata, 
                                                    CRITICALITY_for_delivery);

                if (criticalityAttrType != OM_NO_SUCH_TYPE) {

                   /* Create a value structure for the subobject and link it */
                   if ((status = putvalue(desc,&nextval,MH_T_CRITICAL_FOR_DELIVERY,
                                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                       return(status);
    
                   append_struct(&baseval,&curval,nextval,&count);
                }  /* end if */

            } /* end if extdata->m |= ExtensionMacroBody_criticality */

        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
   
        }
        break;

    case MH_T_CRITICAL_FOR_SUBMISSION: 
        {
        P1ExtensionMacro *extensions ;
        aaListType *list;
        P1ExtensionMacroBody *extdata;
        OM_type    criticalityAttrType ;
    
        extensions = &(rd->extensions);
    
        list_first(extensions); /* set first item on list to current*/
        for (list = (aaListType *) extensions->current;
             list != NULL;
             list = (aaListType *)list_next(extensions))
        {
            extdata = (P1ExtensionMacroBody *)list->data;
            if (extdata->m |= ExtensionMacroBody_criticality) {
                tmp = criticalityAttrType = mhp_get_attr_criticality(extdata, 
                                                    CRITICALITY_for_submission);

                if (criticalityAttrType != OM_NO_SUCH_TYPE) {

                   /* Create a value structure for the subobject and link it */
                   if ((status = putvalue(desc,&nextval,MH_T_CRITICAL_FOR_SUBMISSION,
                                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                       return(status);
    
                   append_struct(&baseval,&curval,nextval,&count);
                }  /* end if */

            } /* end if extdata->m |= ExtensionMacroBody_criticality */

        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
   
        }
        break;

    case MH_T_CRITICAL_FOR_TRANSFER: 
        {
        P1ExtensionMacro *extensions ;
        aaListType *list;
        P1ExtensionMacroBody *extdata;
        OM_type    criticalityAttrType ;
    
        extensions = &(rd->extensions);
    
        list_first(extensions); /* set first item on list to current*/
        for (list = (aaListType *) extensions->current;
             list != NULL;
             list = (aaListType *)list_next(extensions))
        {
            extdata = (P1ExtensionMacroBody *)list->data;
            if (extdata->m |= ExtensionMacroBody_criticality) {
                tmp = criticalityAttrType = mhp_get_attr_criticality(extdata, 
                                                    CRITICALITY_for_transfer);

                if (criticalityAttrType != OM_NO_SUCH_TYPE) {

                   /* Create a value structure for the subobject and link it */
                   if ((status = putvalue(desc,&nextval,MH_T_CRITICAL_FOR_TRANSFER,
                                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                       return(status);
    
                   append_struct(&baseval,&curval,nextval,&count);
                }  /* end if */

            } /* end if extdata->m |= ExtensionMacroBody_criticality */

        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
   
        }
        break;

    case MH_T_EXTENSIONS: 
        break;

     default:
        break;

    } /* end switch */
    } /* end for */

    return(OM_SUCCESS);

} /* end mhp_p_msg_rd() */


/*
 * mhp_p_probe_rd() put P1PerRecipientMessageTransferFields structure
 * into MH_C_RECIPIENT_DESCRIPTOR object, and return the object handle.
 *
 * INPUT:
 *  rd  - address of P1PerRecipientMessageTransferFields.
 *
 * OUTPUT:
 *  object  - handle to the returned Recipient Descriptor object.
 */

int mhp_p_probe_rd(rd, object)
P1PerRecipientProbeTransferFields *rd;
OM_int_private_object_header **object;
{
Uint32 len;
int i, status;
Uint16 count;
OM_enumeration	enumer;
OM_boolean flag;
int *evalue;
char *pvalue;
OM_int_descriptor *desc;
OM_int_private_object_header *subobject;
OM_int_value *baseval, *curval, *nextval;
OM_string string;

    if ((status = omp_template(MH_C_PROBE_RD,object)) != OM_SUCCESS)
    return(status);

    /* Index through Recipient descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; 
		(*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
		i++) 
	{
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_MTA_REPORT_REQUEST:
        if (rd->m & PerRecipientProbeTransferFields_per_recipient_indicators)
		{
        enumer = MH_RQ_NON_DELIVERY;
        if (rd->per_recipient_indicators & 
            	PER_RECIPIENT_INDICATORS_originating_MTA_report) {
            if (rd->per_recipient_indicators & 
                PER_RECIPIENT_INDICATORS_originating_MTA_non_delivery_report) {
                enumer = MH_RQ_ALWAYS_AUDITED;
            }
            else {
                enumer = MH_RQ_ALWAYS;
            }
        }

        if ((status = putvalue(desc,&desc->attr_value,MH_T_MTA_REPORT_REQUEST,
					OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_MTA_RESPONSIBILITY:
        if (rd->m & PerRecipientProbeTransferFields_per_recipient_indicators) {
        if (rd->per_recipient_indicators & PER_RECIPIENT_INDICATORS_responsibility)
            flag = TRUE;
        else
            flag = FALSE;

        if ((status = putvalue(desc,&desc->attr_value,MH_T_MTA_RESPONSIBILITY,
					OM_S_BOOLEAN,(void*)flag)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_RECIPIENT_NUMBER:
        if (rd->m & PerRecipientProbeTransferFields_originally_specified_recipient_number) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_RECIPIENT_NUMBER,
					OM_S_INTEGER, 
					(void*)rd->originally_specified_recipient_number)) 
					!= OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_RECIPIENT_NAME:
        if (rd->m & PerRecipientProbeTransferFields_recipient_name) {
        /* Create and fill in OR Name sub-object */
        if ((status = mhp_p_orname(rd->recipient_name,&subobject))!=OM_SUCCESS)
            return(status);

        if ((status = putvalue(desc,&desc->attr_value,MH_T_RECIPIENT_NAME,
					OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_EXPLICIT_CONVERSION:
        if (rd->m & PerRecipientProbeTransferFields_explicit_conversion)
        enumer = rd->explicit_conversion;
        else
        enumer = MH_EC_NO_CONVERSION;

        if ((status = putvalue(desc,&desc->attr_value,MH_T_EXPLICIT_CONVERSION,
				OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
        	return(status);
        break;


    case MH_T_ALTERNATE_RECIPIENT_NAME:
        if (pvalue = (char *)get_perrecp_probe_extension(rd,
					ETYPE_originator_requested_alternate_recipient)) {

        /* pvalue is a pointer to an int, which in turn is a pointer
         * to P1ORName.  mhp_p_orname() will assume that the content of 
         * pvalue is a pointer to P1ORName. Create and fill in OR Name 
         * sub-object 
         */
            if ((status = mhp_p_orname( 
                       (*(P1OriginatorRequestedAlternateRecipient *)pvalue), 
                       &subobject)) 
                                 != OM_SUCCESS)
                return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&desc->attr_value,
					MH_T_ALTERNATE_RECIPIENT_NAME,OM_S_OBJECT,subobject)) 
					!= OM_SUCCESS)
            	return(status);
        } /* end if */
        break;

    case MH_T_RENDITION_ATTRIBUTES:
        if (pvalue = (char *)get_perrecp_probe_extension(rd,ETYPE_physical_rendition_attributes)) {
        if (len = ((P1PhysicalRenditionAttributes *)pvalue)->length) {
	    mhp_addobj (&string, ((P1PhysicalRenditionAttributes *)pvalue));
            if ((status = p_strcpy(desc,&desc->attr_value,
				MH_T_RENDITION_ATTRIBUTES,
            			OM_S_OBJECT_IDENTIFIER_STRING,
				(void*) string.elements,string.length)
					!= OM_SUCCESS))
            	return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_FORWARDING_ADDR_REQUESTED:
        flag = OM_FALSE;   /* default inital value: required at interface*/
        if (pvalue = (char *)get_perrecp_probe_extension(rd,
                      ETYPE_physical_forwarding_address_request)) {
            if (*(P1PhysicalForwardingAddressRequest *) pvalue)
                flag = OM_TRUE;
        } /* end if */

        if ((status = putvalue(desc,&desc->attr_value,
                MH_T_FORWARDING_ADDR_REQUESTED,OM_S_BOOLEAN,(void*)flag)) 
                != OM_SUCCESS)
            return(status);
        break;

    case MH_T_FORWARDING_PROHIBITED:
        flag = OM_FALSE;
        if (pvalue = (char *)get_perrecp_probe_extension(rd,
                              ETYPE_physical_forwarding_prohibited)) {
            if (* (P1PhysicalForwardingProhibited *) pvalue)
                flag = OM_TRUE;
        } /* end if */

        if ((status = putvalue(desc,&desc->attr_value,
                   MH_T_FORWARDING_PROHIBITED,OM_S_BOOLEAN,(void*)flag)) 
                   != OM_SUCCESS)
            return(status);
        break;

    case MH_T_PROOF_OF_DELIV_REQUESTED:
        flag = OM_FALSE;   /* must set initial attr value at interface */
        if (pvalue = (char *)get_perrecp_probe_extension(rd,
                                    ETYPE_proof_of_delivery_request)) {
            if (* (P1ProofOfDeliveryRequest*)pvalue)
                flag = OM_TRUE;
        } /* end if */

        if ((status = putvalue(desc,&desc->attr_value,
                  MH_T_PROOF_OF_DELIV_REQUESTED,
                  OM_S_BOOLEAN,(void*)flag)) != OM_SUCCESS)
            return(status);
        break;

    case MH_T_BUREAU_FAX_DELIVERY:

        /* only extract the 7-th bit. Other bits in P1PhysicalDeliveryMode
         * go to Postal Mode.
         */
   
        flag = OM_FALSE;   /* must have default initial value at interface */
        if (pvalue = (char *)get_perrecp_probe_extension(rd,
                                ETYPE_physical_delivery_modes)) {
        if ( *(P1PhysicalDeliveryModes *)pvalue 
                & PHYSICALDELIVERYMODES_bureau_fax_delivery)
            flag = OM_TRUE;
        } /* end if */

        if ((status = putvalue(desc,&desc->attr_value,
                    MH_T_BUREAU_FAX_DELIVERY,
                    OM_S_BOOLEAN,(void*)flag)) != OM_SUCCESS)
            return(status);
        break;

    case MH_T_POSTAL_MODE:
        /* no need to generate default values as it's not required */
        if (pvalue = (char *)get_perrecp_probe_extension(rd,
                   ETYPE_physical_delivery_modes)) {
        if (*(P1PhysicalDeliveryModes *)pvalue & 
                     PHYSICALDELIVERYMODES_ordinary_mail)
            enumer = MH_PM_ORDINARY_MAIL;
        else if (*(P1PhysicalDeliveryModes *)pvalue & 
                     PHYSICALDELIVERYMODES_special_delivery)
            enumer = MH_PM_SPECIAL_DELIVERY;
        else if (*(P1PhysicalDeliveryModes *)pvalue & 
                     PHYSICALDELIVERYMODES_express_mail)
            enumer = MH_PM_EXPRESS_MAIL;
        else if (*(P1PhysicalDeliveryModes *)pvalue & 
                     PHYSICALDELIVERYMODES_counter_collection)
            enumer = MH_PM_CC;
        else if (*(P1PhysicalDeliveryModes *)pvalue & 
              PHYSICALDELIVERYMODES_counter_collection_with_telephone_advice)
            enumer = MH_PM_CC_WITH_TELEPHONE_ADVICE;
        else if (*(P1PhysicalDeliveryModes *)pvalue & 
              PHYSICALDELIVERYMODES_counter_collection_with_telex_advice)
            enumer = MH_PM_CC_WITH_TELEX_ADVICE;
        else if (*(P1PhysicalDeliveryModes *)pvalue & 
              PHYSICALDELIVERYMODES_counter_collection_with_teletext_advice)
            enumer = MH_PM_CC_WITH_TELETEX_ADVICE;

        if ((status = putvalue(desc,&desc->attr_value,
                      MH_T_POSTAL_MODE,
                      OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
            return(status);
        } /* end if */

        break;

    case MH_T_POSTAL_REPORT:
        if (pvalue = (char *)get_perrecp_probe_extension(rd,ETYPE_physical_delivery_report_request)) {
        switch ((P1PhysicalDeliveryReportRequest)pvalue) {
        case PHYSICALDELIVERYREPORTREQUEST_return_of_undeliverable_mail_by_pds:
        default:
            enumer = MH_PR_UNDELIVBLE_MAIL_VIA_PDS;
            break;
        case PHYSICALDELIVERYREPORTREQUEST_return_of_notification_by_PDS:
            enumer = MH_PR_NOTIFICN_VIA_PDS;
            break;
        case PHYSICALDELIVERYREPORTREQUEST_return_of_notification_by_MHS:
            enumer = MH_PR_NOTIFICN_VIA_MTS;
            break;
        case PHYSICALDELIVERYREPORTREQUEST_return_of_notification_by_PDS_and_MHS:
            enumer = MH_PR_NOTIFICN_VIA_MTS_AND_PDS;
            break;
        } /* end switch */

        if ((status = putvalue(desc,&desc->attr_value,MH_T_POSTAL_REPORT,OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_RECIP_NUMBER_FOR_ADVICE:
        if (pvalue = (char *)get_perrecp_probe_extension(rd,ETYPE_recipient_number_for_advice)) {
        if (len = strlen((char *)pvalue)) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_RECIP_NUMBER_FOR_ADVICE,
            OM_S_TELETEX_STRING,pvalue,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    case MH_T_REGISTRATION:
        if (pvalue = (char *)get_perrecp_probe_extension(rd,ETYPE_registered_mail_type)) {
        switch ((P1RegisteredMailType)pvalue) {
        case REGISTEREDMAILTYPE_non_registered_mail:
        default:
            enumer = MH_RG_UNREGISTERED_MAIL;
            break;
        case REGISTEREDMAILTYPE_registered_mail:
            enumer = MH_RG_REGISTERED_MAIL;
            break;
        case REGISTEREDMAILTYPE_registered_mail_to_addressee_in_person:
            enumer = MH_RG_REGISTERED_MAIL_IN_PERSON;
            break;
        } /* end switch */

        if ((status = putvalue(desc,&desc->attr_value,MH_T_REGISTRATION,OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_REDIRECTION_HISTORY:
        if (pvalue = (char *)get_perrecp_probe_extension(rd,ETYPE_redirection_history)) {
        for (((P1RedirectionHistory *)pvalue)->current = ((P1RedirectionHistory *)pvalue)->first,
            baseval = curval = nextval = NULL, count = 0;
            ((P1RedirectionHistory *)pvalue)->current != NULL;
            ((P1RedirectionHistory *)pvalue)->current = ((P1RedirectionHistory *)pvalue)->current->next) {
            /* Create and fill in the Redirection History sub-object */
            if ((status = mhp_p_redir(((P1RedirectionHistory *)pvalue)->current->data,&subobject))
            != OM_SUCCESS)
            return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_REDIRECTION_HISTORY,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;
    case MH_T_TOKEN:
        if (pvalue = (char *)get_perrecp_probe_extension(rd,ETYPE_message_token)) {
        } /* end if */
        break;
    case MH_T_INTEGRITY_CHECK:
        if (pvalue = (char *)get_perrecp_probe_extension(rd,ETYPE_content_integrity_check)) {
        } /* end if */
        break;
    case MH_T_PREFERRED_DELIVERY_MODES:
        if (pvalue = (char *)get_perrecp_probe_extension(rd,
                               ETYPE_requested_delivery_method)) 
        {
            int i;   /* Not to be confused with outer loop i */
            P1RequestedDeliveryMethod *prq;

            prq = (P1RequestedDeliveryMethod *)pvalue;

            for (i = 0, 
                   baseval = curval = nextval = desc->attr_value, count = 0; 
                 i < prq->n ; i++)
            {
                if ((status = putvalue(desc, &nextval,
                            MH_T_PREFERRED_DELIVERY_MODES, OM_S_ENUMERATION,
                            (void *) prq->v[i]) != OM_SUCCESS))
                    return(status);
                append_struct(&baseval,&curval,nextval,&count);

            }

            desc->attr_value = baseval;
            desc->count = count;

        } /* end if */

        else /* set default values as required by package */
        {
            if ((status = putvalue(desc, & desc->attr_value, 
                                   MH_T_PREFERRED_DELIVERY_MODES, 
                                   OM_S_ENUMERATION, (void *)MH_DM_ANY))
                                   != OM_SUCCESS)
                return(status);
        } /* else */

        break;



    case MH_T_ORIGINATOR_REPORT_REQUEST:
		enumer = MH_RQ_NON_DELIVERY; /* default */
        if (rd->m & 
            PerRecipientProbeTransferFields_per_recipient_indicators) {
        enumer = MH_RQ_NEVER;
        if (rd->per_recipient_indicators & 
            PER_RECIPIENT_INDICATORS_originator_non_delivery_report)
            enumer = MH_RQ_NON_DELIVERY;
        else if (rd->per_recipient_indicators & 
            PER_RECIPIENT_INDICATORS_originator_report)
            enumer = MH_RQ_ALWAYS;
        } /* end if */

        if ((status = putvalue(desc,&desc->attr_value,
            MH_T_ORIGINATOR_REPORT_REQUEST,OM_S_ENUMERATION,
            (void*)enumer)) != OM_SUCCESS)
            return(status);
        break;
    } /* end switch */
    } /* end for */

    return(OM_SUCCESS);

} /* end mhp_p_probe_rd() */




/*
 * mhp_p_prr() put P1PerRecipientReportTransferFields structure
 * into MH_C_PER_RECIPIENT_REPORT object, and return the object handle.
 *
 * INPUT:
 *  prr - address of P1PerRecipientReportTransferFields
 *
 * OUTPUT:
 *  object  - handle to the returned Per Recipient Report object.
 */

int mhp_p_prr(prr, object)
P1PerRecipientReportTransferFields *prr;
OM_int_private_object_header **object;

{
Uint32 len;
int i, status;
Uint16 count;
OM_enumeration	enumer;
OM_boolean flag;
int *evalue, tmp;
char *pvalue;
OM_int_descriptor *desc;
OM_int_private_object_header *subobject;
OM_int_value *baseval, *curval, *nextval;
P1ExtensionMacroBody *extension;
P1Report *p1report;
char *buf;

    p1report = (P1Report *) &prr->last_trace_information.report;
    switch(p1report->t){ /* delivery or non-delivery choice */
    case Report_delivery:
        if ((status = omp_template(MH_C_PER_RECIP_DR,object)) != OM_SUCCESS)
            return(status);
        break;
    case Report_non_delivery:
        if ((status = omp_template(MH_C_PER_RECIP_NDR,object)) != OM_SUCCESS)
            return(status);
        break;
    default:
        break;
    }

    /* Index through Per Recipient Report descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; 
        (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
        i++) 
    {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_ACTUAL_RECIPIENT_NAME:
        if (prr->m & PerRecipientReportTransferFields_actual_recipient_name) {
        /* Create and fill in OR Name sub-object */
        if ((status = mhp_p_orname(prr->actual_recipient_name,&subobject)) 
                                                    != OM_SUCCESS)
            return(status);
        if ((status = putvalue(desc,&desc->attr_value,
                        MH_T_ACTUAL_RECIPIENT_NAME,OM_S_OBJECT,subobject))
                        != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_ARRIVAL_TIME:
        if ((prr->m & PerRecipientReportTransferFields_last_trace_information)
                && (prr->last_trace_information.m & 
                    LastTraceInformation_arrival_time)) {
            if ((buf = mhp_to_utc(&(prr->last_trace_information.
                        arrival_time))) == NULL){
                log_error(OM_MEMORY_INSUFFICIENT,OMP_DF_MEMORY_INSUFFICIENT);
                return(OM_MEMORY_INSUFFICIENT);
            }
            if ((status = putvalue(desc,&desc->attr_value,MH_T_ARRIVAL_TIME,
                    OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
            return(status);
        }
        break;  

    case MH_T_CONVERTED_EITS:
        if ((prr->m & PerRecipientReportTransferFields_last_trace_information)
             && (prr->last_trace_information.m 
                 & LastTraceInformation_converted_encoded_information_types)){
            if ((status = mhp_p_eits(prr->last_trace_information.
                        converted_encoded_information_types,    
                        &subobject)) != OM_SUCCESS)
                return(status);
            if ((status = putvalue(desc,&desc->attr_value,
                        MH_T_CONVERTED_EITS,OM_S_OBJECT,subobject))
                        != OM_SUCCESS)
                return(status);
        } /* end if */
        break;

    case MH_T_ORIGINALLY_INTENDED_RECIP:
        if (prr->m & 
           PerRecipientReportTransferFields_originally_intended_recipient_name)
        {
        /* Create and fill in OR Name sub-object */
        if ((status = mhp_p_orname(prr->originally_intended_recipient_name,
                    &subobject)) != OM_SUCCESS)
            return(status);
        if ((status = putvalue(desc,&desc->attr_value,
                MH_T_INTENDED_RECIPIENT_NAME,OM_S_OBJECT,subobject))
                != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_INTENDED_RECIPIENT_NUMBER:
        if (prr->m & 
         PerRecipientReportTransferFields_originally_specified_recipient_number)
        {
            if ((status = putvalue(desc,&desc->attr_value,
                    MH_T_INTENDED_RECIPIENT_NUMBER,OM_S_INTEGER,
                    (void*)prr->originally_specified_recipient_number)) 
                    != OM_SUCCESS)
                return(status);
        }
        break;

    case MH_T_ORIGINATOR_REPORT_REQUEST:
		enumer = MH_RQ_NON_DELIVERY; /* default */
        if (prr->m &  PerRecipientReportTransferFields_per_recipient_indicators)
        {
        enumer = MH_RQ_NEVER; /* never unless either bit set */
        if (prr->per_recipient_indicators & 
            PER_RECIPIENT_INDICATORS_originator_non_delivery_report)
            enumer = MH_RQ_NON_DELIVERY;
        else if (prr->per_recipient_indicators &
            PER_RECIPIENT_INDICATORS_originator_report)
            enumer = MH_RQ_ALWAYS;
        } /* end if */
        if ((status = putvalue(desc,&desc->attr_value,
            MH_T_ORIGINATOR_REPORT_REQUEST,OM_S_ENUMERATION,
            (void *)enumer)) != OM_SUCCESS)
            return(status);
        break;

    case MH_T_SUPPLEMENTARY_INFO:
        if (prr->m & PerRecipientReportTransferFields_supplementary_information)        {
        if (len = strlen(prr->supplementary_information)) {
            if ((status = p_strcpy(desc,&desc->attr_value,
                    MH_T_SUPPLEMENTARY_INFO,OM_S_PRINTABLE_STRING,
                    prr->supplementary_information,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_FORWARDING_ADDRESS:
        if (evalue = (int *)get_per_recip_report_extension(prr,ETYPE_physical_forwarding_address)) {
        /* Create and fill in Forwarding Address sub-object */
            if ((status = mhp_p_orname((P1ORName *)(*evalue), &subobject)) != OM_SUCCESS)
               return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&desc->attr_value,MH_T_FORWARDING_ADDRESS,OM_S_OBJECT,subobject))
                   != OM_SUCCESS)
               return(status);
        } /* end if */
        break;

    case MH_T_REDIRECTION_HISTORY:
        if (evalue = (int *)get_per_recip_report_extension(prr, ETYPE_redirection_history)) {
        for (((P1RedirectionHistory *)evalue)->current = ((P1RedirectionHistory *)evalue)->first,
            baseval = curval = nextval = NULL, count = 0;
            ((P1RedirectionHistory *)evalue)->current != NULL;
            ((P1RedirectionHistory *)evalue)->current = ((P1RedirectionHistory *)evalue)->current->next) {
            /* Create and fill in the Redirection History sub-object */
            if ((status = mhp_p_redir(((P1RedirectionHistory *)evalue)->current->data,&subobject))
                    != OM_SUCCESS)
                return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&nextval,MH_T_REDIRECTION_HISTORY,OM_S_OBJECT,subobject)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;


    /* the following applies to Delivery Report */
    case MH_T_DELIVERY_POINT:
		/* if not present, default is public. This is different from 
		** init value in API spec. KDH.
		*/
		enumer =  MH_DP_PUBLIC_UA; 
        if ((prr->m & PerRecipientReportTransferFields_last_trace_information)
                && (p1report->t == Report_delivery)
                && (p1report->v.delivery.m & DeliveryReport_type_of_MTS_user)){
            switch(p1report->v.delivery.type_of_MTS_user){
            case TYPE_OF_MTS_USER_public:
                enumer = MH_DP_PUBLIC_UA;
                break;
            case TYPE_OF_MTS_USER_private:
                enumer = MH_DP_PRIVATE_UA;
                break;
            }
        } /* end if */
        if ((status = putvalue(desc,&desc->attr_value,
                MH_T_DELIVERY_POINT,OM_S_ENUMERATION,
                (void *)enumer)) != OM_SUCCESS)
                    return(status);
        break;

    case MH_T_DELIVERY_TIME:
        if ((prr->m & PerRecipientReportTransferFields_last_trace_information)
                && (p1report->t == Report_delivery)
                && (p1report->v.delivery.m &DeliveryReport_message_delivery_time))      {
            if ((buf = mhp_to_utc(&(p1report->v.delivery.
                        message_delivery_time))) == NULL){
                log_error(OM_MEMORY_INSUFFICIENT, OMP_DF_MEMORY_INSUFFICIENT);
                return(OM_MEMORY_INSUFFICIENT);
            }
            if ((status = putvalue(desc,&desc->attr_value,MH_T_DELIVERY_TIME,
                    OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
            return(status);
        }
        break;
            

    /* the following applies to Non Delivery Report */
    case MH_T_NON_DELIVERY_DIAGNOSTIC:
		enumer = MH_DG_NO_DIAGNOSTIC; /* default */
        if ((prr->m & PerRecipientReportTransferFields_last_trace_information)
                && (p1report->t == Report_non_delivery)
                && (p1report->v.non_delivery.m &  
                    NonDeliveryReport_non_delivery_diagnostic_code)){
            enumer = p1report->v.non_delivery.non_delivery_diagnostic_code;
		}
		if ((status = putvalue(desc,&desc->attr_value,
                    MH_T_NON_DELIVERY_DIAGNOSTIC, OM_S_ENUMERATION,
                    (void*)enumer)) != OM_SUCCESS)
			return(status);
        break;

    case MH_T_NON_DELIVERY_REASON:
        if ((prr->m & PerRecipientReportTransferFields_last_trace_information)
                && (p1report->t == Report_non_delivery)
                && (p1report->v.non_delivery.m &  
                    NonDeliveryReport_non_delivery_reason_code)){
            enumer = p1report->v.non_delivery.non_delivery_reason_code;
            if ((status = putvalue(desc,&desc->attr_value,
                    MH_T_NON_DELIVERY_DIAGNOSTIC, OM_S_ENUMERATION,
                    (void*)enumer)) != OM_SUCCESS)
                return(status);
        }
        break;


    case MH_T_CRITICAL_FOR_DELIVERY:
        {
        P1ExtensionMacro *extensions ;
        aaListType *list;
        P1ExtensionMacroBody *extdata;
        OM_type    criticalityAttrType ;
    
        extensions = &(prr->extensions);
    
        list_first(extensions); /* set first item on list to current*/
        for (list = (aaListType *) extensions->current;
             list != NULL;
             list = (aaListType *)list_next(extensions))
        {
            extdata = (P1ExtensionMacroBody *)list->data;
            if (extdata->m |= ExtensionMacroBody_criticality) {
                tmp = criticalityAttrType = mhp_get_attr_criticality(extdata, 
                                                    CRITICALITY_for_delivery);

                if (criticalityAttrType != OM_NO_SUCH_TYPE) {

                   /* Create a value structure for the subobject and link it */
                   if ((status = putvalue(desc,&nextval,MH_T_CRITICAL_FOR_DELIVERY,
                                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                       return(status);
    
                   append_struct(&baseval,&curval,nextval,&count);
                }  /* end if */

            } /* end if extdata->m |= ExtensionMacroBody_criticality */

        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
   
        }
        break;

    case MH_T_CRITICAL_FOR_SUBMISSION: 
        {
        P1ExtensionMacro *extensions ;
        aaListType *list;
        P1ExtensionMacroBody *extdata;
        OM_type    criticalityAttrType ;
    
        extensions = &(prr->extensions);
    
        list_first(extensions); /* set first item on list to current*/
        for (list = (aaListType *) extensions->current;
             list != NULL;
             list = (aaListType *)list_next(extensions))
        {
            extdata = (P1ExtensionMacroBody *)list->data;
            if (extdata->m |= ExtensionMacroBody_criticality) {
                tmp = criticalityAttrType = mhp_get_attr_criticality(extdata, 
                                                    CRITICALITY_for_submission);

                if (criticalityAttrType != OM_NO_SUCH_TYPE) {

                   /* Create a value structure for the subobject and link it */
                   if ((status = putvalue(desc,&nextval,MH_T_CRITICAL_FOR_SUBMISSION,
                                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                       return(status);
    
                   append_struct(&baseval,&curval,nextval,&count);
                }  /* end if */

            } /* end if extdata->m |= ExtensionMacroBody_criticality */

        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
   
        }
        break;

    case MH_T_CRITICAL_FOR_TRANSFER: 
        {
        P1ExtensionMacro *extensions ;
        aaListType *list;
        P1ExtensionMacroBody *extdata;
        OM_type    criticalityAttrType ;
    
        extensions = &(prr->extensions);
    
        list_first(extensions); /* set first item on list to current*/
        for (list = (aaListType *) extensions->current;
             list != NULL;
             list = (aaListType *)list_next(extensions))
        {
            extdata = (P1ExtensionMacroBody *)list->data;
            if (extdata->m |= ExtensionMacroBody_criticality) {
                tmp = criticalityAttrType = mhp_get_attr_criticality(extdata, 
                                                    CRITICALITY_for_transfer);

                if (criticalityAttrType != OM_NO_SUCH_TYPE) {

                   /* Create a value structure for the subobject and link it */
                   if ((status = putvalue(desc,&nextval,MH_T_CRITICAL_FOR_TRANSFER,
                                            OM_S_INTEGER,(void*)tmp)) != OM_SUCCESS)
                       return(status);
    
                   append_struct(&baseval,&curval,nextval,&count);
                }  /* end if */

            } /* end if extdata->m |= ExtensionMacroBody_criticality */

        } /* end for */
        desc->attr_value = baseval;
        desc->count = count;
   
        }
        break;

    case MH_T_EXTENSIONS: 
        break;

        
    default:
        break;

    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_prr() */






mhp_p_redir(redir,object)
P1Redirection *redir;
OM_int_private_object_header **object;
{
int i, status;
OM_int_descriptor *desc;
OM_int_private_object_header *subobject;
OM_enumeration	enumer;
P1IntendedRecipientName *rn;
char *tm;

    if ((status = omp_template(MH_C_REDIRECTION_RECORD,object)) != OM_SUCCESS)
    return(status);

    /* Index through Redirection Record descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_INTENDED_RECIPIENT_NAME:
        if (redir->m & Redirection_intended_recipient_name) {
            /* Create and fill in OR Name sub-object */
            rn = (P1IntendedRecipientName *)redir->intended_recipient_name ;
           /* if (rn->m & IntendedRecipientName_ORAddressAndOptionalDirectoryName) { */
/* NIST PM               if ((status = mhp_p_orname((P1ORName *)rn->oRAddressAndOptionalDirectoryName,&subobject)) 
                       != OM_SUCCESS)
                  return(status);
PM NIST */
               /* Create a value structure for the subobject and link it */
/* NIST PM               if ((status = putvalue(desc,&desc->attr_value,MH_T_INTENDED_RECIPIENT_NAME,OM_S_OBJECT,subobject))
                       != OM_SUCCESS)
                  return(status);
PM NIST */              /* end if */
        } /* end if */
        break;

    case MH_T_TIME:
        if (redir->m & Redirection_intended_recipient_name) {
        /* Create and fill in UTC time */
            rn = (P1IntendedRecipientName *)redir->intended_recipient_name ;
            /* if (rn->m & IntendedRecipientName_redirection_time) { */
/* NIST PM               tm = mhp_to_utc((struct atime *)&(rn->redirection_time)) ;
PM NIST */
               /* Create a value structure for the subobject and link it */
/* NIST PM               if ((status = putvalue(desc,&desc->attr_value,MH_T_TIME,OM_S_UTC_TIME_STRING,(void*)tm))
                       != OM_SUCCESS)
                  return(status);
PM NIST */              /* end if */
        } /* end if */
        break;

    case MH_T_REASON:
        if (redir->m & Redirection_redirection_reason) {
        switch (redir->redirection_reason) {
        case REDIRECTION_REASON_recipient_assigned_alternate_recipient:
        default:
            enumer = MH_RR_RECIPIENT_ASSIGNED;
            break;
        case REDIRECTION_REASON_originator_requested_alternate_recipient:
            enumer = MH_RR_ORIGINATOR_REQUESTED;
            break;
        case REDIRECTION_REASON_recipient_MD_assigned_alternate_recipient:
            enumer = MH_RR_RECIPIENT_DOMAIN_ASSIGNED;
            break;

        } /* end switch */

        if ((status = putvalue(desc,&desc->attr_value,MH_T_REASON,OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_redir() */





/*
 * mhp_p_eits(eits, object) put P1EncodedInformatinTypes structure
 * into EncodedInformationType object.
 *
 */

mhp_p_eits(p1eit, object)
P1EncodedInformationTypes *p1eit;
OM_int_private_object_header **object;
{
Uint32 len;
int i, status;
Uint16 count;
P1ExternalEncodedInformationTypes *p1extern;
OM_int_descriptor *desc;
OM_int_value *baseval, *curval, *nextval;
OM_int_private_object_header *subobject;
OM_string string;

    if ((status = omp_template(MH_C_EITS,object)) != OM_SUCCESS)
    return(status);

    /* Index through Encoded Information Type descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_BUILTIN_EITS:
        if (p1eit->m & EncodedInformationTypes_built_in_encoded_information_types) {
        /* Initialize value structure */
        baseval = curval = nextval = NULL;

        /* Initialize value count */
        count = 0;

        if (p1eit->built_in_encoded_information_types 
				& BUILT_IN_ENCODED_INFORMATION_TYPES_undefined) {
            if ((status = putvalue(desc,&nextval,MH_T_BUILTIN_EITS,
					OM_S_ENUMERATION,MH_BE_UNDEFINED))
            		!= OM_SUCCESS)
            	return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        if (p1eit->built_in_encoded_information_types 
				& BUILT_IN_ENCODED_INFORMATION_TYPES_telex) {
            if ((status = putvalue(desc,&nextval,MH_T_BUILTIN_EITS,
					OM_S_ENUMERATION,(void*)MH_BE_TELEX)) != OM_SUCCESS)
            	return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        if ( p1eit->built_in_encoded_information_types 
				& BUILT_IN_ENCODED_INFORMATION_TYPES_ia5_text) {
            if ((status = putvalue(desc,&nextval,MH_T_BUILTIN_EITS,
					OM_S_ENUMERATION,(void*)MH_BE_IA5_TEXT)) != OM_SUCCESS)
            	return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        if (p1eit->built_in_encoded_information_types 
				& BUILT_IN_ENCODED_INFORMATION_TYPES_g3_facsimile) {
            if ((status = putvalue(desc,&nextval,MH_T_BUILTIN_EITS,
					OM_S_ENUMERATION,(void*)MH_BE_G3_FAX)) != OM_SUCCESS)
            	return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        if (p1eit->built_in_encoded_information_types 
				& BUILT_IN_ENCODED_INFORMATION_TYPES_g4_class_1) {
            if ((status = putvalue(desc,&nextval,MH_T_BUILTIN_EITS,
					OM_S_ENUMERATION,(void*)MH_BE_G4_CLASS1)) != OM_SUCCESS)
            	return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        if (p1eit->built_in_encoded_information_types 
				& BUILT_IN_ENCODED_INFORMATION_TYPES_teletex) {
            if ((status = putvalue(desc,&nextval,MH_T_BUILTIN_EITS,
					OM_S_ENUMERATION,(void*)MH_BE_TELETEX)) != OM_SUCCESS)
            	return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        if (p1eit->built_in_encoded_information_types 
					& BUILT_IN_ENCODED_INFORMATION_TYPES_videotex) {
            if ((status = putvalue(desc,&nextval,MH_T_BUILTIN_EITS,
					OM_S_ENUMERATION,(void*)MH_BE_VIDEOTEX)) != OM_SUCCESS)
            	return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        if (p1eit->built_in_encoded_information_types  
				& BUILT_IN_ENCODED_INFORMATION_TYPES_mixed_mode) {
            if ((status = putvalue(desc,&nextval,MH_T_BUILTIN_EITS,
					OM_S_ENUMERATION,(void*)MH_BE_MIXED_MODE)) != OM_SUCCESS)
            	return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        if (p1eit->built_in_encoded_information_types 
				& BUILT_IN_ENCODED_INFORMATION_TYPES_iSO6937) {
            if ((status = putvalue(desc,&nextval,MH_T_BUILTIN_EITS,
					OM_S_ENUMERATION,(void*)MH_BE_ISO_6937_TEXT)) != OM_SUCCESS)
            	return(status);

            append_struct(&baseval,&curval,nextval,&count);
        } /* end if */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;

    case MH_T_EXTERNAL_EITS:
        if (p1eit->m & EncodedInformationTypes_external_encoded_information_types) {
        for (p1extern = (P1ExternalEncodedInformationTypes *)(p1eit->external_encoded_information_types),
            p1extern->current = p1extern->first,
            baseval = curval = nextval = NULL, count = 0;
            p1extern->current != NULL; p1extern->current = p1extern->current->next) {
            if (len = ((P1ExternalEncodedInformationType *)p1extern->current->data)->length) {
	    mhp_addobj (&string, ((P1ExternalEncodedInformationType *)p1extern->current->data));
            if ((status = p_strcpy(desc,&nextval,MH_T_EXTERNAL_EITS,OM_S_OBJECT_IDENTIFIER_STRING,(void*)
                string.elements,string.length)) != OM_SUCCESS)
                return(status);

            append_struct(&baseval,&curval,nextval,&count);
            } /* end if */
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;

    case MH_T_G3_FAX_NBPS:
        if (p1eit->m & EncodedInformationTypes_g3_facsimile) {
        /* Create G3 Facsimile NBPS subobject */
        if ((status = mhp_p_g3fax_nbps(p1eit->g3_facsimile,&subobject)) != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_G3_FAX_NBPS,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_G4_FAX_NBPS:
        if (p1eit->m & EncodedInformationTypes_g4_class_1_and_mixed_mode) {
    	  struct any_type *pAnyValue;
			pAnyValue = (struct any_type *) p1eit->g4_class_1_and_mixed_mode;

        	if (len = pAnyValue->len) {
            	if ((status = p_strcpy(desc,&desc->attr_value,MH_T_G4_FAX_NBPS,
					OM_S_OCTET_STRING, pAnyValue->ptr, pAnyValue->len)) 
					!= OM_SUCCESS)
            		return(status);
        	} /* end if */
        } /* end if */
        break;

    case MH_T_TELETEX_NBPS:
        if (p1eit->m & EncodedInformationTypes_teletex) {
        /* Create Teletex NBPS subobject */
        if ((status = mhp_p_teletex_nbps(p1eit->teletex,&subobject)) 
							!= OM_SUCCESS)
            return(status);
        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_TELETEX_NBPS,
								OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    } /* end switch */
    } /* end for */

    return(OM_SUCCESS);
} /* end mhp_p_eits() */





/*
 * mhp_p_g3fax_nbps()
 */

mhp_p_g3fax_nbps(g3_facsimile, object)
bool *g3_facsimile;
OM_int_private_object_header **object;
{
int i, status;
OM_int_descriptor *desc;

    if ((status = omp_template(MH_C_G3_FAX_NBPS,object)) != OM_SUCCESS)
    return(status);

    /* Index through G3 Facsimile Non Basic Parameters descriptor list
     * sequentially and fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_TWO_DIMENSIONAL:
        if (g3_facsimile[G3_FACSIMILE_two_dimensional]) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_TWO_DIMENSIONAL,OM_S_BOOLEAN,(void*)TRUE)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_FINE_RESOLUTION:
        if (g3_facsimile[G3_FACSIMILE_fine_resolution]) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_FINE_RESOLUTION,OM_S_BOOLEAN,(void*)TRUE)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_UNLIMITED_LENGTH:
        if (g3_facsimile[G3_FACSIMILE_unlimited_length]) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_UNLIMITED_LENGTH,OM_S_BOOLEAN,(void*)TRUE)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_B4_LENGTH:
        if (g3_facsimile[G3_FACSIMILE_b4_length]) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_B4_LENGTH,OM_S_BOOLEAN,(void*)TRUE)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_A3_WIDTH:
        if (g3_facsimile[G3_FACSIMILE_a3_width]) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_A3_WIDTH,OM_S_BOOLEAN,(void*)TRUE)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_B4_WIDTH:
        if (g3_facsimile[G3_FACSIMILE_b4_width]) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_B4_WIDTH,OM_S_BOOLEAN,(void*)TRUE)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_UNCOMPRESSED:
        if (g3_facsimile[G3_FACSIMILE_uncompressed]) {
        if ((status = putvalue(desc,&desc->attr_value,MH_T_UNCOMPRESSED,OM_S_BOOLEAN,(void*)TRUE)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_g3fax_nbps() */



/*
 * mhp_p_teletex_nbps()
 *
 */



mhp_p_teletex_nbps(teletex, object)
P1TeletexNonBasicParameters *teletex;
OM_int_private_object_header **object;
{
int i, status, len;
OM_int_descriptor *desc;

    if ((status = omp_template(MH_C_TELETEX_NBPS,object)) != OM_SUCCESS)
    return(status);

    /* Index through Teletex Non Basic Parameters descriptor list
     * sequentially and fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_CONTROL_CHARACTER_SETS:
        if (teletex->m & TeletexNonBasicParameters_control_character_sets) {
        if (len = strlen(teletex->control_character_sets)) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_CONTROL_CHARACTER_SETS,
            OM_S_TELETEX_STRING,teletex->control_character_sets,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_GRAPHIC_CHARACTER_SETS:
        if (teletex->m & TeletexNonBasicParameters_graphic_control_sets) {
        if (len = strlen(teletex->graphic_control_sets)) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_GRAPHIC_CHARACTER_SETS,
            OM_S_TELETEX_STRING,teletex->graphic_control_sets,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_MISCELLANEOUS_CAPABILITIES:
        if (teletex->m & TeletexNonBasicParameters_miscellaneous_terminal_capabilities) {
        if (len = strlen(teletex->miscellaneous_terminal_capabilities)) {
            if ((status = p_strcpy(desc,&desc->attr_value,
					MH_T_MISCELLANEOUS_CAPABILITIES, OM_S_TELETEX_STRING,
					teletex->miscellaneous_terminal_capabilities, len)) != 
					OM_SUCCESS)
            	return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_PAGE_FORMATS:
        if (teletex->m & TeletexNonBasicParameters_page_formats) {
        if (len = teletex->page_formats.length) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_PAGE_FORMATS,OM_S_OCTET_STRING,
            teletex->page_formats.value,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_PRIVATE_USE:
        if (teletex->m & TeletexNonBasicParameters_private_use) {
        if (len = teletex->private_use.length) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_PRIVATE_USE,OM_S_OCTET_STRING,
            teletex->private_use.value,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;

    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_teletex_nbps() */



/*
 * mhp_p_extrace(trace, object) puts P1TraceInformationElement
 * structure into External Trace Entry object.
 */

int mhp_p_extrace(trace, object)
P1TraceInformationElement *trace;
OM_int_private_object_header **object;
{
char *str;
Uint32 len;
int i, status;
char *buf;
OM_enumeration	enumer;
OM_int_descriptor *desc;
OM_int_private_object_header *subobject;

    if ((status = omp_template(MH_C_EXTERNAL_TRACE_ENTRY,object)) != OM_SUCCESS)
    return(status);

    /* Index through External Trace Information descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_COUNTRY_NAME:
            if (trace->global_domain_identifier.m & GlobalDomainIdentifier_country_name) {
        if ((status = mhp_p_country(&str,&len,trace->global_domain_identifier.country_name)) != OM_SUCCESS)
            return(status);

        if (len)
            if ((status = putvalue(desc,&desc->attr_value,MH_T_COUNTRY_NAME,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_ADMD_NAME:
            if (trace->global_domain_identifier.m & GlobalDomainIdentifier_administration_domain_name) {
        if ((status = mhp_p_admdcpy(&str,&len,trace->global_domain_identifier.administration_domain_name))
            != OM_SUCCESS)
            return(status);
        if (len)
            if ((status = putvalue(desc,&desc->attr_value,MH_T_ADMD_NAME,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_PRMD_IDENTIFIER:
        if (trace->global_domain_identifier.m & GlobalDomainIdentifier_private_domain_identifier) {
        if ((status = mhp_p_prmdcpy(&str,&len,trace->global_domain_identifier.private_domain_identifier))
            != OM_SUCCESS)
            return(status);
        if (len)
            if ((status = putvalue(desc,&desc->attr_value,MH_T_PRMD_IDENTIFIER,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_ATTEMPTED_COUNTRY_NAME:
            if (trace->domain_supplied_information.m 
                 & DomainSuppliedInformation_attempted_domain) {
                if (trace->domain_supplied_information.attempted_domain.m &
                          GlobalDomainIdentifier_country_name) {
                    if ((status = mhp_p_country(&str,&len,
                            trace->domain_supplied_information.
                             attempted_domain.country_name)) != OM_SUCCESS)
                        return(status);
                    if (len)
                        if ((status = putvalue(desc,&desc->attr_value,
                                     MH_T_ATTEMPTED_COUNTRY_NAME,
                                     OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
                           return(status);
                } /* end if */
            } /* end if */
        break;

    case MH_T_ATTEMPTED_ADMD_NAME:
        if (trace->domain_supplied_information.m 
                     & DomainSuppliedInformation_attempted_domain) 
        {
            if (trace->domain_supplied_information.attempted_domain.m &
                      GlobalDomainIdentifier_administration_domain_name) {
                if ((status = mhp_p_admdcpy(&str,&len,
                       trace->domain_supplied_information.attempted_domain.
                       administration_domain_name)) != OM_SUCCESS)
                    return(status);
                if (len)
                    if ((status = putvalue(desc,&desc->attr_value,
                                    MH_T_ATTEMPTED_ADMD_NAME,
                                    OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
                        return(status);
            } /* end if */
        } /* end if */
        break;

    case MH_T_ATTEMPTED_PRMD_IDENTIFIER:
            if (trace->domain_supplied_information.m & DomainSuppliedInformation_attempted_domain) {
        if (trace->domain_supplied_information.attempted_domain.m &
            GlobalDomainIdentifier_private_domain_identifier) {
            if ((status = mhp_p_prmdcpy(&str,&len,
            trace->domain_supplied_information.attempted_domain.private_domain_identifier))
            != OM_SUCCESS)
            return(status);
            if (len)
            if ((status = putvalue(desc,&desc->attr_value,
                             MH_T_ATTEMPTED_PRMD_IDENTIFIER,
                             OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
                return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_ACTION:
            if (trace->domain_supplied_information.m & DomainSuppliedInformation_routing_action) {
        switch (trace->domain_supplied_information.routing_action) {
        case ROUTING_ACTION_relayed:
        default:
            enumer = MH_AC_RELAYED;
            break;
        case ROUTING_ACTION_rerouted:
            enumer = MH_AC_REROUTED;
            break;
        } /* end switch */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_ACTION,OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_ARRIVAL_TIME:
            if (trace->domain_supplied_information.m & DomainSuppliedInformation_arrival_time) {
        if ((buf = mhp_to_utc(&(trace->domain_supplied_information.arrival_time))) == NULL)
            return(OM_MEMORY_INSUFFICIENT);

        if ((status = putvalue(desc,&desc->attr_value,MH_T_ARRIVAL_TIME,OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_CONVERTED_EITS:
            if (trace->domain_supplied_information.m & DomainSuppliedInformation_converted_encoded_information_types) {
        /* Create and fill in Converted EITs sub-object */
        if ((status = mhp_p_eits(trace->domain_supplied_information.converted_encoded_information_types,
            &subobject)) != OM_SUCCESS)
            return(status);

        /* Create a value structure for the subobject and link it */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_CONVERTED_EITS,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_DEFERRED_TIME:
            if (trace->domain_supplied_information.m & DomainSuppliedInformation_deferred_time) {
        if ((buf = mhp_to_utc(&(trace->domain_supplied_information.deferred_time))) == NULL)
            return(OM_MEMORY_INSUFFICIENT);

        if ((status = putvalue(desc,&desc->attr_value,MH_T_DEFERRED_TIME,OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_extrace() */



/*
 * mhp_p_intrace(trace, object) puts PerMTATraceInfo1984 structure
 * into Internal Trace Entry object.
 *
 * INPUT:
 *  trace - PerMTATraceInfo1984 structure pointer
 *  object - address for returned handle
 *
 * OUTPUT:
 *  object - containing handle to InternalTraceEntry object
 *
 * Result:
 *  OM_SUCCESS or failure otherwise
 *
 */

int mhp_p_intrace(trace, object)
P1PerMTATraceInfo1984 *trace;
OM_int_private_object_header **object;
{
Uint32 len;
int i, status;
char *buf;
OM_enumeration	enumer;
OM_int_descriptor *desc;

    if ((status = omp_template(MH_C_INTERNAL_TRACE_ENTRY,object)) != OM_SUCCESS)
    return(status);

    /* Index through Internal Trace Information descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_MTA_NAME:
        if (trace->m & PerMTATraceInfo1984_MTAName1984) {
        if (len = strlen(trace->mTAName1984)) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_MTA_NAME,OM_S_IA5_STRING,
            trace->mTAName1984,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    case MH_T_ATTEMPTED_MTA_NAME:
        if (trace->m & PerMTATraceInfo1984_MTAInfo1984) {
        if (trace->mTAInfo1984.m & MTAInfo1984_previous) {
            if (len = strlen(trace->mTAInfo1984.previous)) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_ATTEMPTED_MTA_NAME,
                OM_S_IA5_STRING,trace->mTAInfo1984.previous,len)) != OM_SUCCESS)
                return(status);
            } /* end if */
        } /* end if */
        } /* end if */
        break;
    case MH_T_ACTION:
        if (trace->m & PerMTATraceInfo1984_MTAInfo1984) {
        if (trace->mTAInfo1984.m & MTAInfo1984_action1984) {
            switch (trace->mTAInfo1984.action1984) {
            case ACTION1984_relayed:
            default:
            enumer = MH_AC_RELAYED;
            break;
            case ACTION1984_rerouted:
            enumer = MH_AC_REROUTED;
            break;
            case ACTION1984_recipReassign:
            enumer = MH_AC_REDIRECTED;
            break;
            } /* end switch */
            if ((status = putvalue(desc,&desc->attr_value,MH_T_ACTION,OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    case MH_T_ARRIVAL_TIME:
        if (trace->m & PerMTATraceInfo1984_MTAInfo1984) {
        if (trace->mTAInfo1984.m & MTAInfo1984_arrival) {
            if ((buf = mhp_to_utc(&(trace->mTAInfo1984.arrival))) == NULL)
            return(OM_MEMORY_INSUFFICIENT);

            if ((status = putvalue(desc,&desc->attr_value,MH_T_ARRIVAL_TIME,OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    case MH_T_DEFERRED_TIME:
        if (trace->m & PerMTATraceInfo1984_MTAInfo1984) {
        if (trace->mTAInfo1984.m & MTAInfo1984_deferred) {
            if ((buf = mhp_to_utc(&(trace->mTAInfo1984.deferred))) == NULL)
            return(OM_MEMORY_INSUFFICIENT);

            if ((status = putvalue(desc,&desc->attr_value,MH_T_ARRIVAL_TIME,OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_intrace() */

/*
 * mhp_p_intrace88(trace, object) puts P1InternalTraceInformationElement (88)
 * structure into Internal Trace Entry object.
 *
 * INPUT:
 *  trace - P1InternalTraceInformationElement structure pointer
 *  object - address for returned handle
 *
 * OUTPUT:
 *  object - containing handle to InternalTraceEntry object
 *
 * Result:
 *  OM_SUCCESS or failure otherwise
 *
 */

int mhp_p_intrace88(trace, object)
P1InternalTraceInformationElement *trace;
OM_int_private_object_header **object;
{
Uint32 len;
int i, status;
char *buf;
char *str;
OM_enumeration	enumer;
OM_int_descriptor *desc;
OM_int_private_object_header *subobject;

    if ((status = omp_template(MH_C_INTERNAL_TRACE_ENTRY,object)) != OM_SUCCESS)
    return(status);

    /* Index through Internal Trace Information descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {

    case MH_T_MTA_NAME:
        if (trace->m & InternalTraceInformationElement_mta_name) {
        if (len = strlen(trace->mta_name)) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_MTA_NAME,OM_S_IA5_STRING,
            trace->mta_name,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;

    case MH_T_ADMD_NAME:
        if (trace->m & InternalTraceInformationElement_global_domain_identifier)
        {
        	if (trace->global_domain_identifier.m & GlobalDomainIdentifier_administration_domain_name) 
		{
            		if ((status = mhp_p_admdcpy(&str,&len,
	    		trace->global_domain_identifier.administration_domain_name )) 
			!= OM_SUCCESS)
            			return(status);
            		if (len)
            			if ((status = putvalue(desc,&desc->attr_value,
				MH_T_ADMD_NAME,OM_S_PRINTABLE_STRING,str)) 
				!= OM_SUCCESS)
                			return(status);
        	} /* end if */
        } /* end if */
        break;
    case MH_T_PRMD_IDENTIFIER:
        if (trace->m & InternalTraceInformationElement_global_domain_identifier)
        {
        	if (trace->global_domain_identifier.m & GlobalDomainIdentifier_private_domain_identifier) 
		{
        if ((status = mhp_p_prmdcpy(&str,&len,trace->global_domain_identifier.private_domain_identifier))
            != OM_SUCCESS)
            return(status);
        if (len)
            if ((status = putvalue(desc,&desc->attr_value,MH_T_PRMD_IDENTIFIER,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
            return(status);
        } /* end if */
	}
        break;
    case MH_T_COUNTRY_NAME:
        if (trace->m & InternalTraceInformationElement_global_domain_identifier)
        {
        	if (trace->global_domain_identifier.m & GlobalDomainIdentifier_country_name) {
            		if ((status = mhp_p_country(&str,&len,
	    		trace->global_domain_identifier.country_name )) 
			!= OM_SUCCESS)
            			return(status);
            		if (len)
            			if ((status = putvalue(desc,&desc->attr_value,
				MH_T_COUNTRY_NAME,OM_S_PRINTABLE_STRING,str)) 
				!= OM_SUCCESS)
                			return(status);
             } /* end if */
        } /* end if */
        break;
    case MH_T_ATTEMPTED_MTA_NAME:
        if (trace->m & InternalTraceInformationElement_mta_supplied_information)
	{
        if (trace->mta_supplied_information.m &  
            MTASuppliedInformation_attempted ) 
	{ 
	if   (trace->mta_supplied_information.attempted.t ==
	     MTASuppliedInformation_mta)
	{
            if (len = strlen(trace->mta_supplied_information.attempted.v.mta)) 
	    {
            if ((status = p_strcpy(desc,&desc->attr_value,
	                           MH_T_ATTEMPTED_MTA_NAME,
                	           OM_S_IA5_STRING,
		trace->mta_supplied_information.attempted.v.mta,len)) 
		!= OM_SUCCESS)
                return(status);
            } /* end if */
        } /* end if */
        } /* end if */
	}
        break;

/* 
 * Either Attempted Global Domain or Attempted MTA Name is present, but
 * not both. However, there's no effect here as the p1p2.c parser
 * should behave correctly already.
 */
    case MH_T_ATTEMPTED_ADMD_NAME:
        if ( (trace->m & MTASuppliedInformation_attempted)
             && (trace->mta_supplied_information.attempted.t 
                 == MTASuppliedInformation_domain)
             && (trace->mta_supplied_information.attempted.v.domain.m 
                 & GlobalDomainIdentifier_administration_domain_name)) {
            if ((status = mhp_p_admdcpy(&str, &len, 
                          trace-> mta_supplied_information.attempted.v.domain.
                          administration_domain_name)) != OM_SUCCESS)
                return(status);
            if (len)
                if ((status = putvalue(desc, &desc->attr_value,
                              MH_T_ATTEMPTED_ADMD_NAME, OM_S_PRINTABLE_STRING, 
                              str)) != OM_SUCCESS)
                    return(status);
        } /* end if */
             
	break;

    case MH_T_ATTEMPTED_COUNTRY_NAME:
        if ( (trace->m & MTASuppliedInformation_attempted)
             && (trace->mta_supplied_information.attempted.t 
                 == MTASuppliedInformation_domain)
             && (trace->mta_supplied_information.attempted.v.domain.m 
                 & GlobalDomainIdentifier_country_name)) {
            if ((status = mhp_p_country(&str, &len, 
                          trace-> mta_supplied_information.attempted.v.domain.
                          country_name)) != OM_SUCCESS)
                return(status);
            if (len)
                if ((status = putvalue(desc, &desc->attr_value,
                              MH_T_ATTEMPTED_COUNTRY_NAME, 
                              OM_S_PRINTABLE_STRING, 
                              str)) != OM_SUCCESS)
                    return(status);
        } /* end if */
        
	break;

    case MH_T_ATTEMPTED_PRMD_IDENTIFIER:
        if ( (trace->m & MTASuppliedInformation_attempted)
             && (trace->mta_supplied_information.attempted.t 
                 == MTASuppliedInformation_domain)
             && (trace->mta_supplied_information.attempted.v.domain.m 
                 & GlobalDomainIdentifier_private_domain_identifier)) {
            if ((status = mhp_p_prmdcpy(&str, &len, 
                          trace-> mta_supplied_information.attempted.v.domain.
                          private_domain_identifier)) != OM_SUCCESS)
                return(status);
            if (len)
                if ((status = putvalue(desc, &desc->attr_value,
                              MH_T_ATTEMPTED_PRMD_IDENTIFIER, 
                              OM_S_PRINTABLE_STRING, 
                              str)) != OM_SUCCESS)
                    return(status);
        } /* end if */
	break;

    case MH_T_ACTION:
        if (trace->m & InternalTraceInformationElement_mta_supplied_information)
	{
        	if (trace->mta_supplied_information.m &  
                    MTASuppliedInformation_routing_action)
		{
            	switch (trace->mta_supplied_information.routing_action) 
	    		{
            		case ROUTING_ACTION_relayed:
            		default:
            			enumer = MH_AC_RELAYED;
            			break;
            		case ROUTING_ACTION_rerouted:
            			enumer = MH_AC_REROUTED;
            			break;
            		case ROUTING_ACTION_recipReassign:
            			enumer = MH_AC_REDIRECTED;
            			break;
            		} /* end switch */
            	if ((status = putvalue(desc,&desc->attr_value,MH_T_ACTION,
	 		 OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
            		return(status);
        	} /* end if */
	}
        break;

    case MH_T_ARRIVAL_TIME:
        if (trace->m & InternalTraceInformationElement_mta_supplied_information)
	{
        	if (trace->mta_supplied_information.m &  
                    MTASuppliedInformation_arrival_time)
                {
            		if ((buf = mhp_to_utc
		        	(&(trace->mta_supplied_information.arrival_time))) == NULL)
            			return(OM_MEMORY_INSUFFICIENT);

            		if ((status = putvalue(desc,&desc->attr_value,
		                    	MH_T_ARRIVAL_TIME,OM_S_UTC_TIME_STRING,
					buf)) != OM_SUCCESS)
            			return(status);
                } /* end if */
        } /* end if */
        break;

    case MH_T_CONVERTED_EITS:
        if (trace->m & InternalTraceInformationElement_mta_supplied_information)
	{
        	if (trace->mta_supplied_information.m &  
		MTASuppliedInformation_converted_encoded_information_types) 
		{
		    if ((status = mhp_p_eits( trace->mta_supplied_information.
			converted_encoded_information_types,    
                        &subobject)) != OM_SUCCESS)
                		return(status);
            	    if ((status = putvalue(desc,&desc->attr_value,
                        MH_T_CONVERTED_EITS,OM_S_OBJECT,subobject))
                        != OM_SUCCESS)
                		return(status);
        	} /* end if */
        } /* end if */
        break;
    case MH_T_DEFERRED_TIME:
        if (trace->m & InternalTraceInformationElement_mta_supplied_information)
	{
        	if (trace->mta_supplied_information.m &  
                    MTASuppliedInformation_deferred_time)
                {

            		if ((buf = mhp_to_utc
		           (&(trace->mta_supplied_information.deferred_time))) == NULL)
            			return(OM_MEMORY_INSUFFICIENT);

            		if ((status = putvalue(desc,&desc->attr_value,
		                    	MH_T_ARRIVAL_TIME,OM_S_UTC_TIME_STRING,
					buf)) != OM_SUCCESS)
            			return(status);
        	} /* end if */
        } /* end if */
        break;
    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_intrace() */


/*
 * mhp_p_bilat()
 */

mhp_p_bilat(bilat,object)
P1PerDomainBilateralInformation *bilat;
OM_int_private_object_header **object;
{
char *str;
Uint32 len;
int i, status;
OM_int_descriptor *desc;

    if ((status = omp_template(MH_C_BILATERAL_INFO,object)) != OM_SUCCESS)
    return(status);

    /* Index through Bilateral Information descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_COUNTRY_NAME:
            if (bilat->m & PerDomainBilateralInformation_country_name) {
        if ((status = mhp_p_country(&str,&len,bilat->country_name)) != OM_SUCCESS)
            return(status);

        if (len)
            if ((status = putvalue(desc,&desc->attr_value,MH_T_COUNTRY_NAME,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_ADMD_NAME:
            if (bilat->m & PerDomainBilateralInformation_choice) {
            if (bilat->choice.t == PerDomainBilateralInformation_administration_domain_name) {
            if ((status = mhp_p_admdcpy(&str,&len,bilat->choice.v.administration_domain_name)) != OM_SUCCESS)
            return(status);
            if (len)
            if ((status = putvalue(desc,&desc->attr_value,MH_T_ADMD_NAME,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
                return(status);
        } else if (bilat->choice.t == PerDomainBilateralInformation_sequence) {
                if (bilat->choice.v.sequence.m & PerDomainBilateralInformation_administration_domain_name_with_prmd) {
            if ((status = mhp_p_admdcpy(&str,&len,bilat->choice.v.sequence.administration_domain_name_with_prmd))
                != OM_SUCCESS)
                return(status);
            if (len)
                if ((status = putvalue(desc,&desc->attr_value,MH_T_ADMD_NAME,OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
                return(status);
            } /* end if */
        } /* end if */
        } /* end if */
        break;
    case MH_T_PRMD_IDENTIFIER:
            if (bilat->m & PerDomainBilateralInformation_choice) {
            if (bilat->choice.t == PerDomainBilateralInformation_sequence) {
                if (bilat->choice.v.sequence.m & PerDomainBilateralInformation_private_domain_identifier) {
            if ((status = mhp_p_prmdcpy(&str,&len,bilat->choice.v.sequence.private_domain_identifier))
                != OM_SUCCESS)
                return(status);
            if (len)
                if ((status = putvalue(desc,&desc->attr_value,MH_T_PRMD_IDENTIFIER,
                OM_S_PRINTABLE_STRING,str)) != OM_SUCCESS)
                return(status);
            } /* end if */
        } /* end if */
        } /* end if */
        break;
    case MH_T_INFORMATION:
            if (bilat->m & PerDomainBilateralInformation_bilateral_information) {
        if (len = bilat->bilateral_information.len) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_INFORMATION,OM_S_OCTET_STRING,
            bilat->bilateral_information.ptr,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_bilat() */






/*
 * mhp_p_origincheck()
 */

mhp_p_origincheck(oc,object)
P1ProbeOriginAuthenticationCheck *oc;
OM_int_private_object_header **object;
{
char *str;
Uint32 len;
int i, status;
Uint16 count;
OM_int_descriptor *desc;
OM_enumeration	enumer;
OM_int_value *baseval, *curval, *nextval;

    if ((status = omp_template(MH_C_ALGORITHM_AND_RESULT,object)) != OM_SUCCESS)
    return(status);

    /* Index through Probe Origin Check descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) 
    {
    case MH_T_ALGORITHM_RESULT:
            break;
    default:
            break;
    } /* end switch */

  } /* end for */

  return(OM_SUCCESS);

} /* end mhp_p_origincheck() */

/*
 mhp_p_orig_cert()
 */

mhp_p_orig_cert(orc,object)
P1OriginatorCertificate *orc; 
OM_int_private_object_header **object;
{
char *str;
Uint32 len;
int i, status;
Uint16 count;
OM_int_descriptor *desc;
OM_enumeration	enumer;
OM_int_value *baseval, *curval, *nextval;

    if ((status = omp_template(OMP_O_DS_C_CERTIFICATE,object)) != OM_SUCCESS)

    return(status);

    /* Index through Probe Originator Certificate descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case DS_ISSUER: 
        break; 
    case DS_SIGNATURE:
        break;
    case DS_SIGNATURE_VALUE:
        break;
    case DS_SERIAL_NUMBER:
        break;
    case DS_SUBJECT:
        break;
    case DS_SUBJECT_ALGORITHM:
        break;
    case DS_VALIDITY_NOT_AFTER:
        break;
    case DS_VALIDITY_NOT_BEFORE:
        break;
    case DS_VERSION:
        break;
    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_orig_cert() */

/*
 * mhp_p_seclab()
 */

mhp_p_seclab(sl,object)
P1SecurityLabel *sl;
OM_int_private_object_header **object;
{
char *str;
Uint32 len;
int i, status;
Uint16 count;
OM_int_descriptor *desc;
OM_enumeration	enumer;
OM_int_value *baseval, *curval, *nextval;
OM_string string;

    if ((status = omp_template(MH_C_SECURITY_LABEL,object)) != OM_SUCCESS)
    return(status);

    /* Index through Message Security Label descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_PRIVACY_MARK:
            if (sl->m & SecurityLabel_privacy_mark) {
        if (len = strlen((char *)sl->privacy_mark)) {
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_PRIVACY_MARK,OM_S_PRINTABLE_STRING,
            sl->privacy_mark,len)) != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    case MH_T_SECURITY_POLICY_ID:
            if (sl->m & SecurityLabel_security_policy_identifier) {
        if (len = ((P1SecurityPolicyIdentifier *)sl->security_policy_identifier)->length) {
	    mhp_addobj (&string, ((P1SecurityPolicyIdentifier *)sl->security_policy_identifier));
            if ((status = p_strcpy(desc,&desc->attr_value,MH_T_SECURITY_POLICY_ID,OM_S_OBJECT_IDENTIFIER_STRING,
				(void*) string.elements,string.length))
            != OM_SUCCESS)
            return(status);
        } /* end if */
        } /* end if */
        break;
    case MH_T_SECURITY_CLASSIFICATION:
            if (sl->m & SecurityLabel_security_classification) {
        switch ((P1SecurityClassification)sl->security_classification) {
        case SECURITYCLASSIFICATION_unmarked:
        default:
            enumer = MH_SC_UNMARKED;
            break;
        case SECURITYCLASSIFICATION_unclassified:
            enumer = MH_SC_UNCLASSIFIED;
            break;
        case SECURITYCLASSIFICATION_restricted:
            enumer = MH_SC_RESTRICTED;
            break;
        case SECURITYCLASSIFICATION_confidential:
            enumer = MH_SC_CONFIDENTIAL;
            break;
        case SECURITYCLASSIFICATION_secret:
            enumer = MH_SC_SECRET;
            break;
        case SECURITYCLASSIFICATION_top_secret:
            enumer = MH_SC_TOP_SECRET;
            break;
        } /* end switch */
        if ((status = putvalue(desc,&desc->attr_value,MH_T_ACTION,OM_S_ENUMERATION,(void*)enumer)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;
    case MH_T_SECURITY_CATEGORY_DATA:
            if (sl->m & SecurityLabel_security_categories) {
		int i;   /* Not to be confused with outer loop i */
        for (i = 0, baseval = curval = nextval = NULL, count = 0;
            i < ((P1SecurityCategories *)sl->security_categories)->n; i++) {
            if (((P1SecurityCategory *)((P1SecurityCategories *)sl->security_categories)->v[i])->m &
            SecurityCategory_securityvalue) {
                } /* end if */
        } /* end for */
        } /* end if */
        break;
    case MH_T_SECURITY_CATEGORY_IDS:
            if (sl->m & SecurityLabel_security_categories) {
		int i;   /* Not to be confused with outer loop i */
        for (i = 0, baseval = curval = nextval = NULL, count = 0;
            i < ((P1SecurityCategories *)sl->security_categories)->n; i++) {
            if (((P1SecurityCategory *)((P1SecurityCategories *)sl->security_categories)->v[i])->m &
            SecurityCategory_securitytype) {
            if (len = ((P1SecurityCategory *)((P1SecurityCategories *)
                sl->security_categories)->v[i])->securitytype.length) {
	    	mhp_addobj (&string, & (((P1SecurityCategory *)((P1SecurityCategories *) sl->security_categories)->v[i])->securitytype) );
                if ((status = p_strcpy(desc,&nextval,MH_T_SECURITY_CATEGORY_IDS,OM_S_OBJECT_IDENTIFIER_STRING,(void*)
                		string.elements,string.length)) != OM_SUCCESS)
                    return(status);
                } /* end if */

            append_struct(&baseval,&curval,nextval,&count);
            } /* end if */
        } /* end for */

        desc->attr_value = baseval;
        desc->count = count;
        } /* end if */
        break;
    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_seclab() */






/*
 * mhp_p_exphist()
 */

mhp_p_exphist(eh,object)
P1DLExpansion *eh;
OM_int_private_object_header **object;
{
char *str;
char *buf;
Uint32 len;
int i, status;
OM_int_descriptor *desc;
OM_int_private_object_header *subobject;

    if ((status = omp_template(MH_C_EXPANSION_RECORD,object)) != OM_SUCCESS)
    return(status);

    /* Index through Expansion Record descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_NAME:
         /* mask bit is not set for reqd. elems .. from SB */
            if (DLExpansion_ORAddressAndOptionalDirectoryName) {
        /* Create and fill in OR Name sub-object */
            if ((status = mhp_p_orname(eh->oRAddressAndOptionalDirectoryName,&subobject)) != OM_SUCCESS)
            return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&desc->attr_value,MH_T_NAME,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_TIME:
         /* mask bit is not set for reqd. elems .. from SB */
            if (DLExpansion_redirection_time) {
        if ((buf = mhp_to_utc((struct atime*)&(eh->redirection_time))) == NULL)
            return(OM_MEMORY_INSUFFICIENT);

        if ((status = putvalue(desc,&desc->attr_value,MH_T_TIME,OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_exphist() */




/*
 * mhp_p_origAndexphist()
 */


mhp_p_origAndexphist(eh,object)
P1OriginatorAndDLExpansion *eh;
OM_int_private_object_header **object;
{
char *str;
char *buf;
Uint32 len;
int i, status;
OM_int_descriptor *desc;
OM_int_private_object_header *subobject;

    if ((status = omp_template(MH_C_EXPANSION_RECORD,object)) != OM_SUCCESS)
    return(status);

    /* Index through Expansion Record descriptor list sequentially and
     * fill in attribute if it exists. */
    for (i = TOP; (*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; i++) {
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_NAME:
         /* mask bit is not set for reqd. elems .. from SB */
            if (OriginatorAndDLExpansion_originator_or_dl_name) {
        /* Create and fill in OR Name sub-object */
            if ((status = mhp_p_orname(eh->originator_or_dl_name,&subobject)) != OM_SUCCESS)
            return(status);

            /* Create a value structure for the subobject and link it */
            if ((status = putvalue(desc,&desc->attr_value,MH_T_NAME,OM_S_OBJECT,subobject)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    case MH_T_TIME:
         /* mask bit is not set for reqd. elems .. from SB */
            if (OriginatorAndDLExpansion_origination_or_expansion_time) {
        if ((buf = mhp_to_utc((struct atime*)&(eh->origination_or_expansion_time))) == NULL)
            return(OM_MEMORY_INSUFFICIENT);

        if ((status = putvalue(desc,&desc->attr_value,MH_T_TIME,OM_S_UTC_TIME_STRING,buf)) != OM_SUCCESS)
            return(status);
        } /* end if */
        break;

    } /* end switch */
    } /* end for */
    return(OM_SUCCESS);
} /* end mhp_p_origAndexphist() */





/*
 * mhp_p_binary(content,object) puts the content octect string into
 * General Content object.
 *
 * INPUT:
 *  content - binary content
 *
 * OUTPUT:
 *      General Content object
 *
 * NOTE:
 *      The binary should be of limited size since it resides
 *      in memory (vs. BodyPart which is stored in file).
 *
 * TO DO: not finished.
 */

mhp_p_binary(content, object)
int *content;
OM_int_private_object_header **object;
{
int i, status;
OM_int_descriptor *desc;

    if ((status = omp_template(MH_C_GENERAL_CONTENT,object)) != OM_SUCCESS)
    return(status);

    /* Index through General Content descriptor list sequentially and fill in
     * attribute if it exists. */
    for (i = TOP; 
		(*object)->context.descriptor_list[i].attr_type != OM_NO_MORE_TYPES; 
		i++) 
	{
    desc = (OM_int_descriptor_list)&((*object)->context.descriptor_list[i]);
    switch (desc->attr_type) {
    case MH_T_BINARY_CONTENT:
        if ((status = write_str(desc, &desc->attr_value,
				MH_T_BINARY_CONTENT,OM_S_OCTET_STRING, (char*) content)) 
				!= OM_SUCCESS)
        	return(status);
        break;
    } /* end switch */
    } /* end for */

    return(OM_SUCCESS);

} /* mhp_p_binary() */






/*
 * mhp_p_admdcpy(to, size, from) picks out the appropriate type of ADMD Name
 * (printable or numeric) and copies to "to" buffer.
 *
 * INPUT:
 *  from - address of string to be copied.
 *
 * OUTPUT:
 *  to   - address to string being copied to (null-terminated).
 *  size - size of string (not including null).
 *
 */

int mhp_p_admdcpy(to, size, from)
char **to;
Uint32 *size;
P1AdministrationDomainName *from;
{
char *buf;
int len=0;

    if (from->t ==  AdministrationDomainName_numeric) {
    if (len = strlen(from->v.numeric)) {
        if ((buf = (char *)os_calloc(len+1)) == NULL)
        return(OM_MEMORY_INSUFFICIENT);
        strncpy(buf, from->v.numeric, len);
        buf[len] = '\0';
    } /* end if */
    } /* end if */

    else if (from->t == AdministrationDomainName_printable) {
    if (len = strlen(from->v.printable)) {
        if ((buf = (char *)os_calloc(len+1)) == NULL)
        return(OM_MEMORY_INSUFFICIENT);
        strncpy(buf, from->v.printable, len);
        buf[len] = '\0';
    } /* end if */
    } /* end if */

    if (len) { /* in case zero length field */
		*to = buf;
	}
	else *to = NULL;  /* zero length field */
	
    *size = len;
    return(OM_SUCCESS);
} /* end mhp_p_admdcpy() */



/*
 * mhp_p_prmdcpy(to, from) picks out the appropriate numeric or
 * printable PRMD Name and copies to buffer.
 *
 */

int mhp_p_prmdcpy(to, size, from)
char **to;
Uint32 *size;
P1PrivateDomainName *from;
{
int len=0;
char *buf;

    if (from->t ==  PrivateDomainName_numeric) {
    if (len = strlen(from->v.numeric)) {
        if ((buf = (char *)os_calloc(len+1)) == NULL)
        return(OM_MEMORY_INSUFFICIENT);
        strncpy(buf, from->v.numeric, len);
        buf[len] = '\0';
    } /* end if */
    } /* end if */

    else if (from->t == PrivateDomainName_printable) {
    if (len = strlen(from->v.printable)) {
        if ((buf = (char *)os_calloc(len+1)) == NULL)
        return(OM_MEMORY_INSUFFICIENT);
        strncpy(buf, from->v.printable, len);
        buf[len] = '\0';
    } /* end if */
    } /* end if */

    if (len)
		*to = buf;
	else *to = NULL; /* zero length PRMD */
    *size = len;
    return(OM_SUCCESS);
} /* end mhp_p_prmdcpy() */




/*
 * mhp_p_country(to, size, from) picks out and copies the appropriate
 * (printable or numeric) country name.
 *
 */

int mhp_p_country(to, size, from)
char **to;
Uint32 *size;
P1CountryName *from;
{
int len=0;
char *buf;

    if (from->t == CountryName_x121_dcc_code) {
    if (len = strlen(from->v.x121_dcc_code)) {
        if ((buf = (char *)os_calloc(len+1)) == NULL)
        return(OM_MEMORY_INSUFFICIENT);
        strncpy(buf, from->v.x121_dcc_code, len);
        buf[len] = '\0';
    } /* end if */
    } /* end if */

    if (from->t == CountryName_iso_3166_alpha2_code) {
    if (len = strlen(from->v.iso_3166_alpha2_code)) {
        if ((buf = (char *)os_calloc(len+1)) == NULL)
        return(OM_MEMORY_INSUFFICIENT);
        strncpy(buf, from->v.iso_3166_alpha2_code, len);
        buf[len] = '\0';
    } /* end if */
    } /* end if */

	if (len)
    	*to = buf;
	else *to = NULL; /* zero length field */
    *size = len;
    return(OM_SUCCESS);
} /* end mhp_p_country() */






mhp_p_p1pdsp(desc,orname,type,exttype,value,count)
OM_int_descriptor *desc;
P1ORName *orname;
OM_type type;
sint16 exttype;
OM_int_value *value;
Uint16 *count;
{
Uint32 len;
int status;
int *evalue;
OM_int_value *baseval, *curval, *nextval;

    baseval = curval = nextval = NULL;
    *count = 0;

    if (evalue = (int *)get_address_extension(orname,exttype)) {
    if (((P1PDSParameter *)evalue)->m & PDSParameter_printable_string) {
        if (len = strlen(((P1PDSParameter *)evalue)->printable_string)) {
        if ((status = p_strcpy(desc,&nextval,type,OM_S_PRINTABLE_STRING,(void*)
            ((P1PDSParameter *)evalue)->printable_string,len)) != OM_SUCCESS)
            return(status);

        append_struct(&baseval,&curval,nextval,count);
        } /* end if */
    } /* end if */

    if (((P1PDSParameter *)evalue)->m & PDSParameter_teletex_string) {
        if (len = strlen(((P1PDSParameter *)evalue)->teletex_string)) {
        if ((status = p_strcpy(desc,&nextval,type,OM_S_TELETEX_STRING,(void*)
            ((P1PDSParameter *)evalue)->teletex_string,len)) != OM_SUCCESS)
            return(status);

        append_struct(&baseval,&curval,nextval,count);
        } /* end if */
    } /* end if */
    } /* end if */

    value = baseval;
    return(OM_SUCCESS);
} /* end mhp_p_p1pdsp() */





/*
 * p_strcpy(curval, type, syntax, from) copies a given string
 * from the asn1 structure into the descriptor list element.
 *
 * **NOTE:**
 * This function allocates new memory for the string, so the caller
 * should not used this function if destination string is already
 * allocated. Or, if it's called, make sure the other one is freed.
 * 
 * **WARNING:**
 * (1) len supplied should be greater than zero.
 * (2) this functions called putvalue() which calculates a null-terminated
 * string and set to the string's length component. NULL should always be 
 * set for the end. For binary data, this is not good, so the caller
 * MUST reset the length (for example, in the case of disk-based string,
 * the length should be file size, and not size of "elements" component.
 * KDH 
 * 
 *
 * OUTPUT:
 *  returning success if the string was copies, otherwise report errors.
 *
 */

int p_strcpy( OM_int_descriptor *desc, 
              OM_int_value **curval, 
              OM_type type,
              OM_syntax syntax,
              void *from,
              Uint32 len) /* should always be greater than 0 */
{
    char *buf;

    if ((buf = (char *)os_calloc(len+1)) == NULL)
        return(OM_MEMORY_INSUFFICIENT);
    sl_bcopy(from, buf, len); /* covers both character and binary string */
    buf[len] = '\0';
    return(putvalue(desc,curval, type, syntax, buf));
} /* end p_strcpy()  */


/* Please do not use this routine as it is incorrect; use routines
   mhp_putbits() as a start instead */
int p_bitcpy( OM_int_descriptor *desc, 
              OM_int_value **ivalue, 
              OM_type type,
              struct bits from)
{
    int len;
    OM_int_value *curval;
    OM_descriptor *tmpdesc;
    char *buf;

    /* Create value structure */
    if ((curval = (OM_int_value *)os_calloc(sizeof(OM_int_value))) == NULL)
        return(OM_MEMORY_INSUFFICIENT);

    if ((buf = (char *)os_calloc(sizeof(sint16)+1)) == NULL)
        return(OM_MEMORY_INSUFFICIENT);

    curval->next = NULL;

    len = from.length + 2; /* residue is 2 bytes)
    curval->value.string.length = (OM_string_length) len;
    /* copy the residue first*/
    memcpy(buf, &from.residue, sizeof(sint16)); 
    buf += sizeof (sint16);
    /* copy the remaining value */
    memcpy (buf, &from.data, from.length); 
    curval->value.string.elements = buf;
    return (OM_SUCCESS);

} /* end p_bitcpy()  */

/*
 * putvalue(desc,curval,type,syntax,value) put a value (string, integer,
 * enumeration, object, boolean) into a descriptor.
 * The object must be a (OM_int_private_object_header *).
 * This function will set value.object.object to the 0-th descriptor.
 * count is incremented.
 *
 * ** WARNING **
 * The caller might want to set the "length" component, esp. for binary,
 * in case strlen() does not return the right size.
 *
 *
 */

int putvalue(
    OM_int_descriptor   *desc,
    OM_int_value **ivalue,
    OM_type type,
    OM_syntax syntax,
    void *value)
{
int len;
OM_int_value *curval;
OM_descriptor *tmpdesc;

    /* Create value structure */
    if ((curval = (OM_int_value *)os_calloc(sizeof(OM_int_value))) == NULL)
    return(OM_MEMORY_INSUFFICIENT);

    curval->next = NULL;

    switch (syntax) {
    case OM_S_NUMERIC_STRING:
    case OM_S_OCTET_STRING :
    case OM_S_PRINTABLE_STRING:
    case OM_S_TELETEX_STRING:
    case OM_S_UTC_TIME_STRING:
    case OM_S_VIDEOTEX_STRING:
    case OM_S_IA5_STRING:
    case OM_S_OBJECT_IDENTIFIER_STRING:
    case OM_S_OBJECT_DESCRIPTOR_STRING:
    case OM_S_ENCODING_STRING:
    	len = strlen((char *) value);
    	curval->value.string.length = (OM_string_length) len;
    	curval->value.string.elements = (char *) value;
    	break;
    case OM_S_ENUMERATION:
     	curval->value.enumeration = (OM_enumeration) value;
    	break;
    case OM_S_INTEGER:
    	curval->value.integer = (OM_integer) value;
    	break;
    case OM_S_BOOLEAN:
    	curval->value.boolean = (OM_boolean) value;
    	break;
    case OM_S_OBJECT:
    	tmpdesc = (OM_descriptor *)value;
    	curval->value.object.object = (OM_object) ++tmpdesc;
    	break;
    default: /* should never an error, since used internally only */
    	;
    } /* end switch */
    curval->syntax = syntax;
    *ivalue = (OM_int_value *) curval;
    desc->count++;
    return(OM_SUCCESS);
} /* end putvalue() */



/*
 * append mutliple values to the descriptor.
 the caller MUST MUST initialise base to NULL when calling first time..

 If (basevalue equal to NULL) -- first time
	let base point to "next" -- first value
 else
	let last time's value(cur)'s next  point to "next" -- present value

 remember present value as cur
 set present value's next to NULL
 count ++

 voila! we are done.. -- shekhar
*/

append_struct(base,cur,next,count)
OM_int_value **base;
OM_int_value **cur;
OM_int_value *next;
Uint16 *count;
{
    if (*base == NULL)
        *base = next;
    else
        (*cur)->next = next;
    *cur = next;
    next->next = NULL;
    (*count)++;
} /* end append_struct() */





get_address_extension(orname,type)
P1ORName *orname;
sint16 type;
{
int j;
P1ExtensionAttribute  *ext;
P1ExtensionAttributes *exts;

    exts = (P1ExtensionAttributes *)orname->extension_attributes;

    if (orname->m & ORName_extension_attributes) {
    for (j = 0; j < exts->n; j++) {
        ext = (P1ExtensionAttribute *)exts->v[j];
        if ((ext->m & ExtensionAttribute_extension_attribute_type) &&
        (ext->m & ExtensionAttribute_extension_attribute_value) &&
        (ext->extension_attribute_type == type)) {
        return((int)(ext->extension_attribute_value));
        } /* end if */
    } /* end for */
    } /* end if */
    return(NULL);
} /* end get_address_extension() */


char *get_add_exts(P1ORName *orname, sint16 type)
{
int j;
P1ExtensionAttribute  *ext;
P1ExtensionAttributes *exts;

    exts = (P1ExtensionAttributes *)orname->extension_attributes;

    if (orname->m & ORName_extension_attributes) {
    for (j = 0; j < exts->n; j++) {
        ext = (P1ExtensionAttribute *)exts->v[j];
        if ( ext->extension_attribute_type == type) {
	    parse_address_extensions(ext->extension_attribute_value,
	               ext->extension_attribute_type,
	               &(ext->eattrstruct));
            return((char *)(ext->eattrstruct));
        } /* end if */
    } /* end for */
    } /* end if */
    return(NULL);
} /* end get_add_exts() */




char *get_per_recip_msg_extension(
P1PerRecipientMessageTransferFields *rd,
sint16 type
)
{
    P1ExtensionMacroBody *extdata;
    struct any_type *evalue;

    if (rd->m & PerRecipientMessageTransferFields_extensions) {
    /* Traverse list of extensions */
    for ( list_first( & rd->extensions);
          (rd->extensions.current);
          list_next( & rd->extensions))
    {
        extdata = (P1ExtensionMacroBody *)rd->extensions.current->data;
        if (extdata->etype == type) 
	{
            if (extdata->m & ExtensionMacroBody_evalue)
                evalue = (struct any_type *)(extdata->evalue);
            else 
                evalue = NULL;
	    parse_envelope_extensions(evalue, extdata->etype, 
                                      &(extdata->extstruct));
            return((char *)(extdata->extstruct));
        } /* end if */
    } /* end for */
    } /* end if */

    return(NULL);

} /* end get_per_recip_msg_extension() */


char *get_perrecp_probe_extension(
P1PerRecipientProbeTransferFields *rd,
sint16 type
)
{
    P1ExtensionMacroBody *extdata;
    struct any_type *evalue;

    if (rd->m & PerRecipientProbeTransferFields_extensions) {
    /* Traverse list of extensions */
    for ( list_first( & rd->extensions);
          (rd->extensions.current);
          list_next( & rd->extensions))
    {
        extdata = (P1ExtensionMacroBody *)rd->extensions.current->data;
        if (extdata->etype == type) 
	{
            if (extdata->m & ExtensionMacroBody_evalue)
                evalue = (struct any_type *)(extdata->evalue);
            else 
                evalue = NULL;
	    parse_envelope_extensions(evalue, extdata->etype, 
                                      &(extdata->extstruct));
            return((char *)(extdata->extstruct));
        } /* end if */
    } /* end for */
    } /* end if */

    return(NULL);

} /* end get_perrecp_probe_extension() */


char *get_report_content_extension (P1ReportTransferContent *rte , sint16 type)
{
P1ExtensionMacroBody *extdata;
struct any_type *evalue;

    if (rte->m & ReportTransferContent_extensions) {
    /* Traverse list of extensions */
    for ( list_first(&(rte->extensions));
	(rte->extensions.current);
	list_next(&(rte->extensions) ) ) 
    {
        extdata = (P1ExtensionMacroBody *)rte->extensions.current->data;
        if (extdata->etype == type) 
	{
            if (extdata->m & ExtensionMacroBody_evalue) 
               evalue = (struct any_type *)(extdata->evalue); 
            else
               evalue = NULL;

            parse_envelope_extensions (evalue, extdata->etype,
	 				&(extdata->extstruct));
            return((char *)(extdata->extstruct));
        } /* end if */
    } /* end for */
    } /* end if */
    return(NULL);
} /* end get_report_content_extension() */


char *get_report_extension (P1ReportTransferEnvelope *rte , sint16 type)
{
P1ExtensionMacroBody *extdata;
struct any_type *evalue;

    if (rte->m & ReportTransferEnvelope_extensions) {
    /* Traverse list of extensions */
    for ( list_first(&(rte->extensions));
	(rte->extensions.current);
	list_next(&(rte->extensions) ) ) 
    {
        extdata = (P1ExtensionMacroBody *)rte->extensions.current->data;
        if (extdata->etype == type) 
	{
            if (extdata->m & ExtensionMacroBody_evalue) 
               evalue = (struct any_type *)(extdata->evalue); 
            else
               evalue = NULL;

            parse_envelope_extensions (evalue, extdata->etype,
	 				&(extdata->extstruct));
            return((char *)(extdata->extstruct));
        } /* end if */
    } /* end for */
    } /* end if */
    return(NULL);
} /* end get_report_extension() */


char *get_per_recip_report_extension(P1PerRecipientReportTransferFields *prr,
                                     sint16 type)
{
    P1ExtensionMacroBody *extdata;
    struct any_type *evalue;

    if (prr->m & PerRecipientReportTransferFields_extensions) {
    /* Traverse list of extensions */
    for ( list_first( & prr->extensions);
          (prr->extensions.current);
          list_next( & prr->extensions))
    {
       extdata = (P1ExtensionMacroBody *)prr->extensions.current->data;

       if (extdata->etype == type) {
           if (extdata->m & ExtensionMacroBody_evalue) 
              evalue = (struct any_type *)(extdata->evalue); 
           else
              evalue = NULL;

           parse_envelope_extensions (evalue, extdata->etype,
        				&(extdata->extstruct));
           return((char *)(extdata->extstruct));
       } /* end if */

    } /* end for */
    } /* end if */
    return(NULL);
} /* end get_per_recip_report_extension() */


get_probe_extension_critical(P1MTS_APDU *mpdu)
{
P1ExtensionMacroBody *extdata;

    if (mpdu->v.probe.m & Probe_extensions) {
    /* Traverse list of extensions */
    for (mpdu->v.probe.extensions.current = mpdu->v.probe.extensions.first;
         mpdu->v.probe.extensions.current->next != NULL;
         mpdu->v.probe.extensions.current = mpdu->v.probe.extensions.current->next) {
        extdata = (P1ExtensionMacroBody *)mpdu->v.probe.extensions.current->data;
        if (extdata->m & ExtensionMacroBody_criticality) {
            return((int)(extdata->criticality));
        } /* end if */
    } /* end for */
    } /* end if */
    return(NULL);
} /* end get_probe_extension_critical() */


char *get_msg_extension(P1MTS_APDU *p1ptr, sint16 type)
{
P1ExtensionMacroBody *extdata;
struct any_type *evalue;

    if (p1ptr->v.message.envelope.m & MessageTransferEnvelope_extensions) 
    {
    /* Traverse list of extensions */
    for ( list_first(&(p1ptr->v.message.envelope.extensions));
	(p1ptr->v.message.envelope.extensions.current);
	list_next(&(p1ptr->v.message.envelope.extensions) ) ) 
	{

        extdata = (P1ExtensionMacroBody *) (p1ptr->v.message.
				envelope.extensions.current->data);
        if (extdata->etype == type) 
	{
            if (extdata->m & ExtensionMacroBody_evalue) 
               evalue = (struct any_type *)(extdata->evalue); 
            else
               evalue = NULL;
	    parse_envelope_extensions (evalue, extdata->etype,
	 				&(extdata->extstruct));
            return((char *)(extdata->extstruct));
        } /* end if */
    } /* end for */
    } /* end if */
    return(NULL);
} /* end get_msg_extension() */


char *get_probe_extension(P1MTS_APDU *p1ptr, sint16 type)
{
P1ExtensionMacroBody *extdata;
struct any_type *evalue;

    if (p1ptr->v.probe.m & Probe_extensions) 
    {
    /* Traverse list of extensions */
    for ( list_first(&(p1ptr->v.probe.extensions));
	(p1ptr->v.probe.extensions.current);
	list_next(&(p1ptr->v.probe.extensions) ) ) 
	{

        extdata = (P1ExtensionMacroBody *) (p1ptr->v.probe.
				extensions.current->data);
        if (extdata->etype == type) 
	{
            if (extdata->m & ExtensionMacroBody_evalue) 
               evalue = (struct any_type *)(extdata->evalue); 
            else
               evalue = NULL;
	    parse_envelope_extensions (evalue, extdata->etype,
	 				&(extdata->extstruct));
            return((char *)(extdata->extstruct));
        } /* end if */
    } /* end for */
    } /* end if */
    return(NULL);
} /* end get_probe_extension() */


char *get_delenvl_extension(P22MessageBodyPart *msgbp, sint16 type)
{
P1ExtensionMacroBody *extdata;
struct any_type *evalue;

    if (msgbp->parameters.delivery_envelope.m & OtherMessageDeliveryFields_extensions)
    {
    /* Traverse list of extensions */
    for ( list_first(&(msgbp->parameters.delivery_envelope.extensions));
	(msgbp->parameters.delivery_envelope.extensions.current);
	list_next(&(msgbp->parameters.delivery_envelope.extensions) ) ) 
	{

        extdata = (P1ExtensionMacroBody *) (msgbp->parameters.delivery_envelope.extensions.current->data);
        if (extdata->etype == type) 
	{
            if (extdata->m & ExtensionMacroBody_evalue) 
               evalue = (struct any_type *)(extdata->evalue); 
            else
               evalue = NULL;
	    parse_envelope_extensions (evalue, extdata->etype,
	 				&(extdata->extstruct));
            return((char *)(extdata->extstruct));
        } /* end if */
    } /* end for */
    } /* end if */
    return(NULL);
} /* end get_delenvl_extension() */


